/*******************************************************************************************************************************
*
* Project             GATT Sample Application
* File                Gatt.cpp
* Description         This file provides GATT profile handle creation and deletion, GATT UI menu, and indication call back 
                      functions.
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClxCommon.h"
#include "ClarinoxBlue.h"
#include "ClxTime.h"

#include "Gap.Api.h"
#include "Gap.Ble.Api.h"
#include "Gatt.Ble.Common.Api.h"
#include "Gatt.Ble.Service.h"
#include "Gatt.Ble.Server.Api.h"
#include "Gatt.Ble.Client.Api.h"

#include "GattApp.h"
#include "GattService.h"

#include "string.h"
#include "stdio.h"
#include <assert.h>

#define FIXED_PIN_CODE
#define FIXED_PIN "0000"

extern void printMemory( u1* data, u4 length, u4 lineLength, u4 columnLength, u4 flags );

ClxBleBdAddress     bleAddress;
static s1 handle_in[128];
static ClxHandle    gattHandle     = 0;
extern ClxCharacteristicList cl;
/*
The content of the advertising data must be conserved during the API operation
*/
static u1 advertisingData[32]; 

#define TEST_DATA_READ_SIZE 256
static u1 testData[TEST_DATA_READ_SIZE];
static ClxSize testSize                       = 0;

u2 testServiceHandleBase       = 0;
u4 testSppHandle               = 0;


/* GAP possible major device classes */
const s1* deviceClassList[] = {"Miscellaneous Equipment", 
							   "Computer", 
							   "Phone", 
							   "LAN Device", 
							   "AudioVideo", 
							   "Peripheral", 
                               "Imaging Device", 
                               "Wearable Device",
                               "Toy",
                               "Health Equipment"};


/*******************************************************************************************************************************
*                                                stackMessageHandler
*
* This call-back function is registered for the GAP and GATT profiles, 
* any events raised by GAP profile causes this call-back function executed with
* the associated event and parameters
*
*******************************************************************************************************************************/
boolean stackMessageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
{
    if( messageID == CLX_GAP_INITIALIZATION_COMPLETE_INDICATION)    /* indication of stack initialization completion          */
    {                                                               /* now ready to call Bluetooth API functions              */
        ClxConsoleUIEngine::text("ClarinoxBlue initialization completed\n");
    }
    else if(messageID == CLX_GAP_TERMINATION_COMPLETE_INDICATION)   /* indication of stack termination completion             */
    {                                                               /* now ready to destroy the stack                         */
        if (errorCode == 0)
        { 
            ClxConsoleUIEngine::text("ClarinoxBlue termination completed\n");
            clxReleaseSemaphore (terminationSemaphore);
        }
        else
        {
            ClxConsoleUIEngine::text("\nStack termination was not successful (error : %s)", clxGetErrorCodeText(errorCode));
        }
    }
    else if (messageID == CLX_GAP_DEVICE_DISCOVERED_INDICATION)     /* indication of a new device being found                 */
    {
        clxGapDeviceDiscoveredIndication* arg = (clxGapDeviceDiscoveredIndication*)params;
        ClxConsoleUIEngine::text("\nDevice Discovered : %s (%s)", 
            arg->deviceName, 
            deviceClassList[arg->majorClassOfDevice]);
    }
    else if (messageID == CLX_GAP_LINK_DISCONNECTION_INDICATION)
    {
        clxGapDeviceDiscoveredIndication* arg = (clxGapDeviceDiscoveredIndication*)params;
        ClxConsoleUIEngine::text("Disconnected\n");
            gattHandle = createGattHandle(stack, 0, 0, false);
            clxGapBleStartScan(stack, BleScanType_Passive, 100, 80, FALSE, FALSE);
    }
     

////////////////////////// BLE HANDLING//////////////////////////////////////

    else if (messageID == CLX_GAP_BLE_MANAGE_WHITE_LIST_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_MANAGE_WHITE_LIST_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_START_SCAN_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_START_SCAN_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_STOP_SCAN_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_STOP_SCAN_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_INITIATE_BONDING_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_INITIATE_BONDING_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_CONNECT_TO_PERIPHERAL_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_CONNECT_TO_PERIPHERAL_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_DISCONNECT_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_DISCONNECT_COMPLETE\n");
        return true; 
    }

    else if (messageID == CLX_GAP_BLE_START_ADVERTISING_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_START_ADVERTISING_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_STOP_ADVERTISING_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_STOP_ADVERTISING_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_SET_SCAN_RESPONSE_DATA_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_SET_SCAN_RESPONSE_DATA_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_SET_BONDABLE_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_SET_BONDABLE_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_GET_LIST_OF_PAIRED_DEVICES_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_GET_LIST_OF_PAIRED_DEVICES_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_PASSKEY_REQUEST_REPLY_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_PASSKEY_REQUEST_REPLY_COMPLETE\n");
        return true; 
    }
        else if (messageID == CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE\n");
        return true; 
    }
        else if (messageID == CLX_GAP_BLE_DELETE_PAIRED_DEVICE_INFO_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_DELETE_PAIRED_DEVICE_INFO_COMPLETE\n");
        return true; 
    }
            else if (messageID == CLX_GAP_BLE_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_ENCRYPT_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_ENCRYPT_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_CONNECTION_ESTABLISHED_BY_REMOTE_CENTRAL_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_CONNECTION_ESTABLISHED_BY_REMOTE_CENTRAL_INDICATION\n");
        ClxGapBleConnectionEstablishedByRemoteCentralIndication* arg = (ClxGapBleConnectionEstablishedByRemoteCentralIndication*)params;

        bleAddress.addressType = (&arg->deviceAddr)->addressType;
        memcpy(bleAddress.value, (&arg->deviceAddr)->value, 6);

        return true; 
    }
    else if (messageID == CLX_GAP_BLE_LINK_DISCONNECTION_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_LINK_DISCONNECTION_INDICATION\n");
        return true; 
    }


    else if (messageID == CLX_GAP_BLE_DEVICE_ADVERTISING_INDICATION)/* print BLE advertising packet and connect to first one found */
    {                                                               
        ClxGapBleDeviceAdvertisingIndicationStruct* arg = (ClxGapBleDeviceAdvertisingIndicationStruct*)params;
        ClxConsoleUIEngine::text("BLE address type (%u), address: ",  arg->deviceAddr.addressType);
        clxGapBleStopScan(stack, FALSE);
        bleAddress.addressType = (&arg->deviceAddr)->addressType;
        memcpy(bleAddress.value, (&arg->deviceAddr)->value, 6);

        clxGapConnectToPeripheral( stack,
                                   &arg->deviceAddr,
                                   FALSE);

	    printMemory( ( u1* )&( arg->deviceAddr.value ), 6, 6, 6, 2 );
        
        ClxConsoleUIEngine::text("--> advertising data:\n");
        printMemory( ( u1* )&( arg->data ), arg->dataLength, 64, 8, 3 );
       
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_PASSKEY_REQUEST_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_PASSKEY_REQUEST_INDICATION\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_PASSKEY_NOTIFICATION_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_PASSKEY_NOTIFICATION_INDICATION\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_INDICATION\n");
        return true; 
    }

    else if (messageID == CLX_GATT_VALUE_UPDATED_INDICATION)
    {
        ClxGattValueUpdatedIndication* params_ = (ClxGattValueUpdatedIndication*)params;

        ClxConsoleUIEngine::text("The value of the handle %x updated\n", params_->characteristicHandle);
        if (clxBleGetServiceHandleBaseFromHandle(params_->characteristicHandle) == testServiceHandleBase)
        {
            ClxSize readSize;
            switch(clxBleGetServiceAttributeIndexFromHandle(params_->characteristicHandle))
            {
            case CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_HANDLE_INDEX:
                /* We can read the updated value here in non-blocking mode: */
                readSize = CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_SIZE;
                break;
            case CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_HANDLE_INDEX:
                readSize = CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_SIZE;
                /* We can read the updated value here in non-blocking mode: */
                break;

            }
            clxGattReadLocal(gattHandle, 
                            params_->characteristicHandle, 
                            testData, 
                            readSize,
                            &testSize, 
                            FALSE);

        }
    }
    else if (messageID == CLX_GATT_READ_LOCAL_COMPLETE)
    {
        if (errorCode == CLX_SUCCESS)
        {

            ClxGattReadLocalComplete* params_ = (ClxGattReadLocalComplete*)params;
            ClxConsoleUIEngine::text("The updated value of testData is: ");
            printMemory( (u1*)params_->characteristicValueBuffer, (u4)(*params_->readCharacteristicValueLength), 16, 4, 2 );
        }
        else
        {
            ClxConsoleUIEngine::text("clxGattReadLocal failed with error %u\n", errorCode);
        }
    }
    else if( messageID == CLX_GATT_CLIENT_GET_LIST_OF_SERVICES_COMPLETE)
    {
        if (errorCode == CLX_SUCCESS)
        {
            ClxGattClientGetListOfServicesComplete* params_ = (ClxGattClientGetListOfServicesComplete*)params;            
            ClxConsoleUIEngine::text("The number of services found: %u\n", params_->result->numberOfServices); 
             for(u1 i = 0; i< params_->result->numberOfServices; i++)
            {
                if(params_->result->list[i].uuid.type == ClxGattUuidType_Uuid16){
                   ClxConsoleUIEngine::text("%u: First Handle: %#x  Last Handle: %#x Uuid: %#x (16 byte)\n", i, params_->result->list[i].firstHandle, params_->result->list[i].lastHandle, (params_->result->list[i].uuid.value[0]) );
                }else{
                    ClxConsoleUIEngine::text("%u: First Handle: %#x  Last Handle: %#x Uuid: %#x \n", i, params_->result->list[i].firstHandle, params_->result->list[i].lastHandle, (params_->result->list[i].uuid.value[0]) ); 
                }
           }
        }
        else
        {
            ClxConsoleUIEngine::text("clxGattGetListOfServices failed with error %u\n", errorCode);
        }
    } 
    else if( messageID == CLX_GATT_CLIENT_READ_CHARACTERISTIC_VALUE_COMPLETE)
    {
        if (errorCode == CLX_SUCCESS)
        {
            ClxGattClientReadCharacteristicValueComplete* params_ = (ClxGattClientReadCharacteristicValueComplete*)params;
            ClxConsoleUIEngine::text("Read Completed Successfully: %u bytes\nData: ", *(params_->readLength)); 
            for(u4 i = 0; i<*params_->readLength; i++){
                ClxConsoleUIEngine::text("%02X ", (params_->buffer[i])); 
            }
            
        }
        else
        {
            ClxConsoleUIEngine::text("clxGattReadCharacteristicValue failed with error %u\n", errorCode);
        }
    }  
    else if( messageID == CLX_GATT_CLIENT_WRITE_CHARACTERISTIC_VALUE_COMPLETE)
    {
        if (errorCode == CLX_SUCCESS)
        {                       
            ClxConsoleUIEngine::text("Write completed successfully\n");            
        }
        else
        {
            ClxConsoleUIEngine::text("clxGattWriteCharacteristicValue failed with error %u\n", errorCode);
        }
    }
    else if( messageID == CLX_GATT_CLIENT_DISCOVER_CHARACTERISTICS_COMPLETE)
    {
        if (errorCode == CLX_SUCCESS)
        {
            
            ClxGattClientDiscoverCharacteristicsComplete* params_ = (ClxGattClientDiscoverCharacteristicsComplete*)params;            
            ClxConsoleUIEngine::text("The number of characteristics found: %u\n", params_->characteristics->numberOfCharacteristics);
            for(u1 i = 0; i< params_->characteristics->numberOfCharacteristics; i++)
            {                
                ClxConsoleUIEngine::text("%u: Declaration Handle: %#x  Properties: %u Value Handle: %#x Uuid: %#x \n", i, params_->characteristics->list[i].declarationHandle, params_->characteristics->list[i].properties, params_->characteristics->list[i].valueHandle, (params_->characteristics->list[i].uuid.value[0]) ); 
            }
        }
        else
        {
            ClxConsoleUIEngine::text("clxGattDiscoverCharacteristics failed with error %u\n", errorCode);
        }
    }

    return true;
}


/*******************************************************************************************************************************
*                                                   createGattHandle
*
* create the GATT profile with the server or client role. 
*
*******************************************************************************************************************************/
ClxHandle createGattHandle(ClxStack stack, s1* deviceName, u2 deviceAppearance, bool server)
{
    if(server)
    {
        gattHandle = clxGattCreateServer( stack, deviceName, deviceAppearance, stackMessageHandler);  

        ClxResult ret = clxGattRegisterLocalService(gattHandle, clxGetBleGattTestServiceInterface(), NULL, &testServiceHandleBase, TRUE);

        ClxConsoleUIEngine::text("Registering the Test GATT service is complete with the result %u\n", ret);

    }
    else
    {
        gattHandle = clxGattCreateClient( stack, stackMessageHandler);  
    }

    return gattHandle;
}

/*******************************************************************************************************************************
*                                                    deleteGattHandle
*
* Delete the GATT profile handle when there is no use for this profile.
*
*******************************************************************************************************************************/
void deleteGattHandle(void)
{

}

/*******************************************************************************************************************************
*                                                     connect
*
* Retrieve the list of paired devices, and display it to the user, if the role is server.
*
*******************************************************************************************************************************/
void connect(ClxHandle stack, ClxDeviceId deviceId, ClxHandle gatt, boolean server)
{

}

/*******************************************************************************************************************************
*                                                    menuFunction
*
* Provide the GATT UI menu.
*
*******************************************************************************************************************************/
void menuFunction(ClxStack stack)
{
    const s1* menu =    "Connect to GATT1 \0"
                        "Write To Characteristic \0"
                        "Read Characteristic \0"
                        "Search for Primary Services\0"
                        "Search For Characteristics\0"
                        "Return to previous menu";
    
    while(1)
    {
        /*
        Display the secondary GATT menu and handle user selection
        */
		u4 index = ClxConsoleUIEngine::showMenu("Enter your selection:", menu, 6); 
        if (index == 0)
        {
        }
        else if (index == 1)
        {
             ClxGattUuid u ;
             clxInitGattUuid2(&u, 0x0000);
            ClxServiceList sl;
            sl.list = new ClxServiceDetail[50];
            clxGattClientGetListOfServices(gattHandle, u, &sl, true);
        }
        else if (index == 2)
        {
           
            //if no write response is required, optionsFlag = 0x1
            ClxCharacteristicDetail cd ;           
            ClxConsoleUIEngine::inputBox ("Enter handle of the characteristic (hex) to write to: ", handle_in, 128);
            //u4 numToRead = atoi(handle_in);
            u4 numToRead;
            sscanf(handle_in, "%x", &numToRead);

            cd.valueHandle = numToRead;
//////////////////////
          //  ClxConsoleUIEngine::inputBox ("Enter number of bytes to write: ", handle_in, 128);
          //  u4 numByte = atoi(handle_in);
//////////////////////
            s1 message[128];
            ClxConsoleUIEngine::inputBox ("enter the text value for the characteristic - beware: NO SIZE CHECK:", message, 128);


            clxGattClientWriteCharacteristicValue(gattHandle,&cd,0,(u1*)message,strlen(message),false); 
#if defined(GATT_SPEED_TEST)
            startTime = clxTickTime();
            ClxConsoleUIEngine::text("\nGATT1 Data sending started at %u\n", startTime);
            ClxConsoleUIEngine::text("GATT1 Sending %u bytes to the remote device . . .\n",
                                     NUMBER_OF_TEST_BLOCKS * sizeof(testBlock));
            for(u4 j = 1; j < NUMBER_OF_TEST_BLOCKS; j++)
            {
                /*
                start sending block of data (either client or server could start send operation)
                */
                clxGattSend( gatt_, testBlock, sizeof(testBlock), FALSE );
            }
            endTime = clxTickTime();
            ClxConsoleUIEngine::text("GATT1 Data sending finished at %u\n", endTime);
            ClxConsoleUIEngine::text("GATT1 Time elapsed : %u milliseconds\n", (endTime - startTime));
            ClxConsoleUIEngine::text("GATT1 Average Transfer Speed : %u bytes/sec, %u bits/s\n\n",
                                     NUMBER_OF_TEST_BLOCKS*TEST_BLOCK_SIZE / (endTime - startTime) * 1000, 
                                     NUMBER_OF_TEST_BLOCKS*TEST_BLOCK_SIZE / (endTime - startTime) * 8000);
#endif
        }
        else if (index == 3)
        {
            ClxGattUuid id;
            clxInitGattUuid2(&id, 0x0000);
            ClxConsoleUIEngine::inputBox ("Enter number of characteristics to read: ", handle_in, 128);
            u4 numToRead = atoi(handle_in);           
            
            
            ClxCharacteristicList cl;
            cl.list = new ClxCharacteristicDetail[numToRead];
            
            cl.numberOfCharacteristics = 0;

            for(u4 i = 0; i<numToRead; i++)
            {
                ClxConsoleUIEngine::inputBox ("Enter the characteristic handle index (hex): ", handle_in, 128);
                //u4 handleIndex_ = atoi(handle_in);
                u4 handleIndex_;
                sscanf(handle_in, "%x", &handleIndex_);

                cl.list[i].valueHandle = handleIndex_;
                cl.numberOfCharacteristics++;
            }
            
           
            u1* buf = new u1[400]; 
            u4 length = 0;

            
            clxGattClientReadCharacteristicValue(gattHandle,id,&cl,buf,400, &length,false);
        }
        else if (index == 4)
        {
            ClxGattUuid u;
            ClxGattUuid id;
            clxInitGattUuid2(&id, 0x0000);
            ClxServiceList sl;
            sl.list = new ClxServiceDetail[100]; //MAX_NUM_SERVICES = 20
            clxGattClientGetListOfServices(gattHandle, id, &sl, false);
            int a = 0;
        }
        else if (index == 5)
        {
            ClxGattUuid u;
            ClxGattUuid id;
            clxInitGattUuid2(&id, 0x0000);
            ClxServiceDetail sd;
            ClxConsoleUIEngine::inputBox ("Enter the first handle of the Service (hex) to search: ", handle_in, 128);
            //u4 handleIndex_ = atoi(handle_in);
            u4 handleIndex_;
            sscanf(handle_in, "%x", &handleIndex_);

            sd.firstHandle = (u2)handleIndex_;
            ClxConsoleUIEngine::inputBox ("Enter the last handle of the Service (hex) to search: ", handle_in, 128);
            //handleIndex_ = atoi(handle_in);
            sscanf(handle_in, "%x", &handleIndex_);

            sd.lastHandle = (u2)handleIndex_;
            
            cl.list = new ClxCharacteristicDetail[100];
            cl.numberOfCharacteristics = 0;           

            clxGattClientDiscoverCharacteristics(gattHandle, id, &sd, &cl, false);

            int a = 0;
        }
        else if (index == 6)
        {
            return;
        }
    }
}

