/*******************************************************************************************************************************
*
* Project             GATT Sample Application
* File                Gatt.cpp
* Description         This file provides GATT profile handle creation and deletion, GATT UI menu, and indication call back 
                      functions.
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClxCommon.h"
#include "ClarinoxBlue.h"
#include "ClxTime.h"

#include "Gap.Api.h"
#include "Gap.Ble.Api.h"
#include "Gatt.Ble.Common.Api.h"
#include "Gatt.Ble.Service.h"
#include "Gatt.Ble.Server.Api.h"
#include "Gatt.Ble.Client.Api.h"

#include "GattApp.h"
#include "GattService.h"

#include "string.h"
#include "stdio.h"
#include <assert.h>

#define FIXED_PIN_CODE
#define FIXED_PIN "0000"

extern void printMemory( u1* data, u4 length, u4 lineLength, u4 columnLength, u4 flags );

ClxBleBdAddress     bleAddress;

static ClxHandle    gattHandle     = 0;

/*
The content of the advertising data must be conserved during the API operation
*/
static u1 advertisingData[32]; 

#define TEST_DATA_READ_SIZE 256
static u1 testData[TEST_DATA_READ_SIZE];
static ClxSize testSize                       = 0;

u2 testServiceHandleBase       = 0;
u4 testSppHandle               = 0;


/* GAP possible major device classes */
const s1* deviceClassList[] = {"Miscellaneous Equipment", 
							   "Computer", 
							   "Phone", 
							   "LAN Device", 
							   "AudioVideo", 
							   "Peripheral", 
                               "Imaging Device", 
                               "Wearable Device",
                               "Toy",
                               "Health Equipment"};


/*******************************************************************************************************************************
*                                                stackMessageHandler
*
* This call-back function is registered for the GAP and GATT profiles, 
* any events raised by GAP profile causes this call-back function executed with
* the associated event and parameters
*
*******************************************************************************************************************************/
boolean stackMessageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
{
    if( messageID == CLX_GAP_INITIALIZATION_COMPLETE_INDICATION)    /* indication of stack initialization completion          */
    {                                                               /* now ready to call Bluetooth API functions              */
        ClxConsoleUIEngine::text("ClarinoxBlue initialization completed\n");
    }
    else if(messageID == CLX_GAP_TERMINATION_COMPLETE_INDICATION)   /* indication of stack termination completion             */
    {                                                               /* now ready to destroy the stack                         */
        if (errorCode == 0)
        { 
            ClxConsoleUIEngine::text("ClarinoxBlue termination completed\n");
            clxReleaseSemaphore (terminationSemaphore);
        }
        else
        {
            ClxConsoleUIEngine::text("\nStack termination was not successful (error : %s)", clxGetErrorCodeText(errorCode));
        }
    }
    else if (messageID == CLX_GAP_DEVICE_DISCOVERED_INDICATION)     /* indication of a new device being found                 */
    {
        clxGapDeviceDiscoveredIndication* arg = (clxGapDeviceDiscoveredIndication*)params;
        ClxConsoleUIEngine::text("\nDevice Discovered : %s (%s)", 
            arg->deviceName, 
            deviceClassList[arg->majorClassOfDevice]);
    }
    else if (messageID == CLX_GAP_LINK_DISCONNECTION_INDICATION)
    {
        clxGapDeviceDiscoveredIndication* arg = (clxGapDeviceDiscoveredIndication*)params;
        
        ClxConsoleUIEngine::text("%s Disconnected\n", arg->deviceName);
        ClxBleBdAddress directAddress;
        directAddress.addressType = ClxBleAddressType_Public;
        memset( directAddress.value, 0, 6);
        s1 shortName[] = "Clx Ble Device";
        advertisingData[0] = strlen(shortName) + 1;
        advertisingData[1] = 9;
        memcpy(&advertisingData[2], shortName, strlen(shortName));
        clxGapBleStartAdvertising(stack,
                                    3000,
                                    4000,
                                    ClxBleAdvertisingType_ConnectableUndirected,
                                    ClxBleAddressType_Public,
                                    &directAddress,
                                    ClxBleAdvertisingFilterPolicy_ScanConnectionAnyone,
                                    advertisingData,
                                    strlen(shortName) + 2,
                                    FALSE );
    }
     

////////////////////////// BLE HANDLING//////////////////////////////////////

    else if (messageID == CLX_GAP_BLE_MANAGE_WHITE_LIST_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_MANAGE_WHITE_LIST_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_START_SCAN_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_START_SCAN_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_STOP_SCAN_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_STOP_SCAN_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_INITIATE_BONDING_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_INITIATE_BONDING_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_CONNECT_TO_PERIPHERAL_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_CONNECT_TO_PERIPHERAL_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_DISCONNECT_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_DISCONNECT_COMPLETE\n");
        return true; 
    }

    else if (messageID == CLX_GAP_BLE_START_ADVERTISING_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_START_ADVERTISING_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_STOP_ADVERTISING_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_STOP_ADVERTISING_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_SET_SCAN_RESPONSE_DATA_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_SET_SCAN_RESPONSE_DATA_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_SET_BONDABLE_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_SET_BONDABLE_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_GET_LIST_OF_PAIRED_DEVICES_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_GET_LIST_OF_PAIRED_DEVICES_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_PASSKEY_REQUEST_REPLY_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_PASSKEY_REQUEST_REPLY_COMPLETE\n");
        return true; 
    }
        else if (messageID == CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE\n");
        return true; 
    }
        else if (messageID == CLX_GAP_BLE_DELETE_PAIRED_DEVICE_INFO_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_DELETE_PAIRED_DEVICE_INFO_COMPLETE\n");
        return true; 
    }
            else if (messageID == CLX_GAP_BLE_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_ENCRYPT_COMPLETE)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_ENCRYPT_COMPLETE\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_CONNECTION_ESTABLISHED_BY_REMOTE_CENTRAL_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_CONNECTION_ESTABLISHED_BY_REMOTE_CENTRAL_INDICATION\n");
        ClxGapBleConnectionEstablishedByRemoteCentralIndication* arg = (ClxGapBleConnectionEstablishedByRemoteCentralIndication*)params;

        bleAddress.addressType = (&arg->deviceAddr)->addressType;
        memcpy(bleAddress.value, (&arg->deviceAddr)->value, 6);

        return true; 
    }
    else if (messageID == CLX_GAP_BLE_LINK_DISCONNECTION_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_LINK_DISCONNECTION_INDICATION\n");
        return true; 
    }


    else if (messageID == CLX_GAP_BLE_DEVICE_ADVERTISING_INDICATION)/* print BLE advertising packet and connect to first one found */
    {                                                               
        ClxGapBleDeviceAdvertisingIndicationStruct* arg = (ClxGapBleDeviceAdvertisingIndicationStruct*)params;
        ClxConsoleUIEngine::text("BLE address type (%u), address: ",  arg->deviceAddr.addressType);
        clxGapBleStopScan(stack, FALSE);
        bleAddress.addressType = (&arg->deviceAddr)->addressType;
        memcpy(bleAddress.value, (&arg->deviceAddr)->value, 6);

        clxGapConnectToPeripheral( stack,
                                   &arg->deviceAddr,
                                   FALSE);

	    printMemory( ( u1* )&( arg->deviceAddr.value ), 6, 6, 6, 2 );
        
        ClxConsoleUIEngine::text("--> advertising data:\n");
        printMemory( ( u1* )&( arg->data ), arg->dataLength, 64, 8, 3 );
       
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_PASSKEY_REQUEST_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_PASSKEY_REQUEST_INDICATION\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_PASSKEY_NOTIFICATION_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_PASSKEY_NOTIFICATION_INDICATION\n");
        return true; 
    }
    else if (messageID == CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_INDICATION)
    {                                                               
        ClxConsoleUIEngine::text("CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_INDICATION\n");
        return true; 
    }

    else if (messageID == CLX_GATT_VALUE_UPDATED_INDICATION)
    {
        ClxGattValueUpdatedIndication* params_ = (ClxGattValueUpdatedIndication*)params;

        ClxConsoleUIEngine::text("The value of the handle %x updated\n", params_->characteristicHandle);
        if (clxBleGetServiceHandleBaseFromHandle(params_->characteristicHandle) == testServiceHandleBase)
        {
            ClxSize readSize;
            switch(clxBleGetServiceAttributeIndexFromHandle(params_->characteristicHandle))
            {
            case CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_HANDLE_INDEX:
                /* We can read the updated value here in non-blocking mode: */
                readSize = CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_SIZE;
                break;
            case CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_HANDLE_INDEX:
                readSize = CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_SIZE;
                /* We can read the updated value here in non-blocking mode: */
                break;

            }
            clxGattReadLocal(gattHandle, 
                            params_->characteristicHandle, 
                            testData, 
                            readSize,
                            &testSize, 
                            FALSE);

        }
    }
    else if (messageID == CLX_GATT_READ_LOCAL_COMPLETE)
    {
        if (errorCode == CLX_SUCCESS)
        {

            ClxGattReadLocalComplete* params_ = (ClxGattReadLocalComplete*)params;
            ClxConsoleUIEngine::text("The updated value of testData is: ");
            printMemory( (u1*)params_->characteristicValueBuffer, (u4)(*params_->readCharacteristicValueLength), 16, 4, 2 );
        }
        else
        {
            ClxConsoleUIEngine::text("clxGattReadLocal failed with error %u\n", errorCode);
        }
    }

    return true;
}


/*******************************************************************************************************************************
*                                                   createGattHandle
*
* create the GATT profile with the server or client role. 
*
*******************************************************************************************************************************/
ClxHandle createGattHandle(ClxStack stack, s1* deviceName, u2 deviceAppearance, bool server)
{
    gattHandle = clxGattCreateServer( stack, deviceName, deviceAppearance, stackMessageHandler);  

    ClxResult ret = clxGattRegisterLocalService(gattHandle, clxGetBleGattTestServiceInterface(), NULL, &testServiceHandleBase, TRUE);
    ClxConsoleUIEngine::text("Registering the Test GATT service is complete with the result %u\n", ret);
    return gattHandle;
}



