/******************************************************************************
*
* Project             SPP Sample Application
* File                Spp.cpp
* Description         This file provides SPP profile handle creation / deletion, 
                      SPP UI menu, and indication call back functions.
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/


/*******************************************************************************************************************************
a.	Spp.cpp has a call-back function (stackMessageHandler) that handles stack events. 
    clxSppReceive() needs to be invoked to prepare a receive. 
    The receive completed event (CLX_SPP_RECEIVE_COMPLETE) indicates reception of data and provides the buffer 
    (with the data received from the other device). This call-back function should not be blocked. 
    If the call after receiving data needs to be blocking, then the received buffer can be passed into a queue and 
    signal a semaphore to wake up another task and do the rest of the processing in that task. 

b.	To send a buffer from another task, clxSppSend() needs to be called for sending a buffer (after the connection is made; 
    i.e. event CLX_SPP_CONNECTION_INDICATION is received)  and wait for 	CLX_SPP_SEND_COMPLETE event. 
    This way is preferred for interfacing in a multi-threaded architecture. 
    Send and receive calls can be used in blocking mode from a single thread as well but it is more restrictive.
*******************************************************************************************************************************/

#include "ClxCommon.h"
#include "ClarinoxBlue.h"
#include "ClxTime.h"

#include "Gap.Api.h"
#include "Spp.Api.h"

#include "SppApp.h"

#include "string.h"
#include "stdio.h"
#include <assert.h>
#if defined( CLX_UCOS_III )
#       include  <os.h>  
#endif
#define FIXED_PIN_CODE
#define FIXED_PIN "0000"
#define MAX_TEXT_SIZE 30
extern u2 connectedDeviceId;

bool echoMode     = false;

#define RX_BUFFER_SIZE TEST_BLOCK_SIZE
/*
A buffer to receive and send SPP data
*/
static u1       sppReceive[RX_BUFFER_SIZE];
static ClxSize  sppReceiveDataSize = 0;
static u1       testBlock[TEST_BLOCK_SIZE];

/*
A buffer to store SPP stream data
*/
u1 rxBuffer[2*TEST_BLOCK_SIZE];
static ClxSize totalLength = 0;

static ClxHandle sppHandle = 0;
#define FIXED_PIN_CODE

u4 dataCount = NUMBER_OF_TEST_BLOCKS*TEST_BLOCK_SIZE;
#if defined( SPP_SPEED_TEST )

void viewReceivedSPP(void);
static u4 startTime;
static u4 endTime;
#endif

#define AUTO_ACCEPT_CONNECTION_REQUEST

s1 in[MAX_TEXT_SIZE];

/* SPP current connected device id */
u2 connectedDeviceId = 0;
ClxSemaphore connectionSemaphore;
/* SPP state machine state enumeration */
enum ReceiverState
{
	ReceiverState_Idle,
	ReceiverState_Receiving
} receiverState = ReceiverState_Idle;

/* GAP possible major device classes */
const s1* deviceClassList[] = {"Miscellaneous Equipment", 
							   "Computer", 
							   "Phone", 
							   "LAN Device", 
							   "AudioVideo", 
							   "Peripheral", 
                               "Imaging Device", 
                               "Wearable Device",
                               "Toy",
                               "Health Equipment"};



//extern "C" boolean clxBluetoothIntegrity_HardResetController();

/*******************************************************************************************************************************
*                                                stackMessageHandler
*
* This call-back function is registered for the GAP and SPP profiles, 
* any events raised by GAP profile causes this call-back function executed with
* the associated event and parameters
*
*******************************************************************************************************************************/
boolean stackMessageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
{
   if( messageID == CLX_GAP_INITIALIZATION_COMPLETE_INDICATION)    /* indication of stack initialization completion          */
    {                                                               /* now ready to call Bluetooth API functions              */
        ClxConsoleUIEngine::text("ClarinoxBlue initialization completed\n");
    }
    else if(messageID == CLX_GAP_TERMINATION_COMPLETE_INDICATION)   /* indication of stack termination completion             */
    {                                                               /* now ready to destroy the stack                         */
        if (errorCode == 0)
        { 
            ClxConsoleUIEngine::text("ClarinoxBlue termination completed\n");
            clxReleaseSemaphore (terminationSemaphore);
        }
        else
        {
            ClxConsoleUIEngine::text("\nStack termination was not successful (error : %s)", clxGetErrorCodeText(errorCode));
        }
    }
    else if (messageID == CLX_GAP_START_INQUIRY_COMPLETE)           /* indication of each inquiry cycle completion in the     */
    {                                                               /* periodic inquiry mode                                  */
        return true; 
    }
    else if (messageID == CLX_GAP_DEVICE_DISCOVERED_INDICATION)     /* indication of a new device being found                 */
    {
        clxGapDeviceDiscoveredIndication* arg = (clxGapDeviceDiscoveredIndication*)params;
        s1* deviceClass = "Uncategorized Device";
        if (arg->majorClassOfDevice != MJ_UNCATEGORIZED)
        {
            deviceClass = (s1*)deviceClassList[arg->majorClassOfDevice];
        }
        ClxConsoleUIEngine::text("\nDevice Discovered : %s (%s)", 
            arg->deviceName, 
            deviceClass);
    }
    else if (messageID == CLX_GAP_PIN_CODE_REQUEST_INDICATION)      /* indication of a legacy pin code request during pairing */
    {                                                               /* operation                                              */
        clxGapPinCodeRequestIndication* arg = (clxGapPinCodeRequestIndication*)params;
#if defined(FIXED_PIN_CODE)
        s1 pincode[5] = "0000";
#else
        s1 pincode[16];
        ClxConsoleUIEngine::inputBox ("\nEnter PIN code : ", pincode, sizeof(pincode));
#endif

        clxGapPinCodeRequestReply(stack, arg->deviceId, (const u1*)pincode, strlen(pincode), FALSE);
    }
    else if (messageID == CLX_GAP_USER_PASSKEY_REQUEST_INDICATION)  /* indication of a Simple Secure Pairing pass key request */
    {                                                               /* during pairing operation                               */
        clxGapUserPasskeyRequestIndication* arg = (clxGapUserPasskeyRequestIndication*)params;

        u4 passkey;
        s1 passkeyStr[10];
        ClxConsoleUIEngine::inputBox ("\nEnter User Passkey (numeric) : ", passkeyStr, sizeof(passkeyStr));
        sscanf(passkeyStr, "%u", &passkey);

        clxGapUserPasskeyRequestReply(stack, arg->deviceId, TRUE, passkey, FALSE);
    }
    else if (messageID ==                                           /* indication of a Simple Secure Pairing pass key         */
             CLX_GAP_USER_PASSKEY_NOTIFICATION_INDICATION)          /* notification during pairing operation                  */
    {
        clxGapUserPasskeyNotificationIndication* arg = (clxGapUserPasskeyNotificationIndication*)params;
		
        ClxConsoleUIEngine::text ("\nPasskey for the device \"%s\" is \"%u\"\n", arg->deviceDetail.deviceName, arg->value);
    }
    else if (messageID ==                                           /* indication of a Simple Secure Pairing pass key         */
             CLX_GAP_USER_CONFIRMATION_REQUEST_INDICATION)          /* confirmation request during pairing operation          */
    {
        clxGapUserConfirmationRequestIndication* arg = (clxGapUserConfirmationRequestIndication*)params;

        ClxConsoleUIEngine::text ("\n Please confirm the passkey \"%u\" to connect to \"%s\" ?",
                                  arg->value, 
                                  arg->deviceDetail.deviceName);
        s1 ch = ClxConsoleUIEngine::messageBox("", "yYnN", 4);
        if ((ch == 'y') || (ch == 'Y'))
        {
            clxGapUserConfirmationRequestReply(stack, arg->deviceDetail.deviceId, TRUE, FALSE);
            ClxConsoleUIEngine::text("\nAuthentication completed \n");
        }
        else
        {
            clxGapUserConfirmationRequestReply(stack, arg->deviceDetail.deviceId, FALSE, FALSE);
            ClxConsoleUIEngine::text("\nAuthentication error\n");
        }
        ClxConsoleUIEngine::text ("\n");
    }
    else if (messageID ==                                           /* indication of a remote connection authorization        */
        CLX_GAP_INCOMING_CONNECTION_REQUEST_INDICATION)
    {
        clxGapIncomingConnectionRequestIndication* arg = (clxGapIncomingConnectionRequestIndication*)params;

#if defined(AUTO_ACCEPT_CONNECTION_REQUEST)
        const s1 ch = 'Y';
#else
        ClxConsoleUIEngine::text("\nA connection request received from %s.Accept(y), Reject(n) or don't respond(x)", 
            arg->deviceName);

        s1 ch = ClxConsoleUIEngine::messageBox("", "yYnNxX", 6); 
#endif
        if ((ch == 'y') || (ch == 'Y'))
        {
            connectedDeviceId = arg->deviceId;
            clxGapAcceptConnectionRequest(stack, arg->deviceId, FALSE);			
			
        }
        else if((ch == 'n') || (ch == 'N'))
        {
            clxGapRejectConnectionRequest(stack, arg->deviceId, ClxUnspecified, FALSE);
            ClxConsoleUIEngine::text("\nA connection request rejected from %s.", arg->deviceName);
        }
        else if((ch == 'x') || (ch == 'X'))
        {
            ClxConsoleUIEngine::text("\nConnection request not responded %s.", arg->deviceName);
        }
    }
    else if (messageID == 
            CLX_GAP_ACCEPT_CONNECTION_REQUEST_COMPLETE)             /* indication of accepting an incoming connection         */
    {
        if (errorCode == 0)
		{
            ClxConsoleUIEngine::text("\nConnection was successful");
			
		}
		else
		{
			connectedDeviceId = 0;
			ClxConsoleUIEngine::text("\nConnection was not successful (error : %u)", errorCode);
		}
	}
    /*
    indication of a SPP profile connection
    */
	
	else if (messageID == CLX_SPP_CONNECTION_INDICATION)
	{
        clxSppReceive(serviceHandle, RX_BUFFER_SIZE, sppReceive, &sppReceiveDataSize, FALSE);
        clxGapSetConnectability(stack, FALSE, FALSE);
        clxGapSetDiscoverability(stack, FALSE, FALSE);
        ClxConsoleUIEngine::text ("\nSPP1 Connected to Bluetooth Device.\n");
		if(connectionSemaphore!=NULL)
						clxReleaseSemaphore (connectionSemaphore);
    }
    else if (messageID == CLX_SPP_DISCONNECTION_INDICATION)
    {
        ClxSppConnectionIndication* arg = (ClxSppConnectionIndication*)params;
        ClxConsoleUIEngine::text ("\nSPP1 with the device ID: %d Disconnected from the Bluetooth Device.\n", arg->deviceId);
    }
    else if (messageID == CLX_SPP_DISCONNECT_COMPLETE)
    {
        if (errorCode == 0)
        {
            ClxConsoleUIEngine::text("\nDisconnection was successful");
        }
        else
        {
            ClxConsoleUIEngine::text ("\nclxSppDisconnect failed with error %d (%s)\n",
                                      errorCode,
                                      clxGetErrorCodeText(errorCode));
        }
    }
    else if (messageID == CLX_SPP_SEND_COMPLETE)
    {
		if (errorCode != CLX_SUCCESS)
        {
            ClxConsoleUIEngine::text ("\nSPP1 clxSppSend completed with error %d (%s)\n", 
                                      errorCode, 
                                      clxGetErrorCodeText(errorCode));
        }
        // clxReleaseSemaphore(sendSemaphore);
    }
    /*
    indication of SPP data reception
    */
	else if (messageID == CLX_SPP_RECEIVE_COMPLETE)
	{
		
		if (errorCode == CLX_SUCCESS )
        {
			
            ClxSppReceiveComplete* arg = (ClxSppReceiveComplete*)params;
            u1* rxPointer    = rxBuffer + totalLength;
            totalLength     += *arg->receivedDataLength;
			ClxSize copyLen = *arg->receivedDataLength;
			memcpy(rxPointer, arg->buffer, copyLen);
			printf("\nReceived Text : %s \n",rxPointer); 
            if (totalLength == MAX_TEXT_SIZE)
            {
                //ClxConsoleUIEngine::text("full size received %d\n", arg->dataLen);
                /*
                Copy the buffer, verify checksum and signal the sender
                */
                u4 remaining = totalLength - MAX_TEXT_SIZE;
                ClxSize copyLen = *arg->receivedDataLength;
                if ( remaining != 0 )
                {
                    copyLen = totalLength - MAX_TEXT_SIZE;
                }
                memcpy(rxPointer, arg->buffer, copyLen);
             /*   #if defined(SPP_SPEED_TEST)
                    
                    initialize the test block and calculate checksum 0..3 is used for header, 4..7 used for checksum (endianness must be same)
                    
                    u4 calculatedChecksum = 0;
                    for (u4 i = 8; i < TEST_BLOCK_SIZE; i++)
                    {
                        calculatedChecksum += (u4)(sppReceive[i]);
                    }
                    
                    Checksum field starts after first 4 bytes, first 4 bytes are reserved for signature
                    
                    u4 receivedChecksum = rxBuffer[4] + u4(rxBuffer[5]) * 0x100 + u4(rxBuffer[6]) * 0x10000 + (u4)rxBuffer[7] * 0x1000000;
                    if(receivedChecksum != calculatedChecksum)
                    {
                        ClxConsoleUIEngine::text("SPP1 Checksum mismatch received checksum - 0x%x, calculated checksum - 0x%x\n",
                                                 receivedChecksum, 
                                                 calculatedChecksum);
                    }
                    else
                    {
                        // Comment out to improve the speed
                        //ClxConsoleUIEngine::text("successfully received\n");
                    } 
                #endif*/
                if ( remaining != 0 )
                {
                    copyLen = MAX_TEXT_SIZE;
                    memcpy(rxBuffer, arg->buffer + (*arg->receivedDataLength - remaining), remaining);
                    totalLength = remaining;
                }
                else
                {
                    totalLength = 0;
                }
				u1 i = 0;
				while(rxBuffer[i] != 0xcc)
						{				
							i++;
						}
				s1* in = new s1[i];
				const s1* in1 = in;
				i=0;
			
				while(rxBuffer[i] != 0xcc)
				{
					in[i] = rxBuffer[i];
					i++;
				}
				

				
				ClxConsoleUIEngine::text("\n Received: %s\n",in1);
				
				
                clxReleaseSemaphore(sendSemaphore); 
            }
            else
            {
               // ClxConsoleUIEngine::text("half size received %d\n", arg->dataLen);
                /*
                Copy the buffer and wait for the remaining to be completed
                */
              //  memcpy(rxPointer, arg->buffer, arg->dataLen);
            }
			if (totalLength != 0){
				clxSppReceive(serviceHandle, RX_BUFFER_SIZE, sppReceive, &sppReceiveDataSize, FALSE);
			}
			if(errorCode != CLX_SUCCESS)
			{
				 ClxConsoleUIEngine::text(clxGetErrorCodeText(errorCode));
			}

        }
        else
        {
            ClxConsoleUIEngine::text ("\nSPP1 clxSppReceive completed with error %d (%s)\n", 
                                      errorCode, 
                                      clxGetErrorCodeText(errorCode));
        }
    }
    return true;
}

static boolean server_ = true; //default is server, if client will change on connection
static ClxHandle spp_;

/*******************************************************************************************************************************
*                                                   createSppHandle
*
* create the SPP profile with the server or client role. This will advertise the SPP functionality via the SDP server
*
*******************************************************************************************************************************/
ClxHandle createSppHandle(ClxStack stack, bool server)
{
#if defined(SPP_SPEED_TEST)
    /*
    initialize the test block and calculate checksum 0..3 is used for header, 4..7 used for checksum 
    endianness must be the same for both sending and receiving devices
    */
    memset(testBlock, (s4)HEADER_SIGNATURE, sizeof(u4));
    u4 checksum = 0;
    for (u4 i = sizeof(u4); i < TEST_BLOCK_SIZE; i++)
    {
        if (i > 7)
        {
            checksum += (i & 0xFF);
        }
        testBlock[i] = i;
    }
    *(u4*)(&testBlock[4]) = checksum ;

#endif

    if(!server)
    {
        if( sppHandle == 0)
        {
            sppHandle = clxSppCreate(stack, NULL, "Port1", ClxClient, stackMessageHandler, ClxMediumSecurity);
        }
    }
    else
    {
        if( sppHandle == 0)
        {
            sppHandle = clxSppCreate(stack, NULL, "Port1", ClxServer, stackMessageHandler, ClxMediumSecurity);
        }
    }
    spp_ = sppHandle;
    return sppHandle;
}

void viewReceivedSPP(void){
	
	
	

}

/*******************************************************************************************************************************
*                                                    deleteSppHandle
*
* Delete the SPP profile handle when there is no use for this profile.
*
*******************************************************************************************************************************/
void deleteSppHandle(void)
{
    if( sppHandle != 0)
    {
        clxCloseHandle(sppHandle);
        sppHandle = 0;
        spp_ = 0;
    }
}

/*******************************************************************************************************************************
*                                                     connect
*
* Retrieve the list of paired devices, and display it to the user, if the role is server.
*
*******************************************************************************************************************************/
void connect(ClxHandle stack, ClxDeviceId deviceId, ClxHandle spp, boolean server)
{
    ClxDeviceDetail* list = NULL;
	u4*              size = NULL;
    server_ = server;

	if(!stack)
    {
        ClxConsoleUIEngine::text ("\nStack hasn't been created!\n");
    }

    if (!server_)
    {
		list = new ClxDeviceDetail[10];
		size = new u4;    
		*size = 0;
        u4 numberOfPairedDevices = 0;
        clxGapGetListOfPairedDevices(stack, list, 10, size, TRUE);

		delete[] list;
		list = NULL;
		numberOfPairedDevices = *size;
        
        if (numberOfPairedDevices) 
        {
            list = new ClxDeviceDetail[numberOfPairedDevices];
			clxGapGetListOfPairedDevices(stack, list, numberOfPairedDevices, size, TRUE); 

            while(1)
            {
                const s1 returnText[] = {"Return to previous menu"};

                s1* menuItems = new s1[(numberOfPairedDevices * 128) + sizeof(returnText)];
                
                u4 index = 0;
                for (u4 i = 0; i < numberOfPairedDevices; i++)
                {
                    sprintf (menuItems + index, "%s (%s)", list[i].deviceName, deviceClassList[list[i].majorClassOfDevice]);
                    index += (strlen(menuItems + index) + 1);
                }
                
                strcpy (menuItems + index, returnText);
              
                u4 selection = 0;

                selection = ClxConsoleUIEngine::showMenu("Please select a device", menuItems, numberOfPairedDevices + 1);

                delete[] menuItems;

                connectedDeviceId = list[selection-1].deviceId;
                break;
            }

            delete[] list;
        }
        else
        {
            ClxConsoleUIEngine::text ("\nThere is no paired device\n");
            return;
        }
    }
}

/*******************************************************************************************************************************
*                                                    menuFunction
*
* Provide the SPP UI menu.
*
*******************************************************************************************************************************/
void menuFunction(ClxStack stack, bool waitForConnection)
{
    const s1* menu =    "Connect to SPP1 \0"
                        "Send Text \0"
                        "Send Test Data \0"
                        "Disconnect from SPP1 \0"
                        "Hard reset\0"						
                        "Return to previous menu";
    ClxError ret;
	if(waitForConnection){
		connectionSemaphore = clxCreateSemaphore(0);
		clxAcquireSemaphore(connectionSemaphore);
        clxDeleteSemaphore(connectionSemaphore);
	}
    while(1)
    {
        /*
        Display the secondary SPP menu and handle user selection
        */
		
		u4 index = ClxConsoleUIEngine::showMenu("Enter your selection:", menu, 6); 
        if (index == 0)
        {
        }
        else if (index == 1)
        {
            /*
            Assuming maximum 12 SPP ports on the remote device
            */
            ClxRemoteComPort ports[12];
            u4 numberOfDiscoveredPorts = 0;

            if (!server_)
            {
                ClxConsoleUIEngine::text ("Discovering COM ports on the remote device . . .\n");
                /*
                Discover remote comm ports via SDP operation, if a suitable SPP port is found then 
                connect to that port by using the parameters retrieved via SDP operation
                This is only applicable to client side
                */

                ret = clxSppDiscoverRemoteComPorts(spp_, 
                                                   NULL, 
                                                   connectedDeviceId, 
                                                   sizeof(ports)/sizeof(ClxRemoteComPort),
                                                   ports, 
                                                   &numberOfDiscoveredPorts,
                                                   TRUE);
        
                if (ret != CLX_SUCCESS)
                {
                    ClxConsoleUIEngine::text ("\nError : clxSppDiscoverRemoteComPorts failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
                }

                if (numberOfDiscoveredPorts)
                {
                    ClxConsoleUIEngine::text ("Connecting to COM port \"%s\" on the remote device . . .\n", ports[0].portName);
                    ret = clxSppConnect(spp_, connectedDeviceId, ports[0].portChannelId, TRUE);
                    if (ret != CLX_SUCCESS)
                    {
                        ClxConsoleUIEngine::text("SPP connection failed\n");
                        ClxConsoleUIEngine::text ("\nError : clxSppConnect failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
                    }
                    else
                    {
                        /*
                        Start the asynchronous receive
                        */
                        clxGapSetConnectability(stack, FALSE, FALSE);
                        clxGapSetDiscoverability(stack, FALSE, FALSE);
                        ret = clxSppReceive(spp_, RX_BUFFER_SIZE, sppReceive, &sppReceiveDataSize, FALSE);
                        if (ret != CLX_SUCCESS && ret != CLX_ERROR_COMPLETION_PENDING)
                        {
                            ClxConsoleUIEngine::text("SPP buffer receive failed\n");
                            ClxConsoleUIEngine::text ("\nError : clxSppReceive failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
                        }
                    }
                }
                else
                {
                    ClxConsoleUIEngine::text ("No SPP COM ports discovered\n");
                    return;
                }
            }
            else
            {
                /* 
                Do nothing if client
                */
                ret = CLX_SUCCESS;                                  
            }
        }else if (index == 2)
        {
			/////get input
			const s1 message[21] = "Enter data to send: ";
			ClxConsoleUIEngine::inputBox (message, in, MAX_TEXT_SIZE);

			ret = clxSppSend( spp_, (u1*)in, strlen(in), TRUE );
            if (ret != CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text ("\nError : clxSppSend failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
            }  
		}
        else if (index == 3)
        {
            for (u4 i = 0; i < sizeof(testBlock); i++)
            {
                testBlock[i] = (u1)i;
            }

            u4 start = clxTickTime();

            for (u4 i = 0; i < 100; i++)
            {
                ret = clxSppSend(spp_, testBlock, sizeof(testBlock), TRUE);
                if (ret != CLX_SUCCESS)
                {
                    ClxConsoleUIEngine::text ("\nError : clxSppSend failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
                    break;
                }
            }

            u4 end = clxTickTime();
            ClxConsoleUIEngine::text ("\nSpeed = %u Bytes/Sec", 100*sizeof(testBlock) / (end - start) * 1000);
        }
        else if (index == 4)
        {
            ret = clxSppDisconnect(spp_, TRUE);
            if (ret != CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("Disconnection failed\n");
                ClxConsoleUIEngine::text ("\nError : clxSppDisconnect failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
            }
        }
       
        else if (index == 5)
        {
            //clxBluetoothIntegrity_HardResetController();
        }
      
		else if (index == 6)
        {
            return;
        }
    }
}

