/*******************************************************************************
*
* Project             Bluetooth Application
* File                Bsp.cpp
* Description         Bsp related functions and variables are defined here.
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  1022
// _____________________________________________________________________________
//

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>

#include "ClxBsp.h"
#include "ClarinoxBlue.h"

#include "stm32f4xx_conf.h"

#define DEFAULT_BLUETOOTH_DRIVER_HIGH_TASK_PRIORITY                 1
#define DEFAULT_BLUETOOTH_DRIVER_MEDIUM_HIGH_TASK_PRIORITY          3
#define DEFAULT_BLUETOOTH_DRIVER_MEDIUM_TASK_PRIORITY               5
#define DEFAULT_BLUETOOTH_DRIVER_MEDIUM_LOW_TASK_PRIORITY           7
#define DEFAULT_BLUETOOTH_DRIVER_LOW_TASK_PRIORITY                  9

#define PRIVATE_HEAP_START_ADDRESS                                  0x20000000  
#define PRIVATE_HEAP_SIZE                                           0x2FFFF

/* 
 * Use a custom memory pool set: as follows 
 * The first element is the user buffer size and the second one is the no of buffers required
 * Buffer size must be in ascending order
 * Buffer size must be a multiple of 4
 * Use Clarinox Debugger to tune the buffer size and numbers
 */
USER_SUPPLIED_MEMORY_POOL(
{ 4     , 27   },     //0
{ 8     , 45   },     //1
{ 12    , 25   },     //2
{ 16    , 30   },     //3
{ 20    , 58   },     //4
{ 24    , 31   },     //5
{ 28    , 32   },     //6
{ 32    , 15   },     //7
{ 40    , 40   },     //8
{ 44    , 60   },     //9
{ 56    , 20   },     //10
{ 60    , 5    },     //11
{ 92    , 56   },     //12
{ 128   , 15   },     //13
{ 196   , 15   },     //14
{ 208   , 3    },     //15
{ 224   , 5    },     //16
{ 244   , 4    },     //17
{ 288   , 4    },     //18
{ 292   , 3    },     //19
{ 356   , 6    },     //20
{ 412   , 4    },     //21
{ 452   , 2    },     //22
{ 520   , 5    },     //23
{ 672   , 6    },     //24
{ 724   , 2    },     //25
{ 896   , 3    },     //26
{ 1284  , 7    },     //27
{ 1320  , 1    },     //28
{ 1712  , 2    },     //29
{ 2000  , 3    },     //30
{ 2640  , 1    },     //31
{ 4000  , 3    },     //32
{ 4604  , 2    },     //33
{ 5128  , 1    },     //34
{ 10000 , 1    });    //35

/* 
 * Provide configuration filenames for SoftFrame and Bluetooth components
 */
static const s1* bluetoothConfigFileName                = "ClarinoxBlue.cfg";

#ifdef __cplusplus
extern "C" {
#endif  

 /**
 Assigning Input clock 24MHz and CPU clock ( 180MHz for STM32F429 / 168MHz for STM32F407 ) defined in the compiler.
 */
#if defined(CLX_EXTERNAL_OSC)
u4 Clx_Input_Clock = CLX_EXTERNAL_OSC;
#else
#error "Please define the input clock"
#endif

#if defined(CLX_CPU_CLOCK)
u4 Clx_CPU_Clock = CLX_CPU_CLOCK;
#else
#error "Please define the CPU clock"
#endif

extern const s1*            clxClarinoxBlueConfigFileName;

extern void                 initROMFileSystemMap();
extern ClxFileBspInterface  clxMemoryFileInterface;
extern u1                   clxInitBluetoothController( void );
extern ClxUartBspInterface  clxUartSTM32F407BspInterface;
#ifdef CLX_KOALA_LCD
extern void initializeLcd ( const u1* lcdAppName );
#endif

#ifdef CLX_KOALA_CAMERA
extern void initializeCamera (void);
#endif

extern void (*gnssRxHandler) (const u1* data, u4 length);

void gnssRxHandlerFunction (const u1* data, u4 length)
{
  // here is the handle for the GNSS packets
  // data must be copied to the user buffers upon return from this function 
  //    i.e. obtain some GNSS buffer
  //         memcpy(gnssBuffer, data, length);
  // then the copied buffer can be passed to the GNSS code
  
}

extern u2 clarinoxuCosSoftFrameTaskStackSizeTable[4];
extern u2 clarinoxuCosBluetoothTaskStackSizeTable[3];


/**
The function which is called by ClarinoxBlue during the stack initialization in order to send
vendor specific HCI commands to the Bluetooth controller.

\return 1 : the vendor specific HCI commands were successfully sent to the controller and proper responses were received
from the controller.
        0 : vendor specific command(s) could not be successfully sent to the Bluetooth controller, or a proper response was
		not received in timely manner.

\remark If this function returns 0, the stack initialization will be immediately complete with failure.
*/
extern boolean ( *clxHciSendVendorSpecificCommands)( void );

/**
An array of 5, and type u2, which maps the ClarinoxBlue platform-independent thread priorities
to underlying platform priorities. The first index (index 0) of the array represents the highest priority,
while the last index (index 4) of the array represents the lowest priority.
If the table is not defined in the BSP, the default priority of the underlying platform will be used for all
threads. 

ALL priorities MUST be defined.

ClarinoxBlue threads:
- Driver thread                         : Highest priority (index 0)
- Stack thread                          : Medium priority (index 2)
- Application Callback thread           : Medium priority (index 2)
- Timer thread                          : High priority (index 1)
- Debug thread (if available)           : Lowest priority (index 4)
- Console Engine thread (if available)  : Lowest priority (index 4)
*/
u2 clarinoxBlueIntegrityPlatformTaskPriorityTable[5]	= { DEFAULT_BLUETOOTH_DRIVER_HIGH_TASK_PRIORITY,
                                                            DEFAULT_BLUETOOTH_DRIVER_MEDIUM_HIGH_TASK_PRIORITY,
                                                            DEFAULT_BLUETOOTH_DRIVER_MEDIUM_TASK_PRIORITY,
                                                            DEFAULT_BLUETOOTH_DRIVER_MEDIUM_LOW_TASK_PRIORITY,
                                                            DEFAULT_BLUETOOTH_DRIVER_LOW_TASK_PRIORITY }; 

/*******************************************************************************************************************************
*                                                    PrivateHeap
*
*  Allocating a private heap of size 192KB for STM32F429 / 128KB for STM32F407 
*
*******************************************************************************************************************************/
static char* privateHeapPtr = (char*)PRIVATE_HEAP_START_ADDRESS;


void clxMemoryInitPrivateHeap  (void)
{
	privateHeapPtr = (char*)PRIVATE_HEAP_START_ADDRESS;
}

void clxMemoryDestroyPrivateHeap  (void)
{
	privateHeapPtr = (char*)PRIVATE_HEAP_START_ADDRESS;
}

void* clxMemoryAllocateFromPrivateHeap(unsigned int size)
{
	void* ret = privateHeapPtr;
	privateHeapPtr += size;		
	if( (privateHeapPtr - (char*)PRIVATE_HEAP_START_ADDRESS) > PRIVATE_HEAP_SIZE )
        {
          privateHeapPtr = 0;
	/*
         *Loop when Private Heap overflows
         */
          while(1);
        }
	return ret;
}

void clxMemoryFreeFromPrivateHeap(void* buff)
{}

/*******************************************************************************************************************************
*                                                    userPrintf
*
* User can overload the default printf function used by the stack. if userPrintfFunction (function pointer) is not assigned, 
* then the platform default printf will be used
*
*******************************************************************************************************************************/
void printfFunction ( char* formattedOutput )
{
	if (clxTerminalEmulatorEnabled())
	{
		clxTerminalEmulatorPrint((const u1*)formattedOutput, strlen(formattedOutput));
	}
	else
	{
		printf (formattedOutput);
	}
}

/*******************************************************************************************************************************
*                                                    userScanf
*
* User can overload the default userScanf function used by the stack. if userGetInputFunction (function pointer) is not assigned, 
* then the platform default userScanf will be used
*
*******************************************************************************************************************************/
int getInputFunction ( char* buffer, unsigned int bufferSize )
{
	if (clxTerminalEmulatorEnabled())
	{
		clxTerminalEmulatorGetInput(buffer, 
									bufferSize, 
									0x0d,	/* Reception of carriage return character will cause this function to return */
                                    TRUE,
									TRUE,	/* Consider only ASCII characters (0x20 and above) */
									TRUE);	/* Flush the input queue to get rid of the old characters */
	}
	else
	{
		scanf ("%s", buffer);
	}

	return 0 /* Success */;
}

struct ClarinoxBlueParameters
{
    ClxConfigList       parentList;   /* SHALL be the very first member of the structure */

    /* Add any BSP-specific parameter in here: */
    ClxConfigString     transportType;
    ClxConfigString     comPortName;
    ClxConfigUnsigned   comPortSpeed;
    
    ClxConfigUnsigned   maxNoOfPairedDevices;

    ClxConfigUnsigned   l2capOutgoingBufferSize;
    ClxConfigUnsigned   l2capIncomingBufferSize;
    ClxConfigUnsigned   l2capSignalChannelInputContainerSize;
    ClxConfigUnsigned   l2capSignalChannelOutputContainerSize;
    ClxConfigUnsigned   l2capMaxNumberOfStackWideL2capConnections;
    ClxConfigUnsigned   l2capSignalChannelMtu;
       
    ClxConfigUnsigned   attChannelInputContainerSize;
    ClxConfigUnsigned   attChannelOutputContainerSize;
    ClxConfigUnsigned   attChannelMaxMtu;
    ClxConfigUnsigned   attChannelQueuedWritesContainerSize;

    ClxConfigUnsigned   hciDriverIncomingBufferSize;
    ClxConfigUnsigned   hciMaxNumberOfOutgoingCommandsInQueue;

    ClxConfigUnsigned   rfcommIncomingQueueSize;
    ClxConfigUnsigned   rfcommOutgoingQueueSize;
    ClxConfigUnsigned   rfcommMaxNumberOfL2capConnections;
    ClxConfigUnsigned   rfcommIncomingMtu;
    ClxConfigUnsigned   rfcommMaxOutgoingDataSize;
    ClxConfigUnsigned   rfcommMaxRemoteCredits;

    ClxConfigUnsigned   sdpMaxNumberOfL2capConnections;
    ClxConfigUnsigned   sdpIncomingMtu;
};

ClxConfigList* initializeClarinoxBlueBspConfigParameters()
{
    ClarinoxBlueParameters* configParameters = new ClarinoxBlueParameters;

    /*
    Initialize the parent list object (configParameters->parentList) which is to hold all other configuration objects (SHALL be initialized first):
    */
    clxConfigInitParamsList     (&configParameters->parentList, NULL, NULL);
    clxConfigInitStringParam    (&configParameters->transportType,                              "TransportType",                                "TT_UART_H4",       &configParameters->parentList); 
    clxConfigInitStringParam    (&configParameters->comPortName,                                "ComPortName",                                  "BLUETOOTH_PORT",   &configParameters->parentList); 
    clxConfigInitUnsignedParam  (&configParameters->comPortSpeed,                               "ComPortSpeed",                                 115200,             &configParameters->parentList); 


    clxConfigInitUnsignedParam  (&configParameters->maxNoOfPairedDevices,                       "MaxNoOfPairedDevices",                         4,                  &configParameters->parentList);

    clxConfigInitUnsignedParam  (&configParameters->l2capOutgoingBufferSize,                    "L2cap.OutgoingBufferSize",                     512,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->l2capIncomingBufferSize,                    "L2cap.IncomingBufferSize",                     512,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->l2capSignalChannelInputContainerSize,       "L2cap.SignalChannelInputContainerSize",        256,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->l2capSignalChannelOutputContainerSize,      "L2cap.SignalChannelOutputContainerSize",       256,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->l2capMaxNumberOfStackWideL2capConnections,  "L2cap.MaxNumberOfStackWideL2capConnections",   4,                  &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->l2capSignalChannelMtu,                      "L2cap.SignalChannelMtu",                       190,                &configParameters->parentList);
       
    clxConfigInitUnsignedParam  (&configParameters->attChannelInputContainerSize,               "Ble.AttChannel.InputContainerSize",            512,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->attChannelOutputContainerSize,              "Ble.AttChannel.OutputContainerSize",           512,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->attChannelMaxMtu,                           "Ble.AttChannel.MaxMtu",                        300,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->attChannelQueuedWritesContainerSize,        "Ble.AttChannel.QueuedWritesContainerSize",     1024,               &configParameters->parentList);

    clxConfigInitUnsignedParam  (&configParameters->hciDriverIncomingBufferSize,                "Hci.DriverIncomingBufferSize",                 4096,               &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->hciMaxNumberOfOutgoingCommandsInQueue,      "Hci.MaxNumberOfOutgoingCommandsInQueue",       5,                  &configParameters->parentList);

    clxConfigInitUnsignedParam  (&configParameters->rfcommIncomingQueueSize,                    "Rfcomm.IncomingQueueSize",                     512,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->rfcommOutgoingQueueSize,                    "Rfcomm.OutgoingQueueSize",                     512,                &configParameters->parentList);

    clxConfigInitUnsignedParam  (&configParameters->rfcommMaxNumberOfL2capConnections,          "Rfcomm.MaxNumberOfL2capConnections",           2,                  &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->rfcommIncomingMtu,                          "Rfcomm.IncomingMtu",                           380,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->rfcommMaxOutgoingDataSize,                  "Rfcomm.MaxOutgoingDataSize",                   380,                &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->rfcommMaxRemoteCredits,                     "Rfcomm.RfcommMaxRemoteCredit",                 1,                  &configParameters->parentList);

    clxConfigInitUnsignedParam  (&configParameters->sdpMaxNumberOfL2capConnections,             "Sdp.MaxNumberOfL2capConnections",              4,                  &configParameters->parentList);
    clxConfigInitUnsignedParam  (&configParameters->sdpIncomingMtu,                             "Sdp.IncomingMtu",                              48,                 &configParameters->parentList);

    return &configParameters->parentList;
}

void releaseClarinoxBlueBspConfigParameters(ClxConfigList* configParameters)
{
    delete (ClarinoxBlueParameters*)configParameters;
}

boolean clxBluetooth_HardResetController()
{
    /* 
     *Power cycling the Bluetooth controller: 
     */
    return 0;
}

void clxInitBsp (void)
{
#ifdef CLX_KOALA_LCD
	const u1* lcdAppName = "Clarinox SPP Application";
    initializeLcd(lcdAppName); 
#endif  
#ifdef CLX_KOALA_CAMERA
    initializeCamera();
#endif   
    
    /* 
     *assign user printf and scanf functions for overloading 
     */
    userPrintfFunction                      = printfFunction;                            
    userGetInputFunction                    = getInputFunction;
    
    clxClarinoxBlueConfigFileName           = bluetoothConfigFileName;  
    clxUartBspInterface                     = &clxUartSTM32F407BspInterface;
    clarinoxBluePlatformTaskPriorityTable   = clarinoxBlueIntegrityPlatformTaskPriorityTable;
    clarinoxPlatformBluetoothTaskStackSizeTable      = clarinoxuCosBluetoothTaskStackSizeTable;
    clarinoxPlatformSoftFrameTaskStackSizeTable      = clarinoxuCosSoftFrameTaskStackSizeTable;
    
    clxFileBspInterface                     = &clxMemoryFileInterface;    
    initROMFileSystemMap();
    gnssRxHandler                           = gnssRxHandlerFunction;
    
    /*
    Setting a call-back function which is called during the initialization of ClarinoxBlue stack.
    the call-back function is used to reset the Bluetooth controller, and to download the firmware
    into the controller using vendor specific HCI commands. Must be called before clxInitClarinoxBlue():
    */
    clxHciSendVendorSpecificCommands        = clxInitBluetoothController;    

}


#ifdef __cplusplus
}
#endif

void operator delete ( void* ptr )
{
    if ((ptr >= (void*)PRIVATE_HEAP_START_ADDRESS) &&
        (ptr < (void*)privateHeapPtr))
    {
        /* This is a buffer in ClarinoxSoftFrame private heap: */
        clxPoolsetFree(ptr);
    }
    else
    {
        /* This is a buffer from an external heap: */
        free (ptr);
    }
}

void operator delete[] ( void* ptr )
{
    if ((ptr >= (void*)PRIVATE_HEAP_START_ADDRESS) &&
        (ptr < (void*)privateHeapPtr))
    {
        /* This is a buffer in ClarinoxSoftFrame private heap: */
        clxPoolsetFree(ptr);
    }
    else
    {
        /* This is a buffer from an external heap: */
        free (ptr);
    }
}
