/******************************************************************************
*
* Project             Wlan Sample Application
* File                HttpServer.Interface.cpp
* Description         Implementation of MINI HTTP Server Interface for Thermostat Demo
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#include "ClxBsp.h"
#include "lwip/tcp.h"
#include <string.h>
#include <stdio.h>
#include "index.html.h"
#include "css/bootstrap-theme.min.css.h"
#include "css/bootstrap.min.css.h"
#include "css/thermostat.css.h"
#include "fonts/glyphicons-halflings-regular.woff2.h"
#include "js/bootstrap.min.js.h"
#include "js/jquery-1.11.2.min.js.h"
#include "js/npm.js.h"
#include "js/RGraph.common.core.js.h"
#include "js/RGraph.odo.js.h"



#ifdef __cplusplus
extern "C" {
#endif

#ifdef CLX_TEMPERATURE_SENSOR_ENABLED  
extern float clxMeasureTemperature();
extern void clxSetLEDs(s4 temp, s4 trigger);
#endif

struct WebFile
{
    const struct WebFile *next;
    const s1* name;
    const u1* data;
    const u4  dataLength;
};


/* Entry for the URL /index.html: */
const struct WebFile webFile_data_index_html[] = {{NULL, (const char*)data_index_html, data_index_html + 12, sizeof(data_index_html) - 1 - 12}};
/* Entry for the URL /css/bootstrap-theme.min.css: */
const struct WebFile webFile_data_css_bootstrap_theme_min_css[] = {{webFile_data_index_html, (const char*)data_css_bootstrap_theme_min_css, data_css_bootstrap_theme_min_css + 29, sizeof(data_css_bootstrap_theme_min_css) - 1 - 29}};
/* Entry for the URL /css/bootstrap.min.css: */
const struct WebFile webFile_data_css_bootstrap_min_css[] = {{webFile_data_css_bootstrap_theme_min_css, (const char*)data_css_bootstrap_min_css, data_css_bootstrap_min_css + 23, sizeof(data_css_bootstrap_min_css) - 1 - 23}};
/* Entry for the URL /css/thermostat.css: */
const struct WebFile webFile_data_css_thermostat_css[] = {{webFile_data_css_bootstrap_min_css, (const char*)data_css_thermostat_css, data_css_thermostat_css + 20, sizeof(data_css_thermostat_css) - 1 - 20}};
/* Entry for the URL /fonts/glyphicons-halflings-regular.woff2: */
const struct WebFile webFile_data_fonts_glyphicons_halflings_regular_woff2[] = {{webFile_data_css_thermostat_css, (const char*)data_fonts_glyphicons_halflings_regular_woff2, data_fonts_glyphicons_halflings_regular_woff2 + 42, sizeof(data_fonts_glyphicons_halflings_regular_woff2) - 1 - 42}};
/* Entry for the URL /js/bootstrap.min.js: */
const struct WebFile webFile_data_js_bootstrap_min_js[] = {{webFile_data_fonts_glyphicons_halflings_regular_woff2, (const char*)data_js_bootstrap_min_js, data_js_bootstrap_min_js + 21, sizeof(data_js_bootstrap_min_js) - 1 - 21}};
/* Entry for the URL /js/jquery-1.11.2.min.js: */
const struct WebFile webFile_data_js_jquery_1_11_2_min_js[] = {{webFile_data_js_bootstrap_min_js, (const char*)data_js_jquery_1_11_2_min_js, data_js_jquery_1_11_2_min_js + 25, sizeof(data_js_jquery_1_11_2_min_js) - 1 - 25}};
/* Entry for the URL /js/npm.js: */
const struct WebFile webFile_data_js_npm_js[] = {{webFile_data_js_jquery_1_11_2_min_js, (const char*)data_js_npm_js, data_js_npm_js + 11, sizeof(data_js_npm_js) - 1 - 11}};
/* Entry for the URL /js/RGraph.common.core.js: */
const struct WebFile webFile_data_js_RGraph_common_core_js[] = {{webFile_data_js_npm_js, (const char*)data_js_RGraph_common_core_js, data_js_RGraph_common_core_js + 26, sizeof(data_js_RGraph_common_core_js) - 1 - 26}};
/* Entry for the URL /js/RGraph.odo.js: */
const struct WebFile webFile_data_js_RGraph_odo_js[] = {{webFile_data_js_RGraph_common_core_js, (const char*)data_js_RGraph_odo_js, data_js_RGraph_odo_js + 18, sizeof(data_js_RGraph_odo_js) - 1 - 18}};


const struct WebFile* web_FS_Entry = webFile_data_js_RGraph_odo_js;


s1 responseHeaderSuccess_WithKeepAlive[] = "HTTP/1.0 200 OK\r\n" \
                                           "Connection: Keep-Alive\r\n" \
                                           "Content-Length: 2\r\n" \
                                           "\r\n" \
                                           "xx";

struct HttpRequest
{
    ClxHttpRequestMethod    type;
    const u1*               data;
    u4                      dataLength;
    u4                      currentIndex;
};
    

#ifndef CLX_TEMPERATURE_SENSOR_ENABLED
static u4 tempCounter = 0;
#endif

ClxResult webInterface_Thermostat_init (ClxWebDataBspInterface* this_,
                                      u2* maxNumberOfRequests,
                                      u2* requestHandleBufferSize)
{
    /* No limit on number of requests. A restriction will be applied by the MINI HTTP server: */
    *maxNumberOfRequests = 0xFFFF;
    *requestHandleBufferSize = sizeof(HttpRequest);
    
    return CLX_SUCCESS;
}

void webInterface_Thermostat_destroy (ClxWebDataBspInterface* this_)
{}

ClxHttpResponseStatus webInterface_Thermostat_newRequest (ClxWebDataBspInterface* this_,
                                                          ClxHttpServerConnection connection,
                                                          ClxHttpServerRequestHandle requestHandle,
                                                          enum ClxHttpRequestMethod method,
                                                          ClxHttpVersion requestVersion,
                                                          const s1* url,
                                                          u4 requestBodyLength,
                                                          ClxHttpHeaderFieldsHandle requestHeaderFieldsHandle)
{
    HttpRequest* request = (HttpRequest*)requestHandle;
    request->type = method;
    request->currentIndex = 0;
    request->dataLength = 0;
    
    switch(method)
    {
    case ClxHttpRequestMethod_Get:
        {    
            if (strncmp(url, "/get.html?t=", 12) == 0)
            {
#ifdef CLX_TEMPERATURE_SENSOR_ENABLED              
                u4 threshold = atoi(url + 12);
                u4 temperature = (u4)clxMeasureTemperature();                
                clxSetLEDs(temperature, threshold);
#else
                u4 temperature = tempCounter++ / 100;
#endif                
                sprintf (&responseHeaderSuccess_WithKeepAlive[sizeof(responseHeaderSuccess_WithKeepAlive) - 3], "%02u", temperature);
                request->data = (const u1*)responseHeaderSuccess_WithKeepAlive;
                request->dataLength = sizeof(responseHeaderSuccess_WithKeepAlive) - 1;
                return CLX_HTTP_RESPONSE_STATUS_SUCCESS;              
            }
            else
            {
                if (strcmp(url, "/") == 0)
                {
                    url = "/index.html";
                }
                
                const struct WebFile* current = web_FS_Entry;
                    
                while(current)
                {
                    if (!strcmp(url, current->name))
                    {
                        request->data = current->data;
                        request->dataLength = current->dataLength;
                        return CLX_HTTP_RESPONSE_STATUS_SUCCESS;
                    }
                    
                    current = current->next;
                }    
            
                return CLX_HTTP_RESPONSE_STATUS_NOT_FOUND;
            }
        }
    default:
        return CLX_HTTP_RESPONSE_STATUS_METHOD_NOT_ALLOWED;
    }
}

void webInterface_Thermostat_dataRequired (ClxWebDataBspInterface* this_,
                                         ClxHttpServerRequestHandle requestHandle,
                                         enum ClxHttpRequiredDataType requiredDataType,
                                         struct ClxWebDataBspInterfaceBuffer* buffer)
{
    HttpRequest* request = (HttpRequest*)requestHandle;
    
    if (request->currentIndex < request->dataLength)
    {
        buffer->data = request->data + request->currentIndex;
        buffer->dataLength = request->dataLength - request->currentIndex;
        buffer->flags = (CLX_NETWORK_SOCKET_BUFFER_FLAG_FLUSH_QUEUED_DATA | CLX_NETWORK_SOCKET_BUFFER_FLAG_SET_TCP_PUSH_BIT);
        request->currentIndex += buffer->dataLength;
    }
    else
    {
        buffer->data = NULL;
    }
}

ClxHttpResponseStatus webInterface_Thermostat_dataReceived (ClxWebDataBspInterface* this_,
                                                         ClxHttpServerRequestHandle requestHandle,
                                                         const u1* data, 
                                                         u4 dataLength)
{
    return CLX_HTTP_RESPONSE_STATUS_SUCCESS;
}

void webInterface_Thermostat_requestComplete (ClxWebDataBspInterface* this_, 
                                            ClxHttpServerRequestHandle requestHandle) 
{}

ClxWebDataBspInterface webInterface_Thermostat = {webInterface_Thermostat_init,
                                                   webInterface_Thermostat_destroy,
                                                   webInterface_Thermostat_newRequest,
                                                   webInterface_Thermostat_dataRequired,
                                                   webInterface_Thermostat_dataReceived,
                                                   webInterface_Thermostat_requestComplete};


#ifdef __cplusplus
}
#endif


