/******************************************************************************
*
* Project             Wlan Sample Application
* File                WlanClient.cpp
* Description         Wlan Station role application file
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "ClxBsp.h"
#include "ConsoleUIEngine.h"
#include "ClarinoxWlan.h"
#include "Wlan.Api.h"
#include "Wlan.Config.h"
#include "time.h"
#include "ClxStandard.h"
#include "WiFiApp.h"

ClxSSIDList ssidList;

InterfaceState interfaceState = Stopped;

ClxBSSInfo periodicBssInfo;

ClxBSSInfo bssList[MAX_NO_OF_AP_IN_VICINITY];
ClxBSSInfo allBssList[MAX_NO_OF_AP_IN_VICINITY * NO_OF_SCAN_REPETITION];
            
/*******************************************************************************************************************************
*                                                       printBssInfo
*
* Print out the BSS details.
*
*******************************************************************************************************************************/
void printBssInfo(ClxBSSInfo* bss)
{
    bss->ssid.value[bss->ssid.len] = '\0';
    ClxConsoleUIEngine::text("\n\n\nSSID : %s\n", (s1*)bss->ssid.value);
    ClxConsoleUIEngine::text("BSSID : %02X:%02X:%02X:%02X:%02X:%02X\n", bss->bssid[0],
            bss->bssid[1],
            bss->bssid[2],
            bss->bssid[3],
            bss->bssid[4],
            bss->bssid[5]);
    ClxConsoleUIEngine::text("type : %u\n", (u4)bss->bssType);
    ClxConsoleUIEngine::text("channel : %u\n", (u4)bss->channel);
    ClxConsoleUIEngine::text("frequency band : %u\n", (u4)bss->band);
    ClxConsoleUIEngine::text("signal power level : %d\n", (s4)bss->rssi);

    ClxConsoleUIEngine::text("supported encryption:\n");

    if (bss->encProtocol == (u4)ClxEncProtoNONE)
    {
        ClxConsoleUIEngine::text("  NONE\n");
    }
    if (bss->encProtocol & (u4)ClxEncProtoWEP40)
    {
        ClxConsoleUIEngine::text("  WEP40\n");
    }
    if (bss->encProtocol & (u4)ClxEncProtoWEP104)
    {
        ClxConsoleUIEngine::text("  WEP104\n");
    }    
    if (bss->encProtocol & (u4)ClxEncProtoTKIP)
    {
        ClxConsoleUIEngine::text("  TKIP(WPA)\n");
    }
    if (bss->encProtocol & (u4)ClxEncProtoAES_CCMP)
    {
        ClxConsoleUIEngine::text("  CCMP(WPA2)\n");
    }


    ClxConsoleUIEngine::text("\nSupported authentication:\n");
    if (bss->authType & (u4)ClxAuthTypeOpenSystem)
    {
        ClxConsoleUIEngine::text("  OpenSystem\n");
    }
    if (bss->authType & (u4)ClxAuthTypeLegacy)
    {
        ClxConsoleUIEngine::text("  Legacy(WEP)\n");
    }
    if (bss->authType & (u4)ClxAuthTypePresharedKey)
    {
        ClxConsoleUIEngine::text("  WPA/WPA2 Preshared Key Authentication\n");
    }
    if (bss->authType & (u4)ClxAuthTypeIEEE802_1x)
    {
        ClxConsoleUIEngine::text("  IEEE802.1X (Enterprise) Authentication\n");
    }
    
    printCapabilties(bss->capabilities);

    ClxConsoleUIEngine::text("\n");
}


/*******************************************************************************************************************************
*                                                       showStationMenu
*
* Provide the Station Role UI menu.
*
*******************************************************************************************************************************/
void showStationMenu(ClxStack stack)
{
    ClxError error;
    boolean blocking  = TRUE;
    u2 foundSsidCount = 0;
    
    /*
    Define a structure to contain all the initial configuration parameters to be
    passed to clxWlanStartInterface() for the local station (Station role).

    All objects SHALL be initialized using the initialization functions provided for each configuration type.
    */
    struct StationConfig
    {
        ClxConfigList    parentList;
        struct
        {
            ClxWlanConfigRate rates[21];
            ClxConfigArray    ratesArray;
        }supportedRates;    
        ClxConfigInteger maxTxPower;
        ClxConfigInteger rtsThreshold;
        ClxConfigInteger macLayerMTU;
        ClxConfigInteger wpsEnabled;
        /*
        Driver specific parameters:
        */
        ClxConfigString  sdioDriverName;
        ClxConfigString  firwareFileName;
        ClxConfigString  nvsFileName;
        ClxConfigInteger frameCaptureMask;        
    };

    StationConfig stationConfig;

    /*
    Initialize the parent list object (stationConfig.parentList) which is to hold all other configuration objects (SHALL be initialized first):
    */
    clxConfigInitParamsList(&stationConfig.parentList, NULL, NULL);

    /*
    These values are recommended by the driver:
    */
    clxConfigInitIntegerParam(&stationConfig.maxTxPower,   "MaxTxPower",   15,   &stationConfig.parentList);            /* Maximum TX Power to be used by WL18xx hardware, in dBm */
    clxConfigInitIntegerParam(&stationConfig.macLayerMTU,  "MacLayerMTU",  2352, &stationConfig.parentList);            /* Maximum size of MAC frames which can be sent without fragmentation */
    clxConfigInitIntegerParam(&stationConfig.rtsThreshold, "RTSThreshold", 2353, &stationConfig.parentList);            /* RTS/CTS Threshold */
    clxConfigInitIntegerParam(&stationConfig.wpsEnabled,   "WpsEnabled", 0, &stationConfig.parentList);                 /* Wireless Protected Setup is not enabled */

    /*
    Frames to be captured by ClarinoxDebugger. 0xFFFFFFFF & (~CLX_WLAN_FRAME_TYPE_BEACON) means capture all frames except for beacons. 
    */
    clxConfigInitIntegerParam(&stationConfig.frameCaptureMask, "FrameCaptureMask", 0xFFFFFFFF & (~CLX_WLAN_FRAME_TYPE_BEACON),  &stationConfig.parentList);
     
    /*
    List of data rates which are available and can be used for both sending and receiving of frames. 
    The list also determines what standards (e.g. IEEE802.11b, IEEE802.11g, IEEE802.11n) are enabled for the local station.
    NOTE : When a particular standard is to be used, ALL MANDATORY rates for that standard SHALL be included in the list.
    NOTE : In Station role, the 'Basic Rate' flag (second last argument of the function clxConfigInitRateParam) is not used and is ignored.
    NOTE : ClxWlanConfigRate objects are not directly owned by the parent list. They are owned by an array object of type ClxConfigArray.
    Therefore, we pass NULL as the last argument of clxConfigInitRateParam.
    NOTE : If a rate in the list is not supported by the WLAN hardware, it will be silently ignored.
    */
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[0], NULL, ClxWlanNonHTRate_1Mbps, 0, TRUE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[1], NULL, ClxWlanNonHTRate_2Mbps, 0, TRUE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[2], NULL, ClxWlanNonHTRate_5_5Mbps, 0, TRUE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[3], NULL, ClxWlanNonHTRate_11Mbps, 0, TRUE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[4], NULL, ClxWlanNonHTRate_6Mbps, 0, TRUE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[5], NULL, ClxWlanNonHTRate_9Mbps, 0, FALSE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[6], NULL, ClxWlanNonHTRate_12Mbps, 0, TRUE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[7], NULL, ClxWlanNonHTRate_18Mbps, 0, FALSE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[8], NULL, ClxWlanNonHTRate_24Mbps, 0, FALSE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[9], NULL, ClxWlanNonHTRate_33Mbps, 0, FALSE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[10], NULL, ClxWlanNonHTRate_36Mbps, 0, FALSE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[11], NULL, ClxWlanNonHTRate_48Mbps, 0, FALSE, NULL);
    clxConfigInitRateParam(&stationConfig.supportedRates.rates[12], NULL, ClxWlanNonHTRate_54Mbps, 0, FALSE, NULL);

    /*
    Initialize the configuration array object (stationConfig.supportedRates) to hold the list of supported data rates:
    */
    clxConfigInitParamsArray(&stationConfig.supportedRates.ratesArray, 
                             "SupportedRates", 
                             (ClxConfigParam*)stationConfig.supportedRates.rates, 
                             13, 
                             &stationConfig.parentList);
            
    /* 
    Name of the SDIO driver. This value is passed to the SDIO interface implementation in the BSP (ClxSdioBspInterface),
    and its meaning depends on the BSP interface implementation and/or the underlying platform.
    Refer to the SDIO interface implementation in the BSP
    */
#if defined(IMX6_SABRE_LITE)
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev", &stationConfig.parentList);    
#elif defined(IMX6_SABRE_AUTO)
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &stationConfig.parentList);  
#elif defined(IMELCO)
    clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &goConfig.parentList);        
#elif defined(DM3730_BSP)     
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &stationConfig.parentList);
#elif defined(PROPAK6_BSP)
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &stationConfig.parentList); 
#elif defined(CLARINOX_KOALA_V20)
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &stationConfig.parentList);
#elif defined(LPC1830) /*LPC1830 Xplorer */
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &stationConfig.parentList); 
#elif defined(LPC1837) /*LPC1837 Xplorer */
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &stationConfig.parentList); 
#elif defined(LPC4330) /*LPC4330 Xplorer */
    clxConfigInitStringParam(&stationConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &stationConfig.parentList);    
#else  
#error Select your board
#endif
    
    /* 
    The file containing WL18xx firmware binary. Can be a real file in the underlying file system, or a virtual file 
    through an implementation of ClxFileBspInterface BSP interface:
    */
    clxConfigInitStringParam(&stationConfig.firwareFileName, "FirmwareFileName", "Wl18xx-fw-2-Rev8-8-0-0-13.bin", &stationConfig.parentList);       
    
    while (1)
    {  
        const s1* menu = "Start Interface as Station\0"
            "Scan\0"
            "Scan for a specific SSID\0"
            "Connect to wireless network\0"
            "Disconnect from wireless network\0"
            "Probe the AccessPoint\0"
            "Stop Interface\0"
            "Return to previous menu";

        u4 s = ClxConsoleUIEngine::showMenu("Wlan ST Menu", menu, 8);

        if (s == 1)
        {
            if(interfaceState == Stopped)
            {
                /*
                Before starting the interface, we enable the WL18xx hardware (implementation is in the BSP):
                */
                if (!clxEnableWl18xxWlanHardware())
                {
                    ClxConsoleUIEngine::text("Enabling WLAN hardware failed\n");
                    break;
                }
                
            }
            
            /*
            Start the interface as a station. Since this is the only interface we start in this application, the driver will perform many operations as follows:

            - The Bus driver (e.g. SDIO) is initialized and prepared for communication with the WLAN controller.
            - The controller is booted.
            - The firmware is loaded into the controller.
            - Initial configuration parameters are set.
            - The requested role is enabled in the controller.
            */
            error = clxWlanStartInterface(wlanHandle,                               /* Handle to the virtual interface as returned by clxWlanCreateInterface                                            */
                                          ClxIfTypeSTA,                             /* Start the interface in station role (the role cannot change until the interface is stopped and restarted)        */
                                          clxGetWlanTcpIpInterface(),               /* Implementation of ClxWlanNetworkInterface for this platform (implemented in WlanNetworkStackInterface.cpp)       */
                                          &stationConfig.parentList,                /* A pointer to parent list object containing all the initial configuration parameters                              */ 
                                          TRUE);                                    /* Blocking mode. Do no return until the interface starting procedure is complete either with success or in error   */

            if (error != CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("Starting the WLAN virtual interface failed with error %s\n", 
                                         clxGetWlanErrorCodeText(error)); 
            }
            else
            {
                u1 macAddr[6];

                /* Get the MAC address of this virtual interface (This is only available after the interface is successfully started): */
                clxWlanInterfaceGetMacAddr(wlanHandle, macAddr, TRUE);
                ClxConsoleUIEngine::text("WLAN virtual interface started with MAC address %02X:%02X:%02X:%02X:%02X:%02X\n", 
                        macAddr[0],
                        macAddr[1],
                        macAddr[2],
                        macAddr[3],
                        macAddr[4],
                        macAddr[5]);

                interfaceState = Started;

                /*
                Structure containing the configuration parameter used to obtain the Firmware version of the WL18xx controller.
                NOTE : A parent list object of type ClxConfigList is always required to pass any configuration parameters to the WLAN stack/driver
                even if there is only one parameter being passed:
                */
                struct FwVersion
                {
                    ClxConfigList       parentList;
                    ClxConfigData       fwVersion;
                    u1                  fwVersionBuf[32];
                };

                FwVersion fwVersion;
                
                /* The parent list to be passed to clxConfigInitParamsList. SHALL be initialized first: */
                clxConfigInitParamsList(&fwVersion.parentList, NULL, NULL);

                /* 
                The buffer fwVersion.fwVersionBuf is assigned to the object of type ClxConfigData. 
                When clxWlanGetParametersValue returns, fwVersion.fwVersionBuf will hold WL18xx Firmware version, in ASCII format: 
                */
                clxConfigInitDataParam   (&fwVersion.fwVersion, "Wl18xx.FirmwareVersion",  fwVersion.fwVersionBuf, sizeof(fwVersion.fwVersionBuf), &fwVersion.parentList);
                
                /* Retrieve the requested parameters from the WL18xx driver: */
                
                ClxResult ret = clxWlanGetParametersValue(wlanHandle,                       /* Handle to the virtual interface as returned by clxWlanCreateInterface                                            */
                                                          &fwVersion.parentList,            /* A pointer to parent list object containing the parameter (fwVersion.fwVersion)                                   */ 
                                                          TRUE);                            /* Blocking mode. Do no return until the parameter is read from the driver                                          */
                if (ret == CLX_SUCCESS)
                {
                    ClxConsoleUIEngine::text("Firmware Version : %s\n", fwVersion.fwVersionBuf);
                }

                /*
                Structure containing the list of frequency channels to scan when clxWlanScan() is called:
                */
                struct ScanChannelConfig
                {
                    ClxConfigList               parentList;
                    ClxConfigArray              channelsArray;
                    ClxWlanConfigChannel        channelsToScan[13];
                };

                ScanChannelConfig scanChannelConfig;

                /* Initialize the parent list object first: */
                clxConfigInitParamsList(&scanChannelConfig.parentList, NULL, NULL);

                /* 
                Initializing the channels for scan procedures (based on the local regulatory domain).
                
                NOTE : ClarinoxWLAN does not check the consistency of these parameters with the local regulatory domain. The application has to make sure of that.
                NOTE : If any channel in the list is not supported by the WLAN controller, it will be silently ignored.
                */
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[0],  NULL, 1,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[1],  NULL, 2,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[2],  NULL, 3,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[3],  NULL, 4,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[4],  NULL, 5,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[5],  NULL, 6,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[6],  NULL, 7,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[7],  NULL, 8,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[8],  NULL, 9,  ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[9],  NULL, 10, ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[10], NULL, 11, ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[11], NULL, 12, ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                clxConfigInitChannelParam(&scanChannelConfig.channelsToScan[12], NULL, 13, ClxWlanFreqBand2_4GHz, ClxWlanPassiveScan | ClxWlanActiveScan, 20, NULL);
                                 
                clxConfigInitParamsArray(&scanChannelConfig.channelsArray, 
                                            "OperationalChannels", 
                                            (ClxConfigParam*)scanChannelConfig.channelsToScan, 
                                            11, 
                                            &scanChannelConfig.parentList);

                /*
                Send the list of channels to the WLAN stack:
                */
                error = clxWlanSetParametersValue(wlanHandle, &scanChannelConfig.parentList, TRUE);               
            
                if (error != CLX_SUCCESS)
                {
                    ClxConsoleUIEngine::text ("\nSetting scan channels failed with error %s\n", clxGetWlanErrorCodeText(error));
                    break;
                }            
            }
        }
        else if (s == 2)
        {                  
            u4 j = 0;
            u4 totalSsidFound = 0; 

            /*
            Scan NO_OF_SCAN_REPETITION times to find all the APs 
            */
            for (u4 k = 0; k < NO_OF_SCAN_REPETITION ; k++)
            {
                error = clxWlanScan(wlanHandle,                             /* Handle to the virtual interface as returned by clxWlanCreateInterface                                                */
                                    NULL,                                   /* Do not scan for a specific SSID. Scan for all available networks in the area (Operating on the provided channels)    */
                                    5,                                      /* Return after 5 seconds if the scan operation has not been completed by then (in case of an error)                    */
                                    &foundSsidCount,                        /* On a successful return, will contain the number of networks found                                                    */
                                    TRUE);                                  /* Blocking mode. Do not return until the scan procedure is complete                                                    */

                ClxSize numberOfBSSs;

                /* Get the list of found networks (Basic Service Sets): */

                error = clxWlanGetAvailableBssList(wlanHandle,                              /* Handle to the virtual interface as returned by clxWlanCreateInterface                                                */
                                                    bssList,                                /* The caller-provided list to hold the list of found BSSs on the successful return of this application */
                                                    sizeof(bssList)/sizeof(ClxBSSInfo),     /* The maximum number of elements which can be stored in bssList                                        */
                                                    &numberOfBSSs,                          /* On a successful return, will hold the actual number of BSSs stored in bssList                        */
                                                    TRUE);                                  /* Blocking mode                                                                                        */
                if (error == CLX_SUCCESS)
                {
                    /* add the found BSSs to the end of the mother list (allBssList) : */
                    memcpy((allBssList + j), bssList, (numberOfBSSs* sizeof(ClxBSSInfo)));
                    j = j + numberOfBSSs;
                }
            }
           
            /* Check if there are any duplicates in allBssList (in case a BSS was found in two or more rounds of scanning): */
            boolean duplicated = FALSE;
            for (u4 i = 0; i < j; i++)
            {
                allBssList[i].ssid.value[allBssList[i].ssid.len] = '\0';
                duplicated = FALSE;
                for(u4 l = 0; l < i ; l++)
                {
                    /* Check if already in the list, report duplicated */
                    if( strcmp( (s1*)allBssList[l].ssid.value, (s1*)allBssList[i].ssid.value) == 0)
                    {
                        duplicated = TRUE;
                        break;
                    }
                }
                /* Print just if not duplicated */
                if(duplicated == FALSE)
                {
                    totalSsidFound++;
                    printBssInfo(&allBssList[i]);
                }
            }
            ClxConsoleUIEngine::text ("\nScan is complete with the result: %s - No of BSSs found: %u\n", 
                                       clxGetWlanErrorCodeText(error), 
                                       totalSsidFound);
        }
        else if (s == 3)
        {
            ClxSSID ssid;
            ClxConsoleUIEngine::inputBox("Enter SSID to scan for: ", ssid.value, sizeof(ssid.value)-1);
            ssid.len = strlen(ssid.value);

            error = clxWlanScan(wlanHandle, &ssid, 5 /* Seconds */, &foundSsidCount, blocking);

            if(blocking)
            {
                ClxConsoleUIEngine::text ("\nScan is complete with the result: %s - No of BSSs found: %u\n", 
                                          clxGetWlanErrorCodeText(error), 
                                          foundSsidCount);

                ClxBSSInfo bssList[12];
                ClxSize numberOfBSSs;
                error = clxWlanGetAvailableBssList(wlanHandle, bssList, sizeof(bssList)/sizeof(ClxBSSInfo), &numberOfBSSs, TRUE); 

                if (error == CLX_SUCCESS)
                {
                    for (u4 i = 0; i < numberOfBSSs; i++)
                    {
                        printBssInfo(&bssList[i]);
                    }
                }
            }
        }
        else if (s == 4)
        {
            ClxWlanConnectionProfile connProfile;
            s1 ssid[33];
            ClxConsoleUIEngine::inputBox("Enter SSID to connect to: ", ssid, sizeof(ssid));
            strcpy(connProfile.ssid.value, ssid);
            connProfile.ssid.len = strlen(ssid);
            connProfile.bssConnectionMode = ClxWlanBssConnectionMode_Associate;

            connProfile.securityCredential.authTypeToUse = ClxAuthTypePresharedKey; /* Ignored in this version of ClarinoxWLAN */

            const s1* menu = "No Encryption\0"
                "TKIP (WPA)\0"
                "CCMP (WPA2)\0"
                "Return to main menu";

            u4 p = ClxConsoleUIEngine::showMenu("Encryption Protocol Type", menu, 4);

            switch(p)
            {
            case 1:
                connProfile.securityCredential.encProtocolToUse = ClxEncProtoNONE;
                break;
            case 2:
                connProfile.securityCredential.encProtocolToUse = ClxEncProtoTKIP;
                break;
            case 3:
                connProfile.securityCredential.encProtocolToUse = ClxEncProtoAES_CCMP;
                break;
            case 4:
                return;
            default:
                return;
            }

            /* Initialize the encryption-related parameters with zero: */
            connProfile.securityCredential.keyLen = 0;
            memset(connProfile.securityCredential.passwdPhrase, 0 , sizeof(connProfile.securityCredential.passwdPhrase));
            memset(connProfile.securityCredential.key, 0, sizeof(connProfile.securityCredential.key));

            if (connProfile.securityCredential.encProtocolToUse != ClxEncProtoNONE)
            {
               /* define a larger array in case user enters more than 64 character */
                s1 pwd[70];
                ClxConsoleUIEngine::text ("\nPasskey should be between 8-63, if 64 characters are entered they need to be hex numbers.");
                ClxConsoleUIEngine::inputBox("\nEnter Encryption Passkey (Max 63 characters) or Master Key (64 hex): ", pwd, sizeof(pwd));
                u4 passLength = strlen(pwd);

                if((passLength > 64) || (passLength < 8 ))
                {
                     ClxConsoleUIEngine::text ("\nInvalid Password, it's longer than 64.");
                     return;
                }
                else if(passLength == 64)
                {
                    connProfile.securityCredential.keyLen = 32;                        /* keyLen shall be set to 32 for HEX key                  */ 
                    if(clxAsciiToBinary(connProfile.securityCredential.key, (u1*)pwd, 64) != CLX_SUCCESS)
                    {
                         ClxConsoleUIEngine::text ("\nEnter only hex values(0-9 or a-f)");
                         return;
                    }
                }
                else
                {
                     strcpy(connProfile.securityCredential.passwdPhrase, pwd);
                }        
            }

            /* Connect to the network identified by the information given in connProfile: */
            error = clxWlanConnect(wlanHandle, &connProfile, TRUE);


            /* 
            Show the connection details
            */
            ClxConsoleUIEngine::text ("\nConnection to %s is complete with result %d %s", 
                                      ssid, 
                                      error, 
                                      clxGetWlanErrorCodeText(error));

            if (error == CLX_SUCCESS)
            {
                /* Now that we have successfully connected to the BSS, lets extract some information about the network: */
                struct ConnInfo
                {
                    ClxConfigList               parentList;
                    ClxWlanConfigChannel        channel;        /* The channel on which the network is currently operating */
                    ClxConfigData               bssid;          /* The 6-byte unique ID of the BSS, which is the MAC address of the AccessPoint */
                    u1                          bssidBuf[6];    /* A buffer to store the BSSID (bssid does not have a buffer on its own) */
                };

                ConnInfo connInfo;
                
                /* Initialize the parent list first: */
                clxConfigInitParamsList(&connInfo.parentList, NULL, NULL);
                
                /* 
                The values to which we set the values are not important. They will be set by the stack.
                Only connInfo.channel.channelNumber and connInfo.channel.frequencyBand will be set by the stack: 
                */
                clxConfigInitChannelParam(&connInfo.channel, 
                                          "CurrentChannel", 
                                          0, 
                                          ClxWlanFreqBand2_4GHz, 
                                          0, 
                                          0, 
                                          &connInfo.parentList);

                /* Assign connInfo.bssidBuf to connInfo.bssid object: */
                clxConfigInitDataParam(&connInfo.bssid, 
                                       "CurrentBSSID", 
                                       connInfo.bssidBuf, 
                                       sizeof(connInfo.bssidBuf), 
                                       &connInfo.parentList);

                error = clxWlanGetParametersValue(wlanHandle,                   /* Handle to the virtual interface as returned by clxWlanCreateInterface                                                */
                                                  &connInfo.parentList,         /* A pointer to the parent list object containing the CurrentChannel and CurrentBSSID parameter objects                 */
                                                  TRUE);                        /* Blocking mode                                                                                                        */

                if (error == CLX_SUCCESS)
                {
                    ClxConsoleUIEngine::text ("\nConnection at channel %u. BSSID = %02X:%02X:%02X:%02X:%02X:%02X\n", 
                            connInfo.channel.channelNumber,
                            connInfo.bssidBuf[0],
                            connInfo.bssidBuf[1],
                            connInfo.bssidBuf[2],
                            connInfo.bssidBuf[3],
                            connInfo.bssidBuf[4],
                            connInfo.bssidBuf[5]);
                }
            }
        }
        else if (s == 5)
        {
            /* 
            Disconnect from the network. This will peacefully disconnect the local machine from the network by sending De-Authentication and De-Association
            frames to the AccessPoint:
            */
            error = clxWlanDisconnect(wlanHandle, TRUE);
            ClxConsoleUIEngine::text ("\nDisconnection is complete with result %d %s", error, clxGetWlanErrorCodeText(error));
        }
        else if (s == 6)
        {
            u1 dest[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
            ClxBSSInfo bssInfo;

            /* Send probe requests to the air: */
            error = clxWlanProbe(wlanHandle,            /* Handle to the virtual interface as returned by clxWlanCreateInterface                                                        */
                                 dest,                  /* The destination of probe requests. dest has been set to FF:FF:FF:FF:FF:FF which is the broadcast address                     */
                                 5,                     /* The maximum number of probe requests to be sent if no probe response is received                                             */
                                 1,                     /* The time interval between consecutive probe requests, in the unit of BeaconInterval                                          */
                                 &bssInfo,              /* The caller-provided object which on a successful return will hold information of the BSS which has responded to the probe    */
                                 TRUE);                 /* Blocking mode                                                                                                                */

            if (error == CLX_SUCCESS)
            {
                printBssInfo(&bssInfo);
            }
            else
            {
                ClxConsoleUIEngine::text ("\nProbe failed with result %d %s", error, clxGetWlanErrorCodeText(error));
            }
        }
        else if (s == 7)
        {
            /* Stop the virtual interface: */
            error = clxWlanStopInterface(wlanHandle, TRUE);
            if(error == CLX_SUCCESS)
            {
                interfaceState = Stopped;
            }
            ClxConsoleUIEngine::text ("\nStopping interface is complete with result %s", clxGetWlanErrorCodeText(error));
            
            /* Disable the WL18xx hardware: */
            clxDisableWl18xxWlanHardware();
        }
        else if (s == 8)
        {
            return;
        }
    }
}

