/******************************************************************************
 *
 * Project             ClarinoxWLAN
 * File                IntegrityNetworkStackConnection.c
 * Description         Clarinox WLAN Stack interface implementation For Integrity
 *                     (Separate VAS version)
 *
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2013
*
*******************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
//#include <alloca.h>

#include "ClxBsp.h"
#include "ClxTypes.h"
#include "ClarinoxErrorCodes.h"
#include "WlanErrors.h"
#include "ClarinoxWLanConst.h"
#include "WlanBufferManagerInterface.h"
#include "WlanNetworkStackInterface.h"

#if defined ( CLARINOX_KOALA_V20 )
#include "stm32f4xx.h"
#endif
#if defined ( LPC1830 ) || defined ( LPC1837 ) || defined ( LPC4330 )
#include "cmsis.h"
#endif
#include "lwipnetconfig.h"
#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/tcp.h"
#include "lwip/udp.h"
#include "netif/etharp.h"
#include "lwip/dhcp.h"

#include "Bsp.Const.h"
 
#define DATA_BUFFER_SIZE                    1640    // bytes (This is enough for a single IP packet assuming MTU of 1500 bytes)

extern "C"
{
  
extern void IAP_httpd_init(void);
  
  
/* Network interface name */
#define IFNAME0 's'
#define IFNAME1 't'
  
extern void Periodic_LWIP_Process(__IO uint32_t localtime);
extern boolean clxNetworkStackServiceInterface_RunPendingCallbacks();
extern void clxNetworkStackServiceInterface_Create(u4 numberOfSockets);
extern void clxNetworkStackServiceInterface_Destroy();

extern ClxWebDataBspInterface webInterface_Thermostat;
extern ClxDhcpDataBspInterface DhcpInterface;
extern ClxTcpIperfDataBspInterface IperfInterface;


#define MAX_NUMBER_OF_HTTP_REQUESTS         5
#define MAX_NUMBER_OF_IPERF_TCP_SOCKETS     2
#define MAX_NUMBER_OF_DHCP_SOCKETS          1
#define MAX_NUMBER_OF_HTTP_TCP_SOCKETS      (MAX_NUMBER_OF_HTTP_REQUESTS + MAX_NUMBER_OF_IPERF_TCP_SOCKETS + MAX_NUMBER_OF_DHCP_SOCKETS + 1)
#define HTTP_CONNECTION_TIMEOUT_VALUE       6000    /* ms */


/*******************************
WLAN interface implementation:
********************************/

 ClxResult lwIpTaskProc(void* data);


enum NetworkInterfaceThreadState
{
  NetworkInterfaceThreadState_Idle,
  NetworkInterfaceThreadState_WokenUp,
  NetworkInterfaceThreadState_Running,
  NetworkInterfaceThreadState_Terminated
};

struct LwIpNetworkInterface
{
	ClxWlanNetworkInterface 	networkInterface; 			/* MUST be the first object in this struct */
	ClxHandle 			wlanStackHandle;

	ClxThreadHandle 		lwIpTask;

	NetworkInterfaceThreadState     lwIpTaskkState;
	
	ClxWlanRxPacketDescriptor* 	rxDescList[NUMBER_OF_RX_BUFFERS]; /* List of RX descriptors to be sent over Ethernet */
	u4 				currentNumberOfRxDescriptors;
	u4 				rxDescListPushIndex;
	u4 				rxDescListPopIndex;
        ClxSemaphore 			lwIpTaskSem;
        
        u4                              currentNumberOfTxDescriptors;
        ClxSemaphore 			lwIpTxDescriptorSem;
           
	boolean 			enabled;
	boolean 			terminated;

	ClxMutex 			txMutex;
	ClxMutex 			rxMutex;
};

struct netif;

 struct WlanNetIF
{
  struct netif                  base;
  LwIpNetworkInterface          wlanStackInterface; 
};

static WlanNetIF wlanNetIF;




static void pushRxDescriptor(LwIpNetworkInterface* tcpipStackConnection,ClxWlanRxPacketDescriptor* desc)
{
  assert(tcpipStackConnection->currentNumberOfRxDescriptors < NUMBER_OF_RX_BUFFERS);

  u4 pushIndex = tcpipStackConnection->rxDescListPushIndex;

  tcpipStackConnection->rxDescList[pushIndex] = desc;

  tcpipStackConnection->rxDescListPushIndex = (pushIndex + 1) % NUMBER_OF_RX_BUFFERS;

  ++tcpipStackConnection->currentNumberOfRxDescriptors;
}

static ClxWlanRxPacketDescriptor* popRxDescriptor(LwIpNetworkInterface* tcpipStackConnection)
{
  ClxWlanRxPacketDescriptor* ret = NULL;
    
  if (tcpipStackConnection->currentNumberOfRxDescriptors)
  {
    u4 popIndex = tcpipStackConnection->rxDescListPopIndex;

    tcpipStackConnection->rxDescListPopIndex = (popIndex + 1) % NUMBER_OF_RX_BUFFERS;

    ret = tcpipStackConnection->rxDescList[popIndex];
    
    --tcpipStackConnection->currentNumberOfRxDescriptors;
  }
  
  return ret;
}

static ClxWlanTxPacketDescriptor* getTxDescriptor(LwIpNetworkInterface* tcpipStackConnection)
{
  ClxWlanBuffer* buf = NULL;
  u4 count;

  count = clxWlanGetTxBuffers(tcpipStackConnection->wlanStackHandle, &buf, 1);

  if (count)
  {
    ClxWlanTxPacketDescriptor* desc = NULL;
    clxWlanGetFreeNetworkTxDescriptors(tcpipStackConnection->wlanStackHandle, &desc, 1);

    assert(desc);
    assert(buf);
    assert(buf->length == DATA_BUFFER_SIZE);

    desc->body[0].buffer = buf;

    return desc;
  }
  else
  {
    return NULL;
  }
}


u4 wlanInitGetInterfaceVersion(struct ClxWlanNetworkInterface* thisObj)
{
    return 1;
}

/****************************************************************************/
void networkInterfaceLockTx(struct ClxWlanNetworkInterface* thisObj)
{
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;
  clxAcquireMutex(tcpipStackConnection->txMutex);
}

/****************************************************************************/
void networkInterfaceUnlockTx(struct ClxWlanNetworkInterface* thisObj)
{
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;
  clxReleaseMutex(tcpipStackConnection->txMutex);
}

/****************************************************************************/
void networkInterfaceLockRx(struct ClxWlanNetworkInterface* thisObj)
{
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;
  clxAcquireMutex(tcpipStackConnection->rxMutex);
}

/****************************************************************************/
void networkInterfaceUnlockRx(struct ClxWlanNetworkInterface* thisObj)
{
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;
  clxReleaseMutex(tcpipStackConnection->rxMutex);
}


/****************************************************************************/
ClxResult networkInterfaceInit (_in_ struct ClxWlanNetworkInterface* thisObj,
                           _in_ ClxHandle interfaceHandle,
                           _in_ ClxSize numberOfTxDescriptors,
                           _in_ ClxSize numberOfRxDescriptors)
{    
    LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;
    
    tcpipStackConnection->lwIpTaskkState = NetworkInterfaceThreadState_Idle;
    tcpipStackConnection->lwIpTask = NULL;

    assert(numberOfTxDescriptors == NUMBER_OF_TX_BUFFERS);
    assert(numberOfRxDescriptors == NUMBER_OF_RX_BUFFERS);
 
    tcpipStackConnection->enabled = FALSE;
    
    tcpipStackConnection->currentNumberOfRxDescriptors = 0;
    tcpipStackConnection->rxDescListPushIndex = 0;
    tcpipStackConnection->rxDescListPopIndex = 0;

    tcpipStackConnection->currentNumberOfTxDescriptors = NUMBER_OF_TX_BUFFERS;
    
    tcpipStackConnection->wlanStackHandle = interfaceHandle;
    
    tcpipStackConnection->txMutex = clxCreateMutex();
    tcpipStackConnection->rxMutex = clxCreateMutex();
    tcpipStackConnection->lwIpTaskSem = clxCreateSemaphore(0);
    tcpipStackConnection->lwIpTxDescriptorSem = clxCreateSemaphore(0);
 
    return CLX_SUCCESS;
}

/****************************************************************************/
void networkInterfaceHandleIndication (_in_ struct ClxWlanNetworkInterface* thisObj, _in_ const ClxWlanNetworkInterfaceIndication* indication)
{
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;
  
  switch(indication->indicationID)
  {
  case CLX_WLAN_NETWORK_INTERFACE_INDICATION_STARTED:
      {
          ClxWlanNetworkInterfaceIndication_Started* indication_ = (ClxWlanNetworkInterfaceIndication_Started*)indication;

          wlanNetIF.base.hwaddr_len = ETHARP_HWADDR_LEN;

          /* set MAC hardware address */
          wlanNetIF.base.hwaddr[0] =  indication_->localMacAddress[0];
          wlanNetIF.base.hwaddr[1] =  indication_->localMacAddress[1];
          wlanNetIF.base.hwaddr[2] =  indication_->localMacAddress[2];
          wlanNetIF.base.hwaddr[3] =  indication_->localMacAddress[3];
          wlanNetIF.base.hwaddr[4] =  indication_->localMacAddress[4];
          wlanNetIF.base.hwaddr[5] =  indication_->localMacAddress[5];
      }
      break;

  case CLX_WLAN_NETWORK_INTERFACE_INDICATION_STOPPED:
      /* TODO */
      //tcpipStackConnection->enabled = FALSE;
      //clxReleaseSemaphore(tcpipStackConnection->lwIpTaskSem);
      //clxDeleteThread(tcpipStackConnection->lwIpTask);
      break;

  case CLX_WLAN_NETWORK_INTERFACE_INDICATION_LINK_UP:
      tcpipStackConnection->enabled = TRUE;
      tcpipStackConnection->lwIpTaskkState = NetworkInterfaceThreadState_WokenUp;
      tcpipStackConnection->lwIpTask = clxBeginThread(lwIpTaskProc, (void*) thisObj, "ClxWLAN", 512);

      if (tcpipStackConnection->lwIpTask == NULL)
      {
          printf ("Creating a thread for LwIP stack failed.\n");
      }  
      break;

  case CLX_WLAN_NETWORK_INTERFACE_INDICATION_LINK_DOWN:
      /* TODO */
      tcpipStackConnection->enabled = FALSE;
      clxReleaseSemaphore(tcpipStackConnection->lwIpTaskSem);
      clxDeleteThread(tcpipStackConnection->lwIpTask);
      break;

  default:
      break;
  }
}

/****************************************************************************/
void networkInterfaceDisable (_in_ struct ClxWlanNetworkInterface* thisObj)
{
}

/****************************************************************************/
void networkInterfaceDestroy (struct ClxWlanNetworkInterface* thisObj)
{
}

/****************************************************************************/
boolean networkInterfaceRxPdusReceived (struct ClxWlanNetworkInterface* thisObj, ClxSize totalNumOfPendingRxDescriptors)
{  
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;

  ClxWlanRxPacketDescriptor* rxDescList[NUMBER_OF_RX_BUFFERS];
  u4 descriptorsLeft;

  /* RX path is already protected here: */
  descriptorsLeft = clxWlanGetPendingRxDescriptors(tcpipStackConnection->wlanStackHandle, rxDescList, &totalNumOfPendingRxDescriptors);

  assert(descriptorsLeft == 0);

  for (u4 i = 0; i < totalNumOfPendingRxDescriptors; i++)
  {
    pushRxDescriptor(tcpipStackConnection, rxDescList[i]);
  }

  if (tcpipStackConnection->lwIpTaskkState == NetworkInterfaceThreadState_Idle)
  {
    tcpipStackConnection->lwIpTaskkState = NetworkInterfaceThreadState_WokenUp;
    clxReleaseSemaphore(tcpipStackConnection->lwIpTaskSem);
  }

  return TRUE;
}

/****************************************************************************/
boolean networkInterfaceTxPduSendComplete (struct ClxWlanNetworkInterface* thisObj,
                                        ClxSize totalNumOfCompletedTxDescrptors)
{
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) thisObj;
  
  static void* txCompletedDescriptorsIDsList[NUMBER_OF_TX_BUFFERS];

  ClxSize txCompletedDescriptorsIDListSize = totalNumOfCompletedTxDescrptors;

  clxWlanGetCompletedTxDescriptorsID(tcpipStackConnection->wlanStackHandle, txCompletedDescriptorsIDsList, &txCompletedDescriptorsIDListSize);
  
  boolean releaseSem = (tcpipStackConnection->currentNumberOfTxDescriptors == 0) ? TRUE : FALSE;
    
  tcpipStackConnection->currentNumberOfTxDescriptors += txCompletedDescriptorsIDListSize;
  
  assert(tcpipStackConnection->currentNumberOfTxDescriptors <= NUMBER_OF_TX_BUFFERS);
  
  if (releaseSem)
  {
     clxReleaseSemaphore(tcpipStackConnection->lwIpTxDescriptorSem);     
  }
  
  return TRUE;
}

/****************************************************************************/
ClxResult lwIpTaskProc(void* data)
{
  LwIpNetworkInterface* tcpipStackConnection = (LwIpNetworkInterface*) data;
  
  /*  When the netif is fully configured this function must be called.*/
  netif_set_up(&wlanNetIF.base);

  /* Network Stack Service Interface Implementation: */
  clxNetworkStackServiceInterface_Create(MAX_NUMBER_OF_HTTP_TCP_SOCKETS);
  
#ifdef USE_DHCP_SERVER
  ClxNetworkServiceIp4Address dhcpHostAddr;
  CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS(dhcpHostAddr, 0, CLX_DHCP_SERVER_PORT);
  clxDhcpServerInit(&DhcpInterface, &dhcpHostAddr.base);
#endif

#ifdef USE_HTTP_SERVER
  ClxNetworkServiceIp4Address httpHostAddr;
  CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS(httpHostAddr, 0, 80);
  clxMiniHttpServerInit(&webInterface_Thermostat, MAX_NUMBER_OF_HTTP_REQUESTS, HTTP_CONNECTION_TIMEOUT_VALUE, &httpHostAddr.base);
#endif
  
#ifdef USE_IPERF_SERVER  
  ClxNetworkServiceIp4Address iperfHostAddr;
  CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS(iperfHostAddr, 0, IPERF_SERVER_PORT);
  clxTcpIperfServerInit(&IperfInterface, INTERVAL, &iperfHostAddr.base);
#endif
  
  clxAcquireMutex(tcpipStackConnection->rxMutex);

  while(tcpipStackConnection->enabled)
  {
    tcpipStackConnection->lwIpTaskkState = NetworkInterfaceThreadState_Running;  
    
    /* Running any pending Network service callbacks: */
    while (clxNetworkStackServiceInterface_RunPendingCallbacks()) {}
    
    ClxWlanRxPacketDescriptor* current = popRxDescriptor(tcpipStackConnection);
    
    if (current)
    {
      clxReleaseMutex(tcpipStackConnection->rxMutex);
      
      u4 length = current->totalPacketSize;
      u8_t* data = current->body[0].dataAddress;
      
      u4 l = 0;
      
      /* We allocate a pbuf chain of pbufs from the Lwip buffer pool */
        struct pbuf* p = pbuf_alloc(PBUF_RAW, length, PBUF_POOL);
      
      /* copy received frame to pbuf chain */
      if (p != NULL)
      {
        for (struct pbuf* q = p; q != NULL; q = q->next)
        {
          memcpy((u8_t*)q->payload, (u8_t*)&data[l], q->len);
          l = l + q->len;
        }    
      
        err_t err = wlanNetIF.base.input(p, &wlanNetIF.base);
        
        if (err != ERR_OK)
        {
          pbuf_free(p);
        }        
      }
      
      clxAcquireMutex(tcpipStackConnection->rxMutex);

      /* We release all RX descriptors back to the WLAN stack: */
      clxWlanReleaseRxDescriptors(tcpipStackConnection->wlanStackHandle, &current, 1);     
    }
    else
    {
        if (!tcpipStackConnection->enabled)
        {
           break;
        }

        tcpipStackConnection->lwIpTaskkState = NetworkInterfaceThreadState_Idle;

        clxReleaseMutex(tcpipStackConnection->rxMutex);
        
        /* handle periodic timers for LwIP */
        Periodic_LWIP_Process(clxTickTime());
      
        clxAcquireSemaphoreTimed(tcpipStackConnection->lwIpTaskSem, 100);
        clxAcquireMutex(tcpipStackConnection->rxMutex);
    }   
  }
  
  tcpipStackConnection->lwIpTaskkState = NetworkInterfaceThreadState_Terminated;
  clxReleaseMutex(tcpipStackConnection->rxMutex);
  
  netif_set_down(&wlanNetIF.base);
  
  clxMiniHttpServerDestroy();
  
  clxNetworkStackServiceInterface_Destroy();
  
  return CLX_SUCCESS;  
}



/* WLAN to TCP/IP stack bindings */

struct ClxWlanNetworkInterface* clxGetWlanTcpIpInterface()
{
    wlanNetIF.wlanStackInterface.networkInterface.init                      = networkInterfaceInit;
    wlanNetIF.wlanStackInterface.networkInterface.handleIndication          = networkInterfaceHandleIndication;
    wlanNetIF.wlanStackInterface.networkInterface.destroy                   = networkInterfaceDestroy;
    wlanNetIF.wlanStackInterface.networkInterface.lockTx                    = networkInterfaceLockTx;
    wlanNetIF.wlanStackInterface.networkInterface.unlockTx                  = networkInterfaceUnlockTx;
    wlanNetIF.wlanStackInterface.networkInterface.lockRx                    = networkInterfaceLockRx;
    wlanNetIF.wlanStackInterface.networkInterface.unlockRx                  = networkInterfaceUnlockRx;
    wlanNetIF.wlanStackInterface.networkInterface.rxPdusReceived            = networkInterfaceRxPdusReceived;
    wlanNetIF.wlanStackInterface.networkInterface.txPdusSendingComplete     = networkInterfaceTxPduSendComplete;

    return &wlanNetIF.wlanStackInterface.networkInterface;
}

/* lwIP stuff: */

static err_t clxSendWlanFrame(struct netif *netif, struct pbuf *p)
{
  LwIpNetworkInterface* tcpipStackConnection = &(((WlanNetIF*)netif)->wlanStackInterface);
  
  ClxWlanTxPacketDescriptor* desc = NULL;
      
  while(1)
  {
    clxAcquireMutex(tcpipStackConnection->txMutex);
    
    if (tcpipStackConnection->currentNumberOfTxDescriptors == 0)
    {
       clxReleaseMutex(tcpipStackConnection->txMutex);
       clxAcquireSemaphore(tcpipStackConnection->lwIpTxDescriptorSem);
       
       continue;
    }
    
    desc = getTxDescriptor(tcpipStackConnection);
    assert(desc);
    
    --tcpipStackConnection->currentNumberOfTxDescriptors;
    
    clxReleaseMutex(tcpipStackConnection->txMutex);
    break;
  }
  
  struct pbuf *q;
  int framelength = 0;
  
  /* copy frame from pbufs to driver buffers */
  for(q = p; q != NULL; q = q->next) 
  {
    memcpy((u8_t*)desc->body[0].buffer->address + framelength, q->payload, q->len);
	framelength = framelength + q->len;
  }
  
  
  desc->totalPacketSize = (u4)framelength;
  desc->body[0].dataAddress = desc->body[0].buffer->address;
  desc->body[0].dataLength = desc->totalPacketSize; // set to total including header
  desc->body[1].buffer = NULL;
  desc->body[1].dataAddress = NULL;
  desc->body[1].dataLength = 0;
  desc->userID = NULL;
  
  /* This does not need TX path protection: */
  clxWlanQueueNetworkTxDescriptor(tcpipStackConnection->wlanStackHandle, desc);

  /* Now, we flush the TX descriptor to the WLAN driver : */
  clxWlanFlushPendingNetworkTxDescriptors(tcpipStackConnection->wlanStackHandle);
  
  return ERR_OK;
}

struct netif* clxGetWlanNetIF()
{
    return &wlanNetIF.base;
}

err_t clxInitWlanNetIF(struct netif *netif)
{
#if LWIP_NETIF_HOSTNAME
  /* Initialize interface hostname */
  netif->hostname = "clarinox";
#endif /* LWIP_NETIF_HOSTNAME */

  netif->name[0] = IFNAME0;
  netif->name[1] = IFNAME1;
  /* We directly use etharp_output() here to save a function call.
   * You can instead declare your own function an call etharp_output()
   * from it if you have to do some checks before sending (e.g. if link
   * is available...) */
  netif->output = etharp_output;
  netif->linkoutput = clxSendWlanFrame;
    
  netif->hwaddr_len = ETHARP_HWADDR_LEN;

  /* set MAC hardware address to 0.0.0.0 for now: */
  netif->hwaddr[0] =  0;
  netif->hwaddr[1] =  0;
  netif->hwaddr[2] =  0;
  netif->hwaddr[3] =  0;
  netif->hwaddr[4] =  0;
  netif->hwaddr[5] =  0;

  /* maximum transfer unit */
  netif->mtu = 1500;

  /* device capabilities */
  /* don't set NETIF_FLAG_ETHARP if this device is not an ethernet one */
  netif->flags = NETIF_FLAG_BROADCAST | NETIF_FLAG_ETHARP | NETIF_FLAG_LINK_UP;  
  return ERR_OK;
}



}



