/******************************************************************************
*
* Project             Wlan Sample Application
* File                WlanServer.cpp
* Description         Wlan AP role application file
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "ClxBsp.h"
#include "ConsoleUIEngine.h"
#include "ClarinoxWlan.h"
#include "Wlan.Api.h"
#include "Wlan.Config.h"

#include "WiFiApp.h"


void showAccessPointMenu(ClxStack stack)
{
    ClxError ret;
    boolean blocking = TRUE;

    /*
    Define a structure to contain all the initial configuration parameters to be
    passed to clxWlanStartInterface() for the local station (AccessPoint role).

    All objects SHALL be initialized using the initialization functions provided for each configuration type.
    */
    struct ApConfig
    {
        ClxConfigList    parentList;
        struct
        {
            ClxWlanConfigRate rates[21];
            ClxConfigArray    ratesArray;
        }supportedRates;         
        ClxConfigData    ssid;
        ClxConfigString  freqBand;
        ClxConfigInteger channel;
        ClxConfigInteger hiddenSSID;
        ClxConfigInteger maxTxPower;
        ClxConfigString  channelType;
        ClxConfigInteger beaconInterval;
        ClxConfigInteger dtimInterval;
        ClxConfigInteger maxNumStations;
        ClxConfigInteger wifiSupportedStandards;
        ClxConfigInteger encProtocol;
        ClxConfigInteger clientInactivityTimeout;
        ClxConfigString  passPhrase;
        ClxConfigData    encKey;
        /*
        Driver specific parameters:
        */
        ClxConfigString  sdioDriverName;
        ClxConfigString  firwareFileName;
        ClxConfigData    masterKey;
        ClxConfigInteger frameCaptureMask;
    };

    ApConfig apConfig;


    /*
    Initialize the parent list object (apConfig.parentList) which is to hold all other configuration objects (SHALL be initialized first):
    */
    clxConfigInitParamsList(&apConfig.parentList, NULL, NULL);


    /* 
    Name of the SDIO driver. This value is passed to the SDIO interface implementation in the BSP (ClxSdioBspInterface),
    and its meaning depends on the BSP interface implementation and/or the underlying platform.
    Refer to the SDIO interface implementation in the BSP
    */
#if defined(IMX6_SABRE_LITE)
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev", &apConfig.parentList);    
#elif defined(IMX6_SABRE_AUTO)
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &apConfig.parentList);    
#elif defined(IMELCO)
    clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &goConfig.parentList);      
#elif defined(DM3730_BSP)     
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &apConfig.parentList);
#elif defined(PROPAK6_BSP)
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &apConfig.parentList); 
#elif defined(CLARINOX_KOALA_V20)
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &apConfig.parentList);
#elif defined(LPC1830) /*LPC1830 Xplorer */
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &apConfig.parentList); 
#elif defined(LPC1837) /*LPC1837 Xplorer */
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &apConfig.parentList); 
#elif defined(LPC4330) /*LPC4330 Xplorer */
    clxConfigInitStringParam(&apConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &apConfig.parentList);    
#else     
    #error Select your board
#endif
    
    /* 
    The file containing WL18xx firmware binary. Can be a real file in the underlying file system, or a virtual file 
    through an implementation of ClxFileBspInterface BSP interface:
    */
    clxConfigInitStringParam (&apConfig.firwareFileName, "FirmwareFileName", "Wl18xx-fw-2-Rev8-8-0-0-13.bin", &apConfig.parentList);
    
    /* Maximum TX Power to be used by WL18xx hardware, in dBm */
    clxConfigInitIntegerParam(&apConfig.maxTxPower, "MaxTxPower", 25, &apConfig.parentList);

    /* Frames to be captured by ClarinoxDebugger. 0xFFFFFFFF & (~CLX_WLAN_FRAME_TYPE_BEACON) means capture all frames except for beacons.  */
    clxConfigInitIntegerParam(&apConfig.frameCaptureMask,   "FrameCaptureMask", 0xFFFFFFFF & (~CLX_WLAN_FRAME_TYPE_BEACON),  &apConfig.parentList);
    
    while (1)
    {  
        const s1* menu = "Start Interface as an AccessPoint, with default configuration\0"
            "Start Interface as an AccessPoint, with custom configuration\0"
            "Enable the device\0"
            "Disable the device\0"
            "Get current configuration of AP\0"
            "Get the list of joint STs\0"
            "Get Max rate for a joined client\0"
            "Stop Interface\0"
            "Return to previous menu";
        u4 s = ClxConsoleUIEngine::showMenu("Wlan AP Menu", menu, 9);

        if (s == 1)
        {
            /*
            List of data rates which are available and can be used for both sending and receiving of frames. 
            The list also determines what standards (e.g. IEEE802.11b, IEEE802.11g, IEEE802.11n) are enabled for the local station.
            NOTE : When a particular standard is to be used, ALL MANDATORY rates for that standard SHALL be included in the list.
            NOTE : In AP role, the 'Basic Rate' flag (second last argument of the function clxConfigInitRateParam) defines the rates which must be
            supported by all stations in the BSS as they are used by the AP to send broadcast and multicast frames.
            NOTE : ClxWlanConfigRate objects are not directly owned by the parent list. They are owned by an array object of type ClxConfigArray.
            Therefore, we pass NULL as the last argument of clxConfigInitRateParam.
            NOTE : If a rate in the list is not supported by the WLAN hardware, it will be silently ignored.

            Non-HT (IEEE802.11abg) rates.
            */
            clxConfigInitRateParam(&apConfig.supportedRates.rates[0],  NULL, ClxWlanNonHTRate_1Mbps,  0, TRUE,  NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[1],  NULL, ClxWlanNonHTRate_2Mbps,  0, TRUE,  NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[2],  NULL, ClxWlanNonHTRate_5_5Mbps,0, TRUE,  NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[3],  NULL, ClxWlanNonHTRate_6Mbps,  0, TRUE,  NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[4],  NULL, ClxWlanNonHTRate_9Mbps,  0, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[5],  NULL, ClxWlanNonHTRate_12Mbps, 0, TRUE,  NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[6],  NULL, ClxWlanNonHTRate_18Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[7],  NULL, ClxWlanNonHTRate_24Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[8],  NULL, ClxWlanNonHTRate_33Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[9],  NULL, ClxWlanNonHTRate_36Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[10], NULL, ClxWlanNonHTRate_48Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[11], NULL, ClxWlanNonHTRate_54Mbps, 0, FALSE, NULL);

            /* HT (IEEE802.11n) rates: */
            clxConfigInitRateParam(&apConfig.supportedRates.rates[12], NULL, ClxWlanNonHTRate_NULL, 0, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[13], NULL, ClxWlanNonHTRate_NULL, 1, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[14], NULL, ClxWlanNonHTRate_NULL, 2, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[15], NULL, ClxWlanNonHTRate_NULL, 3, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[16], NULL, ClxWlanNonHTRate_NULL, 4, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[17], NULL, ClxWlanNonHTRate_NULL, 5, FALSE, NULL);
            clxConfigInitRateParam(&apConfig.supportedRates.rates[18], NULL, ClxWlanNonHTRate_NULL, 6, FALSE, NULL);

            /*
            Initialize the configuration array object (stationConfig.supportedRates) to hold the list of supported data rates:
            */
            clxConfigInitParamsArray(&apConfig.supportedRates.ratesArray,
                                     "SupportedRates",
                                     (ClxConfigParam*)apConfig.supportedRates.rates,
                                     19,
                                     &apConfig.parentList);

            if(interfaceState == Stopped)
            {
                if (!clxEnableWl18xxWlanHardware())
                {
                    ClxConsoleUIEngine::text("Enabling WLAN hardware failed\n");
                    break;
                }
            }
            
             /* SSID of the network to be established. It is NOT null-terminated. The length of the SSID is given as the forth argument: */
            clxConfigInitDataParam   (&apConfig.ssid,           "SSID", (u1*)             "Clarinox_AP", 11,    &apConfig.parentList); 

            clxConfigInitStringParam (&apConfig.freqBand,       "FreqBand",               "2.4GHZ",            &apConfig.parentList);           /* The frequency band in which the BSS will operate                             */
            clxConfigInitIntegerParam(&apConfig.channel,        "Channel",                6,                   &apConfig.parentList);           /* The channel on which the BSS will operate (2.4GHZ or 5GHZ)                   */
            clxConfigInitIntegerParam(&apConfig.hiddenSSID,     "HiddenSSID",             0,                   &apConfig.parentList);           /* The SSID will be hidden (if value 1) or will be transparent (if value is 0)  */
            clxConfigInitStringParam (&apConfig.channelType,    "ChannelType",            "NO_HT",             &apConfig.parentList);           /* The frequency channel type (NO_HT for traditional 20MHz channels)            */
            clxConfigInitIntegerParam(&apConfig.beaconInterval, "BeaconInterval",         300,                 &apConfig.parentList);           /* Beacon Interval in milliseconds                                              */
            clxConfigInitIntegerParam(&apConfig.dtimInterval,   "DtimInterval",           2,                   &apConfig.parentList);           /* DTIM Interval in units of BeaconInterval                                     */
            clxConfigInitIntegerParam(&apConfig.maxNumStations, "Ap.MaxNumberOfStations", 8,                   &apConfig.parentList);           /* Max number of clients which can join the BSS at the same time                */
            clxConfigInitStringParam (&apConfig.passPhrase,     "Ap.EncryptionPasskey",   "password999",       &apConfig.parentList);           /* The encryption passkey used to generate the master key                       */
            clxConfigInitIntegerParam(&apConfig.encProtocol,    "Ap.EncryptionProtocol",  (u4)ClxEncProtoAES_CCMP, &apConfig.parentList);       /* The encryption protocol(s) supported by the BSS (values can be ORes)         */
             
            /* The maximum time, in seconds, for an associated client to be inactive before it is de-associated and removed from the BSS: */
            clxConfigInitIntegerParam(&apConfig.clientInactivityTimeout, "ClientInactivityTimeout", 300 /* Seconds */, &apConfig.parentList);  

            /*
            You may also use the parameter "Ap.EncryptionKey" of type ClxConfigData in order to give the 32byte key directly to the stack.
            In this case, Ap.EncryptionPasskey is not required and will be ignored:

            u1 keyData[32]; // store the key in here
            clxConfigInitDataParam(&apConfig.encKey, "Ap.EncryptionKey", keyData, 32, &apConfig.parentList);
            */
            ret = clxWlanStartInterface(wlanHandle, ClxIfTypeAP, clxGetWlanTcpIpInterface(), &apConfig.parentList, blocking);

            if (ret != CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("Starting the WLAN virtual AccessPoint failed with error %s\n", 
                                         clxGetWlanErrorCodeText(ret)); 
            }
            else
            {
                u1 macAddr[6];
                clxWlanInterfaceGetMacAddr(wlanHandle, macAddr, TRUE);
                ClxConsoleUIEngine::text("WLAN virtual interface started with MAC address %02X:%02X:%02X:%02X:%02X:%02X\n", 
                    macAddr[0],
                    macAddr[1],
                    macAddr[2],
                    macAddr[3],
                    macAddr[4],
                    macAddr[5]);

                interfaceState = Started;
            }          
        }
        else if (s == 2)
        {
            u4 selectedAuth = 0;
            if(interfaceState == Stopped)
            {
                if (!clxEnableWl18xxWlanHardware())
                {
                    ClxConsoleUIEngine::text("Enabling WLAN hardware failed\n");
                    break;
                }
            }

            /* default settings */
            clxConfigInitDataParam   (&apConfig.ssid,           "SSID",                     (u1*)"clxTestClx", 10,  &apConfig.parentList);
            clxConfigInitStringParam (&apConfig.freqBand,       "FreqBand",                 "2.4GHZ",               &apConfig.parentList);
            clxConfigInitIntegerParam(&apConfig.channel,        "Channel",                  6,                      &apConfig.parentList);
            clxConfigInitIntegerParam(&apConfig.hiddenSSID,     "HiddenSSID",               1,                      &apConfig.parentList);
            clxConfigInitStringParam (&apConfig.channelType,    "ChannelType",              "NO_HT",                &apConfig.parentList);
            clxConfigInitIntegerParam(&apConfig.beaconInterval, "BeaconInterval",           300,                    &apConfig.parentList);
            clxConfigInitIntegerParam(&apConfig.dtimInterval,   "DtimInterval",             2,                      &apConfig.parentList);
            clxConfigInitIntegerParam(&apConfig.maxNumStations, "Ap.MaxNumberOfStations",   8 ,                     &apConfig.parentList);
            clxConfigInitIntegerParam(&apConfig.wifiSupportedStandards,    "WifiSupportedStandards",  /* (u4)ClxWifiSupportedStandard_WPA  | */(u4)ClxWifiSupportedStandard_WPA2, &apConfig.parentList);
            clxConfigInitIntegerParam(&apConfig.encProtocol,    "Ap.EncryptionProtocol",    (u4)ClxEncProtoAES_CCMP,&apConfig.parentList);
            /* ASCII Key */
            clxConfigInitStringParam (&apConfig.passPhrase,     "Ap.EncryptionPasskey",     "password999",          &apConfig.parentList);
            
            /* ASCII Key 32 bytes*/
           // clxConfigInitStringParam(&apConfig.passPhrase, "Ap.EncryptionPasskey", "password999password999password99", &apConfig.parentList);

            /* Hex Key */
           // u1 masterKey[32] = {0x12,0x34,0x56,0x78,0x90,0x12,0x34,0x56,0x78,0x90,0x12,0x34,0x56,0x78,0x90,0x12,0x34,0x56,0x78,0x90,0x12,0x34,0x56,0x78,0x90,0x12,0x34,0x56,0x78,0x90,0x12,0x34};
           // clxConfigInitDataParam(&apConfig.masterKey, "Ap.EncryptionKey", masterKey, 32, &apConfig.parentList);       
            
            while(1)
            {
                s1* menu =  "Supported Rates\0"
                    "Authentication\0"
                    "Channel\0"
                    "SSID broadcast\0"
                    "Start interface with selected settings\0"
                    "Return to previous menu";

                u4 s = ClxConsoleUIEngine::showMenu("Select the configuration to change:", menu, 6);
                if(s == 1)
                {
                    s1* menu = "g/n/b\0"
                        "b only\0"
                        "g only\0"
                        "n only\0"
                        "Return to main menu";

                    s = ClxConsoleUIEngine::showMenu("Select the Authentication type:", menu, 5);
                    /* common rates */
                    clxConfigInitRateParam(&apConfig.supportedRates.rates[0], NULL, ClxWlanNonHTRate_1Mbps, 0, TRUE, NULL);
                    clxConfigInitRateParam(&apConfig.supportedRates.rates[1], NULL, ClxWlanNonHTRate_2Mbps, 0, TRUE, NULL);
                    switch(s)
                    {
                    case 1:
                        /*
                        Non-HT (IEEE802.11abg) rates.
                        Here, only 1Mbps, 2Mpbs, 5.5Mbps, 11Mbps are set as basic rates. In AccessPoint mode, these are
                        the rates which must be supported by all stations joining this AP. In station mode, the basicRate flag
                        is ignored.
                        */
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[2],  NULL, ClxWlanNonHTRate_5_5Mbps,0, TRUE,  NULL); //802.11b
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[3],  NULL, ClxWlanNonHTRate_6Mbps,  0, FALSE, NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[4],  NULL, ClxWlanNonHTRate_9Mbps,  0, FALSE, NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[5],  NULL, ClxWlanNonHTRate_11Mbps, 0, TRUE,  NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[6],  NULL, ClxWlanNonHTRate_12Mbps, 0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[7],  NULL, ClxWlanNonHTRate_18Mbps, 0, FALSE, NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[8],  NULL, ClxWlanNonHTRate_24Mbps, 0, FALSE, NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[9],  NULL, ClxWlanNonHTRate_33Mbps, 0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[10],  NULL, ClxWlanNonHTRate_36Mbps,0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[11], NULL, ClxWlanNonHTRate_48Mbps, 0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[12], NULL, ClxWlanNonHTRate_54Mbps, 0, FALSE, NULL); //802.11g  

                        /* 802.11 n */
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[13], NULL, ClxWlanNonHTRate_NULL, 0, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[14], NULL, ClxWlanNonHTRate_NULL, 1, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[15], NULL, ClxWlanNonHTRate_NULL, 2, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[16], NULL, ClxWlanNonHTRate_NULL, 3, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[17], NULL, ClxWlanNonHTRate_NULL, 4, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[18], NULL, ClxWlanNonHTRate_NULL, 5, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[19], NULL, ClxWlanNonHTRate_NULL, 6, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[20], NULL, ClxWlanNonHTRate_NULL, 7, FALSE, NULL);

                        clxConfigInitParamsArray(&apConfig.supportedRates.ratesArray, "SupportedRates", (ClxConfigParam*)apConfig.supportedRates.rates, 21 , &apConfig.parentList);
                        break;
                    case 2:
                        /* 802.11 b */
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[2],  NULL, ClxWlanNonHTRate_5_5Mbps, 0, TRUE, NULL); //802.11b
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[3], NULL, ClxWlanNonHTRate_11Mbps, 0, TRUE, NULL);   //802.11b  

                        clxConfigInitParamsArray(&apConfig.supportedRates.ratesArray, "SupportedRates", (ClxConfigParam*)apConfig.supportedRates.rates, 3 , &apConfig.parentList);
                        break;
                    case 3:
                        /* 802.11 g */
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[2], NULL,  ClxWlanNonHTRate_6Mbps,  0, TRUE,  NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[3], NULL,  ClxWlanNonHTRate_9Mbps,  0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[4],  NULL, ClxWlanNonHTRate_12Mbps, 0, TRUE,  NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[5],  NULL, ClxWlanNonHTRate_18Mbps, 0, FALSE, NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[6],  NULL, ClxWlanNonHTRate_24Mbps, 0, FALSE, NULL); //802.11g
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[7],  NULL, ClxWlanNonHTRate_33Mbps, 0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[8],  NULL, ClxWlanNonHTRate_36Mbps, 0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[9], NULL, ClxWlanNonHTRate_48Mbps, 0, FALSE, NULL); //802.11g  
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[10], NULL, ClxWlanNonHTRate_54Mbps, 0, FALSE, NULL); //802.11g  
                        clxConfigInitParamsArray(&apConfig.supportedRates.ratesArray, "SupportedRates", (ClxConfigParam*)apConfig.supportedRates.rates, 11 , &apConfig.parentList);
                        break;
                    case 4:
                        /* 802.11 n */
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[2], NULL, ClxWlanNonHTRate_NULL, 0, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[3], NULL, ClxWlanNonHTRate_NULL, 1, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[4], NULL, ClxWlanNonHTRate_NULL, 2, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[5], NULL, ClxWlanNonHTRate_NULL, 3, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[6], NULL, ClxWlanNonHTRate_NULL, 4, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[7], NULL, ClxWlanNonHTRate_NULL, 5, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[8], NULL, ClxWlanNonHTRate_NULL, 6, FALSE, NULL);
                        clxConfigInitRateParam(&apConfig.supportedRates.rates[9], NULL, ClxWlanNonHTRate_NULL, 7, FALSE, NULL);

                        clxConfigInitParamsArray(&apConfig.supportedRates.ratesArray, "SupportedRates", (ClxConfigParam*)apConfig.supportedRates.rates, 10 , &apConfig.parentList);
                        break;
                    case 5:
                        return;
                    default:
                        return;
                    }
                }
                else if(s == 2)
                {              
                    s1* menu = "Open system authentication\0"
                        "WEP\0"
                        "TKIP\0"
                        "CCMP\0"
                        "TKIP+CCMP\0"
                        "Return to main menu";

                    s = ClxConsoleUIEngine::showMenu("Select the Authentication type:", menu, 6);

                    switch(s)
                    {
                    case 1:
                        selectedAuth =  ClxEncProtoNONE;
                        break;
                    case 2:
                        selectedAuth =  ClxEncProtoWEP40;
                        break;
                    case 3:
                        selectedAuth =  ClxEncProtoTKIP;
                        break;
                    case 4:
                        selectedAuth =  ClxEncProtoAES_CCMP;
                        break;
                    case 5:
                        selectedAuth =  ClxEncProtoTKIP | ClxEncProtoAES_CCMP;
                        break;
                    default:
                        return;
                    }
                    clxConfigInitIntegerParam(&apConfig.encProtocol, "Ap.EncryptionProtocol", selectedAuth, &apConfig.parentList);
                }
                else if(s == 3)
                {
                    s1 requestedChannel[3];
                    ClxConsoleUIEngine::inputBox("Enter the channel number: ", requestedChannel, sizeof(requestedChannel));
                    clxConfigInitIntegerParam(&apConfig.channel, "Channel", atoi(requestedChannel) , &apConfig.parentList);
                    clxWlanSetParametersValue(wlanHandle, &apConfig.parentList, TRUE);
                }
                else if(s == 4)
                {
                    s1* menu = "Disable SSID broadcast\0"
                        "Enable SSID broadcast\0"
                        "Return to main menu";

                    s = ClxConsoleUIEngine::showMenu("Select the SSID type:", menu, 3);

                    switch(s)
                    {
                    case 1:
                        clxConfigInitIntegerParam(&apConfig.hiddenSSID, "HiddenSSID", 1 , &apConfig.parentList);               
                        break;
                    case 2:
                        clxConfigInitIntegerParam(&apConfig.hiddenSSID, "HiddenSSID", 0 , &apConfig.parentList);
                        break;
                    case 3:
                        return;
                    default:
                        return;
                    }   

                    clxWlanSetParametersValue(wlanHandle, &apConfig.parentList, TRUE);
                }
                else if(s == 5)
                {
                    break;
                }
                else if(s == 6)
                {
                    return;
                }
            }

            /*
            You may also use the parameter "Ap.EncryptionKey" of type ClxConfigData in order to give the 32byte key directly to the stack.
            In this case, Ap.EncryptionPasskey is not required and will be ignored:

            u1 keyData[32]; // store the key in here
            clxConfigInitDataParam(&apConfig.encKey, "Ap.EncryptionKey", keyData, 32, &apConfig.parentList);
            */
            ret = clxWlanStartInterface(wlanHandle, ClxIfTypeAP, clxGetWlanTcpIpInterface(), &apConfig.parentList, blocking);

            if (ret != CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("Starting the WLAN virtual AccessPoint failed with error %s\n", clxGetWlanErrorCodeText(ret)); 
            }
            else
            {
                u1 macAddr[6];
                clxWlanInterfaceGetMacAddr(wlanHandle, macAddr, TRUE);
                ClxConsoleUIEngine::text("WLAN virtual interface started with MAC address %02X:%02X:%02X:%02X:%02X:%02X\n", 
                    macAddr[0],
                    macAddr[1],
                    macAddr[2],
                    macAddr[3],
                    macAddr[4],
                    macAddr[5]);

                interfaceState = Started;
            }          
        }
	    else if (s == 3)
        {
            struct InactivityTimeoutConfig
            {
                ClxConfigList    parentList;
                ClxConfigInteger clientInactivityTimeout;
            };

            InactivityTimeoutConfig inactivityTimeoutConfig;
            
            clxConfigInitParamsList(&inactivityTimeoutConfig.parentList, NULL, NULL);
    
            // ClientInactivityTimeout is in seconds:
            clxConfigInitIntegerParam(&inactivityTimeoutConfig.clientInactivityTimeout, "ClientInactivityTimeout", 3000 /* Seconds */, &inactivityTimeoutConfig.parentList);            

            clxWlanSetParametersValue(wlanHandle, &inactivityTimeoutConfig.parentList, TRUE);
            
            ret = clxWlanStartRole(wlanHandle, TRUE);
            
            ClxConsoleUIEngine::text("Enabling the WLAN virtual AccessPoint complete with result %s\n", 
                                     clxGetWlanErrorCodeText(ret));
        }
        else if (s == 4)
        {
            ret = clxWlanStopRole(wlanHandle, TRUE);
            ClxConsoleUIEngine::text("Enabling the WLAN virtual AccessPoint complete with result %s\n", 
                                     clxGetWlanErrorCodeText(ret));            
        }
        else if (s == 5)
        {
            clxWlanGetParametersValue(wlanHandle, &apConfig.parentList, TRUE);

            ClxConfigString* SSID = (ClxConfigString*)clxFindConfigParam(&apConfig.parentList, "SSID", CLX_CONFIG_TYPE(ClxConfigString));
            if (SSID)
            {
                ClxConsoleUIEngine::text("SSID:  %s\n", SSID->paramValue);
            }

            ClxConfigString* passKey = (ClxConfigString*)clxFindConfigParam(&apConfig.parentList, "Ap.EncryptionPasskey", CLX_CONFIG_TYPE(ClxConfigString));
            if (passKey)
            {
                ClxConsoleUIEngine::text("Pass Key:  %s\n", passKey->paramValue);
            }

            ClxConfigInteger* hiddenSSID = (ClxConfigInteger*)clxFindConfigParam(&apConfig.parentList, "HiddenSSID", CLX_CONFIG_TYPE(ClxConfigInteger));
            if ((hiddenSSID) && (hiddenSSID->paramValue != 0))
            {
                ClxConsoleUIEngine::text("Hidden SSID\n");
            }
            else
            {
                ClxConsoleUIEngine::text("Broadcast SSID\n");
            }

            ClxConfigInteger* channel = (ClxConfigInteger*)clxFindConfigParam(&apConfig.parentList, "Channel", CLX_CONFIG_TYPE(ClxConfigInteger));
            if (channel)
            {
                ClxConsoleUIEngine::text("Channel:  %d\n", channel->paramValue);
            }

            ClxConfigInteger* encryptionProtocol = (ClxConfigInteger*)clxFindConfigParam(&apConfig.parentList, "Ap.EncryptionProtocol", CLX_CONFIG_TYPE(ClxConfigInteger));
            if (encryptionProtocol)
            {
                if(encryptionProtocol->paramValue == ClxEncProtoAES_CCMP)
                {
                    ClxConsoleUIEngine::text("Encryption Protocol: WPA2\n");
                }
                else if(encryptionProtocol->paramValue == ClxEncProtoTKIP)
                {
                    ClxConsoleUIEngine::text("Encryption Protocol: WPA\n");
                }
                else if(encryptionProtocol->paramValue == ClxEncProtoWEP40)
                {
                    ClxConsoleUIEngine::text("Encryption Protocol: WEP40\n");
                }
                else if(encryptionProtocol->paramValue == ClxEncProtoWEP104)
                {
                    ClxConsoleUIEngine::text("Encryption Protocol: WEP104\n");
                }
                else if(encryptionProtocol->paramValue == ClxEncProtoNONE)
                {
                    ClxConsoleUIEngine::text("Encryption Protocol: Open\n");
                }
            }
        }

        else if (s == 6)
        {
            ClxConsoleUIEngine::text ("Number of joint STAs: %d\n\n", jointStCount);

            for(int j=0; j<jointStCount; j++)
            {
                ClxConsoleUIEngine::text ("ST %d : ", j+1);
                for(int i=0; i<6; i++)
                {                    
                    ClxConsoleUIEngine::text ("%X", jointStList[j].macAddr[i]);
                    if(i<5)
                    {
                        ClxConsoleUIEngine::text (":");
                    }
                }
                ClxConsoleUIEngine::text ("\n");
            }
        }

        else if (s ==7)
        {
            /* Structure to hold some information to be retrieved from the WLAN stack for a specific client (The client is identified by its MAC address): */
            struct ClientConfig
            {
                ClxConfigList     parentList;
                ClxConfigData     clientMacAddr;
                u1                clientMacAddrBuffer[6];
                ClxWlanConfigRate MaxRate;
                ClxConfigUnsigned capabilities;
            };

            ClientConfig clientConfig;

            s1 requestedChannel[32];
            ClxConsoleUIEngine::inputBox("\nEnter the client MAC address : ", requestedChannel, sizeof(requestedChannel));

            if (!clxWlanParseMacAddressString(requestedChannel, clientConfig.clientMacAddrBuffer))
            {
                ClxConsoleUIEngine::text ("\nPlease enter a MAC address in the format XX:XX:XX:XX:XX:XX\n");
                continue;
            }

            /* The parent list is initialized first: */
            clxConfigInitParamsList(&clientConfig.parentList, "ClientDetails", NULL);

            /* Assign clientConfig.clientMacAddrBuffer to clientConfig.clientMacAddr. This is the MAC address of the client for which the other parameters are to be retrieved */
            clxConfigInitDataParam (&clientConfig.clientMacAddr, "MacAddress", clientConfig.clientMacAddrBuffer, sizeof(clientConfig.clientMacAddrBuffer), &clientConfig.parentList);
            
            /* Initialize clientConfig.MaxRate. The initial value is not important: */
            clxConfigInitRateParam (&clientConfig.MaxRate, "MaxLinkTxRate", ClxWlanNonHTRate_NULL, 0, FALSE, &clientConfig.parentList);
            
            /* Initialize clientConfig.capabilities. The initial value is not important: */
            clxConfigInitUnsignedParam (&clientConfig.capabilities, "Capabilities", 0, &clientConfig.parentList);
            
            /* Retrieve the value of the parameters defined above: */
            clxWlanGetParametersValue(wlanHandle,                           /* Handle to the virtual interface as returned by clxWlanCreateInterface                                            */
                                      &clientConfig.parentList,             /* A pointer to parent list object containing the parameters to be retrieved                                        */ 
                                      TRUE);                                /* Blocking mode                                                                                                    */

            /* if clientConfig.clientMacAddr has been processed by the WLAN stack, then it means a client with this MAC address has been found. Otherwise, there is no such a client: */
            if (clientConfig.clientMacAddr.paramInfo.processed == FALSE)
            {
                ClxConsoleUIEngine::text ("\nA client with the specified MAC address was not found\n");
                continue;
            }

            /*  If clientConfig.MaxRate has been processed by the WLAN stack, it has a valid value: */
            if (clientConfig.MaxRate.paramInfo.processed)
            {
                if (clientConfig.MaxRate.nonHTValue == ClxWlanNonHTRate_NULL)
                {
                    ClxConsoleUIEngine::text("Max Rate = MCS %u\n", (u4)clientConfig.MaxRate.mcsIndex);
                }
                else
                {
                    ClxConsoleUIEngine::text("Max Rate = %u Mbps\n", (u4)clientConfig.MaxRate.nonHTValue/2);
                }
            }

            /*  If clientConfig.capabilities has been processed by the WLAN stack, it has a valid value: */
            if (clientConfig.capabilities.paramInfo.processed)
            {
                 printCapabilties(clientConfig.capabilities.paramValue);
            }
            else
            {
                ClxConsoleUIEngine::text("\nMax Rate has not been negotiated for the specified client, as yet\n");
            }
        }
        else if (s == 8)
        {
            ret = clxWlanStopInterface(wlanHandle, TRUE);
            if(ret == CLX_SUCCESS)
            {
                interfaceState = Stopped;
            }
            ClxConsoleUIEngine::text ("\nStopping interface is complete with result %s", clxGetWlanErrorCodeText(ret));
            clxDisableWl18xxWlanHardware();
           
            for(int i=0; i<jointStCount;i++)                         /* Empty the joint STAs list                             */
            {
                memset(jointStList[i].macAddr, 0, CLX_MAC_ADDR_LEN);
            }
            jointStCount = 0;            
        }
        else if (s == 9)
        {
            return;
        }
    }
}
