/**

  */

#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/tcp.h"
#include "lwip/udp.h"
#include "netif/etharp.h"
#include "lwip/dhcp.h"
#include "ethernetif.h"
#include "lwipnetconfig.h"
#include <stdio.h>
#if defined ( LPC1830 ) || defined ( LPC1837 ) || defined ( LPC4330 )
#include "cmsis.h"
#endif

#define MAX_TRIES       4

typedef enum 
{ 
  DHCP_CLIENT_START=0,
  DHCP_CLIENT_WAIT_ADDRESS,
  DHCP_CLIENT_ADDRESS_ASSIGNED,
  DHCP_CLIENT_TIMEOUT
} 
DHCP_Client_State;

struct netif eth_netif;
uint32_t TCPTimer = 0;
uint32_t ARPTimer = 0;
uint32_t IPaddress = 0;

#ifdef USE_DHCP_CLIENT
uint32_t DHCPfineTimer = 0;
uint32_t DHCPcoarseTimer = 0;
DHCP_Client_State dhcp_client_state = DHCP_CLIENT_START;
#endif

extern struct netif* clxGetWlanNetIF();
extern err_t clxInitWlanNetIF(struct netif *netif);
extern void showLocalIP(const unsigned char* ip);

void Init_LWIP(void)
{
  struct ip_addr ipaddr;
  struct ip_addr netmask;
  struct ip_addr gw;
  struct netif* wlanNetIF = clxGetWlanNetIF();

  /* Initializes the dynamic memory heap defined by MEM_SIZE.*/
  mem_init();

  /* Initializes the memory pools defined by MEMP_NUM_x.*/
  memp_init();

#ifdef USE_DHCP_CLIENT
  ipaddr.addr = 0;
  netmask.addr = 0;
  gw.addr = 0;
#else
  /* Static address used */
  IP4_ADDR(&ipaddr, IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3);
  IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1 , NETMASK_ADDR2, NETMASK_ADDR3);
  IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
#endif /* USE_DHCP_CLIENT */   

  netif_add(wlanNetIF, &ipaddr, &netmask, &gw, NULL, &clxInitWlanNetIF, &ethernet_input);

  /*  Registers the WLAN network interface.*/
  netif_set_default(wlanNetIF);
  
#if USE_DHCP_SERVER  
  /* Static address used */
  IP4_ADDR(&ipaddr, IP_ADDR0 ,IP_ADDR1 , IP_ADDR2 , IP_ADDR3 );
  IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1, NETMASK_ADDR2, NETMASK_ADDR3);
  IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
  netif_set_addr(wlanNetIF, &ipaddr , &netmask, &gw);  
#endif
}


void Periodic_LWIP_Process(volatile uint32_t localtime)
{
#if LWIP_TCP
  /* TCP periodic process every 250 ms */
  if (localtime - TCPTimer >= TCP_TMR_INTERVAL)
  {
    TCPTimer =  localtime;
    tcp_tmr();
  }
#endif
  
  /* ARP periodic process every 5s */
  if ((localtime - ARPTimer) >= ARP_TMR_INTERVAL)
  {
    ARPTimer =  localtime;
    etharp_tmr();
  }

#if defined(USE_DHCP_CLIENT) && !defined(USE_DHCP_SERVER)
  
  /* Fine DHCP periodic process every 500ms */
  if (localtime - DHCPfineTimer >= DHCP_FINE_TIMER_MSECS)
  {
    DHCPfineTimer =  localtime;
    dhcp_fine_tmr();
    if ((dhcp_client_state != DHCP_CLIENT_ADDRESS_ASSIGNED)&&(dhcp_client_state != DHCP_CLIENT_TIMEOUT))
    { 
      struct ip_addr ipaddr;
      struct ip_addr netmask;
      struct ip_addr gw;
      uint8_t ip_addr_assigned[4];

      if(dhcp_client_state == DHCP_CLIENT_START)
      {
        dhcp_start(clxGetWlanNetIF());
        IPaddress = 0;
        dhcp_client_state = DHCP_CLIENT_WAIT_ADDRESS;
      }
      else if(dhcp_client_state == DHCP_CLIENT_WAIT_ADDRESS)
      {
        /* Read the new IP address */
        IPaddress = clxGetWlanNetIF()->ip_addr.addr;

        if (IPaddress!=0) 
        {
          dhcp_client_state = DHCP_CLIENT_ADDRESS_ASSIGNED;	

          /* Stop DHCP */
          dhcp_stop(clxGetWlanNetIF());

          ip_addr_assigned[0] = (uint8_t)(IPaddress >> 24);
          ip_addr_assigned[1] = (uint8_t)(IPaddress >> 16);
          ip_addr_assigned[2] = (uint8_t)(IPaddress >> 8);
          ip_addr_assigned[3] = (uint8_t)(IPaddress);

          showLocalIP(ip_addr_assigned);        

        }
        else
        {
          /* DHCP timeout */
          if (clxGetWlanNetIF()->dhcp->tries > MAX_TRIES)
          {
            dhcp_client_state = DHCP_CLIENT_TIMEOUT;

            /* Stop DHCP */
            dhcp_stop(clxGetWlanNetIF());

            /* Static address used */
            IP4_ADDR(&ipaddr, IP_ADDR0 ,IP_ADDR1 , IP_ADDR2 , IP_ADDR3 );
            IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1, NETMASK_ADDR2, NETMASK_ADDR3);
            IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
            netif_set_addr(clxGetWlanNetIF(), &ipaddr , &netmask, &gw);

            ip_addr_assigned[0] = IP_ADDR3;
            ip_addr_assigned[1] = IP_ADDR2;
            ip_addr_assigned[2] = IP_ADDR1;
            ip_addr_assigned[3] = IP_ADDR0;
            
            showLocalIP(ip_addr_assigned);
            
          }
        }    
      }        
    }
  }

  /* DHCP Coarse periodic process every 60s */
  if (localtime - DHCPcoarseTimer >= DHCP_COARSE_TIMER_MSECS)
  {
    DHCPcoarseTimer =  localtime;
    dhcp_coarse_tmr();
  }
  
#endif
}   

/*****END OF FILE****/
