/*******************************************************************************************************************************
*
* Project             Bluetooth Application
* File                Bsp.cpp
* Description         This file includes Bsp related functions and variables definitions.
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  1140
// _____________________________________________________________________________
//

#include <stdlib.h>
#include <map>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>

#include "ClxBsp.h"
#include "ClarinoxWlanConst.h"
#include "ClxMemoryPoolset.h"
#include "ClarinoxErrorCodes.h"
#include "WlanErrors.h"
#include "ClarinoxWLanConst.h"
#include "WlanBufferManagerInterface.h"
#include "WlanNetworkStackInterface.h"

#include "Bsp.Const.h"
#include "stm32f4xx_conf.h"


#define DEFAULT_WLAN_DRIVER_HIGH_TASK_PRIORITY               1
#define DEFAULT_WLAN_DRIVER_MEDIUM_HIGH_TASK_PRIORITY        3
#define DEFAULT_WLAN_DRIVER_MEDIUM_TASK_PRIORITY             5
#define DEFAULT_WLAN_DRIVER_MEDIUM_LOW_TASK_PRIORITY         7
#define DEFAULT_WLAN_DRIVER_LOW_TASK_PRIORITY                9



#define PRIVATE_HEAP_START_ADDRESS                                  0x20000000
#define PRIVATE_HEAP_END_ADDRESS                                    0x2002FFFF
#define PRIVATE_HEAP_SIZE                                           0x2FFFF

/* 
 * Use a custom memory pool set: as follows 
 * The first element is the user buffer size and the second one is the no of buffers required
 * Buffer size must be in ascending order
 * Buffer size must be a multiple of 4
 * Use Clarinox Debugger to tune the buffer size and numbers
 */
USER_SUPPLIED_MEMORY_POOL(
    { 8     ,   18 },     //1
    { 16    ,   12 },     //2
    { 20    ,   22 },     //3
    { 28    ,   14 },     //4
    { 40    ,   28 },     //5
    { 44    ,   48 },     //6
    { 56    ,   10 },     //7
    { 68    ,   14 },     //8
    { 72    ,   4  },     //9
    { 92    ,   4  },     //10
    { 116   ,   4  },     //11
    { 128   ,   4  },     //12
    { 164   ,   4  },     //13
    { 196   ,   9  },     //14
    { 228   ,   2  },     //15
    { 256   ,   4  },     //16
    { 356   ,   2  },     //17
    { 388   ,   1  },     //18
    { 520   ,   2  },     //19
    { 588   ,   1  },     //20
    { 696   ,   2  },     //21
    { 736   ,   3  },     //22
    { 800   ,   1  },     //23
    { 840   ,   1  },     //24
    { 872   ,   1  },     //25
    { 1024  ,   6  },     //26
    { 1160  ,   1  },     //27 
    { 1280  ,   2  },     //28 
    { 1496  ,   1  },     //29 
    { 1640  ,   15 },     //30
    { 1984  ,   1  },     //31
    { 2048  ,   3  },     //32
    { 2864  ,   1  },     //33
    { 3000  ,   2  },     //34
    { 3044  ,   1  },     //35
    { 3076  ,   1  },     //36
    { 4000  ,   1  },     //37
    { 4096  ,   2  },     //38
    { 4288  ,   1  },     //39
    { 4604  ,   2  },     //40
    { 5596  ,   2  },     //41
    { 8000  ,   1  },     //42
    { 8192  ,   2  });    //43    

extern  void*                   clxPoolsetAlloc  (u2 moduleID, u4 line, u4 size );
extern  void                    clxFree  ( void* ptr );

#ifdef __cplusplus
extern "C" {
#endif 
  
#if defined(CLX_EXTERNAL_OSC)
u4 Clx_Input_Clock = CLX_EXTERNAL_OSC;
#else
#error "Please define the input clock"
#endif

#if defined(CLX_CPU_CLOCK)
u4 Clx_CPU_Clock = CLX_CPU_CLOCK;
#else
#error "Please define the CPU clock"
#endif

/**************************************BSP & OS includes**************************************/  
#include "StmBsp.h"
#include "os.h" 
#include "stm32f4xx_sdio.h"
#include "stm32f4_discovery.h"
/********************************************************************************************/  
typedef void*                   ClxAlignedMemoryRegion;  
extern ClxFileBspInterface      clxMemoryFileInterface;
extern void                     initROMFileSystemMap();
extern ClxSdioBspInterface      clxSdioSTM32F407BspInterface;
extern bool                     (*clxPowerOnChip) ();
extern bool                     (*clxPowerOffChip) ();
void                            EXTILine0_Config(void);
EXTI_InitTypeDef                EXTI_InitStructure;
GPIO_InitTypeDef                GPIO_InitStructure;
void                            CLX_EXTI_ISR_Handler(void);
extern  void*                   SDIO_eventHandlerData;
extern  ClxSdioEventHandler     SDIO_eventHandler;

#ifdef CLX_KOALA_LCD
extern void initializeLcd ( const u1* lcdAppName );
#endif

#ifdef CLX_KOALA_CAMERA
extern void initializeCamera (void);
#endif

#ifdef CLX_TEMPERATURE_SENSOR_ENABLED
extern void clxConfigureSensorAndGpio();
#endif

extern u2 clarinoxuCosSoftFrameTaskStackSizeTable[4];
extern u2 clarinoxuCosWlanTaskStackSizeTable[3];

/**
An array of 5, and type u2, which maps the ClarinoxBlue platform-independent thread priorities
to underlying platform priorities. The first index (index 0) of the array represents the highest priority,
while the last index (index 4) of the array represents the lowest priority.
If the table is not defined in the BSP, the default priority of the underlying platform will be used for all
threads. 

ALL priorities MUST be defined.

ClarinoxBlue threads:
- Driver thread                         : Highest priority (index 0)
- Stack thread                          : Medium priority (index 2)
- Application Callback thread           : Medium priority (index 2)
- Timer thread                          : High priority (index 1)
- Debug thread (if available)           : Lowest priority (index 4)
- Console Engine thread (if available)  : Lowest priority (index 4)
*/
u2 clarinoxWlanTaskPriorityTable[5]	=                     { DEFAULT_WLAN_DRIVER_HIGH_TASK_PRIORITY,
                                                            DEFAULT_WLAN_DRIVER_MEDIUM_HIGH_TASK_PRIORITY,
                                                            DEFAULT_WLAN_DRIVER_MEDIUM_TASK_PRIORITY,
                                                            DEFAULT_WLAN_DRIVER_MEDIUM_LOW_TASK_PRIORITY,
															DEFAULT_WLAN_DRIVER_LOW_TASK_PRIORITY};



extern "C" ClxUartBspInterface clxUartSTM32F407BspInterface;

/*******************************************************************************************************************************
*                                                    PrivateHeap
*
*  Allocating a private heap of size 192KB (0x20000000 - 0x2002FFFF)
*
*******************************************************************************************************************************/
static char* privateHeapPtr_ = (char*)PRIVATE_HEAP_START_ADDRESS; 


void clxMemoryInitPrivateHeap (void)
{
	privateHeapPtr_ = (char*)PRIVATE_HEAP_START_ADDRESS;
}

void clxMemoryDestroyPrivateHeap (void)
{
	privateHeapPtr_ = (char*)PRIVATE_HEAP_START_ADDRESS;
}

static int belongsToClarinoxPool(void* ptr)
{
  int temp = (int)ptr;
  
  if(((temp == PRIVATE_HEAP_START_ADDRESS) || (temp > PRIVATE_HEAP_START_ADDRESS)) && ((temp == PRIVATE_HEAP_END_ADDRESS) || (temp < PRIVATE_HEAP_END_ADDRESS)))
  {
    return 1;
  }
  else
  {
    return 0;
  }
  
}

void* clxMemoryAllocateFromPrivateHeap(unsigned int size)
{
	void* ret = privateHeapPtr_;
	privateHeapPtr_ += size;		
	if( (privateHeapPtr_ - (char*)PRIVATE_HEAP_START_ADDRESS) > PRIVATE_HEAP_SIZE )
    {
        privateHeapPtr_ = 0;
        while(1);
    }
	return ret;
}

void clxMemoryFreeFromPrivateHeap(void* buff)
{}

size_t clxWlanGetPlatformMemoryPageSize()
{
    return MEMORY_PAGE_SIZE;
}

ClxSize clxWL18xxGetSdioBufferSize()
{
  return SDIO_BUFFER_SIZE;
}

ClxSize clxWL18xxGetTxRxBufferSizeInMemoryPageUnits()
{
  return TX_RX_BUFFER_SIZE_IN_MEMORY_PAGE_UNIT;
}

u4 clxWL18xxGetNumberOfTxBuffers()
{
  return NUMBER_OF_TX_BUFFERS;
}

u4 clxWL18xxGetNumberOfRxBuffers()
{
  return NUMBER_OF_RX_BUFFERS;
}

ClxAlignedMemoryRegion clxWl18xxAllocateSdioDriverBuffer(   u1 minNumberOfPagesRequired,
                                                            u4 maxNumberOfPagesRequired,
                                                            u1** startAddress,
                                                            size_t* bufferLength)
{
    *bufferLength = maxNumberOfPagesRequired*MEMORY_PAGE_SIZE; 
    *startAddress = (u1*)clxPoolsetAlloc(0, 0, *bufferLength);  
    return (ClxAlignedMemoryRegion) *startAddress;
}

ClxAlignedMemoryRegion clxWlanAllocateAlignedMemoryRegion(ClxSize minBufferSize,
															ClxSize maxBufferSize, 
															enum ClxWlanBufferType type, 
															enum ClxWlanBufferPath path, 
															ClxSize requiredAlignment,
															u1** startAddress,
															size_t* bufferLength)
{

    *bufferLength = maxBufferSize; 
    *startAddress = (u1*)clxPoolsetAlloc(0, 0, maxBufferSize);  
    return (ClxAlignedMemoryRegion) *startAddress;
}

void clxWL18xxFreeSdioDriverBuffer(ClxAlignedMemoryRegion region)
{
    clxPoolsetFree((void*)region);
}

void clxWlanFreeMemoryRegion(ClxAlignedMemoryRegion region, enum ClxWlanBufferType type)
{  
        clxPoolsetFree((void*)region);
}

void clxWlanUSleep(u4 us)
{
    u4 ms;
    OS_ERR      err;
    ms = (us / 1000) + 1;

    OSTimeDlyHMSM(0, 0, 0, ms, 
                      OS_OPT_TIME_HMSM_STRICT, 
                      &err);
}

bool clxWl18xxEnableIRQ(void)
{ 
    /*Enable external interrupt*/
    EXTILine0_Config();    
    return true ;
}

bool clxWl18xxDisableIRQ(void)
{
    /*Disable external interrupt*/
    BSP_IntDis(BSP_INT_ID_EXTI4);
    return true ;
}

void EXTILine0_Config(void)
{  
  GPIO_InitTypeDef   GPIO_EXTI_InitStructure;

  /* Enable GPIOA clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
  /* Enable SYSCFG clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
  
  /* Configure PA0 pin as input floating */
  GPIO_EXTI_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_EXTI_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_EXTI_InitStructure.GPIO_Pin = GPIO_Pin_4;
  GPIO_Init(GPIOE, &GPIO_EXTI_InitStructure);

  /* Connect EXTI Line4 to PE4 pin */
  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE, EXTI_PinSource4);  

  /* Configure EXTI Line4 */
  EXTI_InitStructure.EXTI_Line = EXTI_Line4;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising; 
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  /* Enable and set EXTI Line4 Interrupt*/
  BSP_IntVectSet(BSP_INT_ID_EXTI4, CLX_EXTI_ISR_Handler);
  BSP_IntEn(BSP_INT_ID_EXTI4);
}

bool clxWl18xxPowerOnChip(void)
{        
    /* PG10 to be toggled */
    GPIO_SetBits(GPIOG, GPIO_Pin_10);
    return true ;
}

bool clxWl18xxPowerOffChip(void)
{  
    /* PG10 to be toggled */
    GPIO_ResetBits(GPIOG, GPIO_Pin_10);   
    return true ;
}

bool clxEnableWl18xxWlanHardware()
{
  
    bool ret = true;        
    /* Enable WLAN hardware by toggling WLAN_EN pin*/    
    ret = clxWl18xxPowerOffChip();
    clxSleep(200);
    ret = clxWl18xxPowerOnChip();
    clxSleep(500);
    ret = clxWl18xxPowerOffChip();
    clxSleep(200);
    ret = clxWl18xxPowerOnChip();
    clxSleep(500);
  
    return ret;
}

void clxDisableWl18xxWlanHardware()
{
    /*Power off WLAN hardware*/
    clxWl18xxPowerOffChip();
}

void CLX_EXTI_ISR_Handler(void)
{
  if(EXTI_GetITStatus(EXTI_Line4) != RESET)
  {
      EXTI_ClearITPendingBit(EXTI_Line4);
      SDIO_eventHandler((ClxSdio)SDIO,
                SDIO_eventHandlerData,
                ClxSdio_InterruptReceived,
               (ClxSdioFunction)0,
               NULL);
  }
  else
  {
    while(1){}
  }
}

/*******************************************************************************************************************************
*                                                    userPrintf
*
* User can overload the default printf function used by the stack. if userPrintfFunction (function pointer) is not assigned, 
* then the platform default printf will be used
*
*******************************************************************************************************************************/
void printfFunction ( char* formattedOutput )
{
	if (clxTerminalEmulatorEnabled())
	{
		clxTerminalEmulatorPrint((const u1*)formattedOutput, strlen(formattedOutput));
	}
	else
	{
		printf (formattedOutput);
	}
}

/*******************************************************************************************************************************
*                                                    userScanf
*
* User can overload the default userScanf function used by the stack. if userGetInputFunction (function pointer) is not assigned, 
* then the platform default userScanf will be used
*
*******************************************************************************************************************************/
int getInputFunction ( char* buffer, unsigned int bufferSize )
{
	if (clxTerminalEmulatorEnabled())
	{
		clxTerminalEmulatorGetInput(buffer, 
									bufferSize, 
									0x0d,	/* Reception of carriage return character will cause this function to return */
                                    TRUE,
									TRUE,	/* Consider only ASCII characters (0x20 and above) */
									TRUE);	/* Flush the input queue to get rid of the old characters */
	}
	else
	{
		scanf ("%s", buffer);
	}

	return 0 /* Success */;
}

boolean clxSerialIo_SetBaudRate(const s1* portName, u4  baudRate)
{
	return TRUE;
}

void clxInitBsp()
{
#ifdef CLX_KOALA_LCD
	const u1* lcdAppName = "Clarinox WiFi LWIP DHCP";
    initializeLcd(lcdAppName); 
#endif  
#ifdef CLX_KOALA_CAMERA
    initializeCamera();
#endif  
    /* Define these two before anything else: */
	userPrintfFunction                      = printfFunction;                            /* assign user printf function for overloading            */
	userGetInputFunction                    = getInputFunction;

	clxSdioBspInterface                     = &clxSdioSTM32F407BspInterface;
	clxUartBspInterface                     = &clxUartSTM32F407BspInterface;

    clarinoxPlatformWlanTaskStackSizeTable      = clarinoxuCosWlanTaskStackSizeTable;
    clarinoxPlatformSoftFrameTaskStackSizeTable      = clarinoxuCosSoftFrameTaskStackSizeTable;
    
	clxPowerOnChip                          = clxWl18xxPowerOnChip;
	clxPowerOffChip                         = clxWl18xxPowerOffChip;

	clxFileBspInterface                     = &clxMemoryFileInterface;    
    initROMFileSystemMap();
	
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOG, ENABLE);

	/* Configure WLAN_EN pin */
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_10;							
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType           = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed           = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd            = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOG, &GPIO_InitStructure);
    
#ifdef CLX_TEMPERATURE_SENSOR_ENABLED    
    clxConfigureSensorAndGpio();
#endif
}


#ifdef __cplusplus
}
#endif

void operator delete ( void* ptr )
{
    if(belongsToClarinoxPool(ptr))
    {
        /* This is a buffer in ClarinoxSoftFrame private heap: */
        clxPoolsetFree(ptr);
    }
    else
    {
        /* This is a buffer from an external heap: */
        free (ptr);
    }
}

void operator delete[] ( void* ptr )
{
    if(belongsToClarinoxPool(ptr))
    {
        /* This is a buffer in ClarinoxSoftFrame private heap: */
        clxPoolsetFree(ptr);
    }
    else
    {
        /* This is a buffer from an external heap: */
        free (ptr);
    }
}

