#ifndef _ClarinoxBlueUartDriverBsp_h_
#define _ClarinoxBlueUartDriverBsp_h_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                ClarinoxBlue.Driver.Bsp.h
* Description         ClarinoxBlue Driver BSP interface
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


/** 
Maximum SCO packet size, used by the stack
*/
#define CLX_BLUE_MAX_RX_SCO_PACKET_SIZE             256


#ifdef __cplusplus
extern "C" {
#endif



/** 
ACL RX Buffer
*/
struct ClxRxAclBuffer
{
    u1* address;                  /*!< buffer containing data received from an ACL channel. The data can be an entire or a fragment of a higher layer (L2CAP) message. 
                                       The data excludes 1-byte packet type header and any HCI/ACL headers */

    u2  bufferSize;               /*!< buffer size */

    u2  connectionHandle;         /*!< the ACL connection handle. SHALL be set to the same value in all fragments of a single Higher layer (L2CAP) packet */

    u1  pbFlag;                   /*!< The Packet Boundary flag */
    u1  bcFlag;                   /*!< The BroadCast flag */

    u2  length;                   /*!< Length of data stored in this buffer */
};



/**
ClarinoxBlue driver interface. An implementation will interface ClarinoxBlue stack to the Bluetooth controller hardware.

NOTE : This is an optional BSP interface. If an implementation does not exist as part of the BSP, an internal implementation
will be used. The internal implementation may employ other BSP interfaces (e.g. UART BSP interface) to communicate with the Bluetooth
controller hardware.

If an implementation exists, it will overwrite any internal implementation. In this case, the variable #clarinoxBlueDriverInterface shall be set to the address of the implementation instance.

NOTE : An implementation shall be able to receive incoming data, find the packet boundaries, and process HCI events, and incoming ACL and SCO packets. In other words, an implementation shall
have a basic knowledge of HCI packets and their structure to be able to process the packets.
*/
struct ClarinoxBlueDriverInterface
{
    /**
    Called by ClarinoxBlue to initialize the driver interface. It shall initialize any local resources required.

	IMPORTANT : When this method returns, this interface is initialized successfully, but not started yet.
	Therefore, driver API functions are not allowed to be called until #start() is called. 

    This method is called during the initialization of ClarinoxBlue. If this method returns any value other than CLX_SUCCESS, the initialization of ClarinoxBlue will fail.
    */
	void (*init) ();

	/**
	Called by ClarinoxBlue to start the driver interface. An implementation shall initialize the Bluetooth controller hardware.
	If an implementation returns CLX_SUCCESS, then the Bluetooth controller hardware is assumed to be up and running, and the interface has started 
	listening for incoming data from the controller hardware.
	Also, when this method returns, the implementation shall be able send packets to the controller hardware (as a result of a call to #sendData method).

	When this function is called, all driver API functions are permitted to be called by this driver interface.

	This method is called during the initialization of ClarinoxBlue. If this method returns any value other than CLX_SUCCESS, the initialization of ClarinoxBlue will fail.

	NOTE : When this function is called, the driver has NOT been locked (refer to #lock method). If required, this shall be done explicitly by the implementation.

	IMPORTANT : The RX path of the interface is assumed to be asynchronous. An implementation shall wait for incoming data and inform the stack of incoming packets
	only when there are completely received. This may be done from the context of a hardware interrupt handlers, a software interrupt handler or a thread which has been started
	by the interface.

	This function is called in the context of ClarinoxBlue stack thread.

	\param[ in ] configList The list of configuration parameters. This is the same list which has been passed to clxInitClarinoxBlue() function. The list may contain configuration parameters
	related to the driver interface (e.g. UART port, baud rate, ...). An implementation may define its own set of configuration parameters.

	\return CLX_SUCCESS if the interface and the Bluetooth controller hardware have both been started successfully.
	Any other value indicates an error. In this case, the method #destroy will be called to destroy the interface object.
	*/
	ClxResult (*start) (_in_ const ClxConfigList* configList);

	/**
	Called by ClarinoxBlue to stop the driver interface. An implementation shall stop listening to incoming data (and cancel any ongoing procedures). When this method
	returns, the interface is NOT allowed to call any driver API functions any more.

	NOTE : If the interface has started a thread to listen for incoming data, it shall stop the thread before this method returns.

	NOTE : When this function is called, the driver has NOT been locked (refer to #lock method). If required, this shall be done explicitly by the implementation.
	*/
	void (*stop) ();

    /**
    Called by ClarinoxBlue to destroy the driver interface. An implementation shall terminate and destroy any software resources owned by the interface.

    This method is called during termination of ClarinoxBlue stack. This method cannot fail.

    NOTE : When this function is called, the driver has NOT been locked (refer to #lock method). If required, this shall be done explicitly by the implementation.
    */
	void (*destroy) ();

    /**
    Called by ClarinoxBlue to send a packet to the controller hardware. The packet may be a HCI command, and ACL packet or a SCO packet. The first byte of data indicates the type of the packet.
    Please refer to Bluetooth spec for more information.

    NOTE : An implementation shall be synchronous (blocking). In other words, this function shall NOT return until the packet transmission is complete, either in error or with success.

    NOTE : This function is called in the context of ClarinoxBlue stack thread. Therefore, an implementation shall NOT block the execution of the stack for a long time. Otherwise, the stack will become
    unresponsive.

    NOTE : When this function is called, the driver has NOT been locked (refer to #lock method). If required, this shall be done explicitly by the implementation.

    \param[ in ] data The packet to send to the controller, as a contiguous buffer in the memory. The memory for this buffer is guaranteed to have been allocated by a call to #allocateMem method.
    The buffer contains the 1-byte packet type header in the beginning.
    \param[ in ] dataLength Total length of the packet. This includes the 1-byte packet type header as well.
    */
	void (*sendData) (_in_ const u1* data, _in_ ClxSize dataLength);

    /**
    Called by ClarinoxBlue to lock access to the driver. When the driver is locked, it is guaranteed that only one context will be able to access ClarinoxBlue IO buffers. 
    
    This function is called in the context of ClarinoxBlue stack thread.

    If an implementation might call any of driver API functions from the context of a thread, it may acquire a mutex to lock access. 
    If an implementation might call any of driver API functions from an interrupt handler, this function shall disable that interrupt.
    */
	void (*lock) ();

    /**
    Called by ClarinoxBlue to unlock access to the driver. This is called only after #lock has previously been called.

    This function is called in the context of ClarinoxBlue stack thread.

    If an implementation might call any of driver API functions from the context of a thread, it may release a mutex to unlock access. 
    If an implementation might call any of driver API functions from an interrupt handler, this function shall enable that interrupt.
    */
	void (*unlock) ();

    /**
    Called by ClarinoxBlue if some memory needs to be allocated. This memory is used for both TX and RX buffers.

	NOTE : Some platforms require TX/RX buffers to have special properties such as DMA-able or non-cache-able. Since all TX/RX buffers are allocated by a call
	to this method, an implementation may need to take these properties into consideration.

	\param[ in ] moduleID The numeric ID of the module (file) inside ClarinoxBlue which has called this method. Used for debugging purposes.
	\param[ in ] line The line of the code in the module which this method has been called. Used for debugging purposes.
	\param[ in ] size Size, in bytes, of the memory buffer to be allocated. The returned buffer shall be at least of this size.

	\return A pointer to the allocated buffer. This pointer will be passed to #freeMem later on to be freed.
	If a free buffer of requested size is not available, the return value shall be NULL.
    */
	void* (*allocateMem) (u2 moduleID, u4 line, u4 size);

    /**
    Called by ClarinoxBlue if allocated memory needs to be freed.

	\param[ in ] ptr Pointer to the buffer to be freed. This buffer was previously allocated by a call to #allocateMem.
    */
	void(*freeMem) (void* ptr);
};

/**
Called by the driver interface to queue an ACL buffer containing a single received ACL packet. This API function may be called in the context of an interrupt handler.

NOTE : This method does NOT send the buffer to ClarinoxBlue stack. It only queues them internally so they can be flushed later on. In order to flush all queued ACL buffers
to ClarinoxBlue stack, #clxBlueDriverFlushRxAclBuffers() shall be called. 

IMPORTANT : In order to avoid synchronization loss, received ACL packets SHALL be queued in the same order that they have been received from the controller hardware.

NOTE : If a single ClarinoxBlue-allocated RX ACL buffer is not large enough to store a single ACL packet. The driver interface implementation SHALL perform fragmentation by storing the ACL
packet into two or more RX ACL buffers. The following rule shall apply:

- The packet boundary flag (ClxRxAclBuffer.pbFlag) of the first RX ACL buffer object shall be set to the packet boundary flag of the ACL packet (refer to Bluetooth specification for more information).
- The packet boundary flag (ClxRxAclBuffer.pbFlag) of the rest of RX ACL buffer objects of this ACL packet SHALL be set to 2 (Continuing packet). 
- The fragments of the same ACL packet shall be queued in the same order as the data has been received from the controller.

\param[ in ] buffer Pointer to a ClarinoxBlue-allocated object of type #ClxRxAclBuffer which contains the received ACL packet. This buffer has been obtained by a previous call to #clxBlueDriverGetFreeRxAclBuffer()
function.
*/
extern void clxBlueDriverQueueRxAclBuffer(_in_ struct ClxRxAclBuffer* buffer);

/**
Called by the driver interface to flush all queued RX ACL packets to ClarinoxBlue stack. This API function may be called in the context of an interrupt handler.

The queued buffers will be released by ClarinoxBlue stack when they are processed.

NOTE : lock() and unlock() methods of the interface are internally called by this function to protect access to the RX buffers.

NOTE : This function shall be called frequently in order to avoid excessive latencies.
*/
extern void clxBlueDriverFlushRxAclBuffers();

/**
Called by the driver interface to submit a received SCO buffer. The buffer has been obtained by a previous call to #clxBlueReleaseRxScoBuffer() API function.

Please refer to documentation of #ClarinoxBlueScoInterface interface for more information.

\param[ in ] buffer A ClarinoxBlue-allocated object of type ClxRxScoBuffer which contains one single RX SCO packet.
*/
extern void clxBlueDriverSubmitRxScoBuffer(_in_ struct ClxRxScoBuffer* buffer);

/**
Called by the driver interface to submit the received HCI event. This API function may be called in the context of an interrupt handler.

This function copies the received HCI event into an internal buffer. Therefore, as soon as this function returns, the HCI event buffer
may be re-used.

NOTE : lock() and unlock() methods of the interface are internally called by this function to protect access to the RX buffers.

\param[ in ] packet A driver-interface-allocated buffer containing the received HCI packet. Please note that this does not include the 1-byte packet type header.
\param[ in ] packetLength The entire length of the received event. This does not include the 1-byte packet type header. 
*/
extern void clxBlueDriverSubmitRxHciEvent(_in_ const u1* packet, _in_ u2 packetLength);

/**
Called by the driver interface to obtain a free RX ACL buffer. The buffer is allocated by ClarinoxBlue stack and may be used by the driver to store incoming ACL data only.
This API function may be called in the context of an interrupt handler.

NOTE : lock() and unlock() methods of the interface are internally called by this function to protect access to the RX buffers.

If there is no free RX ACL buffer, the return value will be NULL.

After storing an incoming ACL packet, or a fragment of it (please refer to #clxBlueDriverQueueRxAclBuffer() documentation), the API function #clxBlueDriverQueueRxAclBuffer() shall be
called to hand the buffer back to ClarinoxBlue stack.

\return A pointer to a free RX ACL buffer. NULL if there is no free RX ACL buffer available at this moment.
*/
extern struct ClxRxAclBuffer* clxBlueDriverGetFreeRxAclBuffer ();

/**
Called by the driver interface to obtain a free RX SCO buffer. The buffer is allocated by ClarinoxBlue stack and may be used by the driver to store incoming SCO data only.

Please refer to documentation of #ClarinoxBlueScoInterface interface for more information.

\return a pointer to a ClarinoxBlue-allocated RX SCO buffer. will be NULL if there is no free RX SCO buffer available at this moment. 
*/
extern struct ClxRxScoBuffer* clxBlueDriverGetFreeRxScoBuffer ();

/**
A pointer to the driver interface implementation instance.
*/
extern struct ClarinoxBlueDriverInterface* clarinoxBlueDriverInterface;


#ifdef __cplusplus
}
#endif


#endif // _ClarinoxBlueUartDriverBsp_h_

