#ifndef __ClxBsp__
#define __ClxBsp__

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxBsp.h
* Description         Board Support Package common
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClxCommon.h"
#include "ClxUnalignedAccess.h"
#include "ClxBsp.Errors.h"
#include "Console.Bsp.h"
#include "ClxFile.Bsp.h"
#include "Uart.Bsp.h"
#include "Sdio.Bsp.h"
#include "NetworkAdaptor.Bsp.h"
#include "MemoryRegion.Bsp.h"
#include "NetworkStackServiceInterface.Bsp.h"
#include "ClarinoxBlue.Driver.Bsp.h"
#include "Iperf.Bsp.h"
#include "HttpServer.Bsp.h"
#include "Dhcp.Bsp.h"


#ifdef __cplusplus
extern "C" {
#endif

typedef struct ClxCapturedPacketDataStruct
{
    u1* data;
    u4 length;
}ClxCapturedPacketData;


/**
This BSP variable is only used in Multi-Processor platforms only, and should not be used in Single-Processor platforms.

This is a pointer which may point to a BSP-defined variable. If set, the value of the assigned variable will hold the index of the
processor core on which all the stack threads are to be bound. If bound successfully, all stack thread will be scheduled and executed only on
the specified core. If this is pointer points to NULL (the default value), no binding will take place and the stack threads may run on any
core available based on the scheduling algorithm used by the operating system.
*/
extern const u4* clxThreadMultiProcessorBindingCoreIndex;

/**
An array of 5 priority levels that is used internally for the ClarinoxBlue implementation. 
These values map the ClarinoxBlue platform-independent thread priorities to underlying platform priorities. 
Each priority level has a 2 byte priority level, the first index (index 0) of the array represents the highest priority,
while the last index (index 4) of the array represents the lowest priority.
If the table is not defined in the BSP, the default priority of the underlying platform will be used for all
threads. 
*/
extern u2* clarinoxBluePlatformTaskPriorityTable;

/**
A total of 9 thread stack sizes are got as input from the BSP and used for thread creation.
The Soft Frame thread stack sizes can be passed in the following order 
1.  SoftFrame Timer Thread stack size 
2.	SoftFrame Debug Thread stack size
3.	SoftFrame Console UI Thread stack size
4.	SoftFrame Terminal Emulator Thread stack size
*/
extern u2* clarinoxPlatformSoftFrameTaskStackSizeTable;
/*
The Bluetooth thread stack sizes can be passed in the following order 
1.	Bluetooth Application Thread stack size
2.	Bluetooth Stack Thread stack size
3.	Bluetooth Uart Rx Thread stack size
*/
extern u2* clarinoxPlatformBluetoothTaskStackSizeTable;
/*
The WLAN thread stack sizes can be passed in the following order 
1.	WLAN Stack Thread stack size
2.	WLAN Application Thread stack size
*/
extern u2* clarinoxPlatformWlanTaskStackSizeTable;

/**
An array of 5 priority levels that is used internally for the ClarinoxWifi implementation. 
These values map the ClarinoxWifi platform-independent thread priorities to underlying platform priorities. 
Each priority level has a 2 byte priority level, the first index (index 0) of the array represents the highest priority,
while the last index (index 4) of the array represents the lowest priority.
If the table is not defined in the BSP, the default priority of the underlying platform will be used for all
threads. 
*/
extern u2* clarinoxWlanPlatformTaskPriorityTable;


/* target processor dependent definitions */
#if defined( CLX_INTEL_X86 ) 
#   if defined(CLX_BIG_ENDIAN)
#       error "CLX_BIG_ENDIAN macro is not consistent with the platform type (Intel x86)"
#   elif !defined(CLX_LITTLE_ENDIAN)
#       define CLX_LITTLE_ENDIAN
#   endif
#   define CLX_UNALIGNED_ACCESS_ALLOWED
#endif


#ifdef CLX_LITTLE_ENDIAN
#   define CONVERT_TO_LITTLEENDIAN_2( x )                   ( ( u2 ) x )
#   define CONVERT_TO_LITTLEENDIAN_4( x )                   ( ( u4 ) x )
#   define CONVERT_TO_BIGENDIAN_2( x )                      ( ( u2 )( ( ( ( u2 )x ) << 8 ) | ( ( ( u2 )x ) >> 8 ) ) )
#   define CONVERT_TO_BIGENDIAN_4( x )                      ( ( u4 )( ( ( ( u4 )x ) >> 24 ) | ( ( ( ( u4 )x ) >> 8 ) & 0xff00 ) | ( ( ( ( u4 )x ) << 8 ) & 0xff0000 ) | ( ( ( u4 )x ) << 24 ) ) )
#elif CLX_BIG_ENDIAN
#   define CONVERT_TO_LITTLEENDIAN_2( x )                   ( ( u2 )( ( ( ( u2 )x ) << 8 ) | ( ( ( u2 )x ) >> 8 ) ) )
#   define CONVERT_TO_LITTLEENDIAN_4( x )                   ( ( u4 )( ( ( ( u4 )x ) >> 24 ) | ( ( ( ( u4 )x ) >> 8 ) & 0xff00 ) | ( ( ( ( u4 )x ) << 8 ) & 0xff0000 ) | ( ( ( u4 )x ) << 24 ) ) )
#   define CONVERT_TO_BIGENDIAN_2( x )                      ( ( u2 ) x )
#   define CONVERT_TO_BIGENDIAN_4( x )                      ( ( u4 ) x )
#else
#   error "Please select the platform byte order"
#endif


#if defined CLX_UNALIGNED_ACCESS_ALLOWED
#   define WRITE_TO_LITTLEENDIAN_2( value, buffer )         (  *( static_cast<u2*> ( buffer ) ) = CONVERT_TO_LITTLEENDIAN_2( value ) )
#   define WRITE_TO_LITTLEENDIAN_4( value, buffer )         (  *( static_cast<u4*> ( buffer ) ) = CONVERT_TO_LITTLEENDIAN_4( value ) )
#   define WRITE_TO_BIGENDIAN_2( value, buffer )            (  *( static_cast<u2*> ( buffer ) ) = CONVERT_TO_BIGENDIAN_2( value ) )
#   define WRITE_TO_BIGENDIAN_4( value, buffer )            (  *( static_cast<u4*> ( buffer ) ) = CONVERT_TO_BIGENDIAN_4( value ) )
#   define READ_FROM_LITTLEENDIAN_2( buffer )               (  CONVERT_TO_LITTLEENDIAN_2( *( static_cast<u2*> ( buffer ) ) ) )
#   define READ_FROM_LITTLEENDIAN_4( buffer )               (  CONVERT_TO_LITTLEENDIAN_4( *( static_cast<u4*> ( buffer ) ) ) )
#   define READ_FROM_BIGENDIAN_2( buffer )                  (  CONVERT_TO_BIGENDIAN_2( *( static_cast<u2*> ( buffer ) ) ) )
#   define READ_FROM_BIGENDIAN_4( buffer )                  (  CONVERT_TO_BIGENDIAN_4( *( static_cast<u4*> ( buffer ) ) ) )

#   define WRITE_TO_LITTLEENDIAN_2_ALIGNED( value, buffer )		WRITE_TO_LITTLEENDIAN_2( value, buffer ) 
#   define WRITE_TO_LITTLEENDIAN_4_ALIGNED( value, buffer )     WRITE_TO_LITTLEENDIAN_4( value, buffer )
#   define WRITE_TO_BIGENDIAN_2_ALIGNED( value, buffer )		WRITE_TO_BIGENDIAN_2( value, buffer )
#   define WRITE_TO_BIGENDIAN_4_ALIGNED( value, buffer )		WRITE_TO_BIGENDIAN_4( value, buffer )
#   define READ_FROM_LITTLEENDIAN_2_ALIGNED( buffer )			READ_FROM_LITTLEENDIAN_2( buffer )
#   define READ_FROM_LITTLEENDIAN_4_ALIGNED( buffer )			READ_FROM_LITTLEENDIAN_4( buffer )
#   define READ_FROM_BIGENDIAN_2_ALIGNED( buffer )				READ_FROM_BIGENDIAN_2( buffer )
#   define READ_FROM_BIGENDIAN_4_ALIGNED( buffer )				READ_FROM_BIGENDIAN_4( buffer )

#else
#   define WRITE_TO_LITTLEENDIAN_2( value, buffer )         (  writeToUnaligned16( CONVERT_TO_LITTLEENDIAN_2( value ), ( buffer ) ) )
#   define WRITE_TO_LITTLEENDIAN_4( value, buffer )         (  writeToUnaligned( CONVERT_TO_LITTLEENDIAN_4( value ), ( buffer ) ) )
#   define WRITE_TO_BIGENDIAN_2( value, buffer )            (  writeToUnaligned16( CONVERT_TO_BIGENDIAN_2( value ), ( buffer ) ) )
#   define WRITE_TO_BIGENDIAN_4( value, buffer )            (  writeToUnaligned( CONVERT_TO_BIGENDIAN_4( value ), ( buffer ) ) )
#   define READ_FROM_LITTLEENDIAN_2( buffer )               (  CONVERT_TO_LITTLEENDIAN_2( readFromUnaligned16( buffer ) ) )
#   define READ_FROM_LITTLEENDIAN_4( buffer )               (  CONVERT_TO_LITTLEENDIAN_4( readFromUnaligned( buffer ) ) )
#   define READ_FROM_BIGENDIAN_2( buffer )                  (  CONVERT_TO_BIGENDIAN_2( readFromUnaligned16( buffer ) ) )
#   define READ_FROM_BIGENDIAN_4( buffer )                  (  CONVERT_TO_BIGENDIAN_4( readFromUnaligned( buffer ) ) )


#   define WRITE_TO_LITTLEENDIAN_2_ALIGNED( value, buffer )         (  *( static_cast<u2*> ( buffer ) ) = CONVERT_TO_LITTLEENDIAN_2( value ) )
#   define WRITE_TO_LITTLEENDIAN_4_ALIGNED( value, buffer )         (  *( static_cast<u4*> ( buffer ) ) = CONVERT_TO_LITTLEENDIAN_4( value ) )
#   define WRITE_TO_BIGENDIAN_2_ALIGNED( value, buffer )            (  *( static_cast<u2*> ( buffer ) ) = CONVERT_TO_BIGENDIAN_2( value ) )
#   define WRITE_TO_BIGENDIAN_4_ALIGNED( value, buffer )            (  *( static_cast<u4*> ( buffer ) ) = CONVERT_TO_BIGENDIAN_4( value ) )
#   define READ_FROM_LITTLEENDIAN_2_ALIGNED( buffer )               (  CONVERT_TO_LITTLEENDIAN_2( *( static_cast<u2*> ( buffer ) ) ) )
#   define READ_FROM_LITTLEENDIAN_4_ALIGNED( buffer )               (  CONVERT_TO_LITTLEENDIAN_4( *( static_cast<u4*> ( buffer ) ) ) )
#   define READ_FROM_BIGENDIAN_2_ALIGNED( buffer )                  (  CONVERT_TO_BIGENDIAN_2( *( static_cast<u2*> ( buffer ) ) ) )
#   define READ_FROM_BIGENDIAN_4_ALIGNED( buffer )                  (  CONVERT_TO_BIGENDIAN_4( *( static_cast<u4*> ( buffer ) ) ) )
#endif


#   define WRITE_TO_LITTLEENDIAN_3( value, buffer )                 \
    buffer[0] = (u1)((u4)value & 0x000000FF);                       \
    buffer[1] = (u1)(((u4)value & 0x0000FF00) >> 8);                \
    buffer[2] = (u1)(((u4)value & 0x00FF0000) >> 16)


#   define WRITE_TO_BIGENDIAN_3( value, buffer )                    \
    buffer[2] = (u1)((u4)value & 0x000000FF);                       \
    buffer[1] = (u1)(((u4)value & 0x0000FF00) >> 8);                \
    buffer[0] = (u1)(((u4)value & 0x00FF0000) >> 16)
    
#   define READ_FROM_LITTLEENDIAN_3( buffer )                       ( (u4)((u1*)buffer[0]) | ((u4)((u1*)buffer[1]) << 8) | ((u4)((u1*)buffer[2]) << 16) ) 
#   define READ_FROM_BIGENDIAN_3( buffer )                          ( (u4)((u1*)buffer[2]) | ((u4)((u1*)buffer[1]) << 8) | ((u4)((u1*)buffer[0]) << 16) ) 

/*  value shall be of type ClxUInteger64 or ClxSInteger64 */
#   define WRITE_TO_LITTLEENDIAN_5( value, buffer )                 \
    buffer[0] = (u1)((u4)value.lsb & 0x000000FF);                   \
    buffer[1] = (u1)(((u4)value.lsb & 0x0000FF00) >> 8);            \
    buffer[2] = (u1)(((u4)value.lsb & 0x00FF0000) >> 16);           \
    buffer[3] = (u1)(((u4)value.lsb & 0xFF000000) >> 24);           \
    buffer[4] = (u1)((u4)value.msb & 0x000000FF)

/*  value shall be of type ClxUInteger64 or ClxSInteger64 */
#   define WRITE_TO_BIGENDIAN_5( value, buffer )                    \
    buffer[4] = (u1)((u4)value.lsb & 0x000000FF);                   \
    buffer[3] = (u1)(((u4)value.lsb & 0x0000FF00) >> 8);            \
    buffer[2] = (u1)(((u4)value.lsb & 0x00FF0000) >> 16);           \
    buffer[1] = (u1)(((u4)value.lsb & 0xFF000000) >> 24);           \
    buffer[0] = (u1)((u4)value.msb & 0x000000FF)

/*  
    value shall be of one of the following types :
    ClxUInteger64 : type shall be set to u4
    ClxSInteger64 : type shall be set to s4 
*/
#   define READ_FROM_LITTLEENDIAN_5( value, type, buffer )                                                                                  \
    value.lsb = (type)( (u4)((u1*)buffer[0]) | ((u4)((u1*)buffer[1]) << 8) | ((u4)((u1*)buffer[2]) << 16) | ((u4)((u1*)buffer[3]) << 24) ); \
    value.msb = (type)((u1*)buffer[4])

/*  
    value shall be of one of the following types :
    ClxUInteger64 : type shall be set to u4
    ClxSInteger64 : type shall be set to s4 
*/
#   define READ_FROM_BIGENDIAN_5( value, type, buffer )                                                                                     \
    value.msb = (type)( (u4)((u1*)buffer[3]) | ((u4)((u1*)buffer[2]) << 8) | ((u4)((u1*)buffer[1]) << 16) | ((u4)((u1*)buffer[0]) << 24) ); \
    value.lsb = (type)((u1*)buffer[4])


/*  value shall be of type ClxUInteger64 or ClxSInteger64 */
#   define WRITE_TO_LITTLEENDIAN_6( value, buffer )                 \
    buffer[0] = (u1)((u4)value.lsb & 0x000000FF);                   \
    buffer[1] = (u1)(((u4)value.lsb & 0x0000FF00) >> 8);            \
    buffer[2] = (u1)(((u4)value.lsb & 0x00FF0000) >> 16);           \
    buffer[3] = (u1)(((u4)value.lsb & 0xFF000000) >> 24);           \
    buffer[4] = (u1)((u4)value.msb & 0x000000FF);                   \
    buffer[5] = (u1)(((u4)value.msb & 0x0000FF00) >> 8)


/*  value shall be of type ClxUInteger64 or ClxSInteger64 */
#   define WRITE_TO_BIGENDIAN_6( value, buffer )                    \
    buffer[5] = (u1)(((u4)value.msb & 0x0000FF00) >> 8);            \
    buffer[4] = (u1)((u4)value.lsb & 0x000000FF);                   \
    buffer[3] = (u1)(((u4)value.lsb & 0x0000FF00) >> 8);            \
    buffer[2] = (u1)(((u4)value.lsb & 0x00FF0000) >> 16);           \
    buffer[1] = (u1)(((u4)value.lsb & 0xFF000000) >> 24);           \
    buffer[0] = (u1)((u4)value.msb & 0x000000FF)

/*  
    value shall be of one of the following types :
    ClxUInteger64 : type shall be set to u4
    ClxSInteger64 : type shall be set to s4 
*/
#   define READ_FROM_LITTLEENDIAN_6( value, type, buffer )                                                                                  \
    value.lsb = (type)( (u4)((u1*)buffer[0]) | ((u4)((u1*)buffer[1]) << 8) | ((u4)((u1*)buffer[2]) << 16) | ((u4)((u1*)buffer[3]) << 24) ); \
    value.msb = (type)( (u4)((u1*)buffer[4]) | ((u4)((u1*)buffer[5]) << 8) )

/*  
    value shall be of one of the following types :
    ClxUInteger64 : type shall be set to u4
    ClxSInteger64 : type shall be set to s4 
*/
#   define READ_FROM_BIGENDIAN_6( value, type, buffer )                                                                                     \
    value.msb = (type)( (u4)((u1*)buffer[3]) | ((u4)((u1*)buffer[2]) << 8) | ((u4)((u1*)buffer[1]) << 16) | ((u4)((u1*)buffer[0]) << 24) ); \
    value.lsb = (type)( (u4)((u1*)buffer[5]) | ((u4)((u1*)buffer[4]) << 8) )


/*  value shall be of type ClxUInteger64 or ClxSInteger64 */
#   define WRITE_TO_LITTLEENDIAN_8( value, buffer )                 \
    WRITE_TO_LITTLEENDIAN_4(value.lsb, buffer);                     \
    WRITE_TO_LITTLEENDIAN_4(value.msb, (buffer + 4))


/*  value shall be of type ClxUInteger64 or ClxSInteger64 */
#   define WRITE_TO_BIGENDIAN_8( value, buffer )                    \
    WRITE_TO_BIGENDIAN_4(value.msb, buffer);                        \
    WRITE_TO_BIGENDIAN_4(value.lsb, (buffer + 4))

/*  
    value shall be of one of the following types :
    ClxUInteger64 : type shall be set to u4
    ClxSInteger64 : type shall be set to s4 
*/
#   define READ_FROM_LITTLEENDIAN_8( value, type, buffer )          \
    value.lsb = (type)READ_FROM_LITTLEENDIAN_4(buffer);             \
    value.msb = (type)READ_FROM_LITTLEENDIAN_4((buffer + 4))

/*  
    value shall be of one of the following types :
    ClxUInteger64 : type shall be set to u4
    ClxSInteger64 : type shall be set to s4 
*/
#   define READ_FROM_BIGENDIAN_8( value, type, buffer )             \
    value.msb = (type)READ_FROM_BIGENDIAN_4(buffer);                \
    value.lsb = (type)READ_FROM_BIGENDIAN_4((buffer + 4))


/*  value shall be of type ClxUInteger128 or ClxSInteger128 */
#   define WRITE_TO_LITTLEENDIAN_16( value, buffer )                \
    WRITE_TO_LITTLEENDIAN_8(value.lsb, buffer);                     \
    WRITE_TO_LITTLEENDIAN_8(value.msb, (buffer + 4))


/*  value shall be of type ClxUInteger128 or ClxSInteger128 */
#   define WRITE_TO_BIGENDIAN_16( value, buffer )                   \
    WRITE_TO_BIGENDIAN_8(value.msb, buffer);                        \
    WRITE_TO_BIGENDIAN_8(value.lsb, (buffer + 4))


/*  
    value shall be of one of the following types :
    ClxUInteger128 : type shall be set to u4
    ClxSInteger128 : type shall be set to s4 
*/
#   define READ_FROM_LITTLEENDIAN_16( value, type, buffer )         \
    READ_FROM_LITTLEENDIAN_8(value.lsb, type, buffer);              \
    READ_FROM_LITTLEENDIAN_8(value.msb, type, (buffer + 4))


/*  
    value shall be of one of the following types :
    ClxUInteger128 : type shall be set to u4
    ClxSInteger128 : type shall be set to s4 
*/
#   define READ_FROM_BIGENDIAN_16( value, type, buffer )            \
    READ_FROM_BIGENDIAN_8(value.msb, type, buffer);                 \
    READ_FROM_BIGENDIAN_8(value.lsb, type, (buffer + 4))


#ifdef __cplusplus
}
#endif


#endif // __ClxBsp__


