#ifndef _MemoryRegion_Bsp_
#define _MemoryRegion_Bsp_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                MemoryRegion.Bsp.h
* Description         BSP Memory Region Management Interface
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
A variable of type ClxAlignedMemoryRegion represents a contiguous buffer in the memory address space of 
an instance of a Clarinox stack, which may be the physical address space, or a virtual address space.

In some cases, a memory region must be used instead of a classic static or dynamic (system heap or ClarinoxSoftFrame pool set) memory buffers.
since memory regions have specific attributes which might not be possessed by the classic memory buffers.
These attributes have been defined in MemoryBuffers.h

At the moment, Memory Region Management BSP must be implemented only for ClarinoxWLAN stack.  
*/
typedef void* ClxMemoryRegion;

/**
Returns the size of physical memory pages of the underlying platform, in bytes.
If the underlying platform does not support MMU (Memory Management Unit), the return value
MUST be set to a minimum of 4096 bytes.
*/
extern size_t clxGetPlatformMemoryPageSize();

/**
Allocates a contiguous memory buffer in the memory address space of the caller code, with specified attributes, 
from the underlying platform's RAM, and provides a pointer to the start of the allocated region, along with the size of the region.

The size of the allocated buffer SHALL not less than minBufferSize, and does not need to be more than maxBufferSize. 
The allocated buffer SHALL have all the attributes as specified in requiredAttributes.

If the function is not able to meet all the requirements, it shall not allocate any memory region.

\param[ in ] minBufferSize The minimum size of the buffer which can be allocated.
\param[ in ] minBufferSize The maximum size of the buffer which can be allocated.
\param[ in ] requiredAttributes The attributes which the allocated buffer must have, as a bit-wise combination of zero
or more attribute values. These values are defined in NetworkBuffers.h
\param[ in ] startAddress A reference to a user-provided variable which, on successful return, will contain the
address of the beginning of the allocated region. If not successful, this variable will be set to NULL.
\param[ in ] bufferLength A reference to a user-provided variable which, on successful return, will contain the length of
the allocated region. If not successful, this value will be set to 0.
        
\return A platform-dependent ID representing the allocated memory region in the underlying platform. This value is only
used to free the allocated region by calling the method freeMemoryPage(). If allocation is not successful, the return value
will be NULL.
*/
extern ClxMemoryRegion clxAllocateMemoryRegion(ClxSize minBufferSize,
                                               ClxSize maxBufferSize,
                                               u4 requiredAttributes,
                                               u1** startAddress,
                                               size_t* bufferLength);

/**
Frees previously-allocated memory region and returns them to the underlying system.

\param[ in ] region The ID of the memory region as returned by the method clxAllocateMemoryRegion(). This value CANNOT be NULL.
*/
extern void clxFreeMemoryRegion(ClxMemoryRegion region);

#ifdef __cplusplus
}
#endif


#endif    // _MemoryRegion_Bsp_

