#ifndef __SDIO_BSP__
#define __SDIO_BSP__

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                Sdio.Bsp.h
* Description         Declares SDIO interface for BSP
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#define CLX_NUM_OF_SDIO_FUNCTIONS        8

#ifdef __cplusplus
extern "C" {
#endif

    /**
    Defines the type of SDIO register addresses. This is a 32bit value but 
    Only 17 least significant bits are valid. The rest shall be set to zero
    */
    typedef u4 ClxSdioRegAddr;

    /**
    Specifies an IO function (0 to 7) in the inserted SDIO card. When this enumeration is used as the type of an array,
    the last element of the array shall be set to ClxSdio_Function_Null in order to mark the end of the array
    (therefore, the size of the array does not need to be passed separately).
    */
    typedef enum ClxSdioFunctionEnum
    {
        ClxSdio_Function_0 = 0x0,
        ClxSdio_Function_1 = 0x1,
        ClxSdio_Function_2 = 0x2,
        ClxSdio_Function_3 = 0x3,
        ClxSdio_Function_4 = 0x4,
        ClxSdio_Function_5 = 0x5,
        ClxSdio_Function_6 = 0x6,
        ClxSdio_Function_7 = 0x7,    
        ClxSdio_Function_Null = 0xFFFFFFFF
    } ClxSdioFunction;   

    /**
    The type of the event for which the eventHandler function of the SDIO object has been called.
    */
    typedef enum ClxSdioEventEnum
    {
        ClxSdio_RequestComplete,            /*!< A pending request has completed (the "request" argument of the event handler points to the request object) */ 
        ClxSdio_InterruptReceived,          /*!< A SDIO interrupt has been received from the device (the "request" argument of the event handler is NULL) */
        ClxSdio_CardInserted,               /*!< A card has been inserted (the "request" argument of the event handler is NULL) */
        ClxSdio_CardRemoved                 /*!< The card has been removed (the "request" argument of the event handler is NULL) */
    } ClxSdioEvent;
    
    /**
    Type of the request which is to be passed to the function clxSdioBspInterface.sendRequest().
    */
    typedef enum ClxSdioRequestTypeStruct
    {
        ClxSdio_Init,                        /*!< Initialize the inserted card. All other members of ClxSdioRequest is ignored */
        ClxSdio_Reset,                       /*!< Reset the card. All other members of ClxSdioRequest is ignored */
        
        /**
        The following requests shall only be made when
        the card reset is complete successfully.
        */
        ClxSdio_GetFunctionBlockSize,        /*!< Return the block size for the specified function */
        ClxSdio_SetFunctionBlockSize,        /*!< Set the block size for the specified function */
        ClxSdio_WriteRegister,               /*!< Write one byte of data to a register using CMD52 */
        ClxSdio_ReadRegister,                /*!< Read one byte of data from a register using CMD52 */
        ClxSdio_WriteDataBuffer_FixedAddr,   /*!< Write 1 to 512 bytes of data to a fixed address using CMD53 */ 
        ClxSdio_WriteDataBuffer_IncAddr,     /*!< Write 1 to 512 bytes of data to an incremental address list using CMD53 */     
        ClxSdio_ReadDataBuffer_FixedAddr,    /*!< Read 1 to 512 bytes of data from a fixed address using CMD53 */
        ClxSdio_ReadDataBuffer_IncAddr,      /*!< Read 1 to 512 bytes of data from an incremental address list using CMD53 */    
        ClxSdio_WriteDataBlocks_FixedAddr,   /*!< Write 1 or more fixed-size blocks of data to a fixed address using CMD53 */
        ClxSdio_WriteDataBlocks_IncAddr,     /*!< Write 1 or more fixed-size blocks of data to an incremental address list using CMD53 */
        ClxSdio_ReadDataBlocks_FixedAddr,    /*!< Read 1 or more fixed-size blocks of data from a fixed address using CMD53 */
        ClxSdio_ReadDataBlocks_IncAddr       /*!< Read 1 or more fixed-size blocks of data from an incremental address list using CMD53 */
    } ClxSdioRequestType;   

    /**
    Defines a request to be sent to the SDIO controller.
    An object of this struct shall be passed to the function clxSdioBspInterface.sendRequest(). The object shall not be deleted or modified
    until the request is complete. If the function clxSdioBspInterface.sendRequest() returns #CLX_SDIO_REQUEST_PENDING, then the request
    object shall not be deleted or modified until the eventHandler call-back function is called for this request.
    */
    typedef struct ClxSdioRequestStruct
    {
        ClxSdioRequestType        type;                /*!< Type of the request. It is of type #ClxSdioRequestType */
        ClxSdioFunction           function;            /*!< The IO function on the SDIO card to which the request is to be sent */ 
        ClxSdioRegAddr            registerAddress;     /*!< The address of the SDIO register to which this request belongs */
        u1*                       data;                /*!< A pointer to the user-allocated data which contains either the data to be written (in case of a writing request),
                                                            or the buffer to which the read data must be stored (in case of a reading request) */
        u4                        length;              /*!< The length of the buffer pointed by "data" parameter of this struct */
        void*                     userSpecific;        /*!< An optional pointer to user-specific data. The contents of this object is user-specific. This parameter can be used
                                                            in order for the developer to recognize different requests */
        ClxResult                 requestResult;       /*!< The result of the request. CLX_SUCCESS if successful. Any other values indicates an error */ 
    } ClxSdioRequest;

    /**
    The type identifies a handle to an implementation-specific and/or platform-specific object which represents a SDIO controller
    on the underlying platform. A SDIO controller is able to support and handle one SDIO card at a time.
    */
    typedef void* ClxSdio;
    
    /** 
    The prototype of the SDIO event handler, as passed to the function pointer openHandle of #ClxSdioBspInterface structure.

    \param[ in ] handle Handle to the SDIO controller as returned by openHandle.
    \param[ in ] userData The user data as passed to openHandle (as the argument eventHandlerUserData).
    \param[ in ] event The event (of type #ClxSdioEvent) for which the event handler function has been called.
    \param[ in ] function the IO function in the SDIO card which has triggered the event. This argument is only 
    valid when the type event is #ClxSdio_RequestComplete. Furthermore, this will be set to #ClxSdio_Function_Null 
    if no specific IO function is the trigger of the event (e.g. request->type is #ClxSdio_Init or #ClxSdio_Reset).
    \param[ in ] request The request to which this event belongs. This is a pointer to the same request object as passed to
    the function clxSdioBspInterface.sendRequest(). This argument is only valid when the type event is #ClxSdio_RequestComplete.
    Otherwise, it will be set to NULL.
   
    \remark The call-back function may be called in the context of any internal thread, as long as it is called in the context of
    the same process (user memory address space) as the rest of the application.

    An implementation of this call-back function SHALL NOT block at any time. The function shall return as soon as possible in order
    to avoid access latency to the device. 
    */
    typedef void (*ClxSdioEventHandler) (ClxSdio handle,
                                         void* userData,
                                         ClxSdioEvent event, 
                                         ClxSdioFunction function, 
                                         ClxSdioRequest* request);

    /**
    The SDIO interface of BSP. All functions defined inside the structure shall be implemented.
    NOTE : If the BSP interface of SDIO is to be used in a system, An object of this structure shall be created by the developer, and
    the members shall be set to proper interface function implementations. Then, the address of this structure shall be set to
    the global pointer variable clxSdioBspInterface. When set, the value of clxSdioBspInterface shall not be changed for the rest of
    system operation (until the  system or the application is restarted). Also, the developer-defined structure object shall NOT be
    deleted or modified until the system or application is restarted.
    */
    typedef struct ClxSdioBspInterfaceStruct
    {
        /**
        Returns a handle (of type #ClxSdio) for a specific SDIO controller present in the underlying platform.

        \param[ in ] deviceName the name of the SDIO controller driver to which a handle is to be returned. This is a null-terminated ASCII
        string, contents of which solely depend on the underlying OS or platform.
        \param[ in ] eventHandler pointer to a function of type #ClxSdioEventHandler which is to be be called when any SDIO events occur
        on this SDIO controller.
        \param[ in ] eventHandlerData pointer to user-defined data which is to be passed to the SDIO event handler function when an event occurs.
        This value can be NULL.
        on this SDIO controller.
        \param[ out ] handle A pointer which, on return, shall be set to an implementation-specific object representing the SDIO controller
        hardware in the underlying platform (only if the return value of the function is CLX_SUCCESS). If an error has occurred during the
        retrieval of the handle, this argument shall be set to NULL, and shall be ignored by the caller.

        \return CLX_SUCCESS: The handle to the SDIO controller has been successfully returned in the argument "handle".
                CLX_SDIO_DRIVER_NOT_EXIST: An SDIO driver with the specified name does not exist.
                CLX_SDIO_FAILED_TO_OPEN_DRIVER: Cannot open the SDIO driver for internal errors.
        */
        ClxResult    (*openHandle) (const s1* deviceName, ClxSdioEventHandler eventHandler, void* eventHandlerUserData, ClxSdio* handle);

        /**
        Initializes the SDIO controller hardware. This function generally needed to be called after #openHandle has been called in order to
        make sure the SDIO controller is in a proper state.

        \param[ in ] handle Handle to the SDIO controller as returned by #openHandle.
        \param[ in ] platformSpecificConfigParams An optional pointer to an object containing the platform-specific parameters needed to initialize
        the SDIO controller hardware. This can be NULL.

        \return CLX_SUCCESS: The controller has been initialized successfully.
                CLX_SDIO_INIT_FAILED: Initialization of the hardware failed.
                CLX_SDIO_DEVICE_DRIVER_SHUT_DOWN: The driver shut down during the operation.
        */
        ClxResult    (*initControllerHardware) (ClxSdio handle, void* platformSpecificConfigParams);

        /**
        Specifies if a card has been inserted in the SDIO slot assigned to the SDIO controller.
        \param[ in ] handle Handle to the SDIO controller as returned by #openHandle.
        \param[ out ] inserted A boolean variable which, on return, shall be set to TRUE (if a card has been inserted),
        and FALSE (if there is no card inserted). If the return value of the function is not CLX_SUCCESS, this argument
        shall be ignored by the caller.

        \return CLX_SUCCESS: The retrieval of information is successful. "inserted" contains valid information.
                CLX_FAIL: The retrieval of information failed. "inserted" shall be ignored.
                CLX_SDIO_DEVICE_DRIVER_SHUT_DOWN: The driver shut down during the operation.

        \remark If the SDIO controller does not support SDIO card insertion and removal (the card has been statically linked to
        the controller), this function shall always set "inserted" to TRUE.
        */
        ClxResult    (*isCardInserted) (ClxSdio handle, boolean& inserted);

        /**
        Returns the list of IO functions which have been implemented on the inserted SDIO card.
        IMPORTANT : this function shall only be called after the inserted card has been both initialized
        (by sending the request of type #ClxSdio_Init to the card using the function clxSdioBspInterface.sendRequest),
        and reset (by sending the request of type #ClxSdio_Reset to the card using the function clxSdioBspInterface.sendRequest).
        Otherwise, an implementation is not required to return a valid response before the card has been initialized and reset.

        \param[ in ] handle Handle to the SDIO controller as returned by #openHandle.
        \param[ out ] funtionsList A pointer to a caller-allocated array of type #ClxSdioFunction, and size of at lease #CLX_NUM_OF_SDIO_FUNCTIONS.
        Upon a successful returning of this function (when the return value is CLX_SUCCESS), the array shall contain the list of supported functions,
        followed by a value of #ClxSdio_Function_Null which marks the end of the functions list.
        If the return value is not CLX_SUCCESS, this argument shall be ignored by the caller.
        NOTE : Please note that the list of IO functions shall be stored in the array from the beginning of the array (index 0), and one after each other. 
        The index of the array element DOES NOT correspond to the the IO function number.

        \return CLX_SUCCESS: The list of the supported IO functions on the inserted card has been retrieved successfully.
                CLX_SDIO_NO_CARD_INSERTED: There is no card inserted.
                CLX_FAIL: The retrieval of the list of the supported IO functions on the inserted card failed.
                CLX_SDIO_DEVICE_DRIVER_SHUT_DOWN: The driver shut down during the operation.
        */
        ClxResult    (*getListOfIOFunctions) (ClxSdio handle, ClxSdioFunction* funtionsList);

        /**
        Sends a request to the inserted SDIO card. An implementation is allowed to perform the request in blocking or non-blocking
        mode. In blocking mode, this function will not return until the request is complete either with success or in error. In non-blocking
        mode, the function SHALL return CLX_SDIO_REQUEST_PENDING if it is not able to immediately complete the request. Then, when the request
        is complete either with success, or in error, the eventHandler call-back function shall be called with the event of type #ClxSdio_RequestComplete,
        and the "request" argument of the call-back function set to the request object provided by the caller of this function. In this case, the
        "requestResult" member of the request object shall be set to the result of the command completion.

        \param[ in ] handle Handle to the SDIO controller as returned by #openHandle.
        \param[ in ] request A caller-allocated object of type #ClxSdioRequest, containing the details of the request
        to be sent to the SDIO card.
        NOTE : All types of requests SHALL be supported by an implementation.
        
        \return CLX_SUCCESS: The request has been processed successfully, and is complete (the eventHandler call-back function
                             will not be called for this request).
                CLX_SDIO_REQUEST_PENDING: The request has been successfully submitted to the driver to be processed. When the
                                          request is complete (either with success or in error), the eventHandler call-back function will be called with the event of type
                                          #ClxSdio_RequestComplete.
                CLX_SDIO_NO_CARD_INSERTED: There is no card inserted.
                CLX_SDIO_DEVICE_DRIVER_SHUT_DOWN: The driver shut down during the operation.
                CLX_SDIO_IO_FUNCTION_NOT_EXIST: The IO function to which this request is to be sent (as the member "function" of the request object) does not exist in the
                                                inserted SDIO card.
                CLX_SDIO_REQUEST_NOT_PROCESSED: The driver could not process the request.
                CLX_SDIO_REQUEST_NOT_SUCCESSFUL: The SDIO card rejected the request.
                Any other negative value: Other Clarinox standard, or an implementation-specific error code.
        */
        ClxResult    (*sendRequest) (ClxSdio handle, ClxSdioRequest* request);

        /**
        Returns the last platform-specific error code. The meaning of the error code solely depends on the underlying platform.
        The function is to be mainly used for debugging purposes.

        \param[ in ] handle Handle to the SDIO controller as returned by #openHandle.

        \return a value of type s4 (signed int) which represents an error code in the underlying platform. If no error has been
        happened in the last operation, the return value shall represent the "Success" code in the underlying platform.
        
        \remark Every time any function of this interface (excluding getLastPlatformDependentError) is called, the last error
        SHALL be reset (set to Success in the underlying platform). This is due to the fact that the last error shall only correspond
        to 
        */
        s4             (*getLastPlatformDependentError) (ClxSdio handle);

        /**
        Masks the SDIO card interrupt. This function shall be called only after the successful initialization of the SDIO controller,
        and SDIO card.

        NOTE : This is an optional feature. If an implementation does not support this feature, this function may not be implemented.

        \param[ in ] handle Handle to the SDIO controller as returned by #openHandle.
        \param[ in ] ioFunctionsMaskFlag The list of IO functions for which the interrupt is to be enabled. This is a bitwise OR-ed combination
        of the IO functions, where each bit represents an IO function (LSB represents IO function 0). If the respective bit is set for a function,
        the interrupt for that function will be enabled. Otherwise, the interrupt for that function will be disabled.

        \return CLX_SUCCESS : The interrupt was successfully enabled.
        Any other value indicates an error.
        
        \remark It is NOT an error to mask an interrupt which is already masked, or unmask an interrupt which is already unmasked. In these cases,
        an implementation of this function shall NOT return an error.
        */
        ClxResult      (*maskCardInterrupt) (ClxSdio handle, u4 ioFunctionsMaskFlag);

        /**
        Closes an already-opened handle to the underlying SDIO controller (driver).
        \param[ in ] handle Handle to the SDIO controller as returned by #openHandle.

        \return CLX_SUCCESS: The handle was closed successfully.
                CLX_FAIL: The closing of the handle failed.

        \remark Regardless of what this function returns, the handle is not useable and shall not be used any more.
        */
        ClxResult    (*closeHandle) (ClxSdio handle);

    } ClxSdioBspInterface;

/**
A pointer to an object of type #ClxSdioBspInterface, which is to be set by the developer in order to transfer all SDIO requests to an implementation
of this interface. The variable is set to NULL by default. If it remains NULL, A default internal implementation (within Clarinox libraries) will be
used instead (if available).
*/
extern ClxSdioBspInterface* clxSdioBspInterface;

#ifdef __cplusplus
}
#endif

#endif // __SDIO_BSP__

