/******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                OSStartApp.c
* Description         OS Start-up file
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/
#include  <stdbool.h>
#include  <os.h>
#include  <StmBsp.h>

#define  MAIN_THREAD_STACK_SIZE                          1024 * 2 
#define  MAIN_THREAD_PRIORITY                                   3
#define  TRUE                                                   1
#define  FALSE                                                  0

extern  int      clarinoxMain(void);
static  OS_TCB   mainThreadTCB;
static  CPU_STK  mainThreadStack[MAIN_THREAD_STACK_SIZE];
static  void     mainThreadEntry  (void *p_arg);

/*
*********************************************************************************************************
*                                                main
*
* Description : main function
*
*********************************************************************************************************
*/

int  main (void)
{    
    
    OS_ERR  err;
    
    /* 
     *Disable all interrupts. 
     */
    BSP_IntDisAll();                                            
    /* 
     *Init uC/OS-III. 
     */
    OSInit(&err);                                               
    /* 
     *Create the Main Thread 
     */
    OSTaskCreate((OS_TCB     *)&mainThreadTCB,                  
                 (CPU_CHAR   *)"Main Thread",
                 (OS_TASK_PTR )mainThreadEntry, 
                 (void       *)0,
                 (OS_PRIO     )MAIN_THREAD_PRIORITY,
                 (CPU_STK    *)&mainThreadStack[0],
                 (CPU_STK_SIZE)MAIN_THREAD_STACK_SIZE / 10,
                 (CPU_STK_SIZE)MAIN_THREAD_STACK_SIZE,
                 (OS_MSG_QTY  )0,
                 (OS_TICK     )0,
                 (void       *)0,
                 (OS_OPT      )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                 (OS_ERR     *)&err);  
    
    /* 
     *Start multitasking (i.e. give control to uC/OS-III). 
     */
    OSStart(&err);                                              
}


/*
*********************************************************************************************************
*                                          MAIN THREAD
*
* Description : This is a main thread entry function. 
*
* Arguments   : p_arg   is the argument passed to the entry thread.
*
*********************************************************************************************************
*/

static  void  mainThreadEntry (void *p_arg)
{
    CPU_INT32U  cpu_clk_freq; 
    CPU_INT32U  cnts;
    OS_ERR      err;
   
    /* 
     *Initialize BSP functions 
     */
    BSP_Init(); 
    /* 
     *Initialize the uC/CPU services 
     */
    CPU_Init();                                                   
    
    /* 
     *Determine SysTick reference freq. 
     */
    cpu_clk_freq = BSP_CPU_ClkFreq();                                                                                                     
    if((cpu_clk_freq != CLX_CPU_CLOCK))
    {
        printf("Wrong CPU clock configuration\n");
        while(1);
    }
    cnts = cpu_clk_freq / (CPU_INT32U)OSCfg_TickRate_Hz; 
    
    /* 
     *Init uC/OS periodic time src (SysTick). 
     */ 
    OS_CPU_SysTickInit(cnts);                                        

#if OS_CFG_STAT_TASK_EN > 0u
    /* 
     *Compute CPU capacity with no task running 
     */
    OSStatTaskCPUUsageInit(&err);                                 
#endif
    
    /* 
     *Main Clarinox Application 
     */
    clarinoxMain();

    while (TRUE) 
    {   
        /* 
         *Never reaches this. 
         */
        OSTimeDlyHMSM(0, 0, 0, 100, 
                      OS_OPT_TIME_HMSM_STRICT, 
                      &err);
    }
}

/*******************************************************************************************************************************
*                                                Hard Fault Handler
*
* Stores the value of the registers before the exception happens.
*
* stacked_pc : Address where the processor was executing before the hard fault occured.
*
* stacked_lr : Contains the return address of the last function call.
*
* Parameter : Stack frame location
*
*******************************************************************************************************************************/

void hard_fault_handler_c (unsigned int * hardfault_args)
{
  unsigned int stacked_r0;
  unsigned int stacked_r1;
  unsigned int stacked_r2;
  unsigned int stacked_r3;
  unsigned int stacked_r12;
  unsigned int stacked_lr;
  unsigned int stacked_pc;
  unsigned int stacked_psr;
 
  stacked_r0 = ((unsigned long) hardfault_args[0]);
  stacked_r1 = ((unsigned long) hardfault_args[1]);
  stacked_r2 = ((unsigned long) hardfault_args[2]);
  stacked_r3 = ((unsigned long) hardfault_args[3]);
 
  stacked_r12 = ((unsigned long) hardfault_args[4]);
  stacked_lr = ((unsigned long) hardfault_args[5]);
  stacked_pc = ((unsigned long) hardfault_args[6]);
  stacked_psr = ((unsigned long) hardfault_args[7]);
 
  printf ("\n\n[Hard fault handler - all numbers in hex]\n");
  printf ("R0 = %x\n", stacked_r0);
  printf ("R1 = %x\n", stacked_r1);
  printf ("R2 = %x\n", stacked_r2);
  printf ("R3 = %x\n", stacked_r3);
  printf ("R12 = %x\n", stacked_r12);
  printf ("LR [R14] = %x  subroutine call return address\n", stacked_lr);
  printf ("PC [R15] = %x  program counter\n", stacked_pc);
  printf ("PSR = %x\n", stacked_psr);
  printf ("BFAR = %x\n", (*((volatile unsigned long *)(0xE000ED38))));
  printf ("CFSR = %x\n", (*((volatile unsigned long *)(0xE000ED28))));
  printf ("HFSR = %x\n", (*((volatile unsigned long *)(0xE000ED2C))));
  printf ("DFSR = %x\n", (*((volatile unsigned long *)(0xE000ED30))));
  printf ("AFSR = %x\n", (*((volatile unsigned long *)(0xE000ED3C))));
  printf ("SCB_SHCSR = %x\n", SCB->SHCSR);
 
  while (1);
}
