/*******************************************************************************
*
* Project             ClarinoxBlue
* File                UartBsp.cpp
* Description         UART interface for Bluetooth Chip
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  1124
// _____________________________________________________________________________
//
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <assert.h>

#include "ClxTypes.h"

#include "ClxBsp.h"	

#include "UartBsp.h"
typedef s4 ClxResult;


#ifdef __cplusplus
extern "C" {
#endif 

#include "StmBsp.h"
#include "stm32f4xx_conf.h" 
#include "os.h"  

DMA_InitTypeDef  DMA_InitStructure_Debug;
DMA_InitTypeDef  DMA_InitStructure_Bluetooth;


void                    CLX_Ser_ISR_Handler_Bluetooth  (void);
void                    CLX_Ser_ISR_Handler_Debug      (void);
void                    CLX_DMA_ISR_Handler_Bluetooth  (void);
void                    CLX_DMA_ISR_Handler_Debug      (void);

void                    Clx_IntVectSet                 (CPU_DATA       int_id, CPU_FNCT_VOID  isr);
CPU_INT08U              BSP_Ser_RdByteUnlocked         (void);
boolean                 clxBluetoothUartSetBaudRate    (const s1* portName, u4  baudRate);

static                  ClxSemaphore   BSP_SerTxWait_Bluetooth;
static                  ClxSemaphore   BSP_SerRxWait_Bluetooth;

static                  ClxSemaphore   BSP_SerTxWait_Debug;
static                  ClxSemaphore   BSP_SerRxWait_Debug;

static                  CPU_INT08U   BSP_SerRxData;
  
// Bluetooth Receive Buffer
u1                      ringBuffer[UART_RX_BUFFER_SIZE];
u4                      readIndex 			        = 0;
u4                      writeIndex 			        = 0;
u4                      readChunkSize 		        = 0;
u4                      noOfUnreadBytes 	        = 0;
u4                      maxNoOfUnreadBytes 	        = 0;  
/***************************************************************************************
  * brief  Initializes both Bluetooth and Debug serial port.
  * param  portName: Specifies the UART/USART port to be initialized.
  * param  baudRate: specifies the baud rate to be used.
  * param  numOfDataBits: Specifies the number of data bits to be used.
  * param  parity: specifies the parity.
  * param  numOfStopBits: specifies the number of stop bits.
  * param  userParams: In case, if the user wants to pass any input.
  * param  handle: Handle refers to the UART/USART port being used.
  * retval CLX_SUCCESS/CLX_FAIL.
  ************************************************************************************/
static ClxResult openPort (const s1* portName,
                            u4 baudRate,
                            ClxUartDataBits numOfDataBits,
                            ClxUartParity parity,
                            ClxUartStopBit numOfStopBits,
                            const ClxConfigParam* userParams,
                            ClxUartPort* handle)
{
    GPIO_InitTypeDef        gpio_init;
    USART_InitTypeDef       usart_init;
    USART_TypeDef* COMport;
    	
    uint16_t USART_Parity;
    uint16_t USART_StopBits;
    uint16_t USART_WordLength;

    if(strcmp( "BLUETOOTH_PORT", portName ) == 0)
    {
#if defined (CLX_UART_BLUETOOTH)   
        /* DMA clock enable */
	    RCC_AHB1PeriphClockCmd(BLUETOOTH_PORT_DMA, ENABLE);	
        COMport = BLUETOOTH_PORT;
 
        RCC_AHB1PeriphClockCmd(BLUETOOTH_PORT_CLOCK_SETUP_GPIO, ENABLE);
        RCC_APB2PeriphClockCmd(BLUETOOTH_PORT_CLOCK_ALTERNATE_FUNCTION, ENABLE);  
      
        GPIO_PinAFConfig(BLUETOOTH_GPIO_GROUP, BLUETOOTH_TX_PIN_SOURCE, BLUETOOTH_PIN_TYPE);
        GPIO_PinAFConfig(BLUETOOTH_GPIO_GROUP, BLUETOOTH_RX_PIN_SOURCE, BLUETOOTH_PIN_TYPE); 
        GPIO_PinAFConfig(BLUETOOTH_GPIO_GROUP, BLUETOOTH_RTS_PIN_SOURCE, BLUETOOTH_PIN_TYPE); 
        GPIO_PinAFConfig(BLUETOOTH_GPIO_GROUP, BLUETOOTH_CTS_PIN_SOURCE, BLUETOOTH_PIN_TYPE);

        gpio_init.GPIO_Pin 	= BLUETOOTH_TX | BLUETOOTH_RX;
        gpio_init.GPIO_Mode 	= GPIO_Mode_AF;
        gpio_init.GPIO_OType 	= GPIO_OType_PP;
        gpio_init.GPIO_PuPd 	= GPIO_PuPd_UP;
        gpio_init.GPIO_Speed 	= GPIO_Speed_50MHz;
        GPIO_Init(BLUETOOTH_GPIO_GROUP, &gpio_init);
          
        gpio_init.GPIO_Pin 	= BLUETOOTH_CTS | BLUETOOTH_RTS;
        gpio_init.GPIO_Mode 	= GPIO_Mode_AF;
        gpio_init.GPIO_OType 	= GPIO_OType_PP;
        gpio_init.GPIO_PuPd 	= GPIO_PuPd_NOPULL;
        gpio_init.GPIO_Speed 	= GPIO_Speed_50MHz;
        GPIO_Init(BLUETOOTH_GPIO_GROUP, &gpio_init);
#endif
    }  	
    else if(strcmp( "DEBUG_PORT", portName ) == 0)
    {
#if defined (CLX_UART_DEBUG)  
        /* DMA clock enable */
	    RCC_AHB1PeriphClockCmd(DEBUG_PORT_DMA, ENABLE);		
        COMport = DEBUG_PORT; 
        RCC_AHB1PeriphClockCmd(DEBUG_PORT_CLOCK_SETUP_GPIO, ENABLE);
        RCC_APB2PeriphClockCmd(DEBUG_PORT_CLOCK_ALTERNATE_FUNCTION, ENABLE);  
      
        GPIO_PinAFConfig(DEBUG_GPIO_GROUP, DEBUG_TX_PIN_SOURCE, DEBUG_PIN_TYPE);
        GPIO_PinAFConfig(DEBUG_GPIO_GROUP, DEBUG_RX_PIN_SOURCE, DEBUG_PIN_TYPE);
      
        gpio_init.GPIO_Pin 	= DEBUG_TX | DEBUG_RX;
        gpio_init.GPIO_Mode 	= GPIO_Mode_AF;
        gpio_init.GPIO_OType 	= GPIO_OType_PP;
        gpio_init.GPIO_PuPd 	= GPIO_PuPd_UP;
        gpio_init.GPIO_Speed 	= GPIO_Speed_50MHz;
        GPIO_Init(DEBUG_GPIO_GROUP, &gpio_init);
#endif
    }
    else
    {
        while(1){}
    }
      
    *handle = (ClxUartPort)COMport;

    /* ------------------ INIT OS OBJECTS ----------------- */
    if (*handle == (ClxUartPort*)BLUETOOTH_PORT)
    {
#if defined (CLX_UART_BLUETOOTH)
		BSP_SerTxWait_Bluetooth = clxCreateSemaphore( 0 );
		BSP_SerRxWait_Bluetooth = clxCreateSemaphore( 0 );
#endif
    }
    else if (*handle == (ClxUartPort*)DEBUG_PORT)
    {
#if defined (CLX_UART_DEBUG)
		BSP_SerTxWait_Debug = clxCreateSemaphore( 0 );
		BSP_SerRxWait_Debug = clxCreateSemaphore( 0 );
#endif
    }    
    /* ------------------ INIT PORT PARAMETERS ----------------- */    
    switch(numOfDataBits)
    {
    case ClxUartDataBits_7:
        USART_WordLength = USART_WordLength_9b;
        break;
    case ClxUartDataBits_8:
        USART_WordLength = USART_WordLength_8b;
        break;
    }
    switch(parity)
    {
    case ClxUartParity_Even:
        USART_Parity = USART_Parity_Even;
        break;
    case ClxUartParity_Odd:
        USART_Parity = USART_Parity_Odd;
        break;
    case ClxUartParity_None:
        USART_Parity = USART_Parity_No;
        break;			
    }
    switch(numOfStopBits)
    {
    case ClxUartStopBit_One:
        USART_StopBits = USART_StopBits_1;
        break;
    case ClxUartStopBit_OneHalf:
        USART_StopBits = USART_StopBits_1_5;
        break;
    case ClxUartStopBit_Two:
        USART_StopBits = USART_StopBits_2;
        break;			
    }
    /* ----------------- INIT USART STRUCT ---------------- */
    usart_init.USART_BaudRate            = baudRate;
    usart_init.USART_WordLength          = USART_WordLength;
    usart_init.USART_StopBits            = USART_StopBits;
    usart_init.USART_Parity              = USART_Parity ;

    if (COMport == DEBUG_PORT)
    {
#if defined (CLX_UART_DEBUG)
        usart_init.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
#endif
    }
    else if (COMport == BLUETOOTH_PORT)
    {
#if defined (CLX_UART_BLUETOOTH)
        usart_init.USART_HardwareFlowControl = USART_HardwareFlowControl_RTS_CTS;
#endif
    }   	
    usart_init.USART_Mode                = USART_Mode_Rx | USART_Mode_Tx;
	
    /* ------------------ SETUP USART -------------------- */
    USART_Init(COMport, &usart_init);
    if(COMport == DEBUG_PORT) 
    { 
#if defined (CLX_UART_DEBUG)
        DMA_DeInit(DEBUG_TX_DMA_STREAM);
        while (DMA_GetCmdStatus(DEBUG_TX_DMA_STREAM) != DISABLE);
        /* Configure DMA controller to manage DEBUG_PORT USART TX DMA request ----------*/  
        DMA_InitStructure_Debug.DMA_PeripheralBaseAddr              = ((uint32_t)COMport + 0x04);
        DMA_InitStructure_Debug.DMA_PeripheralInc                   = DMA_PeripheralInc_Disable;
        DMA_InitStructure_Debug.DMA_MemoryInc                       = DMA_MemoryInc_Enable;
        DMA_InitStructure_Debug.DMA_PeripheralDataSize              = DMA_PeripheralDataSize_Byte;
        DMA_InitStructure_Debug.DMA_MemoryDataSize                  = DMA_MemoryDataSize_Byte;
        DMA_InitStructure_Debug.DMA_Mode                            = DMA_Mode_Normal;
        DMA_InitStructure_Debug.DMA_Priority                        = DMA_Priority_High;
        DMA_InitStructure_Debug.DMA_FIFOMode                        = DMA_FIFOMode_Enable;
        DMA_InitStructure_Debug.DMA_FIFOThreshold                   = DMA_FIFOThreshold_Full;
        DMA_InitStructure_Debug.DMA_MemoryBurst                     = DMA_MemoryBurst_Single;
        DMA_InitStructure_Debug.DMA_PeripheralBurst                 = DMA_PeripheralBurst_Single;
        DMA_Init(DEBUG_TX_DMA_STREAM, &DMA_InitStructure_Debug);  
	  
	    BSP_IntVectSet(DEBUG_TX_DMA_IRQ_VECTOR, CLX_DMA_ISR_Handler_Debug);
        BSP_IntEn(DEBUG_TX_DMA_IRQ_VECTOR);
      
        BSP_IntVectSet(DEBUG_RX_IRQ_VECTOR, CLX_Ser_ISR_Handler_Debug);
        BSP_IntEn(DEBUG_RX_IRQ_VECTOR);
#endif
    }
   else if(COMport == BLUETOOTH_PORT) 
   {
#if defined (CLX_UART_BLUETOOTH)
        DMA_DeInit(BLUETOOTH_TX_DMA_STREAM);
        while (DMA_GetCmdStatus(BLUETOOTH_TX_DMA_STREAM) != DISABLE);
        /* Configure DMA controller to manage BLUETOOTH_PORT USART TX DMA request ----------*/  
        DMA_InitStructure_Bluetooth.DMA_PeripheralBaseAddr       = ((uint32_t)COMport + 0x04);
        DMA_InitStructure_Bluetooth.DMA_PeripheralInc            = DMA_PeripheralInc_Disable;
        DMA_InitStructure_Bluetooth.DMA_MemoryInc                = DMA_MemoryInc_Enable;
        DMA_InitStructure_Bluetooth.DMA_PeripheralDataSize       = DMA_PeripheralDataSize_Byte;
        DMA_InitStructure_Bluetooth.DMA_MemoryDataSize           = DMA_MemoryDataSize_Byte;
        DMA_InitStructure_Bluetooth.DMA_Mode                     = DMA_Mode_Normal;
        DMA_InitStructure_Bluetooth.DMA_Priority                 = DMA_Priority_VeryHigh;
        DMA_InitStructure_Bluetooth.DMA_FIFOMode                 = DMA_FIFOMode_Enable;
        DMA_InitStructure_Bluetooth.DMA_FIFOThreshold            = DMA_FIFOThreshold_Full;
        DMA_InitStructure_Bluetooth.DMA_MemoryBurst              = DMA_MemoryBurst_Single;
        DMA_InitStructure_Bluetooth.DMA_PeripheralBurst          = DMA_PeripheralBurst_Single;
        DMA_Init(BLUETOOTH_TX_DMA_STREAM, &DMA_InitStructure_Bluetooth);
      
	    BSP_IntVectSet(BLUETOOTH_TX_DMA_IRQ_VECTOR, CLX_DMA_ISR_Handler_Bluetooth);
        BSP_IntEn(BLUETOOTH_TX_DMA_IRQ_VECTOR);
       
        BSP_IntVectSet(BLUETOOTH_RX_IRQ_VECTOR, CLX_Ser_ISR_Handler_Bluetooth);
        BSP_IntEn(BLUETOOTH_RX_IRQ_VECTOR);
#endif
    }
    
    USART_Cmd(COMport, ENABLE);
    
    return(CLX_SUCCESS);
}

/****************************************************************************************
  * brief  Write to both Debug and Bluetooth Ports.
  * param  handle: Handle refers to the UART/USART port being used.
  * param  data: specifies the address of the data bus to be written.
  * param  dataLength: Specifies the length of the data to be written.
  * param  lengthWritten: contains the value of the number of data written.
  * retval CLX_SUCCESS/CLX_FAIL.
  ************************************************************************************/						
static ClxResult write (ClxUartPort handle,
                        const u1* data,
                        size_t dataLength,
                        size_t* lengthWritten)
{

    if (data == (u1 *)0) 
    {
        return(CLX_FAIL);
    }

    if (handle == (ClxUartPort)DEBUG_PORT)
    {
#if defined (CLX_UART_DEBUG)
        DMA_InitStructure_Debug.DMA_Channel                       = DEBUG_TX_DMA_CHANNEL;
        DMA_InitStructure_Debug.DMA_DIR                           = DMA_DIR_MemoryToPeripheral;  

        /****************** USART will Transmit Specific Command ******************/ 
        /* Prepare the DMA to transfer the transaction command from the memory to the USART */  
        DMA_InitStructure_Debug.DMA_Memory0BaseAddr               = (uint32_t)data;
        DMA_InitStructure_Debug.DMA_BufferSize                    = (uint16_t)dataLength;
        DMA_Init(DEBUG_TX_DMA_STREAM, &DMA_InitStructure_Debug); 

        /* Enables the USART's DMA interface */
        USART_DMACmd(DEBUG_PORT, USART_DMAReq_Tx, ENABLE);        

        /* Enable the DMA TX Stream, USART will start sending the data */
        DMA_Cmd(DEBUG_TX_DMA_STREAM, ENABLE);

        DMA_ITConfig(DEBUG_TX_DMA_STREAM, DMA_IT_TC, ENABLE);							//Transfer complete interrupt
        WAIT_FOR_DEBUG_PORT_TX_SEMAPHORE 
        // After DMA completion, wait for UART last byte transmission completion as well        
        while (USART_GetFlagStatus(DEBUG_PORT, USART_FLAG_TC) != SET) 
        {     
        }  
        USART_ClearFlag(DEBUG_PORT, USART_FLAG_TC);          
#endif
    }
    else if(handle == (ClxUartPort)BLUETOOTH_PORT)
    {         
#if defined (CLX_UART_BLUETOOTH)
        DMA_InitStructure_Bluetooth.DMA_Channel                    = BLUETOOTH_PORT_TX_DMA_CHANNEL;
        DMA_InitStructure_Bluetooth.DMA_DIR                        = DMA_DIR_MemoryToPeripheral;  

        /****************** USART will Transmit Specific Command ******************/ 
        /* Prepare the DMA to transfer the transaction command from the memory to the USART */  
        DMA_InitStructure_Bluetooth.DMA_Memory0BaseAddr            = (uint32_t)data;
        DMA_InitStructure_Bluetooth.DMA_BufferSize                 = (uint16_t)dataLength;
        DMA_Init(BLUETOOTH_TX_DMA_STREAM, &DMA_InitStructure_Bluetooth); 

        /* Enables the USART's DMA interface */
        USART_DMACmd(BLUETOOTH_PORT, USART_DMAReq_Tx, ENABLE);

        /* Enable the DMA TX Stream, USART will start sending the data */
        DMA_Cmd(BLUETOOTH_TX_DMA_STREAM, ENABLE);
        DMA_ITConfig(BLUETOOTH_TX_DMA_STREAM, DMA_IT_TC, ENABLE);										//Transfer complete interrupt

        WAIT_FOR_BLUETOOTH_PORT_TX_SEMAPHORE
        
        // After DMA completion, wait for UART last byte transmission completion as well        
        while (USART_GetFlagStatus(BLUETOOTH_PORT, USART_FLAG_TC) != SET) 
        {      
        } 
        USART_ClearFlag(BLUETOOTH_PORT, USART_FLAG_TC);       
#endif
    }

    *lengthWritten = dataLength;    
     return(CLX_SUCCESS);
}
/*************************************************************************************
  * brief  Write to both Debug and Bluetooth Ports.
  * param  handle: Handle refers to the UART/USART port being used.
  * param  buf: Contains read values.
  * param  bufLength: Specifies the length of the data to be read.
  * param  lengthRead: contains the value of the number of data read.
  * retval CLX_SUCCESS/CLX_FAIL.
  ************************************************************************************/	
static ClxResult read (ClxUartPort handle,
                        u1* buf,
                        size_t bufLength,
                        size_t* lengthRead)
{
    if (handle == (ClxUartPort)BLUETOOTH_PORT)
    {
#if defined (CLX_UART_BLUETOOTH)      
      static u4 initialized = 0;
      if( initialized == 0 )
      {
         USART_ITConfig((USART_TypeDef *)handle, USART_IT_RXNE, ENABLE);					
         initialized = 1; 
      }
      do{				
            if(writeIndex >= readIndex)
            {
                if ( (writeIndex - readIndex) >= bufLength)
                {
                    readChunkSize = UART_RX_BUFFER_SIZE; 
                    memcpy( buf, &ringBuffer[readIndex], bufLength );
                    readIndex += bufLength;
                    break; 
                }
            }
            else 
            {
                if ( (UART_RX_BUFFER_SIZE - readIndex ) >= bufLength )
                {
                    readChunkSize = UART_RX_BUFFER_SIZE;  
                    memcpy( buf, &ringBuffer[readIndex], bufLength );
                    readIndex += bufLength;
                    break;
                }
                else if ( (UART_RX_BUFFER_SIZE - readIndex + writeIndex ) >= bufLength )
                {
                    readChunkSize = UART_RX_BUFFER_SIZE;
                    memcpy( buf, &ringBuffer[readIndex], UART_RX_BUFFER_SIZE - readIndex );
                    memcpy( UART_RX_BUFFER_SIZE - readIndex + buf, &ringBuffer[0], bufLength - UART_RX_BUFFER_SIZE + readIndex );
                    readIndex = bufLength - UART_RX_BUFFER_SIZE + readIndex;
                    break;
                }
            }
            readChunkSize = bufLength;
            WAIT_FOR_BLUETOOTH_PORT_DATA_READ_SEMAPHORE	    
            if( (writeIndex | readIndex) == 0)
            {
                return CLX_ERROR; // this is the condition where we closed the UART
            }
        } while(1);
#if 1
// TODO just to measure max size required for the RX buffer		  
        USART_ITConfig((USART_TypeDef *)handle, USART_IT_RXNE, DISABLE);
	    noOfUnreadBytes -= bufLength;		  
	    USART_ITConfig((USART_TypeDef *)handle, USART_IT_RXNE, ENABLE);
#endif		  
#endif
        *lengthRead = bufLength; 
    }
    else
    {
#if defined (CLX_UART_DEBUG)
        USART_ITConfig(DEBUG_PORT, USART_IT_RXNE, ENABLE);          /* Enable the Receive not empty interrupt             */
        WAIT_FOR_DEBUG_PORT_DATA_READ_SEMAPHORE 
        USART_ITConfig(DEBUG_PORT, USART_IT_RXNE, DISABLE);         /* Disable the Receive not empty interrupt            */	
        buf[0] = BSP_SerRxData;                                     /* Read the data from the temporary register          */
#endif
        *lengthRead = 1; 
    }
       
    return(CLX_SUCCESS);
}


/**************************************************************************************
  * brief  Write to both Debug and Bluetooth Ports.
  * param  handle: Handle refers to the UART/USART port being used.
  * retval CLX_SUCCESS/CLX_FAIL.
  ************************************************************************************/	
static ClxResult closePort (ClxUartPort handle)
{
    CPU_BOOLEAN   err;
    OS_SEM*   uartTxSem;
    OS_SEM*   uartRxSem;
    
    if (handle == (ClxUartPort*)DEBUG_PORT)
    {
        uartTxSem = (OS_SEM*)BSP_SerTxWait_Debug;
        uartRxSem = (OS_SEM*)BSP_SerRxWait_Debug;
    }
    else if (handle == (ClxUartPort*)BLUETOOTH_PORT)
    {
        uartTxSem = (OS_SEM*)BSP_SerTxWait_Bluetooth;
        uartRxSem = (OS_SEM*)BSP_SerRxWait_Bluetooth;
        writeIndex = 0;
        readIndex  = 0;
    }
    USART_Cmd((USART_TypeDef*)handle, DISABLE);
    clxYieldCurrentThread();

    if (handle == (ClxUartPort*)DEBUG_PORT)
    {
        RELEASE_DEBUG_PORT_DATA_READ_SEMAPHORE
    }
    else if (handle == (ClxUartPort*)BLUETOOTH_PORT)
    {
        writeIndex = 0;
        readIndex  = 0;
        RELEASE_BLUETOOTH_PORT_DATA_READ_SEMAPHORE
    }
    clxYieldCurrentThread();

    OSSemDel ((OS_SEM  *)uartTxSem,
            (OS_OPT   )OS_OPT_DEL_NO_PEND,
            (OS_ERR  *)&err);
    free( uartTxSem );
    OSSemDel ((OS_SEM  *)uartRxSem,
            (OS_OPT   )OS_OPT_DEL_NO_PEND,
            (OS_ERR  *)&err);
    free( uartRxSem );
    return(CLX_SUCCESS);
}
/*************************************************************************************
  * brief  Interrupt handler for Debug Port DMA.
  * param  none.
  * retval none.
  ************************************************************************************/	
void CLX_DMA_ISR_Handler_Debug (void)
{
    /* Test on DMA Stream Transfer Complete interrupt */
    if(DMA_GetITStatus(DEBUG_TX_DMA_STREAM, DEBUG_TX_DMA_TC_IRQ))
    {
         /* Clear DMA Stream Transfer Complete interrupt pending bit */
        DMA_ClearITPendingBit(DEBUG_TX_DMA_STREAM, DEBUG_TX_DMA_TC_IRQ);  
        DMA_ITConfig(DEBUG_TX_DMA_STREAM, DMA_IT_TC, DISABLE); 
        
        RELEASE_DEBUG_PORT_TX_SEMAPHORE
    }
}

/*************************************************************************************
  * brief  Interrupt handler for Bluetooth DMA.
  * param  none.
  * retval none.
  ************************************************************************************/	
void CLX_DMA_ISR_Handler_Bluetooth (void)
{
    /* Test on DMA Stream Transfer Complete interrupt */
    if(DMA_GetITStatus(BLUETOOTH_TX_DMA_STREAM, BLUETOOTH_TX_DMA_TC_IRQ))
    {
        /* Clear DMA Stream Transfer Complete interrupt pending bit */
        DMA_ClearITPendingBit(BLUETOOTH_TX_DMA_STREAM, BLUETOOTH_TX_DMA_TC_IRQ);  
        DMA_ITConfig(BLUETOOTH_TX_DMA_STREAM, DMA_IT_TC, DISABLE);

        RELEASE_BLUETOOTH_PORT_TX_SEMAPHORE
    }
}

/*************************************************************************************
  * brief  Interrupt handler for BLUETOOTH_PORT RX.
  * param  none.
  * retval none.
  ************************************************************************************/	
void  CLX_Ser_ISR_Handler_Bluetooth (void)
{
    FlagStatus rxne_status;

    rxne_status = USART_GetFlagStatus(BLUETOOTH_PORT, USART_FLAG_RXNE);
    if (rxne_status == SET) 
    {
        ringBuffer[writeIndex++] = USART_ReceiveData(BLUETOOTH_PORT) & 0xFF;    
        if(writeIndex == UART_RX_BUFFER_SIZE)
        {
            writeIndex = 0;
        }
        USART_ClearITPendingBit(BLUETOOTH_PORT, USART_IT_RXNE);         
				
        if(writeIndex > readIndex)
        {
              if((writeIndex - readIndex) >= readChunkSize)
              {
                    readChunkSize = UART_RX_BUFFER_SIZE;
                    RELEASE_BLUETOOTH_PORT_DATA_READ_SEMAPHORE
              }
        }
        else
        {
              if ( (UART_RX_BUFFER_SIZE - readIndex + writeIndex ) >= readChunkSize )
              {
                    readChunkSize = UART_RX_BUFFER_SIZE;
                    RELEASE_BLUETOOTH_PORT_DATA_READ_SEMAPHORE
              }
        }
#if 1
// TODO just to measure max size required for the RX buffer		  
		noOfUnreadBytes++;
		if ( noOfUnreadBytes > UART_RX_BUFFER_SIZE)
		{
            // TODO needs fixing..
            while(1)
            {
            }			
		}
		if( noOfUnreadBytes > maxNoOfUnreadBytes )
		{
		    maxNoOfUnreadBytes = noOfUnreadBytes;
		}
#endif
    }
}

/**************************************************************************************
  * brief  Interrupt handler for DEBUG_PORT RX.
  * param  none.
  * retval none.
  ************************************************************************************/	
void  CLX_Ser_ISR_Handler_Debug (void)
{
    FlagStatus rxne_status;

    rxne_status = USART_GetFlagStatus(DEBUG_PORT, USART_FLAG_RXNE);
    if (rxne_status == SET) {
        BSP_SerRxData = USART_ReceiveData(DEBUG_PORT) & 0xFF;       /* Read one byte from the receive data register.      */
        USART_ClearITPendingBit(DEBUG_PORT, USART_IT_RXNE);         /* Clear the DEBUG_PORT receive interrupt.            */
        RELEASE_DEBUG_PORT_DATA_READ_SEMAPHORE
    }
}

/*************************************************************************************************************
  * brief  Once the reset command is sent to the Bluetooth chip the baud rate can be increased to 921600 bps.
  * param  portName: Base address of the Bluetooth USART port
  * param  baudRate: Baud rate to be set ( Currently it is 921600 bps )
  * retval none.
  ************************************************************************************************************/	

boolean clxBluetoothUartSetBaudRate(const s1* portName, u4  baudRate)
{
    USART_InitTypeDef       usart_init;
    if(strcmp( "BLUETOOTH_PORT", portName ) == 0)
    {
      usart_init.USART_BaudRate            = baudRate; 
      usart_init.USART_WordLength          = USART_WordLength_8b;
      usart_init.USART_StopBits            = USART_StopBits_1;
      usart_init.USART_Parity              = USART_Parity_No ;
      usart_init.USART_HardwareFlowControl = USART_HardwareFlowControl_RTS_CTS;
      usart_init.USART_Mode                = USART_Mode_Rx | USART_Mode_Tx;
      USART_Init(BLUETOOTH_PORT, &usart_init);
      return TRUE;
    }
    else
    {
      return FALSE;
    }
}
                           
ClxUartBspInterface clxUartSTM32F407BspInterface = {openPort,
                                                    write,
                                                    read,
                                                    closePort};


#ifdef __cplusplus
}
#endif
