/******************************************************************************
*
* Project             Wlan Sample Application
* File                NetworkStackServiceInterface.LWIP.Bsp.cpp
* Description         Implementation of Network Stack Service Interface for LWIP stack
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#include "ClxBsp.h"
#include "lwip/tcp.h"
#include "lwip/udp.h"
#include <string.h>
#include <stdio.h>


extern "C" 
{
 

#define interface  ClxLwipStackServiceInterface::instance


#define TCP_POLL_INTERVAL                                                   1 /* around 2 seconds */
#define TCP_MAX_DISCONNECTION_POLL_COUNT                                    2
    
#define SOCKET_OBJECT_SIGNATURE                                             0xBEAFFAEB

#define TCP_CLIENT_SOCKET_MAX_NO_MEM_RETRY_COUNT                            3

#define TCP_CLIENT_SOCKET_STATE_SENDING_CONNECT_REQUEST                     0x01
#define TCP_CLIENT_SOCKET_STATE_WAITING_FOR_CONNECT_RESPONSE                0x02
#define TCP_CLIENT_SOCKET_STATE_CONNECTED                                   0x03
#define TCP_CLIENT_SOCKET_STATE_DISCONNECTING_WAITING_FOR_SEND_COMPLETE     0x04
#define TCP_CLIENT_SOCKET_STATE_SENDING_DISCONNECT_REQUEST                  0x05
#define TCP_CLIENT_SOCKET_STATE_WAITING_FOR_DISCONNECTION_COMPLETE          0x06
    
#define NETWORK_SERVICE_CALLBACK_CONNECTION_COMPLETE                        0x01
#define NETWORK_SERVICE_CALLBACK_NEW_CONNECTION                             0x02
#define NETWORK_SERVICE_CALLBACK_SEND_COMPLETE                              0x04
#define NETWORK_SERVICE_CALLBACK_DATA_RECEIVED                              0x08
#define NETWORK_SERVICE_CALLBACK_CONNECTION_IDLE                            0x10
    

/****************************
Queueable
****************************/
class Queueable
{
public:
    Queueable* prev_;
    Queueable* next_;

protected:
    Queueable()
    {
        prev_ = next_ = this;
    }
    
public:
    boolean isQueued() const
    {
        return (next_ == this) ? FALSE : TRUE;
    }
};


/****************************
SocketTx
****************************/
class SocketTx
{
public:
    const u1* buffer;
    size_t    remainingLength;           /* the length of the remaining data to be sent: */
    size_t    currentIndex;              /* the index of the data which is to be sent next. The data before currentIndex is not necessarily acknowledged yet */
    size_t    pendingLength;             /* Length of the data which has been submitted to LWIP stack but not sent yet (or has been sent but not acknowledged yet) */     
    u1        flags;
    boolean   flush;                     /* Ask the LWIP stack to send all the queued TX data when remainingLength becomes 0 */

public:
    SocketTx()
    {
        reset();
    }

    void reset()
    {
        buffer = NULL;
        flags = 0;
        remainingLength = 0;
        currentIndex = 0;
        pendingLength = 0;
        flags = 0;
        flush = FALSE;
    }
};


/****************************
SocketRx
****************************/
class SocketRx
{
public:
    pbuf* begin;
    pbuf* end;
    pbuf* current;      /* The buffer which is currently being processed by the network service */

public:
    SocketRx()
    {
        reset();
    }

    void reset()
    {
        begin = NULL;
        end = NULL;
        current = NULL;
    }

    void push(pbuf* buf)
    {
        assert(buf);
    
        if (end)
        {
            end->next = buf;
        }
        else
        {
           begin = buf;
        }
    
        while (buf)
        {
            end = buf;
            buf = buf->next;
        }
    }
                       
    pbuf* pop()
    {
        pbuf* ret = begin;
    
        if (ret)
        {
            begin = ret->next;
        
            if (begin == NULL)
            {
               end = NULL; 
            }
        }

        return ret;
    }
}; 


/****************************
Socket
****************************/
class Socket : public Queueable, public ClxNetworkSocketHandle, public ClxPrivatePoolAllocator
{
private:
    u4                  signature_;
    ClxNetworkService&  service_;
    u2                  type_;

public:
    u2                  pendingCallbacks;

protected:
    Socket(ClxNetworkService& service, u2 type)
    :
    signature_(SOCKET_OBJECT_SIGNATURE),
    service_(service),
    type_(type),
    pendingCallbacks(0)
    {}

public:
    virtual ~Socket()
    {
        signature_ = 0;
    }

public:
    virtual void call_connectionComplete() = 0;
    virtual void call_newConnection() = 0;
    virtual void call_sendComplete() = 0;
    virtual void call_dataReceived() = 0;
    virtual void call_connectionIdle() = 0;
    
    virtual ClxResult terminate() = 0;
    virtual void close() = 0;
    virtual ClxResult send(const u1* data, u4 dataLength, u4 flags, const struct ClxNetworkServiceAddress* remoteAddr) = 0;
    virtual ClxResult recvComplete() = 0;

public:
    u2 type() const
    {
        return type_;
    }

    ClxNetworkService&  service()
    {
        return service_;
    }

    boolean isValidHandle() const
    {
        return (signature_ == SOCKET_OBJECT_SIGNATURE) ? TRUE : FALSE;
    }
};


/****************************
TcpClientSocket
****************************/
class TcpClientSocket : public Socket
{
private:
    tcp_pcb*    pcb;

public:
    u2          state;   
    u2          disconnectionPollCount;
    u4          lastActivityTime;
    
    SocketTx    tx;
    SocketRx    rx;

public:
    TcpClientSocket(ClxNetworkService& service)
    :
    Socket(service, CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP),
    state(0), pcb(NULL)
    {}

public:
    /* LWIP callbacks: */
    static err_t lwip_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *p, err_t err);
    static void lwip_connErr(void *arg, err_t err);
    static err_t lwip_poll(void *arg, struct tcp_pcb *pcb);
    static err_t lwip_dataSent(void *arg, struct tcp_pcb *pcb, u16_t len);

    /* Network stack interface calls: */
    void close();
    ClxResult terminate();
    ClxResult send(const u1* data, u4 dataLength, u4 flags, const struct ClxNetworkServiceAddress* remoteAddr);
    ClxResult recvComplete();

public:
    void updateActivityTimeValue()
    {
        lastActivityTime = clxTickTime();
        Socket::pendingCallbacks &= (~NETWORK_SERVICE_CALLBACK_CONNECTION_IDLE);        
    }
    
    err_t connected(struct tcp_pcb* connectionPcb)
    {
        state = TCP_CLIENT_SOCKET_STATE_CONNECTED;          
        pcb = connectionPcb;
        
        /* We store the pointer to the new socket descriptor in the PCB structure: */
        tcp_arg(connectionPcb, this);
        
        tcp_recv(connectionPcb, lwip_recv);
        tcp_err(connectionPcb, lwip_connErr);
        tcp_poll(connectionPcb, lwip_poll, TCP_POLL_INTERVAL);
        tcp_sent(connectionPcb, lwip_dataSent);
        
        disconnectionPollCount = 0;
        
        updateActivityTimeValue();
        
        return ERR_OK;
    }

public:
    void call_connectionComplete();
    void call_newConnection();
    void call_sendComplete();
    void call_dataReceived();
    void call_connectionIdle();
};


/****************************
TcpListenerSocket
****************************/
class TcpListenerSocket : public Socket
{
public:
    tcp_pcb* pcb;

public:
    TcpListenerSocket(ClxNetworkService& service)
    :
    Socket(service, CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP_LISTENER),
    pcb(NULL)
    {}

public:
    ClxResult listen(TcpListenerSocket& socket, const ClxNetworkServiceAddress* localAddr);
    static err_t accept(void *arg, struct tcp_pcb *pcb, err_t err);

public:
    void call_connectionComplete() { assert(0); }
    void call_newConnection() { assert(0); }
    void call_sendComplete() { assert(0); }
    void call_dataReceived() { assert(0); }
    void call_connectionIdle() { assert(0); }
    
    ClxResult send(const u1* data, u4 dataLength, u4 flags, const struct ClxNetworkServiceAddress* remoteAddr) { return CLX_SOCKET_EOPNOTSUPP; }
    ClxResult recvComplete() { return CLX_SOCKET_EOPNOTSUPP; }

public:
    ClxResult terminate()
    {
        if (pcb)
        {
            tcp_close(pcb);
        }
        return CLX_SUCCESS;
    }
    
    void close()
    {
       if (pcb)
       {
           tcp_abort(pcb);
       }
    }
};


/****************************
UdpSocket
****************************/
class UdpSocket : public Socket
{
private:
    udp_pcb*    pcb;

public:
    u2          state;   
    u2          disconnectionPollCount;
    u4          lastActivityTime;
    
    SocketTx    tx;
    SocketRx    rx;

public:
    UdpSocket(ClxNetworkService& service)
    :
    Socket(service, CLX_NETWORK_SOCKET_HANDLE_TYPE_UDP),
    state(0), pcb(NULL)
    {}

public:
  
    ClxResult bind(UdpSocket& socket, const ClxNetworkServiceAddress* localAddr);
  
    /* LWIP callbacks: */
    static void  lwip_udp_recv(void *arg, struct udp_pcb *pcb_lwip, struct pbuf *p, struct ip_addr *addr, u2 port);

    /* Network stack interface calls: */
    void close();
    ClxResult terminate();
    ClxResult send(const u1* data, u4 dataLength, u4 flags, const struct ClxNetworkServiceAddress* remoteAddr);
    ClxResult recvComplete();

public:
    void call_connectionComplete();
    void call_newConnection();
    void call_sendComplete();
    void call_dataReceived();
    void call_connectionIdle();
};


/****************************
ClxLwipStackServiceInterface
****************************/
class ClxLwipStackServiceInterface
{   
public:
    static ClxLwipStackServiceInterface* instance;

public:
    class SocketQueue : public ClxQueueTemplate<Queueable>
    {
    public:
        void pushBack(Socket* item)
        {
            appendBefore (item, this);
        }

        Socket* popFromFront()
        {
            if (isEmpty()) 
            {
                return NULL;
            }

            Queueable* ret = next_;
            ClxQueueTemplate<Queueable>::detach(ret);
            return static_cast<Socket*>(ret);
        }

        void detachItem(Socket* item)
        {
            ClxQueueTemplate<Queueable>::detach(item);
        }
    };

public:
    ClxPrivateMemoryPoolHandle  socketPool;
    SocketQueue                 pendingSocketsQueue;

public:
    ClxLwipStackServiceInterface(u4 numberOfSockets)
    {
        /* We have to find the maximum size of a socket descriptor object: */
        u4 maxSocketDescriptorObjectSize = MAX(sizeof(TcpListenerSocket), MAX(sizeof(TcpClientSocket), sizeof(UdpSocket)));
        // TODO : Add UDP and others

        socketPool = clxCreatePrivateMemoryPool(numberOfSockets, maxSocketDescriptorObjectSize, NULL);
    }

    ~ClxLwipStackServiceInterface()
    {
        clxCheckPrivateMemoryPoolForCorruption(socketPool);
        
        clxDestroyPrivateMemoryPool(socketPool, NULL);
    }
    
    void callbackPendingForSocket(Socket& socket, u2 pendingCallback)
    {
        if (socket.pendingCallbacks == 0)
        {
            assert(!socket.isQueued());

            pendingSocketsQueue.pushBack(&socket);
        }

        socket.pendingCallbacks |= pendingCallback;
    }
    
    
    ClxResult bind_udp(const struct ClxNetworkServiceAddress* localAddr, struct udp_pcb** pcb)
    {
        *pcb = udp_new();  

        if (*pcb == NULL)
        {
            return CLX_SOCKET_ENOBUFS;
        }

        switch(localAddr->type)
        {
        case ClxNetworkServiceAddressType_IP4:
            {
                ClxNetworkServiceIp4Address* localAddr_ = (ClxNetworkServiceIp4Address*)localAddr;

                ip_addr ipaddr;
                ipaddr.addr = localAddr_->addr;

                err_t err = udp_bind(*pcb, &ipaddr, localAddr_->port);
                //udp_bind(pcb, IP_ADDR_ANY, CLX_DHCP_SERVER_PORT)
            
                switch(err)
                {
                case ERR_OK:
                    return CLX_SUCCESS;
                case ERR_USE:
                    return CLX_SOCKET_EADDRINUSE;
                default:
                    ClxConsoleUIEngine::text("\nUDP bind failed!");
                    return CLX_ERROR_INTERNAL_ERROR;
                }
            }
            break;
        default:
            return CLX_SOCKET_EAFNOSUPPORT;
        }         
    }

    ClxResult bind(const struct ClxNetworkServiceAddress* localAddr, struct tcp_pcb** pcb)
    {
        *pcb = tcp_new();  

        if (*pcb == NULL)
        {
            return CLX_SOCKET_ENOBUFS;
        }

        switch(localAddr->type)
        {
        case ClxNetworkServiceAddressType_IP4:
            {
                ClxNetworkServiceIp4Address* localAddr_ = (ClxNetworkServiceIp4Address*)localAddr;

                ip_addr ipaddr;
                ipaddr.addr = localAddr_->addr;

                err_t err = tcp_bind(*pcb, &ipaddr, localAddr_->port);
            
                switch(err)
                {
                case ERR_OK:
                    return CLX_SUCCESS;
                case ERR_USE:
                    tcp_close(*pcb);
                    return CLX_SOCKET_EADDRINUSE;
                default:
                    tcp_close(*pcb);
                    return CLX_ERROR_INTERNAL_ERROR;
                }
            }
            break;
        default:
            tcp_close(*pcb);
            return CLX_SOCKET_EAFNOSUPPORT;
        }         
    }
 
    void deleteSocket(Socket* socket, boolean informService)
    {
        if (socket->pendingCallbacks)
        {
            pendingSocketsQueue.detachItem(socket);
            socket->pendingCallbacks = 0;
        }

        /* Now the descriptor is free: */
        assert(!socket->isQueued());

        ClxNetworkService& service = socket->service();
        void* userData = socket->userData;

        delete socket;

        if (informService)
        {
            service.terminated(&service,userData, CLX_ERROR_CONNECTION_FAILED); 
        }
    }
};

ClxLwipStackServiceInterface* ClxLwipStackServiceInterface::instance = NULL;



/****************************
TcpListenerSocket
****************************/
ClxResult TcpListenerSocket::listen(TcpListenerSocket& socket, const ClxNetworkServiceAddress* localAddr)
{  
    struct tcp_pcb* tempPCB;

    ClxResult ret = interface->bind(localAddr, &tempPCB);
    if (ret != CLX_SUCCESS)
    {
        return ret;
    }
    
    /* We have to listen for incoming connections: */
    socket.pcb = tcp_listen(tempPCB);

    if (!socket.pcb)
    {
        return CLX_SOCKET_ENOBUFS;
    }

    tcp_accept(socket.pcb, accept);

    /* We store the socket pointer in the PCB structure: */
    tcp_arg(socket.pcb, this);                
   
    return CLX_SUCCESS;
}

err_t TcpListenerSocket::accept(void *arg, struct tcp_pcb *pcb, err_t err)
{
    TcpListenerSocket* listeningSocket = (TcpListenerSocket*)arg;
 
    assert(listeningSocket->type() == CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP_LISTENER);              
    
    /* We accept this connection only if we have a free socket descriptor: */

    if (clxIsPrivateMemoryPoolBufferAvailable(interface->socketPool, sizeof(TcpClientSocket)))
    {
        TcpClientSocket* newSocket = new (interface->socketPool) TcpClientSocket(listeningSocket->service());
    
        assert(newSocket);
        assert(newSocket->pendingCallbacks == 0);
            
        /* 
        We queue this new descriptor for calling call_newConnection() call-back function
        later on:
        */
        interface->callbackPendingForSocket(*newSocket, NETWORK_SERVICE_CALLBACK_NEW_CONNECTION);

        /* 
        Informing the stack that we have accepted the new connection request.
        (If the network service is not happy with this new connection, it can close it
        when call_newConnection() call-back function is called):
        */
        tcp_accepted(listeningSocket->pcb);
        
        return newSocket->connected(pcb);
    }
    else
    {
        return ERR_MEM;
    }
}



/****************************
TcpClientSocket
****************************/
err_t TcpClientSocket::lwip_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *p, err_t err)
{
    TcpClientSocket* socket = (TcpClientSocket*)arg;

    assert(socket->type() == CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP);
    assert(socket->pcb == pcb);

    if (err == ERR_OK)
    {
        if (p)
        {
            socket->updateActivityTimeValue();
            
            if (socket->state == TCP_CLIENT_SOCKET_STATE_CONNECTED)
            {
                if(!socket->rx.current)
                {
                    socket->rx.current = p;

                    /* 
                    The service is not processing any RX data.
                    We queue this socket for calling call_dataReceived() call-back function
                    later on:
                    */
                    interface->callbackPendingForSocket(*socket, NETWORK_SERVICE_CALLBACK_DATA_RECEIVED);
                }
                else
                {
                    socket->rx.push(p);
                }
            }
            else
            {
                pbuf_free(p);
            }
        }
        else
        {
            socket->close();
            interface->deleteSocket(socket, TRUE);
        }
    }
    else
    {
        pcb = NULL;
        socket->close();
        interface->deleteSocket(socket, TRUE);  
    }
         
    return ERR_OK;   
}

void TcpClientSocket::close()
{
    /* 
    We free any pending RX buffers:
    */
    pbuf* rxBuf;
    while ((rxBuf = rx.pop()) != NULL)
    {
        pbuf_free(rxBuf);
    }

    if (rx.current)
    {
        pbuf_free(rx.current);        
    }

    if (pcb)
    {            
        /* We don't want to receive any indication when we abort the connection: */
        tcp_recv(pcb, NULL);
        tcp_err(pcb, NULL);
            
        /* This de-allocates the pcb:*/
        tcp_abort(pcb);
        
        pcb = NULL;
    }
}

ClxResult TcpClientSocket::terminate()
{
    /* If there is a connection, we will disconnect it: */
    switch(state)
    {
    case TCP_CLIENT_SOCKET_STATE_CONNECTED:
        {
            assert(pcb);
            
            /* 
            We free any pending RX buffers (except for the one being currently processed by
            the application):
            */
            pbuf* rxBuf;
            while ((rxBuf = rx.pop()) != NULL)
            {
                pbuf_free(rxBuf);
            }
                               
            pendingCallbacks &= ~(NETWORK_SERVICE_CALLBACK_DATA_RECEIVED | 
                                  NETWORK_SERVICE_CALLBACK_NEW_CONNECTION | 
                                  NETWORK_SERVICE_CALLBACK_CONNECTION_COMPLETE | 
                                  NETWORK_SERVICE_CALLBACK_CONNECTION_IDLE);

            if (tx.pendingLength)
            {
                /* We have to wait until all pending has been sent and acknowledged by the other side before we are able to issue disconnect request: */
                state = TCP_CLIENT_SOCKET_STATE_DISCONNECTING_WAITING_FOR_SEND_COMPLETE;
            }
            else if (tcp_close(pcb) == ERR_OK)
            {
                state = TCP_CLIENT_SOCKET_STATE_WAITING_FOR_DISCONNECTION_COMPLETE;
            }
            else
            {
                /* We wait for a poll or TCP sent indication from the stack and try again: */                    
                state = TCP_CLIENT_SOCKET_STATE_SENDING_DISCONNECT_REQUEST;                  
            }
        }
        break;
    
    case TCP_CLIENT_SOCKET_STATE_DISCONNECTING_WAITING_FOR_SEND_COMPLETE:
    case TCP_CLIENT_SOCKET_STATE_SENDING_DISCONNECT_REQUEST:  
    case TCP_CLIENT_SOCKET_STATE_WAITING_FOR_DISCONNECTION_COMPLETE:
        break;
        
    case TCP_CLIENT_SOCKET_STATE_SENDING_CONNECT_REQUEST:      
    case TCP_CLIENT_SOCKET_STATE_WAITING_FOR_CONNECT_RESPONSE:  
         close();
         return CLX_SUCCESS;
         
    default:
        assert(0);
    }
    
    return CLX_ERROR_COMPLETION_PENDING;
}

ClxResult TcpClientSocket::send(const u1* data, u4 dataLength, u4 flags, const struct ClxNetworkServiceAddress* remoteAddr)
{
    if (remoteAddr)
    {
        return CLX_ERROR_INVALID_COMMAND_ARGUMENT;
    }

    switch(state)
    {
    case TCP_CLIENT_SOCKET_STATE_CONNECTED:
        if (tx.buffer)
        {
            return CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS;
        }
        else
        {
            updateActivityTimeValue();
            
            tx.buffer = data;
            tx.currentIndex = 0;
            tx.flags = 0;
            tx.remainingLength = dataLength;

            if (flags & CLX_NETWORK_SOCKET_BUFFER_FLAG_COPY)
            {
                tx.flags = TCP_WRITE_FLAG_COPY;
            }

            if ((flags & CLX_NETWORK_SOCKET_BUFFER_FLAG_SET_TCP_PUSH_BIT) == 0)
            {
                tx.flags |= TCP_WRITE_FLAG_MORE;
            }

            if (flags & CLX_NETWORK_SOCKET_BUFFER_FLAG_FLUSH_QUEUED_DATA)
            {
                tx.flush = TRUE;
            }
            else
            {
                tx.flush = FALSE;
            }

            /* We see if we can send some data in here: */
            u2 data2Send = MIN(tcp_sndbuf(pcb), dataLength);

            if (tcp_write(pcb, tx.buffer, data2Send, tx.flags) == ERR_OK)
            {
                tx.currentIndex += data2Send;
                tx.remainingLength -= data2Send;
                tx.pendingLength += data2Send;

                if (tx.remainingLength == 0)
                {
                    tx.buffer = NULL;

                    if (tx.flush)
                    {
                        tcp_output(pcb);
                    }

                    /* 
                    The current buffer has been completely sent.
                    */
                    return CLX_SUCCESS;
                }               
            }

            /* We wait for tcp_sent callback or a poll callback to continue: */
            return CLX_ERROR_COMPLETION_PENDING;
        }
    default:
        break;
    }

    return CLX_SOCKET_ENOTCONN; 
}

ClxResult TcpClientSocket::recvComplete()
{
    if (rx.current)
    {
        /* Now we can release this buffer: */
        tcp_recved(pcb, rx.current->tot_len);
        pbuf_free(rx.current);
        rx.current = NULL;

        /* Lets see if there are more buffers pending: */
        pbuf* recvBuf = rx.pop();

        if (recvBuf)
        {
            rx.current = recvBuf;

            /* 
            We queue this socket for calling call_dataReceived() call-back function
            later on:
            */
            interface->callbackPendingForSocket(*this, NETWORK_SERVICE_CALLBACK_DATA_RECEIVED);
        }
      
        return CLX_SUCCESS;       
    }
    else
    {
        return CLX_ERROR_INVALID_REQUEST;
    }
}

void TcpClientSocket::lwip_connErr(void *arg, err_t err)
{
    TcpClientSocket* socket = (TcpClientSocket*)arg;

    assert(socket->type() == CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP);

    /* pcb object is already deleted: */
    socket->pcb = NULL;
    socket->close();
    interface->deleteSocket(socket, TRUE);
}

err_t TcpClientSocket::lwip_poll(void *arg, struct tcp_pcb *pcb)
{
    TcpClientSocket* socket = (TcpClientSocket*)arg;

    assert(socket->type() == CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP);

    switch(socket->state)
    {
    case TCP_CLIENT_SOCKET_STATE_CONNECTED:
        {
            /* We check if we can send more data: */
            if (socket->tx.remainingLength)
            {
                u2 data2Send = MIN(tcp_sndbuf(pcb), socket->tx.remainingLength);

                if (tcp_write(pcb, socket->tx.buffer + socket->tx.currentIndex, data2Send, socket->tx.flags) == ERR_OK)
                {
                    socket->tx.currentIndex += data2Send;
                    socket->tx.remainingLength -= data2Send;
                    socket->tx.pendingLength += data2Send;

                    if (socket->tx.remainingLength == 0)
                    {
                        if (socket->tx.flush)
                        {
                            tcp_output(pcb);
                        }

                        /* 
                        The current buffer has been completely sent.
                        We queue this socket for calling call_sendComplete() call-back function
                        later on:
                        */
                        interface->callbackPendingForSocket(*socket, NETWORK_SERVICE_CALLBACK_SEND_COMPLETE);
                    }     
                }

                /* If tcp_write fails, we wait for another call to lwip_dataSent or a call to lwip_poll and try again: */
            }
            else
            {
                /* The connection is IDLE. We inform the service of this fact: */
                interface->callbackPendingForSocket(*socket, NETWORK_SERVICE_CALLBACK_CONNECTION_IDLE);
            }
        }
        break;
        
    case TCP_CLIENT_SOCKET_STATE_SENDING_DISCONNECT_REQUEST:
        /* Lets see if we can send disconnect request now: */
        if (tcp_close(pcb) == ERR_OK)
        {
            /* pcb is already freed: */
            pcb = NULL;
            socket->close();
            interface->deleteSocket(socket, TRUE);
        }
        else if (++socket->disconnectionPollCount >=  TCP_MAX_DISCONNECTION_POLL_COUNT)
        {
            /* Disconnection attempt took too long: */
            socket->close();
            interface->deleteSocket(socket, TRUE);            
        }
        break;
        
    case TCP_CLIENT_SOCKET_STATE_WAITING_FOR_DISCONNECTION_COMPLETE:     
        if (++socket->disconnectionPollCount >=  TCP_MAX_DISCONNECTION_POLL_COUNT)
        {
            /* Disconnection attempt took too long: */
            socket->close();
            interface->deleteSocket(socket, TRUE);            
        }
        break;
        
            
    default:
        break;
    }
    
    return ERR_OK;
}

err_t TcpClientSocket::lwip_dataSent(void *arg, struct tcp_pcb *pcb, u16_t len)
{
    TcpClientSocket* socket = (TcpClientSocket*)arg;

    assert(socket->type() == CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP);

    assert(len <= socket->tx.pendingLength);

    socket->tx.pendingLength -= len;

    switch(socket->state)
    {
    case TCP_CLIENT_SOCKET_STATE_CONNECTED:
        if (socket->tx.remainingLength)
        {
            assert(socket->tx.buffer);
                
            /* We see if we can send more in here: */
            u2 data2Send = MIN(tcp_sndbuf(pcb), socket->tx.remainingLength);

            if (tcp_write(pcb, socket->tx.buffer + socket->tx.currentIndex, data2Send, socket->tx.flags) == ERR_OK)
            {
                socket->tx.currentIndex += data2Send;
                socket->tx.remainingLength -= data2Send;
                socket->tx.pendingLength += data2Send;

                if (socket->tx.remainingLength == 0)
                {
                    if (socket->tx.flush)
                    {
                        tcp_output(pcb);
                    }

                    /* 
                    The current buffer has been completely sent.
                    We queue this socket for calling call_sendComplete() call-back function
                    later on:
                    */
                    interface->callbackPendingForSocket(*socket, NETWORK_SERVICE_CALLBACK_SEND_COMPLETE);
                }               
            }

            /* If tcp_write fails, we wait for another call to lwip_dataSent or a call to lwip_poll and try again: */
        }
        break;
    case TCP_CLIENT_SOCKET_STATE_DISCONNECTING_WAITING_FOR_SEND_COMPLETE:
        if (socket->tx.pendingLength == 0)
        {
            socket->state = TCP_CLIENT_SOCKET_STATE_SENDING_DISCONNECT_REQUEST;
            /* Falling through to the next case */
        }
        else
        {
            break;
        }
    case TCP_CLIENT_SOCKET_STATE_SENDING_DISCONNECT_REQUEST:
        /* Also, Lets see if we can send disconnect request now: */
        if (tcp_close(pcb) == ERR_OK)
        {
            /* pcb is already freed: */
            pcb = NULL;
            socket->close();
            interface->deleteSocket(socket, TRUE);
        }
        else
        {
            /* We wait for a poll or TCP sent indication from the stack and try again: */              
        }
        break;
    default:
        break;
    }

    return ERR_OK;
}

void TcpClientSocket::call_connectionComplete()
{
    assert(state == TCP_CLIENT_SOCKET_STATE_CONNECTED);

    ClxNetworkServiceIp4Address remoteAddress;
    CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS(remoteAddress, pcb->remote_ip.addr, pcb->remote_port); 

    service().connectionComplete(&service(), this, &remoteAddress.base);
}

void TcpClientSocket::call_newConnection()
{
    assert(state == TCP_CLIENT_SOCKET_STATE_CONNECTED);

    ClxNetworkServiceIp4Address localAddress;
    CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS(localAddress, pcb->local_ip.addr, pcb->local_port); 

    ClxNetworkServiceIp4Address remoteAddress;
    CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS(remoteAddress, pcb->remote_ip.addr, pcb->remote_port); 

    service().newConnection(&service(), this, &localAddress.base, &remoteAddress.base);
}

void TcpClientSocket::call_sendComplete()
{
    assert(tx.buffer);
    assert(tx.remainingLength == 0);

    const u1* sentBuf = tx.buffer;

    /* 
    We set tx.buffer to NULL before calling call_sendComplete, in case the call-back
    wants to send more data:
    */
    tx.buffer = NULL;
        
    service().sendComplete(&service(),this, sentBuf);
}

void TcpClientSocket::call_dataReceived()
{
    assert(rx.current);

    service().dataReceived(&service(),this, (u1*)rx.current->payload, rx.current->tot_len, 0, NULL);
}

void TcpClientSocket::call_connectionIdle()
{
    u4 idleTime = clxTickTime() - lastActivityTime;

    service().connectionIdle(&service(),this, idleTime);
}







/****************************
UdpSocket
****************************/
void UdpSocket::call_connectionComplete()
{
  
}

void UdpSocket::call_newConnection()
{
  
}

void UdpSocket::call_sendComplete()
{
  
}

void UdpSocket::call_dataReceived()
{ 
    /* forward received data to the service e.g. DHCP server services */
    service().dataReceived(&service(),this, (u1*)rx.current->payload, rx.current->tot_len, 0, NULL);
      
    if (rx.current)
    {
        /* Now we can release this buffer: */
        pbuf_free(rx.current);
        rx.current = NULL;

        /* Lets see if there are more buffers pending: */
        pbuf* recvBuf = rx.pop();

        if (recvBuf)
        {
            rx.current = recvBuf;

            /* 
            We queue this socket for calling call_dataReceived() call-back function
            later on:
            */
            interface->callbackPendingForSocket(*this, NETWORK_SERVICE_CALLBACK_DATA_RECEIVED);
        }      
    }
}

void UdpSocket::call_connectionIdle()
{
  
}

void UdpSocket::close()
{
  
}

ClxResult UdpSocket::terminate()
{
  
}


/*   Before calling UdpSocket::send you need to use a ClxNetworkServiceIp4Address together 
with  CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS  
and pass in the base structure ClxNetworkServiceAddress                                    */

ClxResult UdpSocket::send(const u1* data, u4 dataLength, u4 flags, const struct ClxNetworkServiceAddress* remoteAddr)
{
  ClxResult result = CLX_FAIL;
  
  ClxNetworkServiceIp4Address* remoteAddr_ = (ClxNetworkServiceIp4Address*)remoteAddr;
  
  ip_addr ipaddr;
  ipaddr.addr = remoteAddr_->addr;
  
  struct pbuf* p1 = pbuf_alloc(PBUF_TRANSPORT,dataLength,PBUF_RAM);

  memcpy(p1->payload, data, dataLength); 

  //result = udp_sendto(pcb, p1, IP_ADDR_BROADCAST, CLX_DHCP_CLIENT_PORT); //for DHCP
  result = udp_sendto(pcb, p1, &ipaddr, remoteAddr_->port);  

  pbuf_free(p1); 
  
  return result;   
}


ClxResult UdpSocket::recvComplete()
{
  
}


void  UdpSocket::lwip_udp_recv(void *arg, struct udp_pcb *pcb_lwip, struct pbuf *p, struct ip_addr *addr, u2 port)
{
    UdpSocket* socket = (UdpSocket*)arg;
    socket->pcb = pcb_lwip;
    
    if (p)
    {
        if(!socket->rx.current)
        {
            socket->rx.current = p;

            /* 
            The service is not processing any RX data.
            We queue this socket for calling call_dataReceived() call-back function
            later on:
            */
            interface->callbackPendingForSocket(*socket, NETWORK_SERVICE_CALLBACK_DATA_RECEIVED);
        }
        else
        {
            socket->rx.push(p);
        }
    }
    else
    {
        socket->close();
        interface->deleteSocket(socket, TRUE);
    }
}



ClxResult UdpSocket::bind(UdpSocket& socket, const ClxNetworkServiceAddress* localAddr)
{
    struct udp_pcb* tempPCB;

    ClxResult ret = interface->bind_udp(localAddr, &tempPCB);
    if (ret != CLX_SUCCESS)
    {
        return ret;
    }
    else
    {
      /*setup the lwip call back function for receiving udp packets */
      
      udp_recv(tempPCB, lwip_udp_recv, this);
      
    }
    
    return CLX_SUCCESS;   
}



/*************************************************
clxNetworkStackServiceInterface_xxxx API functions
*************************************************/
void clxNetworkStackServiceInterface_Create(u4 numberOfSockets)
{
    ClxLwipStackServiceInterface::instance = new ClxLwipStackServiceInterface(numberOfSockets);
}

void clxNetworkStackServiceInterface_Destroy()
{
    delete ClxLwipStackServiceInterface::instance;
    ClxLwipStackServiceInterface::instance = NULL;
}

ClxResult clxNetworkStackServiceInterface_Open(struct ClxNetworkService* service,
                                               u1 socketType,
                                               const struct ClxNetworkServiceAddress* localAddr,
                                               ClxNetworkSocketHandle** handle)
{
    switch(socketType)
    {
    case CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP:
        // TODO
        break;
    case CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP_LISTENER:
        {
            if (localAddr == NULL)
            {
                return CLX_SOCKET_INVALID_ADDR;
            }

            if (!clxIsPrivateMemoryPoolBufferAvailable(interface->socketPool, sizeof(TcpListenerSocket)))
            {
                return CLX_SOCKET_ENOMEM;
            }

            TcpListenerSocket* newSocket = new (interface->socketPool) TcpListenerSocket(*service);

            ClxResult ret = newSocket->listen(*newSocket, localAddr);

            if (ret == CLX_SUCCESS)
            {
                *handle = newSocket;
            }
            else
            {
                delete newSocket;
            }

            return ret;
        }
        
    case CLX_NETWORK_SOCKET_HANDLE_TYPE_UDP:
        {
            if (localAddr == NULL)
            {
                return CLX_SOCKET_INVALID_ADDR;
            }

            if (!clxIsPrivateMemoryPoolBufferAvailable(interface->socketPool, sizeof(UdpSocket)))
            {
                return CLX_SOCKET_ENOMEM;
            }

            UdpSocket* newSocket = new (interface->socketPool) UdpSocket(*service);

            ClxResult ret = newSocket->bind(*newSocket, localAddr);

            if (ret == CLX_SUCCESS)
            {
                *handle = newSocket;
            }
            else
            {
                delete newSocket;
            }

            return ret;
        }      
      
        
    default:
        // TODO
        break;
    }

    return CLX_SOCKET_ESOCKTNOSUPPORT;
}


ClxResult clxNetworkStackServiceInterface_Connect(struct ClxNetworkSocketHandle* handle,
                                                  const struct ClxNetworkServiceAddress* remoteAddr)
{
    // TODO
    return CLX_SOCKET_ESOCKTNOSUPPORT;
}

ClxResult clxNetworkStackServiceInterface_Terminate(struct ClxNetworkSocketHandle* handle)
{
    Socket* socket = static_cast<Socket*>(handle);

    if (socket->isValidHandle())
    {
        ClxResult ret = socket->terminate();

        if (ret == CLX_SUCCESS)
        {
            interface->deleteSocket(socket, FALSE);
        }
        
        return ret;
    }
    else
    {
        return CLX_SOCKET_EBADF;
    }
}

void clxNetworkStackServiceInterface_Close(struct ClxNetworkSocketHandle* handle)
{
    Socket* socket = static_cast<Socket*>(handle);

    if (socket->isValidHandle())
    {
        socket->close();
        interface->deleteSocket(socket, FALSE);
    }
}

ClxResult clxNetworkStackServiceInterface_Send(struct ClxNetworkSocketHandle* handle, 
                                               const u1* buf,
                                               size_t length,
                                               u4 flags,
                                               const struct ClxNetworkServiceAddress* remoteAddr)
{
    Socket* socket = static_cast<Socket*>(handle);

    if (!socket->isValidHandle())
    {
        return CLX_SOCKET_EBADF;
    }

    return socket->send(buf, length, flags, remoteAddr);
}

ClxResult clxNetworkStackServiceInterface_RecvComplete(struct ClxNetworkSocketHandle* handle)
{
    Socket* socket = static_cast<Socket*>(handle);

    if (!socket->isValidHandle())
    {
        return CLX_SOCKET_EBADF;
    }

    return socket->recvComplete();
}



/* Run the pending callbacks for one socket only: */
boolean clxNetworkStackServiceInterface_RunPendingCallbacks()
{
    Socket* socket = interface->pendingSocketsQueue.popFromFront();
    
    if (socket)
    {
        assert(socket->pendingCallbacks);

        /* 
        IMPORTANT : the socket is already out of the pending queue. A call-back function
        is allowed to close this socket and even re-use the same descriptor for a new socket.
        Also, a call-back function may result in the socket pending again for call-backs:
        */
        if (socket->pendingCallbacks & NETWORK_SERVICE_CALLBACK_CONNECTION_COMPLETE)
        {
            socket->pendingCallbacks &= (~NETWORK_SERVICE_CALLBACK_CONNECTION_COMPLETE);
            socket->call_connectionComplete();
        }

        /* Last callback might have already deleted this socket: */
        if ((socket->isValidHandle()) && (socket->pendingCallbacks & NETWORK_SERVICE_CALLBACK_NEW_CONNECTION))
        {
            socket->pendingCallbacks &= (~NETWORK_SERVICE_CALLBACK_NEW_CONNECTION);
            socket->call_newConnection();
        }
        
        /* Last callback might have already deleted this socket: */
        if ((socket->isValidHandle()) && (socket->pendingCallbacks & NETWORK_SERVICE_CALLBACK_SEND_COMPLETE))
        {
            socket->pendingCallbacks &= (~NETWORK_SERVICE_CALLBACK_SEND_COMPLETE);
            socket->call_sendComplete();
        }

        /* Last callback might have already deleted this socket: */
        if ((socket->isValidHandle()) && (socket->pendingCallbacks & NETWORK_SERVICE_CALLBACK_DATA_RECEIVED))
        {
            socket->pendingCallbacks &= (~NETWORK_SERVICE_CALLBACK_DATA_RECEIVED);
            socket->call_dataReceived();
        }

        /* Last callback might have already deleted this socket: */
        if ((socket->isValidHandle()) && (socket->pendingCallbacks & NETWORK_SERVICE_CALLBACK_CONNECTION_IDLE))
        {
            socket->pendingCallbacks &= (~NETWORK_SERVICE_CALLBACK_CONNECTION_IDLE);
            socket->call_connectionIdle();
        }
        
        /* If the queue is empty, we return FALSE, indicating that there is no more callback pending: */
        return (interface->pendingSocketsQueue.isEmpty() ? FALSE : TRUE);
    }

    return FALSE;
}


}  // extern "C"



