#ifndef _A2DP_API_H_
#define _A2DP_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                A2dp.Api.h
* Description         Declares ClarinoxBlue A2DP Profile API for both roles
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************************************************************//**
 * Used in ClxA2dpStreamCallbackFunc as an input argument, user can check it inside the call back function, to be aware of 
 * the reason of the call.
 ******************************************************************************************************************************/
enum ClxStreamCallbackReason
{
    ClxStreamCallbackReason_Init            = 0,
    ClxStreamCallbackReason_ProvideData     = 1,
    ClxStreamCallbackReason_DataReceived    = 2,
    ClxStreamCallbackReason_StreamPaused    = 3,
    ClxStreamCallbackReason_Terminate       = 4
};


/*!
 * Refer to #clxA2dpAudioStreamSetup function.
 */
#define CLX_A2DP_AUDIO_STREAM_SETUP_COMPLETE                         0x4F00

/*!
 * Refer to #clxA2dpPlay function.
 */
#define CLX_A2DP_PLAY_COMPLETE                                       0x4F01

/*!
 * Refer to #clxA2dpPause function.
 */
#define CLX_A2DP_PAUSE_COMPLETE                                      0x4F02

/*!
 * Refer to #clxA2dpStop function.
 */
#define CLX_A2DP_STOP_COMPLETE                                       0x4F03



/***************************************************************************************************************************//**
* This indication is received when a service Level connection is made to a remote A2DP service.
* This indication is only received when the service connection and audio connection request has been initiated by the
* remote device. The parameter of this indication (params argument of the call-back function) will be NULL.
******************************************************************************************************************************/
#define CLX_A2DP_AUDIO_STREAM_SETUP_INDICATION                       0x8F00

/*!
 * This indication is received when a start request received from a remote device.
 */
#define CLX_A2DP_START_INDICATION                                    0x8F01

/*!
 * This indication is received when a suspend request received from a remote device.
  */
#define CLX_A2DP_SUSPEND_INDICATION                                  0x8F02

/*!
 *This indication is received when a stop request received from a remote device.
 */
#define CLX_A2DP_STOP_INDICATION                                     0x8F03

/*!
 * This indication is received when an abort request received from a remote device.
 */
#define CLX_A2DP_ABORT_INDICATION                                    0x8F04

/*!
 * This indication is received when an AVDTP signaling disconnect happens.
 */
#define CLX_AVDTP_SIGNALING_DISCONNECT_INDICATION                    0x8F05

/***************************************************************************************************************************//**
 * Defines the role of the profile whether it is used as Source or Sink.
 ******************************************************************************************************************************/
enum ClxA2dpEndpointType
{
    ClxA2dpEndpointType_Source,                                     /*!< source role                                          */
    ClxA2dpEndpointType_Sink                                        /*!< sink role                                            */
};

/***************************************************************************************************************************//**
 * Input of the #clxA2dpAudioStreamSetup. Input of the #clxA2dpStop.
 ******************************************************************************************************************************/
typedef struct clxA2dpAudioSetupStopAbortInputStruct
{
    ClxDeviceId remoteDeviceId;
}clxA2dpAudioSetupStopAbortInput;

/***************************************************************************************************************************//**
 * List of the Capabilities or codec configuration.
 ******************************************************************************************************************************/
struct ClxA2dpSbcConfig
{
    u1 samplingFrequency;                                           /*!< sampling frequency                                   */
    u1 channelMode;                                                 /*!< channel mode                                         */
    u1 blockLength;                                                 /*!< block length                                         */
    u1 subbands;
    u1 allocationMethod;
    u1 minBitpool;
    u1 maxBitpool;
};

/***************************************************************************************************************************//**
* \brief Definition of the streaming call-back function, that has been used for both sink and source functions.
* e.g. If the application is going to act as source, source call-back function shall be defined in the application, 
* with the same prototype,
* and the pointer of it shall be passed to clxA2dpCreate function, while creating the A2dpHandle
******************************************************************************************************************************/
typedef ClxSize (*ClxA2dpStreamCallbackFunc) (enum ClxStreamCallbackReason reason, ClxA2dpSbcConfig* sbcConfig, void* data, ClxSize dataLen, void** userData);

/***************************************************************************************************************************//** 
 * \brief Callback function prototype is used by application to display selected capabilities of the remote device to the user 
 * \param[ in ] config A pointer to the configuration 
 ******************************************************************************************************************************/
typedef boolean (*ClxA2dpCapabilitiesCallbackFunc) (ClxA2dpSbcConfig* config);


/****************************************************************************************************************************//**
* \brief Creates an  A2dp service on the local machine.
* \param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
* \param[ in ] endpointType type of the endpoint is going to be created.
* \param[ in ] sbcConfig A reference to the capabilities to be set for the local device.
* \param[ in ] streamCallbackFunc A pointer to the call-back function which will be called upon a streaming request.
* \param[ in ] indicationCallbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
* \param[ in ] capabilitiesCallbackFunc A pointer to the call-back function which will be called to display the local codec configuration.
* \return ClxHandle A2dp handle. This handle must be used for any further operations on this instance of A2dp profile.
*********************************************************************************************************************************/
ClxHandle clxA2dpCreate(_in_ ClxStack                        stack,
                        _in_ ClxA2dpEndpointType             endpointType,
                        _in_ ClxA2dpSbcConfig&               sbcConfig,
                        _in_ ClxA2dpStreamCallbackFunc       streamCallbackFunc,
                        _in_ ClxApplicationCallbackFunc      indicationCallbackFunc,
                        _in_ ClxA2dpCapabilitiesCallbackFunc capabilitiesCallbackFunc);

/****************************************************************************************************************************//**
 * \brief This function is called to
 *   - perform a SDP query 
 *   - to get the AVDTP version on the remote device 
 *   - perform discovery, 
 *   - get capability 
 *   - set configuration.
 *
 * Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type 
 *                     CLX_A2DP_SETUP_COMPLETE. The indication does not have any parameters.
 *
 * \param[ in ]  a2dp handle.
 * \param[ in ]  remoteDeviceId ID of the remote Bluetooth device, this id must be obtained via GAP APIs.
 * \param[ in ]  block type of the operation.
 *               - TRUE:  API will be blocked until this command is completed (successfully or failed).
 *               - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
 * \return ClxError error code for the operation.
 *               - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
 *               - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
 *               - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
 *               - CLX_ERROR_INVALID_HANDLE: invalid handle.
 *               - CLX_SUCCESS: operation is successful
 *               - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
 *******************************************************************************************************************************/
ClxError clxA2dpAudioStreamSetup(_in_ ClxHandle   a2dp,
                                 _in_ ClxDeviceId remoteDeviceId,
                                 _in_ boolean     block);

/***************************************************************************************************************************//**
 * This function is called when PLAY music is requested. This function shall be called after #clxA2dpAudioStreamSetup().
 *
 * Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type CLX_A2DP_PLAY_COMPLETE.
 * The indication does not have any parameters.
 *
 * \param[ in ]  a2dp handle.
 * \param[ in ]  block type of the operation.
 *               - TRUE:  API will be blocked until this command is completed (successfully or failed).
 *               - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
 * \return ClxError error code for the operation.
 *               - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
 *               - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
 *               - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
 *               - CLX_ERROR_INVALID_HANDLE: invalid handle.
 *              In case of blocking mode the following error codes may also be returned.
 *               - CLX_SUCCESS: operation is successful
 *               - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
 ******************************************************************************************************************************/
ClxError clxA2dpPlay(_in_ ClxHandle a2dp,
                     _in_ boolean   block);

/***************************************************************************************************************************//**
 * This function is called when Music needs to be disabled temporarily. It performs PAUSE activity and suspends music.
 *
 * Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
 *                     type CLX_A2DP_PAUSE_COMPLETE. The indication does not have any parameters.
 *
 * \param[ in ]  a2dp handle.
 * \param[ in ]  block type of the operation.
 *               - TRUE:  API will be blocked until this command is completed (successfully or failed).
 *               - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
 * \return       ClxError error code for the operation.
 *               - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
 *               - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
 *               - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
 *               - CLX_ERROR_INVALID_HANDLE: invalid handle.
 *               In case of blocking mode the following error codes may also be returned.
 *               - CLX_SUCCESS: operation is successful
 ******************************************************************************************************************************/
ClxError clxA2dpPause(_in_ ClxHandle a2dp,
                      _in_ boolean   block);

/***************************************************************************************************************************//**
 * \brief This function is called when a STOP request is received to stop the music.
 *
 * Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type CLX_A2DP_STOP_COMPLETE.
 * The indication does not have any parameters.
 *
 * \param[ in ]  a2dp handle.
 * \param[ in ]  remoteDeviceId ID of the remote Bluetooth device, this id must be obtained via GAP APIs.
 * \param[ in ]  block type of the operation.
 *               - TRUE:  API will be blocked until this command is completed (successfully or failed).
 *               - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
 * \return       ClxError error code for the operation.
 *                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode).
 *                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode).
 *                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
 *                - CLX_ERROR_INVALID_HANDLE: invalid handle.
 *
 *               In case of blocking mode the following error codes may also be returned.
 *                - CLX_SUCCESS: operation is successful
 ******************************************************************************************************************************/
ClxError clxA2dpStop(_in_ ClxHandle   a2dp,
                     _in_ ClxDeviceId remoteDeviceId,
                     _in_ boolean     block);

/***************************************************************************************************************************//**
 * \brief It performs AVDTP abort query, and it disconnects the L2cap streaming and signaling connection.
 *
 * Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type CLX_A2DP_STOP_COMPLETE.
 *The indication does not have any parameters.
 *
 * \param[ in ]  a2dp handle.
 * \param[ in ]  remoteDeviceId ID of the remote Bluetooth device, this id must be obtained via GAP APIs.
 * \param[ in ]  block type of the operation.
 *               - TRUE:  API will be blocked until this command is completed (successfully or failed).
 *               - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
 * \return       ClxError error code for the operation.
 *               - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode).
 *               - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
 *               - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
 *               - CLX_ERROR_INVALID_HANDLE: invalid handle.
 *               In case of blocking mode the following error codes may also be returned.
 *               - CLX_SUCCESS: operation is successful
 ****************************************************************************************************************************/
ClxError clxA2dpAbort(_in_ ClxHandle   a2dp,
                      _in_ ClxDeviceId remoteDeviceId,
                      _in_ boolean     block);

#ifdef __cplusplus
}
#endif


#endif // _A2DP_API_H_

