#ifndef _ClarinoxBlue_h_
#define _ClarinoxBlue_h_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                ClarinoxBlue..h
* Description         ClarinoxBlue Bluetooth Protocol Stack common include file
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClarinoxBlueConst.h"
#include "ClarinoxBlueErrorCodes.h"

/**
External reference for initialization and release of Bluetooth configuration parameters
*/
extern "C" ClxConfigList*   initializeClarinoxBlueBspConfigParameters();
extern "C" void             releaseClarinoxBlueBspConfigParameters(ClxConfigList* configList);

#ifdef __cplusplus
extern "C" {
#endif

/**
Initiates ClarinoxBlue stack. When this function is used in non-blocking mode returns immediately. Upon initialization of the stack is complete, 
an indication of type CLX_GAP_INITIALIZATION_COMPLETE_INDICATION will be sent to the GAP call-back function. 
The errorCode parameter of the call-back function must be checked, to see if the stack has been initialized successfully. 

Below shows error handling when a non-blocking initialization call is made;;
\code
if (clxInitClarinoxBlue(&params, FALSE, stackMessageHandler, NULL, &stack, FALSE) != CLX_ERROR_COMPLETION_PENDING)
{
       ClxConsoleUIEngine::text ("Initialization of ClarinoxBlue failed\n");
       return 0;
}
\endcode


If this function is used in blocking mode, then it returns upon successful completion of the Bluetooth stack.
\code
if (clxInitClarinoxBlue(&params, FALSE, stackMessageHandler, NULL, &stack, TRUE) != CLX_SUCCESS)
{
       ClxConsoleUIEngine::text ("Initialization of ClarinoxBlue failed\n");
       return 0;
}
\endcode


\param[ in ] parameterConfigList The global parameters of the stack. If a configuration parameter does not exist in the list,
                                 the default value will be used for that parameter. This argument may be NULL. In this case, the default value will be used for all parameters.
\param[ in ] callBacksInUserThread The presence of a call back function in user defined thread. Takes Boolean expression TRUE/FALSE
\param[ in ] gapCallbackFunc a pointer to the GAP call-back functions which will receive GAP-related indications. This parameter CANNOT be NULL.
\param[ in ] exceptionCallback a pointer to the exception handling function. Refer to remarks for more information. This argument can be NULL, in that case internal default handlers
                               will be used. It is strongly recommended that user assigns a handler for managing stack exceptions. 
                               The exception handler prototype is of type #ClxExceptionCallback. Handler provides moduleId and lineNumber to the call-back function.
                               This information would be useful if there is an unexpected exception while user application development.
\param[ out ] stack a pointer to clarinoxBlue stack object.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: initialization is in progress, operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful

\remark exceptionCallback is a user-defined function which is called when a fatal error occurs inside the
stack. The reason for the fatal error is a BLACKBOX which represents an error condition from which the stack cannot recover.
Upon reception of an exception, the user should:
- record the moduleID and line information passed to the callback function, and present it to Clarinox for investigation.
- restart the entire application without trying to terminate or destroy the stack instance.

The application must not continue execution when an exception has happened. The following example shows an exception handler;
\code
void userExceptionHandler( s4 moduleId, s4 lineNumber )
{
    printf("ClarinoxBlue raised an exception in module: %u, line: %u\n", moduleId, lineNumber);
}
\endcode

A more complete example detailing initialization is shown below;
\code
    // configuration parameters that would override default parameter 
    struct GenericConfigParameters
    {
        ClxConfigInteger supportSecureSimplePairing;
        ClxConfigInteger enableLinkLevelAuthentication;
        ClxConfigInteger ioCapabilities;
        ClxConfigInteger linkRequestTimeout;
        ClxConfigInteger majorClassOfDevice;
        ClxConfigInteger minorClassOfDevice;
        ClxConfigInteger serviceClasses;
        ClxConfigString localDeviceName;
    };

    GenericConfigParameters genericConfigParameters;

    clxConfigInitIntegerParam(&genericConfigParameters.supportSecureSimplePairing,      "SupportSecureSimplePairing",       TRUE,             configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.enableLinkLevelAuthentication,   "EnableLinkLevelAuthentication",    FALSE,            configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.ioCapabilities,                  "IoCapabilities",                   IO_NONE,          configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.linkRequestTimeout,              "LinkRequestTimeout",               16000,            configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.majorClassOfDevice,              "MajorClassOfDevice",               MJ_AUDIO_VIDEO,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.minorClassOfDevice,              "MinorClassOfDevice",               AVMI_CAR_AUDIO,   configList); 
    clxConfigInitStringParam (&genericConfigParameters.localDeviceName,                 "LocalDeviceName",                  deviceName,       configList); 

    clxConfigInitIntegerParam(&genericConfigParameters.serviceClasses,                  "ServiceClasses",                   ( ST_LIMITED_DISCOVERABLE_MODE |
                                                                                                                              ST_POSITIONING               |
                                                                                                                              ST_NETWORKING                |
                                                                                                                              ST_RENDERING                 |
                                                                                                                              ST_CAPTURING                 |
                                                                                                                              ST_OBJECT_TRANSFER           |
                                                                                                                              ST_AUDIO                     |
                                                                                                                              ST_TELEPHONY                 |
                                                                                                                              ST_INFORMATION ),   configList); 

    if (clxInitClarinoxBlue(0,                    // Initialize the Bluetooth stack                
                            FALSE,
                            stackMessageHandler,
                            userExceptionHandler,
                            stack, 
                            TRUE ) != CLX_SUCCESS)
        {
            ClxConsoleUIEngine::text("Initialization of ClarinoxBlue failed\n");
            ret = FALSE;
        }
        else
        {
            ClxConsoleUIEngine::text("ClarinoxBlue version %s.%u \n",
                                        clxGetClarinoxBlueVersion(),
                                        clxGetClarinoxBlueRevision());
        }

\endcode



*/
ClxError clxInitClarinoxBlue(_user_in_ ClxConfigList* parameterConfigList,
                             _in_      boolean callBacksInUserThread,
                             _in_      ClxApplicationCallbackFunc gapCallbackFunc,
                             _in_      ClxExceptionCallback exceptionCallback,
                             _out_     ClxStack* stack,
                             _in_      boolean block);


/**
Initiates the first phase of the stack termination. The first phase includes cleaning up the stack by disconnecting from the remote services.
This function is non-blocking and returns immediately. When the first phase is complete, an indication of type CLX_GAP_TERMINATION_COMPLETE_INDICATION
will be received by the GAP call-back function. At that moment, the second phase of ClarinoxBlue termination must be performed by calling #clxDestroyClarinoxBlue.

IMPORTANT : All open handles belonging to this instance of stack MUST BE CLOSED before this function is called. Otherwise, the behaviour will be undefined, and the stack
may even crash.

Below example demonstrates how to terminate stack;

\code
// This code must be executed in the context of a user_thread
    clxCloseHandle(someSppHandle);                  // close all the handles for open profiles
    clxTerminateClarinoxBlue(stack);                // terminate ClarinoxBlue
    clxAcquireSemaphore(terminationSemaphore);      // wait for termination completion
    clxDeleteSemaphore(terminationSemaphore);       // semaphore is not required anymore
        
    clxDestroyClarinoxBlue(stack);                  // Destroy the stack, after this call is completed, stack can be initialized again
    stack = 0;
    ClxConsoleUIEngine::destroy();                  // Whole SoftFrame is destroyed, after this point only user applications (threads) are active
\endcode

To be able to achieve this code the stack call-back handler must implement 

\code
// stackMessageHandler must be assigned as stack call-back handler
boolean stackMessageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
{
    if(messageID == CLX_GAP_TERMINATION_COMPLETE_INDICATION)    // indication of stack termination completion
    {                                                           // now ready to destroy the stack, so synchronize with the user thread             
        if (errorCode == 0)
        { 
            ClxConsoleUIEngine::text("ClarinoxBlue termination completed\n");
            clxReleaseSemaphore (terminationSemaphore);         // synchronize with the user thread for destruction
        }
        else
        {
            ClxConsoleUIEngine::text("\nStack termination was not successful (error : %s)", clxGetErrorCodeText(errorCode));
        }
    }   
    return TRUE;
}
\endcode

\param[ in ] clarinoxBlue the ClarinoxBlue stack object.
*/
void clxTerminateClarinoxBlue(_in_ ClxStack clarinoxBlue);

/**
Performs the second (last) phase of ClarinoxBlue termination by deleting the objects, threads, and timers. It also releases all the memory used by the stack.
This function MUST ONLY be called when the first phase of stack termination is complete (an indication of type CLX_GAP_TERMINATION_COMPLETE_INDICATION has been
received). When this function returns, the stack is completely terminated.

IMPORTANT : Note that this function CANNOT be called within the context of the GAP call-back function (or any other service call-back functions) thread.
This is due to the fact that the call-back thread must also be terminated and cleaned up by this function. The developer must devise a mechanism in order to 
call this function outside of the call-back function.
\see clxTerminateClarinoxBlue for an example code snippet for termination and destruction of the Bluetooth stack

\param[ in ] clarinoxBlue the ClarinoxBlue stack object.
*/
void clxDestroyClarinoxBlue(_in_ ClxStack clarinoxBlue);


/**
Sends a vendor specific command to the Bluetooth controller. THIS FUNCTION MUST ONLY BE CALLED INSIDE the function clxHciSendVendorSpecificCommands().
Refer to the BSP implementation file for your target platform for more information on how to send vendor specific commands to the chipset. 

\param[ in ] ogf The OGF of the command. For vendor specific commands, this must be set to 0x3F.
\param[ in ] ocf The OCF of the command as provided in vendor's/manufacturer's documentations.
\param[ in ] data Pointer to data section of the command with the size "length". Can be NULL if the command has no data section.
\param[ in ] length The length of the data section of the command. Must be set to 0 if "data" is NULL. Otherwise, must NOT be 0.
\param[ out ] eventPacket Pointer to a pointer variable which will be set to the response event packet. The buffer containing the response
packet will be allocated by the stack and MUST NOT be modified or deleted by the user. The response buffer will remain valid until #clxSendVendorSpecificCommand is called again.
\param[ in ] responseTimeout The maximum time (in milliseconds) which the function will block waiting for a response to the command before it returns with an error. Set to 0 to indicate
an infinite time.

\return CLX_ERROR_INVALID_COMMAND_ARGUMENT the argument eventPacket is NULL.
        CLX_ERROR_COMMAND_NOT_COMPLETE a previous #clxSendVendorSpecificCommand function call has not returned yet.
        CLX_ERROR_TIMEOUT_OCCURRED Timeout occurred before a response to the command was received.
        CLX_SUCCESS The command was successfully sent to the Bluetooth controller and a response was received with a pointer to the response buffer
        being returned as the argument eventPacket.

\remark The function can be used to send any command (vendor specific or standard) to the Bluetooth controller. But the function should only be used to
send a vendor specific command. The only standard command which this function may be used to send should be RESET command. The user should avoid sending
any standard command other than RESET command since it might conflict with the commands the stack sends, which may result in the stack not working properly
or even crashing.

Calling this function outside of clxHciSendVendorSpecificCommands() function is strictly prohibited as it will result in undefined behaviour.
*/
ClxError clxSendVendorSpecificCommand(  _in_ u2 ogf,
                                        _in_ u2 ocf, 
                                        _in_ const /*@null@*/ u1* data, 
                                        _in_ u4 length, 
                                        _out_ u1** eventPacket,
                                        _in_ u4 responseTimeout );


/**
Returns a text string containing the name of the error code. This function only returns the string in debug mode.
In release mode, the return string is always an empty string.

\param[ in ] errorCode The error code for which a string representation must be returned.

\return A stack-allocated string containing the string representation of the error code. This string MUST not be modified or deleted.
*/
const s1* clxGetErrorCodeText(ClxError errorCode);

/**
Returns the ClarinoxBlue version.
\return A function-allocated buffer containing the version as a null-terminated string. The buffer
must NOT be modified by the caller.
*/
const s1* clxGetClarinoxBlueVersion(void);

/**
Returns the ClarinoxBlue repository revision.
\return the repository revision as an unsigned integer number. 
*/
u4 clxGetClarinoxBlueRevision(void);


#ifdef __cplusplus
}
#endif


#endif // _ClarinoxBlue_h_

