#ifndef _FTC_API_H_
#define _FTC_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Ftc.Api.h
* Description         Declares ClarinoxBlue FTP Client API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "Ftp.Common.Api.h"

#ifdef __cplusplus
extern "C" {
#endif

/** 
COMMAND COMPLETE MESSAGES 
*/

/**
Refer to #clxFtpClientConnect function.
*/
#define CLX_FTP_CLIENT_CONNECT_COMPLETE                        0x5100  

/**
Refer to #clxFtpClientDisconnect function.
*/
#define CLX_FTP_CLIENT_DISCONNECT_COMPLETE                     0x5101

/**
Refer to #clxFtpClientSetPath function.
*/
#define CLX_FTP_CLIENT_SETPATH_COMPLETE                        0x5102  

/**
Refer to #clxFtpClientPushFile function.
*/
#define CLX_FTP_CLIENT_PUSH_OBJECT_COMPLETE                    0x5103  

/**
Refer to #clxFtpClientPullFile function.
*/
#define CLX_FTP_CLIENT_PULL_OBJECT_COMPLETE                    0x5104  

/**
Refer to #clxFtpClientDeleteObject function.
*/
#define CLX_FTP_CLIENT_DELETE_OBJECT_COMPLETE                  0x5105  

/**
Refer to clxFtpClientCopyObject function.
*/
#define CLX_FTP_CLIENT_COPY_OBJECT_COMPLETE                    0x5106  
        
/**
Refer to clxFtpClientRenameObject function.
*/
#define CLX_FTP_CLIENT_RENAME_OBJECT_COMPLETE                  0x5107  

/**
Refer to clxFtpClientMoveObject function.
*/
#define CLX_FTP_CLIENT_MOVE_OBJECT_COMPLETE                    0x5108  

/**
Refer to clxFtpClientSetPermissions function.
*/
#define    CLX_FTP_CLIENT_SET_PERMISSIONS_COMPLETE             0x5109  

/**
Refer to #clxFtcGetCurrentFolderListing function.
*/
#define CLX_FTP_CLIENT_GET_CURRENT_FOLDER_LISTING_COMPLETE     0x510A

/**
Refer to #clxFtpClientAbort function.
*/
#define CLX_FTP_CLIENT_ABORT_COMPLETE                          0x510B


/* INDICATIONS*/


/**
This indication is received when a service is disconnected due to internal event. The parameter of this indication (params argument of the call-back
function) will be NULL.
*/
#define    CLX_FTP_CLIENT_CONNECTION_FAILED_INDICATION         0x9100  


/**
Specifies the reason why the data call-back function has been called. 
*/
typedef enum ClxFtpClientCallBackReasonEnum
{
     ClxFtpClient_DataReceived,
     ClxFtpClient_DataNeeded
}ClxFtpClientCallBackReason;

/**
This is a callback function used while transferring data between client and server.
*/
typedef void (*ClxFtpClientDataTransferCallbackFunc) (void*                      objectID,
                                                      ClxFtpClientCallBackReason reason,
                                                      const u1*                  dataReceived,
                                                      u1**                       bufferToSend,
                                                      u4*                        len);

/**
Creates a File Transfer service (Client role) on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] indicationCallbackFunc Callback function used by application to handle indications
\param[ in ] dataTransferCallbackFunc Callback function used by application to send/receive data to/from remote device.

\return ClxHandle File Transfer Profile (Client) handle. This handle must be used for any further operations on this instance of Hands free profile.
*/
ClxHandle clxFtpClientCreate(_in_ ClxStack                             stack,
                             _in_ ClxApplicationCallbackFunc           indicationCallbackFunc,
                             _in_ ClxFtpClientDataTransferCallbackFunc dataTransferCallbackFunc);

/**
Connect an instance of FTP client profile to a remote FTP (Server) service. 
Pairing with the remote device may be completed before connecting to the device. 
If there are no pairing at the time of this API, the authentication process will be executed during this API. 
See GAP API for the details of pairing.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP/Client profile handle.
\param[ in ]  remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_NOT_SUPPORTED_BY_REMOTE_MACHINE : the remote machine does not support FTP server role (or it not available at the moment).
                - CLX_ERROR_CONNECTION_FAILED : operation is unsuccessful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_ERROR_OBEX_BAD_REQUEST : the FTP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_UNAUTHERIZED : the local machine is not authorized to access the remote FTP server.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote FTP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the FTP server has rejected the request as forbidden.
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxFtpClientConnect( _in_ ClxHandle    ftc,
                              _in_ ClxDeviceId  remoteDeviceId, 
                              _in_ boolean      block);

/**
Disconnect an instance of FTP client profile to a remote FTP (Server) service. This function is initiated by client.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_DISCONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP/Client profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: An operation is in progress. The operation must be aborted before
                  the channel can be disconnected.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the FTP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote FTP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the FTP server has rejected the request as forbidden.
*/
ClxError clxFtpClientDisconnect( _in_ ClxHandle ftc,
                                 _in_ boolean   block);


/**
This function is called to SET the current folder to the subfolder of interest.This function is initiated by client.
This function also performs:

 - Setting the current folder back to the root folder(if the parent is root folder) 
 OR
 - If the parent of the sub-folder is not the root folder, then set the current folder to the parent folder.
 - Creates new folder in the Server's current folder by setting 'createIfNotExisting' = TRUE

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_SETPATH_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP/Client profile handle.
\param[ in ]  pathName pointer to the directory path.
\param[ in ] backUpALevel if TRUE the setpath will point to parent folder of the current folder else the setpath points to the current folder.
\param[ in ] createIfNotExisting if TRUE creates a folder same as in setpath(if folder does not exist) else returns error.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to the remote FTP server.
                - CLX_ERROR_CONNECTION_FAILED: connection to the FTP server failed during the operation.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another request is in progress.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the FTP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote FTP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the FTP server has rejected the request as forbidden.
                - CLX_ERROR_OBEX_NOT_FOUND : the requested folder was not found.
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before completion.
*/
ClxError clxFtpClientSetPath(_in_ ClxHandle ftc,
                             _in_ const s1* pathName,
                             _in_ boolean   backUpALevel,
                             _in_ boolean   createIfNotExisting,
                             _in_ boolean   block);
/**
This function is called to retrieve listing of the current folder in XML format. This function is initiated by client and only the names of files and folders is retrieved.
No name is sent since the folder path is set by #clxFtpClientSetPath function.

Precondition:
Folder/File path needs to set first using #clxFtpClientSetPath function.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_GET_CURRENT_FOLDER_LISTING_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP/Client profile handle.
\param[ in ]  userObjectID pointer to the variable which can be used to store object specific information and can be passed in the call back function. If this option is not used then value can be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to the remote FTP server.
                - CLX_ERROR_CONNECTION_FAILED: connection to the FTP server failed during the operation.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another request is in progress.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the FTP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote FTP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the FTP server has rejected the request as forbidden.
                - CLX_ERROR_OBEX_NOT_FOUND : the requested object was not found.
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before completion.
*/
ClxError clxFtcGetCurrentFolderListing(_in_ ClxHandle ftc,
                                       _in_ void*     userObjectID,
                                       _in_ boolean   block);

/**
This function is called to transfer a file from Client to the Server. 
The file is transferred to the current folder on the Server.

Pre-conditions:
Create a new folder(if it does not already exist or necessary) in the Server's current folder using #clxFtpClientSetPath function.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_PUSH_OBJECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP/Client profile handle.
\param[ in ]  fileName pointer to the name of the file to be transferred. This cannot be the name of a folder.
\param[ in ]  userFileID pointer to the variable which can be used to store file specific information and can be passed in the call back function. If this option is not used then value can be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to the remote FTP server.
                - CLX_ERROR_CONNECTION_FAILED: connection to the FTP server failed during the operation.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another request is in progress.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the FTP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote FTP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the FTP server has rejected the request as forbidden.
                - CLX_ERROR_OBEX_NOT_FOUND : the requested object was not found.
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before completion.
*/
ClxError clxFtpClientPushFile(_in_ ClxHandle  ftc,
                              _in_ const s1*  fileName,
                              _in_ void*      userFileID,
                              _in_ boolean    block);

/**
This function is called to transfer a file from Server to the Client. 
If the folder contains other folders, then these other files are transferred using the same method.
The file is transferred to the Client's current folder.

Pre-conditions:
Create a new folder(if it does not already exist or necessary) in the Client's current folder.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_PULL_OBJECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP/Client profile handle.
\param[ in ]  fileName pointer to the name of the File to be transferred.
\param[ in ]  userFileID pointer to the variable which can be used to store file specific information and can be passed in the call back function. If this option is not used then value can be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to the remote FTP server.
                - CLX_ERROR_CONNECTION_FAILED: connection to the FTP server failed during the operation.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another request is in progress.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the FTP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote FTP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the FTP server has rejected the request as forbidden.
                - CLX_ERROR_OBEX_NOT_FOUND : the requested object was not found.
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before completion.
*/
ClxError clxFtpClientPullFile(_in_ ClxHandle   ftc,
                              _in_ const s1*   fileName,
                              _in_ void*       userFileID,
                              _in_ boolean     block);

/**
This function is called by the Client to delete a folder/file on the Server. Following are the functions performed by delete function:
- Delete a file.
- Delete empty folder.
- A non-empty folder can be deleted in the same way as empty folder provided the Server allows this kind of operation. 
  If not supported a response code is received which tells to delete all elements in the folder individually before deleting the folder.

Precondition:
File/Folder path needs to set first using #clxFtpClientSetPath function.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_DELETE_OBJECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP/Client profile handle.
\param[ in ]  objectName pointer to the name of the Folder/File to be deleted.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to the remote FTP server.
                - CLX_ERROR_CONNECTION_FAILED: connection to the FTP server failed during the operation.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another request is in progress.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the FTP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote FTP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the FTP server has rejected the request as forbidden.
                - CLX_ERROR_OBEX_NOT_FOUND : the requested object was not found.
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before completion.
*/
ClxError clxFtpClientDeleteObject(_in_ ClxHandle ftc,
                                  _in_ const s1* objectName,
                                  _in_ boolean   block);

/**
Aborts an ongoing operation. If there is an any ongoing operation, this command will abort it.
Therefore, the respective command for the aborted operation (#clxFtpClientSetPath, #clxFtcGetCurrentFolderListing, #clxFtpClientPushFile,
#clxFtpClientPullFile, or #clxFtpClientDeleteObject) will be immediately complete with the error CLX_ERROR_OBEX_OPERATION_ABORTED.

NOTE : This command CANNOT be used to abort #clxFtpClientConnect or #clxFtpClientDisconnect command.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_FTP_CLIENT_ABORT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  ftc FTP client handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_COMMAND_NOT_COMPLETE the operation is cancelled by the user
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful (the ongoing operation has been aborted)
                - CLX_ERROR_COMMAND_UNSUCCESSFUL : there is no ongoing operation to be aborted
*/
ClxError clxFtpClientAbort(_in_ ClxHandle ftc,
                           _in_ boolean   block);




ClxError clxFtpClientCopyObject(_in_ ClxHandle  ftc,
                                _in_ const s1*  objectName,
                                _in_ boolean    block);

ClxError clxFtpClientRenameObject(_in_ ClxHandle ftc,
                                  _in_ const s1* objectName,
                                  _in_ boolean   block);

ClxError clxFtpClientMoveObject(_in_ ClxHandle ftc,
                                _in_ const s1* objectName,
                                _in_ boolean   block);

ClxError clxFtpClientSetPermissions(_in_ ClxHandle ftc,
                                    _in_ const s1* objectName,
                                    _in_ boolean   block);    


#ifdef __cplusplus
}
#endif


#endif // _FTC_API_H_
