#ifndef _FTS_API_H_
#define _FTS_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Fts.Api.h
* Description         Declares ClarinoxBlue FTP Server API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "Ftp.Common.Api.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
Callback function prototypes used by Clarinox Ftp Server to handle stack callback function calls. 
*/
typedef struct ClarinoxFtpServerCallbackFunctions
{
    /**
    This Callback function called when FTP connection request is received from FTP Client.
    This function does not return any parameters.
    */
    void (*obexConnected) (_in_ ClxHandle fts,
                           _in_ ClxDeviceId remoteDeviceId);

    /**
    This Callback function called when FTP disconnection request is received from FTP Client.
    This function does not return any parameters.
    */
    void (*obexDisconnected) (_in_ ClxHandle fts);

    /**
    This Callback function called when FTP Push File request is received from FTP Client.
    This function does not return any parameters.
    */
    void (*fileDataReceived) (_in_ ClxHandle fts,
                              _in_ void* userObjectID,
                              _in_ const u1* data, 
                              _in_ u4 dataLength);

    /**
    This Callback function called when FTP Pull File request is received from FTP Client.
    This function does not return any parameters.
    */
    void (*fileDataNeeded) (_in_ ClxHandle fts,
                            _in_ void* userObjectID);

    /**
    This Callback function called when FTP Client request is completed (esp. during file transfers).
    This function does not return any parameters.
    */
    void (*operationComplete) (_in_ ClxHandle fts,
                               _in_ void* userObjectID);

    /**
    This Callback function called when FTP Client request is aborted. This function does not return any parameters.
    */
    void (*operationAborted) (_in_ ClxHandle fts,
                              _in_ void* userObjectID);

    /**
    This Callback function called when FTP SetPath request is received from FTP Client.
    SetPath request is made either to set the current folder to the sub-folder or setting current folder back to the root folder.
    The callback function returns response of type #ClxObexResponse
    */
    ClxObexResponse (*setPathRequestReceived) (_in_ ClxHandle fts,
                                               _in_ const s1* pathUnicodeName,
                                               _in_ u4 pathLength,
                                               _in_ boolean backupOneLevel,
                                               _in_ boolean createFolder);

    /**
    This Callback function called when FTP Pull File request is received from FTP Client.
    The callback function returns response of type #ClxObexResponse
    */
    ClxObexResponse (*pullFileRequestReceived) (_in_ ClxHandle fts,
                                                _in_ const s1* objectName,
                                                _in_ u4 objectNameLength,
                                                _in_ const s1* type,
                                                _in_ u4 typeLength,
                                                _out_ void** userObjectID,
                                                _out_ u4* totalDataLength);

    /**
    This Callback function called when FTP Push File request is received from FTP Client.
    The callback function returns response of type #ClxObexResponse
    */
    ClxObexResponse (*pushFileRequestReceived)(_in_ ClxHandle fts,
                                               _in_ const s1* objectName,
                                               _in_ u4 objectNameLength,
                                               _in_ const s1* type,
                                               _in_ u4 typeLength,
                                               _in_ boolean deleteFile,
                                               _out_ void** userObjectID);

} ClarinoxFtpServerCallbackFunctions;

/**
Creates a File Transfer service (Server role) on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] ftpServerCallbackFunctions Callback function used by application to send/receive data to/from remote device.

\return ClxHandle File Transfer Profile (Client) handle. This handle must be used for any further operations on this instance of FTP profile.
*/
ClxHandle clxFtpServerCreate(_in_ ClxStack stack,
                             _in_ const ClarinoxFtpServerCallbackFunctions* ftpServerCallbackFunctions);

/**
This function is called when data needs to be sent to the FTP client. This function sends the data immediately.
This function MUST ONLY be called inside the call-back function fileDataNeeded.
\param[ in ] fts FTP/Server profile handle.
\param[ in ] data pointer to buffer containing data sent to client.
\param[ in ] dataLength length of data in the data buffer. Note: When data transmission is finished this function is called with dataLength = 0.

\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INVALID_REQUEST : there is no data request pending (this has not been called as a response to call-back function 'fileDataNeeded')
                - CLX_ERROR_CONNECTION_NOT_EXIST : connection does not exist
                - CLX_ERROR_INTERNAL_ERROR: An internal error has occurred
*/
ClxError clxFtpServerSendDataUnblock(_in_ ClxHandle fts,
                                     _in_ const u1* data,
                                     _in_ u4 dataLength);

/**
This function is called when requested data from FTP Client. This function is called when server is not ready for data transmission immediately.
This function CANNOT be called inside any of the server call-back functions. It must be called in the context of a user thread.

This function MUST ONLY be called as a response to the call-back function 'fileDataNeeded'.

\param[ in ] fts FTP/Server profile handle.
\param[ in ] data pointer to buffer containing data to be sent to client.
\param[ in ] dataLength length of data in the data buffer. Note: When data transmission is finished this function is called with dataLength = 0.

\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST : there is no data request pending (this has not been called as a response to call-back function fileDataNeeded 'fileDataNeeded')
                - CLX_ERROR_CONNECTION_NOT_EXIST : connection does not exist
                - CLX_ERROR_INTERNAL_ERROR: An internal error has occurred
*/
ClxError clxFtpServerSendDataBlock(_in_ ClxHandle fts,
                                   _in_ const u1* data,
                                   _in_ u4 dataLength);


#ifdef __cplusplus
}
#endif


#endif // _FTS_API_H_
