#ifndef _HSP_AG_API_H_
#define _HSP_AG_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                HspAg.Api.h
* Description         Declares ClarinoxBlue Headset Profile AudioGateway role API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Refer to #clxHspAgConnect function.
*/
#define CLX_HSP_AG_CONNECT_COMPLETE                                     0x4900

/**
Refer to #clxHspAgSetSpeakerGain function.
*/
#define CLX_HSP_AG_SET_SPEAKER_GAIN_COMPLETE                            0x4901

/**
Refer to #clxHspAgSetMicrophoneGain function.
*/
#define CLX_HSP_AG_SET_MICROPHONE_GAIN_COMPLETE                         0x4902

/**
Refer to #clxHspAgStartRinging function.
*/
#define CLX_HSP_AG_START_RINGING_COMPLETE                               0x4903

/**
Refer to #clxHspAgStopRinging function.
*/
#define CLX_HSP_AG_STOP_RINGING_COMPLETE                                0x4904

/**
Refer to #clxHspAgDisconnect function.
*/
#define CLX_HSP_AG_DISCONNECT_COMPLETE                                  0x4905

/**
Refer to #clxHspAgConnectAudio function.
*/
#define CLX_HSP_AG_CONNECT_AUDIO_COMPLETE                               0x4906

/**
Refer to #clxHspAgDisconnectAudio function.
*/
#define CLX_HSP_AG_DISCONNECT_AUDIO_COMPLETE                            0x4907

/**
This indication is received when a connection is made to a remote HSP service.
This indication is only received when the connection has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will be of type #ClxHspConnectionIndication.
*/
#define CLX_HSP_AG_CONNECTION_INDICATION                                0x8900

/**
This indication is received when the user of the remote headset device presses the
button on the device. The button can be pressed (and accordingly, this indication
can be received) at any time when there is a connection established to the remote
headset device. The local application is to react to this indication according to
the current state of the local device:

- If this indication is received when the local device is sending ringing signals
to the remote headset device, the local device must stop sending ringing signals
and initiate an audio connection to the remote headset device.

- If this indication is received when there is an audio connection established to
the remote device, the audio connection must be disconnected.

- In any other states, this indication may be ignored.

This indication does not have any parameter (params argument of the call-back function
will be NULL).
*/
#define CLX_HSP_AG_BUTTON_PRESS_INDICATION                              0x8901

/**
This indication is received when an already-established HSP connection to the remote
device fails unexpectedly (e.g. the remote headset device goes out of range, gets turned off, ...). 
This indication is only received when the disconnection procedure
has NOT been initiated by the local application.

Upon reception of this indication, the local application may: 

- try to reconnect to the same device.
- connect to a different device. 
- wait for a connection request from the same or a different remote headset device.
- inform the local user of the failure, and provide them with a list of options.
- close the HSP/AG handle by calling #clxHspAgDisconnect function.

This indication does not have any parameter (params argument of the call-back
function) will be of type #ClxHspConnectionIndication.
*/
#define CLX_HSP_AG_CONNECTION_FAILED_INDICATION                         0x8902

/**
This indication is received when the volume level of the speaker on the remote
headset device is changed.

The parameter of this indication is of type #ClxHspGainReport.
*/
#define CLX_HSP_AG_SPEAKER_GAIN_REPORT_INDICATION                       0x8903

/**
This indication is received when the volume level of the microphone on the remote
headset device is changed.

The parameter of this indication is of type #ClxHspGainReport.
*/
#define CLX_HSP_AG_MICROPHONE_GAIN_REPORT_INDICATION                    0x8904

/**
This indication is received when the an already-established audio connection to the
remote headset device fails unexpectedly. This indication is NOT received when the
audio channel disconnection procedure has been initiated by the local application.

Upon reception of this indication, the local application may try to reconnect the
audio connection, or disconnect the HSP data channel.

This indication does not have any parameter (params argument of the call-back function
will be NULL).
*/
#define CLX_HSP_AG_AUDIO_CONNECTION_FAILED_INDICATION                   0x8905

/**
Creates a Headset service (Audio Gateway role) on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] dualModeAudioChannelParams pointer to the list of audio channel parameters.
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\return ClxHandle Headset audio gateway profile handle. This handle must be used for any further operations on this instance of Headset profile.
*/
ClxHandle clxHspAgCreate(_in_ ClxStack stack,
                         _in_ const ClxBluetoothAudioChannelParams* dualModeAudioChannelParams,
                         _in_ ClxApplicationCallbackFunc callbackFunc);

/**
Connect an instance of Headset audio gateway profile to a remote headset service. Pairing with the remote server may be completed before connecting to the server. 
If there are no pairing at the time of this API, the authentication process will be executed during this API. See GAP API for the details of pairing.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.
\param[ in ]  remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHspAgConnect(_in_ ClxHandle hspAg,
                         _in_ ClxDeviceId remoteDeviceId, 
                         _in_ boolean block);

/**
Sends a request to the remote service to adjust speaker volume level.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_SET_SPEAKER_GAIN_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.
\param[ in ]  speakerLevel the level to which the remote headset speaker volume is to be adjusted, has to be 0-15, where 0 = Minimum gain and 15 = Maximum gain.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote headset service.
*/
ClxError clxHspAgSetSpeakerGain(_in_ ClxHandle hspAg,
                                _in_ u1 speakerLevel, 
                                _in_ boolean block);

/**
Sends a request to the remote service to adjust microphone volume level.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_SET_MICROPHONE_GAIN_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.
\param[ in ]  micLevel the level to which the remote headset microphone volume is to be adjusted, has to be 0-15, where 0 = Minimum gain and 15 = Maximum gain.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote headset service.
*/
ClxError clxHspAgSetMicrophoneGain(_in_ ClxHandle hspAg,
                                   _in_ u1        micLevel, 
                                   _in_ boolean   block);

/**
Starts sending ring signal to the remote headset. This function must be called as a result of an internal event indicating there is an incoming phone call. 
If this command is successful, the stack will keep on sending ring signals periodically (every 3 seconds) until the function #clxHspAgStopRinging is called.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_START_RINGING_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote headset service.
*/
ClxError clxHspAgStartRinging(_in_ ClxHandle hspAg,
                              _in_ boolean block);

/**
Stops sending ring signal to the remote headset. This function must only be called when the function #clxHspAgStartRinging has previously been called.
Typically, this function is called when one of the following events occurs:

    - The remote headset user responds to the ringing signals by pressing the button the headset. As a result of the button being
      pressed, an indication of type #CLX_HSP_AG_BUTTON_PRESS_INDICATION is sent to the call-back function.

    - An internal event indicates that the incoming phone call has been cancelled before the headset user has had a chance
      to respond to it.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_STOP_RINGING_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote headset service.
*/
ClxError clxHspAgStopRinging(_in_ ClxHandle hspAg,
                             _in_ boolean block);

/**
Disconnects from the remote headset service. If there is an audio connection established to the remote headset, it will be disconnected before
the data channel is disconnected.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_DISCONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote headset service.
*/
ClxError clxHspAgDisconnect(_in_ ClxHandle hspAg,
                            _in_ boolean block);

/**
Connects the audio channel to the remote headset service. An audio channel can be connected at any time when data channel is already
established.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_CONNECT_AUDIO_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.
\param[ in ] transmitBandwidth Transmit bandwidth (e.g. 8000 bits per second)
\param[ in ] receiveBandwidth Receive bandwidth (e.g. 8000 bits per second)
\param[ in ] maxLatency Maximum latency (e.g. 0xFFFF; best effort)
\param[ in ] voiceSetting Voice settings are described in the chapter 6.12 of core package version 2.1 + EDR specification.

A brief explanation of these bits are as follows:

2 LSB bits: air coding

- 00: CVSD
- 01: uLaw
- 10: aLaw
- 11: transparent

Next three bits:

bit positions that MSB of sample is away from starting at MSB  (only for linear PCM)

Next bit:

- 0: sample size 8 bit (for linearPCM)
- 1: sample size 16 bit (for linearPCM)

Next two bits: input data format

- 00: 1s complement
- 01: 2s complement
- 10: Sign-Magnitude
- 11: Unsigned

Next (MSB) two bits: input coding

- 00: linear
- 01: uLaw
- 10: aLaw
- 11: reserved

\param[ in ] retransmissionEffort Retransmission effort (e.g. 0xFF; don't care)
\param[ in ] packetType connection packet type. Can be a combination of packet types as defined in #ScoPacketTypeParameter.
Also, the special value #CLX_ALL_AVAILABLE_SCO_PACKETS can be used which allows the stack to automatically select all the
packet types which are supported by both the local Bluetooth controller and the remote Bluetooth controller.


\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote headset service.
                - CLX_ERROR_AUDIO_CONNECTION_IN_PROGRESS : Audio connection is already in progress.
                - CLX_ERROR_AUDIO_CONNECTION_EXISTS : An audio connection to the remote headset already exists.
                - CLX_ERROR_AUDIO_DISCONNECTION_IN_PROGRESS : Audio channel disconnection is in progress.
*/
ClxError clxHspAgConnectAudio(_in_ ClxHandle hspAg,
                              _in_ u4 transmitBandwidth, 
                              _in_ u4 receiveBandwidth, 
                              _in_ u2 maxLatency, 
                              _in_ u2 voiceSetting, 
                              _in_ u1 retransmissionEffort, 
                              _in_ u2 packetType,
                              _in_ boolean block);

/**
Disconnects the audio channel from the remote headset service. An already-established audio channel can be disconnected at any time when data channel is not disconnected
yet.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HSP_AG_DISCONNECT_AUDIO_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hspAg HSP/AG profile handle.

\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote headset service.
                - CLX_ERROR_AUDIO_CONNECTION_IN_PROGRESS : Audio connection is already in progress.
                - CLX_ERROR_AUDIO_CONNECTION_NOT_EXIST : An audio connection to the remote headset does not exist.
                - CLX_ERROR_AUDIO_DISCONNECTION_IN_PROGRESS : Audio channel disconnection is in progress.
*/
ClxError clxHspAgDisconnectAudio(_in_ ClxHandle hspAg,
                                 _in_ boolean block);


#ifdef __cplusplus
}
#endif

#endif // _HSP_AG_API_H_
