#ifndef __Ipsp_Api_h__
#define __Ipsp_Api_h__

/*******************************************************************************
*
* Project             Ipsp
* File                Ipsp.Api.h
* Description         Declares API Functions and Definitions For Ipsp
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Refer to #clxIpspStartInterface function.
*/
#define CLX_IPSP_START_INTERFACE_COMPLETE                                                   0x6100

/**
Refer to #clxIpspStoptInterface function.
*/
#define CLX_IPSP_STOPT_INTERFACE_COMPLETE                                                   0x6101

/**
Refer to #clxIpspConnectToRemoteNode function.
*/
#define CLX_IPSP_CONNECT_TO_REMOTE_NODE_COMPLETE                                            0x6102

/**
Refer to #clxIpspDisconnectLink function.
*/
#define CLX_IPSP_DISCONNECT_LINK_COMPLETE                                                   0x6103

/**
This indication is received when an IPSP channel is established between the local node and the remote router.
The parameter of this indication is of type #ClxIpspRouterConnectedIndication.
*/
#define CLX_IPSP_ROUTER_CONNECTED_INDICATION                                                0xa100

/**
This indication is received when the IPSP channel to a remote device is disconnected.
This indication may be received in any local role.
The parameter of this indication is of type #ClxIpspLinkDisconnectedIndication.
*/
#define CLX_IPSP_LINK_DISCONNECTED_INDICATION                                               0xa101

/**
Enumerates different roles of IPSP
*/
typedef enum ClxIpspRoleEnum
{
    ClxIpspRole_Node = 0,      /*!< Node role (only accepts a single connection from a router) */
    ClxIpspRole_Router = 1     /*!< Router role (Establish connections to one or more nodes in order to route IPv6 packets for them) */
} ClxIpspRole;

/**
Data Structure for the indication #CLX_IPSP_ROUTER_CONNECTED_INDICATION
*/
typedef struct ClxIpspRouterConnectedIndicationStruct
{
    u1    remoteDeviceAddress[CLX_BLUETOOTH_ADDRESS_LENGTH];        /*!< The address of the remote router to which an IPSP channel has been established */
} ClxIpspRouterConnectedIndication;

/**
Data Structure for the indication #CLX_IPSP_LINK_DISCONNECTED_INDICATION
*/
typedef struct ClxIpspLinkDisconnectedIndicationStruct
{
    u1    remoteDeviceAddress[CLX_BLUETOOTH_ADDRESS_LENGTH];        /*!< The address of the remote router */
} ClxIpspLinkDisconnectedIndication;


ClxHandle clxIpspCreate(_in_ ClxStack stack,
                        _in_ ClxIpspRole role,
                        _in_ struct ClxNetworkAccessInterface* networkInterface,
                        _in_ ClxApplicationCallbackFunc callbackFunc);



/**
                Starts the network interface associated to an Ipsp service. If this procedure is successful, the service will be ready to accept connections
                (in case of the node role), or initiate connections to remote nodes (in case of the router role).
                
                As part of this command, ClxNetworkAccessInterface.init() method of the associated network interface is called to initialize the interface.
                If this method returns an error, this command will fail with the same error code.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_IPSP_START_INTERFACE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] ipsp        Service handle
\param[  in   ] configList  Optional list of configuration parameters which will passed to ClxNetworkAccessInterface.init() method of the associated network
                            interface. May be NULL if no configuration parameter is required.
\param[  in   ] block       Indicates mode of operation:
                            TRUE : Blocking mode
                            FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_IPSP_INTERFACE_ALREADY_STARTED: The interface is already started
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - Any other error code: Initialization of the network interface has failed with this error code
*/
ClxResult clxIpspStartInterface(_in_ ClxHandle                    ipsp,
                                _user_in_ const ClxConfigList*    configList,
                                _in_ boolean                      block);

/**
                Stops the network interface associated to an Ipsp service. If this procedure is successful, any existing IPSP channels will be disconnected
                before this command is complete. When the interface is stopped, no more connections may be initiated or accepted until the interface is started again.
                
                As part of this command, ClxNetworkAccessInterface.destroy() method of the associated network interface is called to destroy the interface.
                
                IMPORTANT : Before the handle to a local service is closed (by a call to #clxCloseHandle), the network interface associated to the service
                shall be stopped by a call to this function (if it is already started).

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_IPSP_STOPT_INTERFACE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] ipsp   Service handle
\param[  in   ] block  Indicates mode of operation:
                       TRUE : Blocking mode
                       FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_IPSP_INTERFACE_NOT_STARTED: The interface is not started
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxIpspStoptInterface(_in_ ClxHandle    ipsp,
                                _in_ boolean      block);

/**
                Initiates a connection to a remote IPSP service with the Node role. The local IPSP service shall be in Router role.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_IPSP_CONNECT_TO_REMOTE_NODE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] ipsp                 Service handle
\param[  in   ] remoteDeviceAddress  The address of the remote node
\param[  in   ] block                Indicates mode of operation:
                                     TRUE : Blocking mode
                                     FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_IPSP_INTERFACE_NOT_STARTED: The interface is not started
                - CLX_ERROR_IPSP_NOT_SUPPORTED_BY_LOCAL_ROLE: The local service is not in Router role
                - CLX_ERROR_CONNECTION_FAILED: operation has failed
                - CLX_ERROR_NOT_SUPPORTED_BY_REMOTE_MACHINE: The IPSP Node role has not been implemented in the remote machine
*/
ClxResult clxIpspConnectToRemoteNode(_in_ ClxHandle         ipsp,
                                     _user_in_ const u1*    remoteDeviceAddress,
                                     _in_ boolean           block);

/**
                Terminates an existing connection to a remote IPSP service. The local IPSP service may be in any role.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_IPSP_DISCONNECT_LINK_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] ipsp                 Service handle
\param[  in   ] remoteDeviceAddress  The address of the remote node
\param[  in   ] block                Indicates mode of operation:
                                     TRUE : Blocking mode
                                     FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_IPSP_INTERFACE_NOT_STARTED: The interface is not started
                - CLX_ERROR_IPSP_NOT_SUPPORTED_BY_LOCAL_ROLE: The local service is not in Router role
                - CLX_ERROR_CONNECTION_NOT_EXIST: There is no IPSP connection to the remote machine
*/
ClxResult clxIpspDisconnectLink(_in_ ClxHandle         ipsp,
                                _user_in_ const u1*    remoteDeviceAddress,
                                _in_ boolean           block);

#ifdef __cplusplus
}
#endif

#endif  // __Ipsp_Api_h__

