#ifndef _OPC_API_H_
#define _OPC_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Opc.Api.h
* Description         Declares ClarinoxBlue OPP Client API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Opcodes for Command Complete confirmations:
*/

/**
Refer to #clxOppClientConnect function.
*/
#define CLX_OPP_CLIENT_CONNECT_COMPLETE                       0x5000 

/**
Refer to #clxOppClientDisconnect function.
*/
#define CLX_OPP_CLIENT_DISCONNECT_COMPLETE                    0x5001

/**
Refer to #clxOppClientPushObject function.
*/
#define CLX_OPP_CLIENT_PUSH_OBJECT_COMPLETE                   0x5002

/**
Refer to #clxOppClientPullBcard function.
*/
#define CLX_OPP_CLIENT_PULL_BUSINESS_CARD_COMPLETE            0x5003

/**
Refer to #clxOppClientAbort function.
*/
#define CLX_OPP_CLIENT_ABORT_COMPLETE                         0x5004


/** INDICATIONS*/

/**
This indication is received when a service is disconnected due to internal event. The parameter of this indication (params argument of the call-back
function) will be NULL.
*/
#define    CLX_OPP_CLIENT_CONNECTION_FAILED_INDICATION        0x9000

typedef enum ClxOppClientCallBackReasonEnum
{
     ClxOppClient_DataReceived,
     ClxOppClient_DataNeeded
}ClxOppClientCallBackReason;

typedef enum ClxOppPushObjectTypeEnum
{
    ClxOppVCard     = 0,
    ClxOppVCalender = 1,
    ClxOppVMessage  = 2,
    ClxOppVNote     = 3
}ClxOppPushObjectType;

/**
This is a callback function used while transferring data between client and server.
*/
typedef void (*ClxOppClientDataTransferCallbackFunc) (void* objectID,
                                                      ClxOppClientCallBackReason reason,
                                                      const s1* objectName,
                                                      u4 nameLen,
                                                      const u1* dataReceived,
                                                      u1** bufferToSend,
                                                      u4* len);

/**
Creates a  OPP client service on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] indicationcallbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\param[ in ] dataTransferCallbackFunc Callback function used by application to send/receive data to/from remote device.
\return ClxHandle OPP client handle. This handle must be used for any further operations on this instance of OPP client profile.
*/
ClxHandle clxOppClientCreate(_in_ ClxStack stack,
                             _in_ ClxApplicationCallbackFunc indicationcallbackFunc,
                             _in_ ClxOppClientDataTransferCallbackFunc dataTransferCallbackFunc);

/**
Connect an instance of OPP Push Client to a remote OPP Push Server. 
Pairing with the remote device may be completed before connecting to the device. 
If there are no pairing at the time of this API, the authentication process will be executed during this API. 
See GAP API for the details of pairing.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_OPP_CLIENT_CONNECT_COMPLETE.
The indication takes remoteDeviceId of type #ClxDeviceId as parameters.

\param[ in ]  opc OPP Push Client profile handle.
\param[ in ]  remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_NOT_SUPPORTED_BY_REMOTE_MACHINE : the remote machine does not support OPP server role (or it not available at the moment).
                - CLX_ERROR_CONNECTION_FAILED : operation is unsuccessful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_ERROR_OBEX_BAD_REQUEST : the OPP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_UNAUTHERIZED : the local machine is not authorized to access the remote OPP server.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote OPP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the OPP server has rejected the request as forbidden.
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxOppClientConnect( _in_ ClxHandle opc,
                              _in_ ClxDeviceId remoteDeviceId, 
                              _in_ boolean block);

/**
Disconnect an instance of OPP client profile to a remote OPP Push server. This function is initiated by OPP push client.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_OPP_CLIENT_DISCONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  opc OPP Push Client profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: An operation is in progress. The operation must be aborted before
                                                              the channel can be disconnected.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the OPP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote OPP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the OPP server has rejected the request as forbidden.
*/
ClxError clxOppClientDisconnect( _in_ ClxHandle opc,
                                 _in_ boolean block);

/**
This function is called to Push object (The object can, for example, be a business card or an appointment) from Object Push Client to the Object Push Server. 

Pre-conditions:
A list of Servers which may support the Object Push Service is displayed to the user by doing service discovery procedure.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_OPP_CLIENT_PUSH_OBJECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  opc OPP Push Client profile handle.
\param[ in ]  objectName pointer to the name of the object( for e.g. vCard) to be transferred.
\param[ in ]  objectType is of type ClxOppPushObjectType which defines the type of the file to be pushed.
\param[ in ]  userObjectID pointer to the variable which can be used to store object specific information and can be passed in the call back function. If this option is not used then value can be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
               - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to the remote OPP server.
                - CLX_ERROR_CONNECTION_FAILED: connection to the OPP server failed during the operation.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another request is in progress.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the OPP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote OPP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the OPP server has rejected the request as forbidden.
                - CLX_ERROR_OBEX_NOT_FOUND : the requested object was not found.
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before completion.
*/
ClxError clxOppClientPushObject( _in_ ClxHandle opc,
                                 _in_ const s1* objectName,
                                 _in_ ClxOppPushObjectType objectType,
                                 _in_ void* userObjectID,
                                 _in_ boolean block);

/**
This function is called to pull a a business card from a Push Server.
The business card is transferred to the Client's current folder.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_OPP_CLIENT_PULL_BUSINESS_CARD_COMPLETE.
The indication does not have any parameters.

\param[ in ]  opc OPP/Client profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to the remote OPP server.
                - CLX_ERROR_CONNECTION_FAILED: connection to the OPP server failed during the operation.
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another request is in progress.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_OBEX_BAD_REQUEST : the OPP server has rejected the connection request as a bad request.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : an internal error occurred in the remote OPP server.
                - CLX_ERROR_OBEX_FORBIDDEN : the OPP server has rejected the request as forbidden.
                - CLX_ERROR_OBEX_NOT_FOUND : the requested object was not found.
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before completion.
*/
ClxError clxOppClientPullBcard(_in_ ClxHandle opc,
                               _in_ boolean block);

/**
Aborts an ongoing operation. If there is an any ongoing operation, this
command will abort it. Therefore, the respective command for the aborted operation (#clxOppClientPushObject) will be immediately complete with the error CLX_ERROR_OBEX_OPERATION_ABORTED.

NOTE : This command CANNOT be used to abort #clxOppClientConnect or #clxOppClientDisconnect command.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_OPP_CLIENT_ABORT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  opc OPP Client client handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_COMMAND_NOT_COMPLETE the operation is cancelled by the user
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful (the ongoing operation has been aborted)
                - CLX_ERROR_COMMAND_UNSUCCESSFUL : there is no ongoing operation to be aborted
*/
ClxError clxOppClientAbort(_in_ ClxHandle opc,
                           _in_ boolean block);


#ifdef __cplusplus
}
#endif


#endif // _OPC_API_H_
