#ifndef _OPS_API_H_
#define _OPS_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Ops.Api.h
* Description         Declares ClarinoxBlue OPP Server API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Callback function prototypes used by Clarinox OPP Server to handle stack callback function calls. 
*/
typedef struct ClarinoxOppServerCallbackFunctions
{
    /**
    This Callback function called when OPP connection request is received from OPP Client.
    This function does not return any parameters.
    */
    void (*obexConnected) (_in_ ClxHandle ops,
                           _in_ ClxDeviceId remoteDeviceId);

    /**
    This Callback function called when OPP disconnection request is received from OPP Client.
    This function does not return any parameters.
    */
    void (*obexDisconnected) (_in_ ClxHandle ops,
                              _in_ ClxDeviceId remoteDeviceId);

    /**
    This Callback function called when OPP Push Object request is received from OPP Client.
    This function does not return any parameters.
    */
    void (*objectDataReceived) (_in_ ClxHandle ops,
                                _in_ void* userObjectID,
                                _in_ const u1* data, 
                                _in_ u4 dataLength);

    /**
    This Callback function called when OPP Pull Object request is received from OPP Client.
    This function does not return any parameters.
    */
    void (*objectDataNeeded) (_in_ ClxHandle ops,
                              _in_ void* userObjectID);

    /**
    This Callback function called when OPP Client request is completed (esp. during object transfers).
    This function does not return any parameters.
    */
    void (*operationComplete) (_in_ ClxHandle ops,
                               _in_ void* userObjectID);

    /**
    This Callback function called when OPP Client request is aborted. This function does not return any parameters.
    */
    void (*operationAborted) (_in_ ClxHandle ops,
                              _in_ void* userObjectID);

    /**
    This Callback function called when OPP Pull Object request is received from OPP Client.
    The callback function returns response of type #ClxObexResponse
    */
    ClxObexResponse (*getObjectRequestReceived) (_in_ ClxHandle ops,
                                                 _in_ const s1* objectName,
                                                 _in_ u4 objectNameLength,
                                                 _out_ void** userObjectID);

    /**
    This Callback function called when OPP Push Object request is received from OPP Client.
    The callback function returns response of type #ClxObexResponse
    */
    ClxObexResponse (*putObjectRequestReceived)(_in_ ClxHandle ops,
                                                _in_ const s1* objectName,
                                                _in_ u4 objectNameLength,
                                                _in_ const s1* type,
                                                _in_ u4 typeLength,
                                                _in_ boolean deleteFile,
                                                _out_ void** userObjectID);

} ClarinoxOppServerCallbackFunctions;


/**
Creates a Object Push service (Server role) on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] oppServerCallbackFunctions Callback function used by OPP server application remote device.

\return ClxHandle File Transfer Profile (Server) handle. This handle must be used for any further operations on this instance of OPP profile.
*/
ClxHandle clxOppServerCreate(_in_ ClxStack stack,
                             _in_ const ClarinoxOppServerCallbackFunctions* oppServerCallbackFunctions);


/**
This function is called when data needs to be sent to the OPP client. This function sends the data immediately.
This function MUST ONLY be called inside the call-back function 'objectDataNeeded'.
\param[ in ] ops OPP/Server profile handle.
\param[ in ] data pointer to buffer containing data sent to client.
\param[ in ] dataLength length of data in the data buffer. Note: When data transmission is finished this function is called with dataLength = 0.

\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INVALID_REQUEST : there is no data request pending (this has not been called as a response to call back function 'objectDataNeeded')
                - CLX_ERROR_CONNECTION_NOT_EXIST : connection does not exist
                - CLX_ERROR_INTERNAL_ERROR: An internal error has occurred
*/
ClxError clxOppServerSendDataUnblock(_in_ ClxHandle ops,
                                     _in_ const u1* data,
                                     _in_ u4 dataLength);

/**
This function is called when requested data from OPP Client. This function is called when server is not ready for data transmission immediately.
This function CANNOT be called inside any of the server call-back functions. It must be called in the context of a user thread.

This function MUST ONLY be called as a response to the call-back function call back function 'objectDataNeeded'.

\param[ in ] ops OPP/Server profile handle.
\param[ in ] data pointer to buffer containing data to be sent to client.
\param[ in ] dataLength length of data in the data buffer. Note: When data transmission is finished this function is called with dataLength = 0.

\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST : there is no data request pending (this has not been called as a response to call back function 'objectDataNeeded')
                - CLX_ERROR_CONNECTION_NOT_EXIST : connection does not exist
                - CLX_ERROR_INTERNAL_ERROR: An internal error has occurred
*/
ClxError clxOppServerSendDataBlock(_in_ ClxHandle ops,
                                   _in_ const u1* data,
                                   _in_ u4 dataLength);


#ifdef __cplusplus
}
#endif


#endif // _OPS_API_H_
