#ifndef __Smp_Ble_Api_h__
#define __Smp_Ble_Api_h__

/*******************************************************************************
*
* Project             Security Manager Protocol
* File                Smp.Ble.Api.h
* Description         Declares API Functions and Definitions For Ble
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#if defined(CLARINOX_BLE_SMP_ENABLED)
/**
Refer to #clxBleSmpInitiateBonding function.
*/
#define CLX_BLE_SMP_INITIATE_BONDING_COMPLETE                                               0x6400

/**
Refer to #clxBleSmpUserConfirmationRequestReply function.
*/
#define CLX_BLE_SMP_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE                                0x6401

/**
Refer to #clxBleSmpUserPasskeyRequestReply function.
*/
#define CLX_BLE_SMP_USER_PASSKEY_REQUEST_REPLY_COMPLETE                                     0x6402

/**
This indication is received during the authentication procedure when the remote device asks the local device to confirm or
reject the connection. This indication contains the 6 digits passkey. If the local device has the display capabilities then 
it displays the passkey to confirm or reject it by pressing either yes or no button.
The parameter of this indication is of type #ClxBleSmpUserPasskeyConfirmationIndication.
*/
#define CLX_BLE_SMP_USER_PASSKEY_CONFIRMATION_INDICATION                                    0xa400

/**
This indication is received during the authentication procedure when the remote device asks the local device to enter the
passkey. This indication contains the 6 digits passkey. If the local device has the the display and keyboard capabilities then
it enters the passkey for accept.
The parameter of this indication is of type #ClxBleSmpUserPasskeyRequestIndication.
*/
#define CLX_BLE_SMP_USER_PASSKEY_REQUEST_INDICATION                                         0xa401

/**
This indication is received when there is a failure during the pairing procedure between local and remote devices.
The parameter of this indication is of type #ClxBleSmpPairingFailedIndication.
*/
#define CLX_BLE_SMP_PAIRING_FAILED_INDICATION                                               0xa402

typedef enum ClxBleSmpRoleEnum
{
    ClxBleSmpRole_Slave     = 0x00,                         /*!< Security Manager Slave Role                                   */
    ClxBleSmpRole_Master    = 0x01                          /*!< Security Manager Master Role                                  */
} ClxBleSmpRole;

typedef enum ClxBleSmpBondingTypeEnum
{
    ClxBleSmpBondingType_NoBonding  = 0x00,                 /*!< Device in non bondable mode                                   */
    ClxBleSmpBondingType_Bonding    = 0x01                  /*!< Device in bondable mode                                       */
} ClxBleSmpBondingType;

typedef enum ClxBleSmpMITMProtectionEnum
{
    ClxBleSmpMITMProtection_Off = 0x00,                     /*!< Device doesn't need MITM protection for bonding               */
    ClxBleSmpMITMProtection_On  = 0x01                      /*!< Device needs MITM protection for bonding                      */
} ClxBleSmpMITMProtection;

typedef enum ClxBleSmpKeyDistributionEnum
{
    ClxBleSmpKeyDistribution_EncryptionKey  = 0x01,
    ClxBleSmpKeyDistribution_IdentityKey    = 0x02,
    ClxBleSmpKeyDistribution_SignatureKey   = 0x04
} ClxBleSmpKeyDistribution;

/**
Data Structure for the indication #CLX_BLE_SMP_USER_PASSKEY_CONFIRMATION_INDICATION
*/
typedef struct ClxBleSmpUserPasskeyConfirmationIndicationStruct
{
    u4    passkeyConfirmValue;
} ClxBleSmpUserPasskeyConfirmationIndication;

/**
Data Structure for the indication #CLX_BLE_SMP_USER_PASSKEY_REQUEST_INDICATION
*/
typedef struct ClxBleSmpUserPasskeyRequestIndicationStruct
{
    u4    passkeyValue;
} ClxBleSmpUserPasskeyRequestIndication;

/**
Data Structure for the indication #CLX_BLE_SMP_PAIRING_FAILED_INDICATION
*/
typedef struct ClxBleSmpPairingFailedIndicationStruct
{
    ClxError    errorCode;
} ClxBleSmpPairingFailedIndication;

/**
This function is used to initiate the bonding procedure to remote device. In order to bonding operation to be successful, both
the local device and remote device must be in bondable mode.

Blocking mode : This function will not return until the command is complete. The function return value indicates the result of 
the command

Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
When the command is complete, the call-back function will be called with an indication of 
type #CLX_BLE_SMP_INITIATE_BONDING_COMPLETE. This indication does not have any parameters.

\param[  in   ] stack                 Local device stack handle
\param[  in   ] role                  Local device role
\param[  in   ] bondingType           Local device bonding type
\param[  in   ] mitmProtection        Defines the local device MITM protection
\param[  in   ] maxEncryptionKeySize  Defines the encryption key size. The maximum size is 16 bytes
\param[  in   ] localDistributeKeys   Local device key distribution for authentication. please refer #ClxBleSmpKeyDistribution
\param[  in   ] remoteDistributeKeys  Remote device key distribution for authentication. please refer #ClxBleSmpKeyDistribution
\param[  in   ] block                 Indicates mode of operation:
                                      TRUE : Blocking mode
                                      FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxBleSmpInitiateBonding(_in_ ClxStack                   stack,
                                   _in_ ClxBleSmpRole              role,
                                   _in_ ClxBleSmpBondingType       bondingType,
                                   _in_ ClxBleSmpMITMProtection    mitmProtection,
                                   _in_ u1                         maxEncryptionKeySize,
                                   _in_ u1                         localDistributeKeys,
                                   _in_ u1                         remoteDistributeKeys,
                                   _in_ boolean                    block);

/**
This function replies to confirm the authentication request from a remote device, by either confirming or rejecting it. 
This function must only be called once receiving of the indication #CLX_BLE_SMP_USER_PASSKEY_CONFIRMATION_INDICATION

Blocking mode : This function will not return until the command is complete. The function return value indicates the result
of the command

Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
When the command is complete, the call-back function will be called with an indication of 
type #CLX_BLE_SMP_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE. This indication does not have any parameters.

\param[  in   ] stack            Local device stack handle
\param[  in   ] userSelectedYes  if TRUE, the user has confirmed the request otherwise rejected the request.
\param[  in   ] block            Indicates mode of operation:
                                 TRUE : Blocking mode
                                 FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxBleSmpUserConfirmationRequestReply(_in_ ClxStack    stack,
                                                _in_ boolean     userSelectedYes,
                                                _in_ boolean     block);

/**
This function replies to the authentication request to enter a passkey, by either providing a passkey, or rejecting the request.
This function must only be called once receiving of the indication #CLX_BLE_SMP_USER_PASSKEY_REQUEST_INDICATION.

Blocking mode : This function will not return until the command is complete. The function return value indicates the result
of the command.

Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
When the command is complete, the call-back function will be called with an indication of
type #CLX_BLE_SMP_USER_PASSKEY_REQUEST_REPLY_COMPLETE. This indication does not have any parameters.

\param[  in   ] stack               Local device stack handle
\param[  in   ] userEnteredPasskey  if TRUE, the user has entered a passkey otherwise the user has rejected the passkey request.
\param[  in   ] userPasskey         The passkey entered by the user, as an integer. The value must be 0 through 999999 decimal.
                                    This parameter will be ignored if userEnteredPasskey is FALSE.
\param[  in   ] block               Indicates mode of operation:
                                    TRUE : Blocking mode
                                    FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxBleSmpUserPasskeyRequestReply(_in_ ClxStack    stack,
                                           _in_ boolean     userEnteredPasskey,
                                           _in_ u4          userPasskey,
                                           _in_ boolean     block);

#endif /* CLARINOX_BLE_SMP_ENABLED */

#ifdef __cplusplus
}
#endif

#endif  // __Smp_Ble_Api_h__

