#ifndef _ClarinoxWlan_h_
#define _ClarinoxWlan_h_

/*******************************************************************************
*
* Project             ClarinoxWLAN
* File                ClarinoxWlan.h
* Description         ClarinoxWlan Protocol Stack
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClarinoxWlanConst.h"
#include "WlanBufferManagerInterface.h"
#include "WlanNetworkStackInterface.h"
#include "WlanStackPacketCaptureInterface.h"

#ifdef __cplusplus
extern "C" {
#endif


/**
Initiates WLAN stack. When this function is used in non-blocking mode returns immediately. Upon initialization of the stack is complete, 
an indication of type #CLX_WLAN_STACK_INITIALIZATION_COMPLETE_INDICATION will be sent to the call-back function. 
The errorCode parameter of the call-back function must be checked, to
see if the stack has been initialized successfully. 
\code
if (clxInitClarinoxWlan(&params, FALSE, stackMessageHandler, NULL, NULL, &stack, FALSE) != CLX_ERROR_COMPLETION_PENDING)
{
       ClxConsoleUIEngine::text ("Initialization of ClarinoxWlan failed\n");
       return 0;
}
\endcode


If this function is used in blocking mode, then it returns upon successful completion of the WLAN stack.
\code
if (clxInitClarinoxWlan(&params, FALSE, stackMessageHandler, NULL, NULL, &stack, TRUE) != CLX_SUCCESS)
{
       ClxConsoleUIEngine::text ("Initialization of ClarinoxWlan failed\n");
       return 0;
}
\endcode

\param[ in ] physicalInterface WLAN controller hardware on the target platform. This argument CANNOT be NULL.
\param[ in ] callBacksInUserThread The presence of a call back function in user defined thread. Takes Boolean expression True/False
\param[ in ] indicationCallbackFunc a pointer to the call-back functions which will receive the indications. This parameter CANNOT be NULL.
\param[ in ] packetCaptureInterface a pointer to the packet capture interface of type #ClxWlanStackPacketCaptureInterface. This interface is used
to monitor all TX/RX frames. This parameter is NULL if monitoring feature is not required.
\param[ in ] exceptionCallback a pointer to the exception handling function.
\param[ out ] stack pointer to WLAN stack object.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful

\return must be #CLX_ERROR_COMPLETION_PENDING which indicates initialization is in progress.
*/
ClxError clxInitClarinoxWlan(_in_   void*                                        physicalInterface,
                             _in_   boolean                                      callBacksInUserThread,
                             _in_   ClxApplicationCallbackFunc                   indicationCallbackFunc,
                             _in_   ClxExceptionCallback                         exceptionCallback,
                             _out_  ClxStack*                                    stack,
                             _in_   boolean                                      block);


/**
Initiates the first phase of the stack termination. The first phase includes cleaning up the stack by disconnecting from the remote services.
This function is non-blocking and returns immediately. When the first phase is complete, an indication of type CLX_WLAN_STACK_TERMINATION_COMPLETE_INDICATION
will be received by the indication call-back function. At that moment, the second phase of WLAN termination must be performed by calling #clxDestroyClarinoxWlan.

IMPORTANT : All open handles belonging to this instance of stack MUST BE CLOSED before this function is called. Otherwise, the behaviour will be undefined, and the stack
may even crash.

\param[ in ] clarinoxWlan the ClarinoxWlan stack object.
*/
void clxTerminateClarinoxWlan(_in_ ClxStack clarinoxWlan);

/**
Performs the second (last) phase of WLAN termination by deleting the objects, threads, and timers. It also releases all the memory used by the stack.
This function MUST ONLY be called when the first phase of stack termination is complete (i.e. an indication of type CLX_WLAN_STACK_TERMINATION_COMPLETE_INDICATION has been
received). When this function returns, the stack is completely terminated.

IMPORTANT : Please note that this function CANNOT be called within the context of the call-back function (or any other service call-back functions) thread.
This is due to the fact that the call-back thread must also be terminated and cleaned up by this function. The developer must devise a mechanism in order to 
call this function outside of the call-back function.

\param[ in ] clarinoxWlan the ClarinoxWlan stack object.
*/
void clxDestroyClarinoxWlan(_in_ ClxStack clarinoxWlan);

/**
Returns a text string containing the name of the error code. This function only returns the string in debug mode.
In release mode, the return string is always an empty string.

\param[ in ] errorCode The error code for which a string representation must be returned.

\return A stack-allocated string containing the string representation of the error code. This string MUST not be modified or deleted.
*/

const s1* clxGetWlanErrorCodeText(ClxError errorCode);



/* Useful Tools: */
/**
Parses the string containing a MAC address in the format XX:XX:XX:XX:XX:XX (Null terminated),
and returns the MAC address in binary format (6 bytes, no null termination).

\param[ in ] str The Null-terminated string buffer containing the MAC address in HEX format.
\param[ out ] macAddrBuf A caller-allocated buffer which on return will carry the MAC address in binary format (only if the return
value of the function is TRUE). The size of this buffer MUST BE at least 6 bytes.

\return TRUE The conversion has been successful.
FALSE The string is not in the correct HEX format.
*/
boolean clxWlanParseMacAddressString(const s1* str, u1* macAddrBuf);


#ifdef __cplusplus
}
#endif

#endif // _ClarinoxWlan_h_

