#ifndef _WlanConst_h_
#define _WlanConst_h_

/*******************************************************************************
*
* Project             ClarinoxWLAN
* File                ClarinoxWlanConst.h
* Description         Clarinox WLAN Protocol Stack constants and types
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClarinoxConst.h"

#ifdef __cplusplus
extern "C" {
#endif


/**
An array of size 5, and type u2, which maps the ClarinoxWlan platform-independent thread priorities
to underlying platform priorities. The first index (index 0) of the array represents the highest priority,
while the last index (index 4) of the array represents the lowest priority.
If the table is not defined in BSP, the default priority of the underlying platform will be used for all
threads. 

ALL priorities MUST be defined.

ClarinoxWlan threads:

- Driver thread : Uses Highest priority (index 0).

- Stack thread : Uses Medium priority (index 2).

- Application Callback thread : Uses Medium priority (index 2).

- Timer thread : Uses High priority (index 1).

- Debug thread (if available) : Uses Lowest priority (index 4).

- Console Engine thread (if available) : Uses Lowest priority (index 4).
*/
extern u2* clarinoxWlanPlatformTaskPriorityTable;



typedef enum ClxWlanPacketStatusStruct
{
    ClxWlanPacketStatus_Success = 0,            /*!< Successfully submitted to the driver/hardware, 
                                                     or successfully sent to the remote station (when the flag
                                                     CLX_WLAN_TX_DESC_FLAG_RELEASE_ON_HW_TX_COMPLETE is set) */
    ClxWlanPacketStatus_NoLink,                 /*!< No wireless link is available to send this MSDU */
    ClxWlanPacketStatus_AckFailed,              /*!< Failed due to acknowledgement failure from the destination
                                                     (when the flag CLX_WLAN_TX_DESC_FLAG_RELEASE_ON_HW_TX_COMPLETE is set) */
    ClxWlanPacketStatus_Cancelled,              /*!< The stack cancelled sending of the TX MSDU */
    ClxWlanPacketStatus_Timeout,                /*!< Hardware-Specific or Software-Specific timeout occurred before the TX MSDU is sent or successfully
                                                     acknowledged */                
    ClxWlanPacketStatus_NoResources,            /*!< Dropped due to not enough resources on the driver/hardware */ 
    ClxWlanPacketStatus_HardwareError,          /*!< Failed due to an unknown hardware error */
    ClxWlanPacketStatus_CRC_Failed,             /*!< CRC verification failed for the RX packet */
    ClxWlanPacketStatus_IntegrityFailed,        /*!< Packet integrity test failed for the RX packet (applies to encrypted packets only) */
    ClxWlanPacketStatus_UnknownError            /*!< Unknown error occurred during transfer or reception of this packet */
} ClxWlanPacketStatus;


/**
Wireless Protected Setup (WPS) authentication methods supported by the local or remote device.
*/
enum ClxWpsSupportedMethod
{
    ClxWpsSupportedMethod_Invalid               = 0x0000,               /*!< Invalid value */
    ClxWpsSupportedMethod_USBA                  = 0x0001,               /*!< (Flash Drive) Deprecated */
    ClxWpsSupportedMethod_Ethernet              = 0x0002,               /*!< Deprecated */
    ClxWpsSupportedMethod_Label                 = 0x0004,               /*!< 8 digit static PIN typically available on device */
    ClxWpsSupportedMethod_Display               = 0x0008,               /*!< A dynamic 4 or 8 digit PIN is available from a display. Version 2.0 devices must qualify
                                                                             the display as Virtual (0x2008) or Physical (0x4008) */
    ClxWpsSupportedMethod_ExternalNfcToken      = 0x0010,               /*!< A physical passive NFC token is used to transfer the configuration or device password */
    ClxWpsSupportedMethod_IntegratedNfcToken    = 0x0020,               /*!< The NFC passive token is integrated in the device */
    ClxWpsSupportedMethod_NfcInterface          = 0x0040,               /*!< The device contains an NFC interface */
    ClxWpsSupportedMethod_PushButton            = 0x0080,               /*!< The device contains either a physical or virtual pushbutton. Version 2.0 devices
                                                                              must qualify the pushbutton as Virtual (0x0280) or Physical (0x0480) */
    ClxWpsSupportedMethod_Keypad                = 0x0100,               /*!< Device is capable of entering a PIN */
    ClxWpsSupportedMethod_VirtualPushButton     = 0x0280,               /*!< Push button functionality is available through a software user interface */
    ClxWpsSupportedMethod_PhysicalPushButton    = 0x0480,               /*!< A physical push button is available on the device */
    ClxWpsSupportedMethod_VirtualDisplayPIN     = 0x2008,               /*!< The dynamic 4 or 8 digit PIN is displayed through a remote user interface. For
                                                                             example using the management html page of an AP to obtain the dynamic PIN */
    ClxWpsSupportedMethod_PhysicalDisplayPIN    = 0x4008                /*!< The dynamic 4 or 8 digit PIN is shown on a display/screen that is part of the device. For
                                                                             example obtaining the dynamic PIN from the LCD screen on a printer */
};

/*
Device types as defined by Wi-Fi Alliance for WPS.
*/
enum ClxWpsDeviceType
{
    ClxWps_Computer_PC                                          = 0x00010001,
    ClxWps_Computer_Server                                      = 0x00010002,
    ClxWps_Computer_MediaCenter                                 = 0x00010003,
    ClxWps_Computer_UltraMobile                                 = 0x00010004,
    ClxWps_Computer_Notebook                                    = 0x00010005,
    ClxWps_Computer_Desktop                                     = 0x00010006,
    ClxWps_Computer_MobileInternetDevice                        = 0x00010007,
    ClxWps_Computer_Netbook                                     = 0x00010008,
    ClxWps_Computer_Tablet                                      = 0x00010009,
    ClxWps_InputDevice_Keyboard                                 = 0x00020001,
    ClxWps_InputDevice_Mouse                                    = 0x00020002,
    ClxWps_InputDevice_Joystick                                 = 0x00020003,
    ClxWps_InputDevice_Trackball                                = 0x00020004,
    ClxWps_InputDevice_GamingController                         = 0x00020005,
    ClxWps_InputDevice_Remote                                   = 0x00020006,
    ClxWps_InputDevice_Touchscreen                              = 0x00020007,
    ClxWps_InputDevice_BiometricReader                          = 0x00020008,
    ClxWps_InputDevice_BarcodeReader                            = 0x00020009,
    ClxWps_Printer                                              = 0x00030001,
    ClxWps_Scanner                                              = 0x00030002,
    ClxWps_Fax                                                  = 0x00030003,
    ClxWps_Copier                                               = 0x00030004,
    ClxWps_Printer_Fax_Scanner_Copier                           = 0x00030005,       /* All-In-One Device */
    ClxWps_Camera_DigitalStillCamera                            = 0x00040001,
    ClxWps_Camera_VideoCamera                                   = 0x00040002,
    ClxWps_Camera_WebCamera                                     = 0x00040003,
    ClxWps_Camera_SecurityCamera                                = 0x00040004,
    ClxWps_Storage_NAS                                          = 0x00050001,
    ClxWps_Network_AP                                           = 0x00060001,
    ClxWps_Network_Router                                       = 0x00060002,
    ClxWps_Network_Switch                                       = 0x00060003,
    ClxWps_Network_Gateway                                      = 0x00060004,
    ClxWps_Network_Bridge                                       = 0x00060005,
    ClxWps_Display_Television                                   = 0x00070001,
    ClxWps_Display_ElectronicPictureFrame                       = 0x00070002,
    ClxWps_Display_Projector                                    = 0x00070003,
    ClxWps_Display_Monitor                                      = 0x00070004,
    ClxWps_MultimediaDevice_DAR                                 = 0x00080001,
    ClxWps_MultimediaDevice_PVR                                 = 0x00080002,
    ClxWps_MultimediaDevice_MCX                                 = 0x00080003,
    ClxWps_MultimediaDevice_SetTopBox                           = 0x00080004,
    ClxWps_MultimediaDevice_MediaClientServerAdapterExtender    = 0x00080005,
    ClxWps_MultimediaDevice_PortableVideoPlayer                 = 0x00080006,
    ClxWps_GamingDevices_Xbox                                   = 0x00090001,
    ClxWps_GamingDevices_Xbox360                                = 0x00090002,
    ClxWps_GamingDevices_Playstation                            = 0x00090003,
    ClxWps_GamingDevices_GameConsole_GameConsoleAdapter         = 0x00090004,
    ClxWps_GamingDevices_PortableGamingDevice                   = 0x00090005,
    ClxWps_Telephone_WindowsMobile                              = 0x000A0001,
    ClxWps_Telephone_PhoneSingleMode                            = 0x000A0002,
    ClxWps_Telephone_PhoneDualMode                              = 0x000A0003,
    ClxWps_Telephone_SmartphoneSingleMode                       = 0x000A0004,
    ClxWps_Telephone_SmartphoneDualMode                         = 0x000A0005,
    ClxWps_AudioDevice_AudioTunerReceiver                       = 0x000B0001,
    ClxWps_AudioDevice_Speakers                                 = 0x000B0002,
    ClxWps_AudioDevice_PortableMusicPlayer                      = 0x000B0003,
    ClxWps_AudioDevice_Headset                                  = 0x000B0004,
    ClxWps_AudioDevice_Headphone                                = 0x000B0005,
    ClxWps_AudioDevice_Microphone                               = 0x000B0006,
    ClxWps_AudioDevice_HomeTheaterSystem                        = 0x000B0007,
    ClxWps_VendorSpecificDeviceType                             = 0x0FFFFFFF        /*!< used only when a vendor-specific device type value is received from the remote device.
                                                                                         CANNOT be used when setting the device type of the local device */
};


/**
The Type of Scanning
*/
enum ClxScanType
{
    ClxScanTypeActive,              /*!< Active Scanning. */
    ClxScanTypePassive              /*!< Passive Scanning. */
};

 /** User configurable clarinoxWlan parameters */
struct ClarinoxWlanParameters
{
  //  ClxBSSType      bssType;
    s1                  BSSID[6];           /*!< Any valid individual or broadcast MAC address or wildcard BSSID. */
    s1                  SSID[32];           /*!< Specifies the desired SSID or the wildcard SSID. */
    enum ClxScanType    scanType;
    u4                  probeDelay;         /*!< This is the delay, in microseconds, before the procedure to probe a channel in active scanning begins.*/
    //ChannelList
    u4                  minChannelTime;
    u4                  maxChannelTime;
    //VendorSpecificInfo

};


#define CLARINOX_WLAN_CLASS_ID                                    55
#define CLARINOX_WLAN_VIRTUAL_INTERFACE_CLASS_ID                  56


#ifdef __cplusplus
}
#endif

#endif // _WlanConst_h_

