#ifndef _WLAN_CONFIG_H_
#define _WLAN_CONFIG_H_

/*******************************************************************************
*
* Project             ClarinoxWLAN
* File                Wlan.Config.h
* Description         Contains WLAN configuration parameters
*
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
ClxFreuencyBand enumerates different types of WLAN frequency bands.
*/
enum ClxWlanFrequencyBand
{
    ClxWlanFreqBand2_4GHz,
    ClxWlanFreqBand5GHz,
    ClxWlanFreqBand60GHz
};

/**
ClxWlanStandard enumerates the different WLAN standards. More than one value may be Bit-wise combined, 
if a relevant configuration parameter allows this.
*/
enum ClxWlanStandard
{
    ClxWlanStandardB    = 0x1,
    ClxWlanStandardAG   = 0x2,                                  /* Both IEEE802.11g and IEEE802.11a (The only difference is the frequency band in which they operate) */
    ClxWlanStandardN    = 0x4,
    ClxWlanStandardAC   = 0x08
};


/**
ClxWlanStandard enumerates the different scan options. More than one value may be Bit-wise combined, 
if a relevant configuration parameter allows this. The value ClxWlanNoScan shall not be combined with
the other values.
*/
enum ClxWlanScanType
{
    ClxWlanNoScan = 0x0,
    ClxWlanPassiveScan = 0x1,
    ClxWlanActiveScan = 0x2
};


/* Operating channel type: */
enum ClxWlanChannelWidth
{
    ClxWlanChannelWidth_20MHZ                = 0,               /*!< 20MHz channel used with IEEE802.11abg and IEEE802.11n */
    ClxWlanChannelWidth_40MHZ_SecondaryAbove = 1,               /*!< A HT (IEEE802.11n) channel with the bandwidth of 20MHz+20MHz (Secondary channel is above primary channel) */
    ClxWlanChannelWidth_40MHZ_SecondaryBelow = 3                /*!< A HT (IEEE802.11n) channel with the bandwidth of 20MHz+20MHz (Secondary channel is below primary channel) */    
};


/**
General capabilities of a WLAN station. Can be bit-wise-combined into a single variable: 
*/
#define CLX_WLAN_CAPABILITY_SHORT_PREAMBLE_SUPPORTED                        0x0001
#define CLX_WLAN_CAPABILITY_QOS_SUPPORTED                                   0x0002
#define CLX_WLAN_CAPABILITY_SHORT_SLOT_TIME_SUPPORTED                       0x0004
#define CLX_WLAN_CAPABILITY_CHANNEL_AGILITY_SUPPORTED                       0x0008
#define CLX_WLAN_CAPABILITY_DSSS_OFDM_SUPPORTED                             0x0010    /*!< The station supports sending and receiving OFDM (IEEE802.11g) frames starting with a DSSS (IEEE802.11b) header 
                                                                                         which provides a full compatibility between IEEE802.11b and IEEE802.11g standards without additional protection mechanisms */ 
/* WIFI Alliance standard capabilities: */
#define CLX_WLAN_CAPABILITY_WPA_SUPPORTED                                   0x0020    /*!< The station supports Wifi Protected Access */
#define CLX_WLAN_CAPABILITY_WPA2_SUPPORTED                                  0x0040    /*!< The station supports Wifi Protected Access version 2 */
#define CLX_WLAN_CAPABILITY_WPS_SUPPORTED                                   0x0080    /*!< The station supports Wifi Protected Setup */
#define CLX_WLAN_CAPABILITY_WMM_SUPPORTED                                   0x0100    /*!< The station supports Wifi MultiMedia (based on IEEE802.11 QOS features) */
#define CLX_WLAN_CAPABILITY_WIFI_DIRECT_SUPPORTED                           0x0200    /*!< The station supports WIFI Direct - also known as P2P (Point to Point) mode */

/* IEEE802.11n HT (High Throughput) capabilities : */
#define CLX_WLAN_CAPABILITY_HT_40MHZ_CHANNEL_WIDTH_SUPPORTED                0x0400    /*!< The station supports 40MHz channel width to double the data throughput */
#define CLX_WLAN_CAPABILITY_HT_SGI_FOR_20MHZ_SUPPORTED                      0x0800    /*!< Short Guard Interval (400ns) is supported in 20MHz channels */
#define CLX_WLAN_CAPABILITY_HT_SGI_FOR_40MHZ_SUPPORTED                      0x1000    /*!< Short Guard Interval (400ns) is supported in 40MHz channels */
#define CLX_WLAN_CAPABILITY_HT_GREENFIELD_MODE_SUPPORTED                    0x2000    /*!< Greenfield mode (fast mode but disruptive to IEEE802.11abg stations in the area) is supported */
#define CLX_WLAN_CAPABILITY_HT_40MHZ_INTOLERANT                             0x4000    /*!< The station is not able to operate in an environment where 40MHz channels are in use */
#define CLX_WLAN_CAPABILITY_HT_STATIC_POWER_SAVING_MODE_SUPPORTED           0x8000    /*!< The static power saving mode is supported (mutually exclusive with dynamic mode) */
#define CLX_WLAN_CAPABILITY_HT_DYNAMIC_POWER_SAVING_MODE_SUPPORTED          0x10000   /*!< The dynamic power saving mode is supported (mutually exclusive with static mode) */
#define CLX_WLAN_CAPABILITY_HT_TWO_SPATIAL_STREAMS_SUPPORTED                0x20000   /*!< Two spatial streams (with each containing different data) are supported */
#define CLX_WLAN_CAPABILITY_HT_THREE_SPATIAL_STREAMS_SUPPORTED              0x40000   /*!< Three spatial streams (with each containing different data) are supported */
#define CLX_WLAN_CAPABILITY_HT_FOUR_SPATIAL_STREAMS_SUPPORTED               0x80000   /*!< Four spatial streams (with each containing different data) are supported */


/**
ClxWlanStandard enumerates non-HT (IEE802.11a, IEEE802.11g, and IEEE802.11b) data rates.
*/
enum ClxWlanNonHTRate
{
    ClxWlanNonHTRate_NULL       = 0,                 /*!< Indicates that the relevant parameter is not a non-HT data rate */
    ClxWlanNonHTRate_1Mbps      = 2,                 /*!< Mandatory in all standards */
    ClxWlanNonHTRate_2Mbps      = 4,                 /*!< Mandatory in all standards */
    ClxWlanNonHTRate_5_5Mbps    = 11,                /*!< 5.5Mpbs - Mandatory in IEEE802.11b standard */
    ClxWlanNonHTRate_6Mbps      = 12,                /*!< Mandatory in IEEE802.11ag standard */
    ClxWlanNonHTRate_9Mbps      = 18,                /*!< IEEE802.11ag standard */
    ClxWlanNonHTRate_11Mbps     = 22,                /*!< Mandatory in IEEE802.11b standard */
    ClxWlanNonHTRate_12Mbps     = 24,                /*!< Mandatory in IEEE802.11ag standard */
    ClxWlanNonHTRate_18Mbps     = 36,                /*!< IEEE802.11ag standard */
    ClxWlanNonHTRate_22Mbps     = 44,                /*!< Deprecated */
    ClxWlanNonHTRate_24Mbps     = 48,                /*!< IEEE802.11ag standard */
    ClxWlanNonHTRate_33Mbps     = 66,                /*!< Deprecated */
    ClxWlanNonHTRate_36Mbps     = 72,                /*!< IEEE802.11ag standard */
    ClxWlanNonHTRate_48Mbps     = 96,                /*!< IEEE802.11ag standard */
    ClxWlanNonHTRate_54Mbps     = 108                /*!< IEEE802.11ag standard */
};


/**
Bit-Wise definition of Non-HT rates. Used when a value contains a combination of rates. 
*/
#define    CLX_WLAN_NON_HT_RATE_BIT_1MBPS           0x0001    
#define    CLX_WLAN_NON_HT_RATE_BIT_2MBPS           0x0002        
#define    CLX_WLAN_NON_HT_RATE_BIT_5_5MBPS         0x0004    
#define    CLX_WLAN_NON_HT_RATE_BIT_6MBPS           0x0008        
#define    CLX_WLAN_NON_HT_RATE_BIT_9MBPS           0x0010        
#define    CLX_WLAN_NON_HT_RATE_BIT_11MBPS          0x0020    
#define    CLX_WLAN_NON_HT_RATE_BIT_12MBPS          0x0040    
#define    CLX_WLAN_NON_HT_RATE_BIT_18MBPS          0x0080
#define    CLX_WLAN_NON_HT_RATE_BIT_22MBPS          0x0100
#define    CLX_WLAN_NON_HT_RATE_BIT_24MBPS          0x0200
#define    CLX_WLAN_NON_HT_RATE_BIT_33MBPS          0x0400
#define    CLX_WLAN_NON_HT_RATE_BIT_36MBPS          0x0800
#define    CLX_WLAN_NON_HT_RATE_BIT_48MBPS          0x1000
#define    CLX_WLAN_NON_HT_RATE_BIT_54MBPS          0x2000


/**
ClxWlanConfigRate defines a single TX/RX rate as defined by IEEE802.11 standard. An object of type #ClxWlanConfigRate may refer
to a non-HT rate (as defined IEEE802.11a, IEEE802.11b or IEEE802.11g standard), or a HT (High Throughput) rate as defined by
IEEE802.11n standard. A non-HT rate is defined by the actual over-the-air data rate (in Mbps). A HT rate is defined by a MCS
(Modulation and Coding Scheme) index which is mapped to the actual over-the-air data rate.

paramInfo : provides information of the parameter. This is to be set by a call to clxConfigInitRateParam(). 
This SHALL not be set manually.

nonHTValue : The data rate value, only if this is a non-HT rate. The value shall be selected from #ClxWlanNonHTRate enumerator.
If this object defines a HT rate, this member SHALL be set to ClxWlanNonHTRate_NULL.
mcsIndex : The MCS value of this rate, only if this a HT rate. This member is ignored if "nonHTValue" of type #ClxWlanNonHTRate is not set to ClxWlanNonHTRate_NULL.
Please refer to IEEE802.11n (2009) specification for more information on MCS rates and their meaning.
basicRate : This value only applies when the local interface is set to AccessPoint or AD-HOC mode. In these modes, setting this member to TRUE indicates
that this object specifies a Basic Rate. A basic rate is a rate which must be supported by all clients in a BSS/IBSS. If a client does not support one or 
more of basic rates, it will not be able to join the WLAN network. Basic rates are used to send multicast/broadcast packets in the network.
In Station mode, this member has no effect and is ignored by ClarinoxWLAN.
*/
typedef struct ClxWlanConfigRateStruct
{
    ClxConfigParam                paramInfo;
    enum ClxWlanNonHTRate         nonHTValue;
    u4                            mcsIndex;
    boolean                       basicRate;
} ClxWlanConfigRate;


CLX_DEFINE_CONFIG_TYPE(ClxWlanConfigRate);


/**
Initializes a member of parentList with the given object of type #ClxWlanConfigRate.
*/
extern void clxConfigInitRateParam (ClxWlanConfigRate* object, 
                                    const s1* paramName, 
                                    enum ClxWlanNonHTRate nonHTValue,
                                    u4 mcsIndex,
                                    boolean    basicRate,
                                    ClxConfigList* parentList);


/**
ClxWlanConfigChannel defines relevant information of a given WLAN channel. The structure also includes regulatory
domain-related information on a channel basis. The API user shall make sure these parameters are set appropriately
according to the regulatory rules of the domain/country in which the product is used. ClarinoxWLAN will NOT examine these
values for consistency with the regulatory rules.

paramInfo : provides information of the parameter. This is to be set by a call to clxConfigInitChannelParam(). 
This SHALL not be set manually.

channelNumber : An unsigned number allocated to every WLAN channel.
frequencyBand : The frequency band in which the channel is located. This parameter is of type #ClxFrequencyBand.
allowedScanType : The type of scanning which is permitted in the current regulatory domain for this channel. The parameter
shall contain an OR-ed combination of values of type #ClxWlanScanType. If scanning is not permitted on this channel, this
parameter shall be set to ClxWlanNoScan.
maxAllowedTxPower : The maximum TX power (in dBm) allowed on this channel according to the regulatory rules.
*/
typedef struct ClxWlanConfigChannelStruct
{
    ClxConfigParam                paramInfo;
    u2                            channelNumber;
    enum ClxWlanFrequencyBand     frequencyBand;
    u4                            allowedScanType;
    u4                            maxAllowedTxPower;
} ClxWlanConfigChannel;


CLX_DEFINE_CONFIG_TYPE(ClxWlanConfigChannel);


/**
Initializes a member of parentList with the given object of type #ClxWlanConfigChannel.
*/
extern void clxConfigInitChannelParam    (ClxWlanConfigChannel* object, 
                                           const s1* paramName, 
                                           u2 channelNumber, 
                                           ClxWlanFrequencyBand frequencyBand,
                                           u4 allowedScanType,
                                           u4 maxAllowedTxPower,
                                           ClxConfigList* parentList);


#ifdef __cplusplus
}
#endif


#endif // _WLAN_CONFIG_H_
