#ifndef _WlanNetworkStackInterface_
#define _WlanNetworkStackInterface_

/******************************************************************************
 *
 * Project             ClarinoxWLAN
 * File                WlanNetworkStackInterface.h
 * Description         Clarinox WLAN network-stack interface
 *
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#define CLX_WLAN_MAXIMUM_PAYLOAD_CHUNKS_PER_DESCRIPTOR                  6
#define CLX_WLAN_IEEE802_3_HEADER_SIZE                                  14    /*!< in Bytes */ 

#define CLX_WLAN_TX_PROTECTED
#define CLX_WLAN_RX_PROTECTED


/**
The indication CLX_WLAN_NETWORK_INTERFACE_INDICATION_STARTED is received by the network interface implementation when the parent virtual interface
has been successfully started. When this indication is received, the implementation of network interface is able to call any API functions defined
in this file. 

NOTE : When this indication is received, the wireless link is still down. The link should be considered up only after reception of the indication
#CLX_WLAN_NETWORK_INTERFACE_INDICATION_LINK_UP.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxWlanNetworkInterfaceIndication_Started.

Handling this indication is MANDATORY.
*/
#define CLX_WLAN_NETWORK_INTERFACE_INDICATION_STARTED                       0x0001

/**
The indication CLX_WLAN_NETWORK_INTERFACE_INDICATION_STOPPED is received by the network interface implementation when the parent virtual interface
has been stopped. When this indication is received, the implementation of network interface MUST cancel any ongoing TX/RX activity and return any
resources of the WLAN stack it currently possesses (e.g. TX/RX buffers and descriptors) to the WLAN stack via API functions. When the implementation of 
handleIndication() returns, the implementation of network interface must already have called off any TX/RX activity, and all resources must have been returned
to the WLAN stack. Also, when handleIndication() returns, the implementation of the network stack is NOT allowed to call the API functions defined in this file
any longer.

NOTE : This indication might be received when the link is up. In this case, reception of this indication also implies that the link has gone down.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxWlanNetworkInterfaceIndication.

Handling this indication is MANDATORY.
*/
#define CLX_WLAN_NETWORK_INTERFACE_INDICATION_STOPPED                       0x0002

/**
The indication CLX_WLAN_NETWORK_INTERFACE_STATUS_LINK_UP is received by the network interface implementation when there is an active wireless link and TX packets
may be sent to the WLAN stack or RX packets may be received by the network interface implementation.

When this indication is received, the link should be considered up until the indication #CLX_WLAN_NETWORK_INTERFACE_INDICATION_LINK_DOWN or #CLX_WLAN_NETWORK_INTERFACE_INDICATION_STOPPED
is received.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxWlanNetworkInterfaceIndication.

Handling this indication is optional.
*/
#define CLX_WLAN_NETWORK_INTERFACE_INDICATION_LINK_UP                       0x0003

/**
The indication CLX_WLAN_NETWORK_INTERFACE_STATUS_LINK_DOWN is received by the network interface implementation when there is no longer an active wireless link and TX packets
cannot be sent to the WLAN stack any more.

NOTE : When the link is down, any TX packets passed to the WLAN stack will be completed immediately.
NOTE : After receiving this indication, the implementation will not receive any RX frames any more.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxWlanNetworkInterfaceIndication.

Handling this indication is optional.
*/
#define CLX_WLAN_NETWORK_INTERFACE_INDICATION_LINK_DOWN                     0x0004

/**
The indication CLX_WLAN_NETWORK_INTERFACE_INDICATION_NEW_CLIENT_ASSOCIATED is received by the network interface implementation when a new station has joined the BSS. This indication is received
ONLY if the local virtual interface has the AccessPoint or P2P-GO role.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxWlanNetworkInterfaceIndication_NewClientAssociatedStruct.

Handling this indication is optional.
*/
#define CLX_WLAN_NETWORK_INTERFACE_INDICATION_NEW_CLIENT_ASSOCIATED         0x0005


/**
The indication CLX_WLAN_NETWORK_INTERFACE_INDICATION_CLIENT_DEASSOCIATED is received by the network interface implementation when a station has left the BSS. This indication is received
ONLY if the local virtual interface has the AccessPoint or P2P-GO role.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxWlanNetworkInterfaceIndication_ClientDeassociated.

Handling this indication is optional.
*/
#define CLX_WLAN_NETWORK_INTERFACE_INDICATION_CLIENT_DEASSOCIATED           0x0006


#ifdef __cplusplus
extern "C" {
#endif


    /**
    Base type for all indications
    */
    typedef struct ClxWlanNetworkInterfaceIndicationStruct
    {
        u2 indicationID;
    } ClxWlanNetworkInterfaceIndication;


    /**
    Object type for the indication CLX_WLAN_NETWORK_INTERFACE_INDICATION_STARTED
    */
    typedef struct ClxWlanNetworkInterfaceIndication_StartedStruct
    {
        ClxWlanNetworkInterfaceIndication base;     /*!< SHALL be the first member */

        const u1* localMacAddress;                  /*!< The local MAC address which is going to be used by the parent virtual interface */
    } ClxWlanNetworkInterfaceIndication_Started;


    /**
    Object type for the indication CLX_WLAN_NETWORK_INTERFACE_INDICATION_NEW_CLIENT_ASSOCIATED
    */
    typedef struct ClxWlanNetworkInterfaceIndication_NewClientAssociatedStruct
    {
        ClxWlanNetworkInterfaceIndication base;     /*!< SHALL be the first member */

        const u1*   clientMacAddress;               /*!< MAC address of the client associated to the local AccessPoint or P2P GO */
        u2          clientID;                       /*!< The numeric unique ID the client associated to the local AccessPoint or P2P GO (assigned by ClarinoxWLAN) */
    }  ClxWlanNetworkInterfaceIndication_NewClientAssociated;


    /**
    Object type for the indication CLX_WLAN_NETWORK_INTERFACE_INDICATION_CLIENT_DEASSOCIATED
    */
    typedef struct ClxWlanNetworkInterfaceIndication_ClientDeassociatedStruct
    {
        ClxWlanNetworkInterfaceIndication base;     /*!< SHALL be the first member */

        const u1*   clientMacAddress;               /*!< MAC address of the client de-associated from the local AccessPoint or P2P GO */
        u2          clientID;                       /*!< The numeric unique ID the client de-associated from the local AccessPoint or P2P GO (assigned by ClarinoxWLAN) */
    }  ClxWlanNetworkInterfaceIndication_ClientDeassociated;


    /**
    Describes a chunk of data which is part of a packet body. The data may have been stored in a WLAN-Stack-allocated buffer 
    (#ClxWlanBuffer structure), or stored in an arbitrary buffer. In the latter case, "buffer" parameter of this structure 
    shall be set to NULL.
    */
    typedef struct ClxWlanPacketDataStruct
    {
        ClxWlanBuffer*    buffer;                             /*!<  Optional; if not NULL, dataAddress and (dataAddress + dataLength) 
                                                                  SHALL be within the memory range of the buffer                    */
        u1*             dataAddress;                        /*!<  The address of the data chunk defined by this object              */
        u4              dataLength;                         /*!<  The length of the data chunk defined by this object               */
    } ClxWlanPacketData;


    /**
    A TX descriptor of type #ClxWlanTxPacketDescriptor represents a single PDU which is to be sent to the WLAN driver. 
    An instance of this structure is only allocated by ClarinoxWLAN stack.
    An implementation may obtain one or more TX descriptors by calling #clxWlanGetFreeNetworkTxDescriptors() function. 
    TX descriptors are re-usable objects. They are allocated only once when a new virtual interface is created, 
    and used for the data transfers in the TX path until the virtual interface is closed.

    Note: An object of type ClxWlanTxPacketDescriptor is valid only as long as the virtual interface is not closed.
    The virtual interface owns ClxWlanTxPacketDescriptor objects.
    */
    typedef struct ClxWlanTxPacketDescriptorStruct
    {
        void*             userID;                                               /*!< An identifier associated to this descriptor, this id is given by the Network stack interface. 
                                                                                     The implementation can use this value to identify the PDUs when the sending of a PDU is completed */
        ClxWlanPacketData body[CLX_WLAN_MAXIMUM_PAYLOAD_CHUNKS_PER_DESCRIPTOR]; /*!< The PDU described by this descriptor, as an array of #ClxWlanPacketData objects. 
                                                                                     A maximum number of CLX_WLAN_MAXIMUM_PAYLOAD_CHUNKS_PER_DESCRIPTOR data chunk may be 
                                                                                     used to define a single PDU. If fewer #ClxWlanPacketData objects are used, then the 
                                                                                     "dataAddress" parameter of the chunk right after the last used chunk SHALL be set to NULL         */
        ClxSize           totalPacketSize;                                      /*!< The total size of the PDU described by this descriptor. Total size SHALL be the sum of the 
                                                                                     all data chunk sizes as defined in #body parameter                                                */
    } ClxWlanTxPacketDescriptor;


    /**
    A RX descriptor of type #ClxWlanRxPacketDescriptor represents a single PDU which is received from the WLAN driver. 
    An instance of this structure is only allocated by ClarinoxWLAN stack.
    RX descriptors are re-usable objects. They are allocated only once when a new virtual interface is created,
    and used for the data transfers in the RX path until the virtual interface is closed.

    Note: An object of type ClxWlanRxPacketDescriptor is valid only as long as the virtual interface is not closed.
    The virtual interface owns ClxWlanRxPacketDescriptor objects.
    */
    typedef struct ClxWlanRxPacketDescriptorStruct
    {
        ClxWlanPacketData    body[CLX_WLAN_MAXIMUM_PAYLOAD_CHUNKS_PER_DESCRIPTOR];       /*!< The PDU described by this descriptor, as an array of #ClxWlanPacketData objects. 
                                                                                             A maximum number of CLX_WLAN_MAXIMUM_PAYLOAD_CHUNKS_PER_DESCRIPTOR data chunk may be 
                                                                                             used to define a single PDU. If fewer #ClxWlanPacketData objects are used, then the 
                                                                                             "dataAddress" parameter of the chunk right after the last used chunk SHALL be set to NULL */
        ClxSize             totalPacketSize;                                            /*!< The total size of the PDU described by this descriptor. Total size SHALL be the sum of the
                                                                                             all data chunk sizes as defined in #body parameter                                        */
        ClxSize             packetAlignment;                                            /*!< The alignment of the first byte of the body (body[0].address[0]), in bytes                */
    } ClxWlanRxPacketDescriptor;
 

    /**
    ClxWlanNetworkInterface defines an interface between the ClarinoxWLAN stack and an external network stack (usually TCP/IP). 
    An object of this structure has to be allocated, and a pointer to the allocated object must be passed to the
    #clxWlanCreateInterface() API function. This object contains a number of function pointers that needs to be implemented.
    The implementation of these functions will be called by the ClarinoxWLAN stack when necessary.

    An object of this structure is bound to a specific virtual interface. Therefore, all TX/RX activities of this interface 
    are related to the owning virtual interface.

    Note : All functions implemented in this interface are called ONLY in the context of ClarinoxWLAN stack thread.
    */
    struct ClxWlanNetworkInterface
    {
        /**
        Initializes this interface. An implementation shall initializes all internal variables and initiate the connection 
        to the external network stack. This function is called when #clxWlanStartInterface() API function has been called 
        to create a new interface. If any error occurs during the initialization, an implementation must return an error 
        (any value other than CLX_SUCCESS). The returned error value will be directly returned by #clxWlanStartInterface() API function.

        If an implementation of this function returns CLX_SUCCESS, the interface is successfully initialized, but in disabled state. This means
        that the interface is not able to send and receive packets as yet.

        IMPORTANT : None of the network interface API functions may be called within the implementation of this function.
        An implementation shall wait until an indication of #CLX_WLAN_NETWORK_INTERFACE_INDICATION_STARTED is received before it is able
        to call any API function declared in this file.

        THIS FUNCTION MUST ALWAYS BE DEFINED.

        Note : An implementation may block if it cannot immediately initialize the interface. This will lead to the call to 
        #clxWlanStartInterface() API function to block as well.

        \param[ in ] thisObj A pointer to this object.
        \param[ in ] interfaceHandle API handle for the owning virtual interface. 
        This is the same handle which has been returned by #clxWlanCreateInterface() API function.
        \param[ in ] numberOfTxDescriptors Fixed number of TX descriptors which can be used for this virtual interface. 
        This value is determined by the WLAN driver.
        \param[ in ] numberOfRxDescriptors Fixed number of RX descriptors which can be used for this virtual interface. 
        This value is determined by the WLAN driver.
        
        \return CLX_SUCCESS if the initialization has been successful. In this case, the interface is started but disabled.
        The call-back function #handleIndication with the indication of type CLX_WLAN_NETWORK_INTERFACE_INDICATION_STARTED will be called shortly 
		to indicate the successful initialization of the stack.
        Any other value indicates an error.
        */
        ClxResult (*init) (_in_ struct ClxWlanNetworkInterface* thisObj,
                           _in_ ClxHandle                       interfaceHandle,
                           _in_ ClxSize                         numberOfTxDescriptors,
                           _in_ ClxSize                         numberOfRxDescriptors);


        /**
        This function is called when an indication has been received from the WLAN stack. An indication has the base type #ClxWlanNetworkInterfaceIndication.
        The indicationID member of #ClxWlanNetworkInterfaceIndication determined the actual type of the indication. Based on indicationID, the indication object
        can be cast to the correct type (if there is a special structure defined for that indication) and the other members of the indication can be accessed.

        NOTE : An implementation is able to call network interface API functions within this function, or after this function returns.

        THIS FUNCTION MUST ALWAYS BE DEFINED and it must handle the mandatory indications. However, optional indications might be not handled. The documentation
        of each indication specifies whether or the indication is mandatory to handle.

        \param[ in ] thisObj A pointer to this object.
        \param[ in ] indication A pointer to the indication object. The indicationID member of the object determines the actual type of the object.
        */
        void (*handleIndication) (_in_ struct ClxWlanNetworkInterface* thisObj, _in_ const ClxWlanNetworkInterfaceIndication* indication);


        /**
        Destroys the given interface. An implementation must destroy all of the internal variables, and the implementation object (if dynamically allocated).

        When this function is called, the indication #CLX_WLAN_NETWORK_INTERFACE_INDICATION_STOPPED has already been received and must have been handled by
        the implementation of #handleIndication function.

        THIS FUNCTION MUST ALWAYS BE DEFINED.

        IMPORTANT : NONE of the network interface API functions may be called within the implementation of this function.

        Note : This function will not be called if #(*init) has returned an error.

        \param[ in ] thisObj A pointer to this object.
        */
        void (*destroy) (_in_ struct ClxWlanNetworkInterface* thisObj);

        /**
        Locks the TX path to protect any activity in the TX path. This function must be defined if the TX path, 
        from the network stack to the WLAN stack, consists of more than one thread. Otherwise it is optional.
        It must be set to 0 when not defined. This function is called internally by the WLAN stack when protection is required. 
        An implementation may simply acquire a mutex. 

        Note: This function is required to be called by the parent interface when the TX path needs to be protected.

        IMPORTANT NOTE: If there is any TX or RX descriptor held by the interface at the time this function is called, 
        the implementation must make sure they are not accessed or used when this function returns.
        This is due to the fact that all descriptor objects will be deleted as soon as this function returns. 
        The implementation does not need to release all RX descriptors before this function returns.

        \param[ in ] thisObj A pointer to this object.
        */
        void (*lockTx) (_in_ struct ClxWlanNetworkInterface* thisObj);

        /**
        Unlocks the TX path. This function is only called when #lockTx has previously been called.

        This function must be defined only if #lockTx has been defined.

        \param[ in ] thisObj A pointer to this object.
        */
        void (*unlockTx) (_in_ struct ClxWlanNetworkInterface* thisObj);

        /**
        Locks the RX path to protect any activity in the TX path. This function must be defined if the RX path,
        from the WLAN stack to the network stack, consists of more than one thread. Otherwise it is optional.
        It must be set to 0 when not defined. This function is called internally by the WLAN stack when protection is required.
        An implementation may simply acquire a mutex.

        Note: This function is required to be called by the parent interface when the RX path needs to be protected.

        \param[ in ] thisObj A pointer to this object.
        */
        void (*lockRx) (_in_ struct ClxWlanNetworkInterface* thisObj);

        /**
        Unlocks the RX path. This function is only called when #lockRx has previously been called.

        This function shall be defined only if #lockRx has been defined.

        \param[ in ] thisObj A pointer to this object.
        */
        void (*unlockRx) (_in_ struct ClxWlanNetworkInterface* thisObj);

        /**
        Called when there are one or more packets received from the wireless link. The interface must use 
        #clxWlanGetPendingRxDescriptors() function to retrieve the packets. The interface must be in enabled
        state when this function is called.
        
        THIS FUNCTION MUST ALWAYS BE DEFINED.

        IMPORTANT NOTE: This call is RX protected. This means that #lockRx has been called before this function is called. 
        #unlockRx will be called when this function returns.

        Note: The function #clxWlanGetPendingRxDescriptors() may be called inside the implementation of this function, 
        alternatively it may called later on from the context of another thread. In the latter case, the RX path 
        must be manually locked by the interface before the function #clxWlanGetPendingRxDescriptors() is called. 

        \param[ in ] thisObj A pointer to this object.
        \param[ in ] totalNumOfPendingRxDescriptors Total number of RX packets which have been received but not obtained by the interface yet.

        \return TRUE if the implementation has already obtained all the pending RX descriptors using #clxWlanGetPendingRxDescriptors() 
        function within this call-back function.
        FALSE if the implementation will try to obtain the RX descriptors later on.
        */
        boolean CLX_WLAN_RX_PROTECTED (*rxPdusReceived) (_in_ struct ClxWlanNetworkInterface* thisObj,
                                                                     ClxSize                       totalNumOfPendingRxDescriptors);


        /**
        Called when there are one or more TX packets have been sent to the wireless link either with success or in error. 
        The interface must call #clxWlanGetCompletedTxDescriptorsID() function to retrieve the TX descriptors IDs.
        The interface must be in enabled state when this function is called.

        THIS FUNCTION SHALL MUST BE DEFINED.

        IMPORTANT NOTE: This call is TX protected. This means that #lockTx has been called before this function is called. 
        #unlockTx will be called when this function returns.

        Note: The function #clxWlanGetCompletedTxDescriptorsID() may be called inside the implementation of this function, 
        alternatively it may called later on from the context of another thread. In the latter case, The TX path 
        must be manually locked by the interface before the function #clxWlanGetCompletedTxDescriptorsID() is called. 

        \param[ in ] thisObj A pointer to this object.
        \param[ in ] totalNumOfCompletedTxDescriptors Total number of TX packets which have been sent.

        \return TRUE if the implementation has already obtained all the completed TX descriptors (by a call to #clxWlanGetCompletedTxDescriptorsID() 
        function within this call-back function.
        FALSE if the implementation will try to obtain the TX descriptors later on.
        */
        boolean CLX_WLAN_TX_PROTECTED  (*txPdusSendingComplete) (_in_ struct ClxWlanNetworkInterface* thisObj,
                                                                 ClxSize     totalNumOfCompletedTxDescriptors);
    };

    
    /**
    Retrieves a list of one or more TX descriptors. These descriptors may be used to store the TX packets for
    sending them to the wireless link.

    IMPORTANT NOTE: TX descriptor objects are allocated by the WLAN stack. They MUST NOT be deleted by the interface.

    IMPORTANT NOTE: TX path SHALL always be protected when this function is called.

    \param[ in ] interfaceHandle handle for the parent virtual interface as passed to #(*init) implementation of the interface.
    \param[ out ] list A pointer to an array of type ClxWlanTxPacketDescriptor* with the size of numberOfRequiredDescriptors. 
    On return, the array will contain the TX descriptor objects.
    \param[ in ] numberOfRequiredDescriptors maximum number of descriptors required by the interface. 
    The actual number of returned descriptors may be less than this number if there is not enough 
    free descriptors in the list of free descriptors.

    \return The actual number of TX descriptors returned in list. A value of 0 to numberOfRequiredDescriptors may be returned. 
    A value of 0 means there are no free descriptor available (not an error condition).
    */
    ClxSize clxWlanGetFreeNetworkTxDescriptors(_in_  ClxHandle                   interfaceHandle, 
                                               _out_ ClxWlanTxPacketDescriptor** list, 
                                               _in_  ClxSize                     numberOfRequiredDescriptors);

    /**
    Queues a packet, described by a TX descriptor, into the WLAN stack. Note that this function will not send the packet 
    to the wireless link. The function #clxWlanFlushPendingNetworkTxDescriptors() must be called to flush all 
    of the queued TX descriptors to the WLAN driver.
    
    NOTE: TX path protection is not required if the functions #clxWlanQueueNetworkTxDescriptor(), 
    #clxWlanQueueNetworkTxDescriptors(), and #clxWlanFlushPendingNetworkTxDescriptors() are always called in the context of 
    the same thread. Otherwise, TX path protection is mandatory.

    NOTE: When a descriptor is passed to this function to be queued, it is not free any more. 
    The interface must not modify the contents of the descriptor after it has been passed to this function.

    IMPORTANT NOTE: The interface MUST NOT pass an interface-allocated TX descriptor object to this function. 
    Only TX descriptor object obtained by a call to #clxWlanGetFreeNetworkTxDescriptors() function may be used.

    \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
    \param[ in ] txDescriptor A pointer to a TX descriptor object. This object has been obtained by a call to 
    #clxWlanGetFreeNetworkTxDescriptors() function, and has been filled with the information of the TX packet.
    */
    void clxWlanQueueNetworkTxDescriptor(_in_ ClxHandle                  interfaceHandle,
                                         _in_ ClxWlanTxPacketDescriptor* txDescriptor);

    /**
    Queues a list of packets, described by TX descriptors, into the WLAN stack. Note that this function will not send
    the packets to the wireless link. The function #clxWlanFlushPendingNetworkTxDescriptors() must be called to flush 
    all of the queued TX descriptors to the WLAN driver.
    
    NOTE: TX path protection is not required if the functions #clxWlanQueueNetworkTxDescriptor(), 
    #clxWlanQueueNetworkTxDescriptors(), and #clxWlanFlushPendingNetworkTxDescriptors() are always called in the context of 
    the same thread. Otherwise, TX path protection is mandatory.

    NOTE: When a descriptor is passed to this function to be queued, it is not free any more. 
    The interface must not modify the contents of the descriptor after it has been passed to this function.

    IMPORTANT NOTE: The interface MUST NOT pass an interface-allocated TX descriptor object to this function. 
    Only TX descriptor object obtained by a call to #clxWlanGetFreeNetworkTxDescriptors() function may be used.

    \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
    \param[ in ] txDescriptorsList A pointer to an array of type ClxWlanTxPacketDescriptor and the size numberOfDescriptors.
    \param[ in ] numberOfDescriptors Number of descriptors inside txDescriptorsList array.
    */
    void clxWlanQueueNetworkTxDescriptors(_in_                             ClxHandle interfaceHandle,
                                          _in_ ClxWlanTxPacketDescriptor** txDescriptorsList,
                                          _in_ ClxSize                     numberOfDescriptors);

    /**
    Flushes all pending (queued) TX descriptors to the WLAN driver. An implementation must call this function frequently 
    in order to make sure the packets will be sent to the wireless link. When the packets are sent to the wireless link 
    (either with success or in error), the associated descriptor objects will be marked as complete. These descriptors 
    will be free for reuse after #clxWlanGetCompletedTxDescriptorsID() function is called.

    NOTE: TX path protection is not required if the functions #clxWlanQueueNetworkTxDescriptor(), 
    #clxWlanQueueNetworkTxDescriptors(), and #clxWlanFlushPendingNetworkTxDescriptors() are always called in the context of 
    the same thread. Otherwise, TX path protection is mandatory.
    
    \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.

    \return The number of pending TX descriptors which could not be sent to the driver since the wireless link was not operational. 
    These descriptors are marked complete and will be free after the function #clxWlanGetCompletedTxDescriptorsID() is called. 
    Note that the interface call-back function txPdusSendingComplete inside #ClxWlanNetworkInterface WILL NOT be called for these descriptors.
    */
    ClxSize clxWlanFlushPendingNetworkTxDescriptors(_in_ ClxHandle interfaceHandle);

    /**
    Retrieves one or more pending RX descriptor. Each descriptor points to a packet received from the wireless link.

    IMPORTANT NOTE: RX descriptor objects are allocated by the WLAN stack. They MUST NOT be deleted by the interface.

    IMPORTANT NOTE: RX path MUST always be protected when this function is called.

    \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
    \param[ in ] rxDescriptorsList An array of type ClxWlanRxPacketDescriptor* and size numberOfDescriptors. 
    On return, it will contain zero or more RX descriptors.
    \param [ out ] numberOfDescriptors Contains the maximum number of RX descriptors which the caller is prepared to obtain. 
    On return, it will contain the actual number of RX descriptors returned in the rxDescriptorsList.
    \return The current number of pending RX descriptors upon this function's return. This number does not include 
    the RX descriptors just returned in rxDescriptorsList.
    */
    ClxSize clxWlanGetPendingRxDescriptors(_in_     ClxHandle interfaceHandle,
                                            _out_   ClxWlanRxPacketDescriptor** rxDescriptorsList, 
                                            _inout_ ClxSize* numberOfDescriptors);

    /**
    Retrieves the list of completed TX descriptor IDs. The descriptors returned by this function will be marked as free, 
    and can be obtained again by a call to #clxWlanGetFreeNetworkTxDescriptors() function for re-use.

    IMPORTANT NOTE: TX path SHALL always be protected when this function is called.

    \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
    \param[ in ] txCompletedDescriptorsIDsList An array of type void* and size txCompletedDescriptorsIDListSize. 
    On return, it will contain zero or more TX descriptor user-defined IDs  as defined by the "userID" parameter of 
    ClxWlanTxPacketDescriptor objects
    \param [ out ] txCompletedDescriptorsIDListSize Contains the maximum of number of TX descriptor IDs which the caller is prepared to obtain. 
    On return, it will contain the actual number of TX descriptors IDs returned in txCompletedDescriptorsIDsList.
    \return The current number of pending TX descriptors upon this function's returned. This number of does not include 
    the TX descriptors just returned in txCompletedDescriptorsIDsList.
    */
    ClxSize clxWlanGetCompletedTxDescriptorsID(_in_             ClxHandle interfaceHandle,
                                               _out_   void**   txCompletedDescriptorsIDsList,
                                               _inout_ ClxSize* txCompletedDescriptorsIDListSize);

    /**
    Releases the used RX descriptors back to the WLAN stack. All RX descriptors returned by a call to #clxWlanGetPendingRxDescriptors() 
    function shall be released after the interface is no longer using them.

    IMPORTANT NOTE: RX PATH MUST always be protected when this function is called.

    \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
    \param[ in ] rxDescriptorsList A pointer to an array of type ClxWlanRxPacketDescriptor and the size numberOfDescriptors.
    \param[ in ] numberOfDescriptors Number of descriptors inside rxDescriptorsList array.
    */
    void clxWlanReleaseRxDescriptors(_in_ ClxHandle interfaceHandle,
                                     _in_ ClxWlanRxPacketDescriptor** rxDescriptorsList, 
                                     _in_ ClxSize numberOfDescriptors);

    /**
    Retrieves a list of one or more TX Buffers. These buffers may be used to store the TX packets for
    sending them to the wireless link.

    IMPORTANT NOTE: TX buffer objects are allocated by the WLAN stack. They MUST NOT be deleted by the interface.

    IMPORTANT NOTE: TX path SHALL always be protected when this function is called.

    \param[ in ] interfaceHandle handle for the parent virtual interface as passed to #(*init) implementation of the interface.
    \param[ out ] buffersList A pointer to an array of type ClxWlanBuffer* with the size of numberOfRequiredBuffers. 
    On return, the array will contain the TX buffer objects.
    \param[ in ] numberOfRequiredBuffers maximum number of TX buffers required by the interface. 
    The actual number of returned buffers may be less than this number if there is not enough 
    free buffers in the queue of free TX buffers.

    \return The actual number of TX buffers returned in list. A value of 0 to numberOfRequiredBuffers may be returned. 
    A value of 0 means there are no free buffer available (not an error condition).
    */
    ClxSize clxWlanGetTxBuffers(_in_  ClxHandle       interfaceHandle, 
                                _out_ ClxWlanBuffer** buffersList, 
                                _in_  ClxSize         numberOfRequiredBuffers);


#ifdef __cplusplus
}
#endif

#endif  // _WlanNetworkStackInterface_

