#ifndef _A2l_h_
#define _A2l_h_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame 
* File                A2l.h
* Description         Declares Clarinox Application Adaptation Layer related 
*                     types and macros
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


/**
Duplicates a service handle by incrementing its internal reference counter. This function can be used in multi-threaded applications in order to provide a safe mechanism for
deleting the service object.

NOTE : When a handle is created, its reference counter is set to 1. Every time clxDuplicateHandle() is called, the reference counter of the handle is incremented by 1.
On the other hand, every time the handle is passed to clxCloseHandle(), and clxCloseHandle returns CLX_SUCCESS, the reference counter is decremented by 1. The service object
is physically deleted only when the reference counter reaches 0.

The first time clxCloseHandle() returns CLX_SUCCEESS, the handle is in Closed state (it is fully closed). But, the corresponding service will not be deleted until the reference count
reaches 0.

NOTE : It is not possible to duplicate a handle which is not in Active state (A handle is not in Active state if it has already closed or is being closed).

\param[ in ] handle The handle to be duplicated.

\return CLX_SUCCESS                 The handle was duplicated successfully. 
        CLX_ERROR_INVALID_HANDLE    The handle specified is not valid (e.g. it is not in Active state)
*/
ClxResult clxDuplicateHandle(_in_ ClxHandle handle);

/**
Closes a service handle (of type ClxHandle). 

The service associated to this handle enters closed state if this function returns CLX_SUCCESS. A service in closed state does any accept any command from the application.
Also, it will not send any indication to the application any longer.

NOTE : Assume clxDuplicateHandle() has been called N times for a service handle. Then for that handle, clxCloseHandle() shall be called (N + 1) times, and clxCloseHandle() shall have returned CLX_SUCCESS for (N + 1) times
before the service is closed and successfully deleted.

NOTE : When the service is physically deleted from the memory, any reference to the handle may result in undefined behavior.

Pending Commands:
If there is any pending commands (in either blocking or non-blocking mode), they will be cancelled only and only if the service support this feature. 
If this feature is not supported, clxCloseHandle() will return CLX_ERROR_HANDLE_COMMANDS_PENDING if there is any commands pending.

IMPORTANT : When clxCloseHandle() operation is progress, other threads are NOT allowed to call new API functions for this handle. This may result in undefined behaviour or a crash.
In multi-threaded designs, the application shall make sure this will not happen by setting a flag, or modifying the state of a state machine, in a manner that implies that that handle is being closed, 
before calling clxCloseHandle().

NOTE : Please refer to documentation of a service API set in order to find out whether or not the service supports this feature. If the documentation does not mention support of this feature,
then it must be assumed that this feature is not supported.

NOTE : When services, which do not support cancelling the pending commands, are used, it is the application responsibility to wait until all pending commands
are completed peacefully, before calling clxCloseHandle().

The function may be used with any service handle which is of type ClxHandle.

\param[ in ] handle a service handle which is to be closed.

\return CLX_SUCCESS                         The handle was closed successfully. 
                                            The service might have been also physically deleted from the memory if all duplicates, along with the original handle, have also been closed.
        CLX_ERROR_INVALID_HANDLE            The handle specified is not valid (it is in a wrong state, or it is already closed)
        CLX_ERROR_HANDLE_COMMANDS_PENDING   The handle does not support cancellation of pending commands, and one or more commands are pending at the time of calling this function.
        CLX_ERROR_INTERNAL_ERROR            Internal error occurred.
*/
ClxError clxCloseHandle(_in_ ClxHandle handle);

/**
Stores an application specific "data" in the given service handle for later use.
\code
Comm::Comm(ClxStack stack, DataFormat currentDataFormat, u1 ExpansionOptionEnabled, boolean server)
{
    // create a SPP profile handle and store "this" pointer (Comm object) in the handle. 
    // This information could be retrieved later on by using #clxGetHandleUserData
    spp_ = clxSppCreate(stack, NULL, "SPPCommPort", ClxServer, messageHandler, ClxAutomaticSecurity);
    clxSetHandleUserData(spp_, this);
}
\endcode
\param[ in ] handle The service handle that the data will be stored in.
\param[ in ] data user data that will be stored.
*/
void clxSetHandleUserData (ClxHandle handle, void* data);

/**
Returns the ClarinoxBlue repository revision.
\code
boolean Comm::messageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
{
    // retrieve saved Comm object in the message handler call back stored by using #clxSetHandleUserData
    Comm* this_ = reinterpret_cast<Comm*>(clxGetHandleUserData(serviceHandle));
}
\endcode
\return the repository revision as an unsigned integer number. 
*/
void* clxGetHandleUserData (ClxHandle handle);

#ifdef __cplusplus
}
#endif


#endif // _A2l_h_
