#ifndef _ClxCommon_
#define _ClxCommon_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxCommon.h
* Description         ClarinoxSoftFrame Common header file
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include <string.h>
#include <errno.h>
#include <assert.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stddef.h>

#include "ClxTypes.h"


/*
Calculates a/b. If the result is not an integer, the result is rounded up to next integer value.
*/
#define CLX_DIVIDE_AND_ROUND(a, b)        ((a + b - 1) / b)

/* 
Defines a data buffer of the given size (in byes), which is guaranteed to be 16bit aligned in the memory.
*/
#define CLX_ALIGNED_BUFFER_2(name, size)  u2 name[(size + 1)/2] 

/* 
Defines a data buffer of the given size (in byes), which is guaranteed to be 32bit aligned in the memory.
*/
#define CLX_ALIGNED_BUFFER_4(name, size)  u4 name[(size + 3)/4] 

/* 
The qualifier _Clx16bitAligned_ is added to a buffer pointer definition to specify that the variable points to
a buffer in the memory which is at least 16bit aligned:
*/
#define _Clx16bitAligned_

/* 
The qualifier _Clx32bitAligned_ is added to a buffer pointer definition to specify that the variable points to
a buffer in the memory which is at least 32bit aligned:
*/
#define _Clx32bitAligned_

/*
Returns a value with only the specified bit set, and all other bits are reset. The total number
of bits in the return value depends on the type casting used:
*/
#define CLX_SET_BIT(bit)  (1 << (bit))


/**
Returns a value of equal to or larger than originalSize which is aligned with (divisible by) requiredAlignment.
IMPORTANT : requiredAlignment MUST be a power of 2 (e.g. 2, 4, 16, 256, 2048, ...).
*/
#define CLX_ALIGNED_SIZE(originalSize, requiredAlignment)   (originalSize + ((requiredAlignment - (originalSize & (requiredAlignment-1))) & (requiredAlignment-1)))

#define GET_16BIT_ALIGNED_SIZE(size)  CLX_ALIGNED_SIZE(size, 2)
#define GET_32BIT_ALIGNED_SIZE(size)  CLX_ALIGNED_SIZE(size, 4)

/*
At this moment, we only support 32bit platforms:
*/
#define GET_PLATFORM_ALIGNED_SIZE(size)   GET_32BIT_ALIGNED_SIZE(size)

#ifndef MAX
#   define MAX( a, b ) ( ( ( a ) > ( b ) ) ? ( a ) : ( b ) )
#endif
#ifndef MIN
#   define MIN( a, b ) ( ( ( a ) < ( b ) ) ? ( a ) : ( b ) )
#endif


/*
Returns a value which uniquely identifies the type of a C structure at run time.
*/
#define CLX_C_STRUCTURE_TYPE(StructureName)              (void*)&_ClxCObjectType_##StructureName##_

/*
Defines RunTime Type Information (RTTI) for a C structure and associates it to the structure. 
RTTI is a unique number across the application which is used to identify the type of a C object at run time.
The macro CLX_C_STRUCTURE_TYPE returns the type for a structure which has been associated RTTI.
*/
#define CLX_DECLARE_C_OBJECT_RTTI(StructureName)                 extern const u1 _ClxCObjectType_##StructureName##_

/*
Instantiates RunTime Type Information for a C structure which is associated RTTI using the macro CLX_DECLARE_C_OBJECT_RTTI.
This macro must be called in a C or C++ file in order to assign a unique number to the C structure at link time. The
number can then be used to identify the type of a C object at run time.

Example:

Define a base structure which RTTI capability as follows:

struct BaseStruct
{
    void* rtti;
};

Now, you can define different structures of different types which all derive from BaseStruct as follows:

struct Struct1
{
    struct BaseStruct;      // Simulation of C++ inheritance in C. SHALL be the very first member in the derived structure
};

struct Struct2
{
    struct BaseStruct;      // Simulation of C++ inheritance in C. SHALL be the very first member in the derived structure
};

...

struct StructN
{
    struct BaseStruct;      // Simulation of C++ inheritance in C. SHALL be the very first member in the derived structure
};

Now, define RTTI for each structure:

CLX_DECLARE_C_OBJECT_RTTI(Struct1);
CLX_DECLARE_C_OBJECT_RTTI(Struct2);
...
CLX_DECLARE_C_OBJECT_RTTI(StructN);

Now, in a C or C++ file instantiate RTTI for these structures:
CLX_INSTANTIATE_C_OBJECT_RTTI(Struct1);
CLX_INSTANTIATE_C_OBJECT_RTTI(Struct2);
...
CLX_INSTANTIATE_C_OBJECT_RTTI(StructN);

Now, you can identify the type of an object at run time as long as you know the object derives from BaseStruct:

void getObjectType(BaseStruct* object)
{
    if (object->rtti == CLX_C_STRUCTURE_TYPE(Struct1))
    {
        printf ("object is of type Struct1\n");
    }
    else if (object->rtti == CLX_C_STRUCTURE_TYPE(Struct2))
    {
        printf ("object is of type Struct2\n");
    }
    ...
    else if (object->rtti == CLX_C_STRUCTURE_TYPE(StructN))
    {
        printf ("object is of type StructN\n");
    }
}
*/
#define CLX_INSTANTIATE_C_OBJECT_RTTI(StructureName)     const u1 _ClxCObjectType_##StructureName##_ = 0



extern void handleBlackBox ( s4 moduleID, s4 line );

#if !defined( CLX_MODULE_ID)
#define CLX_MODULE_ID 0xFFFFFFFF
#endif

#if !defined( BLACKBOX )
#define BLACKBOX                                                                \
do                                                                              \
{                                                                               \
    int line = __LINE__;                                                        \
    clxSleep( 1000 );                                                           \
    handleBlackBox( CLX_MODULE_ID, line );    /* report and terminate */        \
}while( 0 )

#endif

#if !defined(BLACKBOX_IF)
#define BLACKBOX_IF(x) if(x) BLACKBOX
#endif

#ifdef __cplusplus
extern "C" {
#endif


/**
Returns the ClarinoxSoftFrame version.
\return A function-allocated buffer containing the version as a null-terminated string. The buffer
must NOT be modified by the caller.
*/
extern const s1* clxGetClarinoxSoftFrameVersion(void);

/**
Returns the ClarinoxSoftFrame repository revision.
\return the repository revision as an unsigned integer number. 
*/
extern u4 clxGetClarinoxSoftFrameRevision(void);

/**
Returns the ClarinoxTasks version.
\return A function-allocated buffer containing the version as a null-terminated string. The buffer
must NOT be modified by the caller.
*/
extern const s1* clxGetClarinoxTasksVersion(void);


#ifdef __cplusplus
}
#endif

#include "ClxMemoryPoolset.h"
#include "ClarinoxConst.h"
#include "ClarinoxErrorCodes.h"
#include "ClxConsole.h"
#include "ClxSemaphore.h"
#include "ClxMutex.h"
#include "ClxThread.h"
#include "ClxTime.h"
#include "ConsoleUIEngine.h"
#include "ClxPrivateMemoryPool.h"
#include "A2L.h"
#include "ClxDataConversion.h"
#include "ClxConfigParams.h"
#include "TerminalEmulator.h"
#include "ClxCQueueable.h"
#include "ClxQueueTemplate.h"
#include "ClxPrivateMemoryPool.h"


/** \file ClxCommon.h
    \brief ClxCommon.h contains the ClarinoxSoftFrame common include files
*/


#endif    // _ClxCommon_
