#ifndef _ClxConfigParams_
#define _ClxConfigParams_

/*******************************************************************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxConfigParams.h
* Description         ClarinoxSoftFrame Configuration parameters common structure
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************************************************************/



#define CLX_CONFIG_TYPE(ParamStruct)                ((void*)&C_clx##ParamStruct##_ConfigType_)

#define CLX_DEFINE_CONFIG_TYPE(ParamStruct)         extern ParamStruct C_clx##ParamStruct##_ConfigType_

#define CLX_INSTANTIATE_CONFIG_TYPE(ParamStruct)    ParamStruct C_clx##ParamStruct##_ConfigType_

#define CLX_IS_CONFIG_TYPE(paramVar, ParamStruct)   (paramVar.paramTypeID == (void*)&C_clx##ParamStruct##_ConfigType_)



#ifdef __cplusplus
extern "C" {
#endif

/**
 * ClxConfigParamStruct which contains parameter name, parameter TYPEID and boolean flag called processed.
 */
typedef struct ClxConfigParamStruct
{
    const s1*                       paramName;
    void*                           paramTypeID;
    struct ClxConfigParamStruct*    next;
    boolean                         processed;
} ClxConfigParam;

/**
 * ClxConfigInteger which stores the parameters of type integer type.
 */
typedef struct ClxConfigIntegerStruct
{
    ClxConfigParam    paramInfo;
    s4                paramValue;
} ClxConfigInteger;

/**
 * ClxConfigUnsignedStruct which stores the parameters of type unsigned integer type. 
 */
typedef struct ClxConfigUnsignedStruct
{
    ClxConfigParam    paramInfo;
    u4                paramValue;
} ClxConfigUnsigned;

/**
 * ClxConfigReal which stores the parameters of type real/double. 
 */

#if defined(CLX_FLOATING_POINT_SUPPORTED)

typedef struct ClxConfigRealStruct
{
    ClxConfigParam    paramInfo;
    r8                paramValue;
} ClxConfigReal;
#endif


/**
 * ClxConfigString which stores the parameters of type string. 
 */
typedef struct ClxConfigStringStruct
{
    ClxConfigParam   paramInfo;
    const s1*        paramValue;
} ClxConfigString;

/**
 * ClxConfigData which contains a pointer to the parameter data (of type unsigned char) and size of the data (of type integer). 
 */
typedef struct ClxConfigDataStruct
{
    ClxConfigParam    paramInfo;
    u1*               paramData;
    ClxSize           paramDataSize;
} ClxConfigData;

/**
 * Array of configurations, array members shall be of the same type.
 * IMPORTANT : The parameter array shall be cast to the correct type (type of the members) before being browsed for members.
 */
typedef struct ClxConfigArrayStruct
{
    ClxConfigParam        paramInfo;
    void*                 array;
    ClxSize               numOfParams;
} ClxConfigArray;

/**
 * ClxConfigList which contains a pointer(of type ClxConfigParam) to the list of  objects which are of different types 
 * and size(of type integer) of the parameters in the list.
 */
typedef struct ClxConfigListStruct
{
    ClxConfigParam        paramInfo;
    ClxConfigParam*       first;
    ClxConfigParam*       last;
} ClxConfigList;

/*******************************************************************************************************************************
 * Initializes the subsequent member of parentList with the given object of type #ClxConfigInteger.
 * 
 * \param[ in ] object a pointer to the wireless stack object of type ClxConfigInteger  
 * \param[ in ] paramName string pointer to name of the parameter.
 * \param[ in ] paramValue a pointer to value of the parameter of type unsigned integer
 * \param[ in ] parentList pointer to parent List of type #ClxConfigList which we are willing to add
 *****************************************************************************************************************************/
extern void clxConfigInitIntegerParam   (ClxConfigInteger* object, const s1* paramName, s4 paramValue, ClxConfigList* parentList);

/*******************************************************************************************************************************
 * Initializes the paramList with the given parameter of type ClxConfigUnsigned.
 * 
 * \param[ in ] object pointer to ClxConfigUnsigned
 * \param[ in ] paramName pointer to parameter name
 * \param[ in ] paramValue parameter value 
 * \param[ in ] parentList pointer to ClxConfigList which we are willing to fill
 *****************************************************************************************************************************/
extern void clxConfigInitUnsignedParam   (ClxConfigUnsigned* object, const s1* paramName, u4 paramValue, ClxConfigList* parentList);

/*******************************************************************************************************************************
 * Initializes the subsequent member of parentList with the given object of type #ClxConfigReal.
 *
 * \param[ in ] object a pointer to wireless stack object of type ClxConfigReal  
 * \param[ in ] paramName a string pointer to name of the parameter.
 * \param[ in ] paramValue a pointer to value of the parameter of type double
 * \param[ in ] parentList pointer to parent List of type #ClxConfigList which we are willing to add
 *****************************************************************************************************************************/
#if defined(CLX_FLOATING_POINT_SUPPORTED)
extern void clxConfigInitRealParam      (ClxConfigReal* object, const s1* paramName, r8 paramValue, ClxConfigList* parentList);
#endif

/*******************************************************************************************************************************
 * Initializes the subsequent member of parentList with the given object of type #ClxConfigString.
 *
 * \param[ in ] object a pointer to wireless stack object of type ClxConfigString  
 * \param[ in ] paramName a string pointer to name of the parameter.
 * \param[ in ] paramValue a string pointer to value of the parameter 
 * \param[ in ] parentList pointer to parent List of type #ClxConfigList which we are willing to add
 *****************************************************************************************************************************/
extern void clxConfigInitStringParam    (ClxConfigString* object, const s1* paramName, const s1* paramValue, ClxConfigList* parentList);

/*******************************************************************************************************************************
 * Initializes the subsequent member of parentList with the given object of type #ClxConfigString.
 *
 * \param[ in ] object a pointer to wlan object of type ClxConfigData  
 * \param[ in ] paramName a string pointer to name of the parameter.
 * \param[ in ] paramData a pointer to data of the parameter of type unsigned char
 * \param[ in ] paramDataSize size of the parameter data of type unsigned integer
 * \param[ in ] parentList pointer to parent List of type #ClxConfigList which we are willing to add
 *****************************************************************************************************************************/
extern void clxConfigInitDataParam      (ClxConfigData* object, const s1* paramName, u1* paramData, ClxSize paramDataSize, ClxConfigList* parentList);

/*******************************************************************************************************************************
 * Initializes the subsequent member of parentList with the given object of type #ClxConfigArray.
 *
 * \param[ in ] object a pointer to wlan object of type ClxConfigArray  
 * \param[ in ] paramName pointer to paramName of type string
 * \param[ in ] array a pointer to a list of type a specific type.
 * \param[ in ] numOfParams number of parameters of size and type of unsigned integer
 * \param[ in ] parentList pointer to parent List of type #ClxConfigList which we are willing to add
 *****************************************************************************************************************************/
extern void clxConfigInitParamsArray    (ClxConfigArray* object, const s1* paramName, void* array, ClxSize numOfParams, ClxConfigList* parentList);

/*******************************************************************************************************************************
 * Initializes the first member of the List with the given object of type #ClxConfigList.
 *
 * \param[ in ] object a pointer to wlan object of type ClxConfigList  
 * \param[ in ] paramName a string pointer to parameter name.
 * \param[ in ] parentList pointer to parent List of type #ClxConfigList which we are willing to add.
 *****************************************************************************************************************************/
extern void clxConfigInitParamsList     (ClxConfigList* object, const s1* paramName, ClxConfigList* parentList);

/*******************************************************************************************************************************
 * Searches in a configuration list for a specific configuration parameter.
 *
 * \param[ in ] list The list in which the specific parameter is to be searched for. 
 * \param[ in ] paramName The name of the parameter to be searched for. This argument cannot be NULL.
 * \param[ in ] paramType Type of the parameter to be searched for. This is an optional argument. If set to NULL, the list will
 be searched for a parameter with the given name, but any type.
 * return A pointer to the found parameter object. If no parameter with the specified name and (optionally) type is found, the 
 return value will be NULL.
 *****************************************************************************************************************************/
extern ClxConfigParam* clxFindConfigParam(const ClxConfigList* list, const s1* paramName, void* paramType = NULL);

/*
Definitions of the configuration parameters. Used internally. 
*/
CLX_DEFINE_CONFIG_TYPE(ClxConfigInteger);
CLX_DEFINE_CONFIG_TYPE(ClxConfigUnsigned);

#if defined (CLX_FLOATING_POINT_SUPPORTED)
CLX_DEFINE_CONFIG_TYPE(ClxConfigReal);
#endif

CLX_DEFINE_CONFIG_TYPE(ClxConfigString);
CLX_DEFINE_CONFIG_TYPE(ClxConfigData);
CLX_DEFINE_CONFIG_TYPE(ClxConfigArray);
CLX_DEFINE_CONFIG_TYPE(ClxConfigList);

#ifdef __cplusplus
}
#endif



#ifdef __cplusplus

class CppClxConfigList : public ClxConfigList
{
public:
    CppClxConfigList(const s1* name, ClxConfigList* parentList)
    {
        clxConfigInitParamsList(this, name, parentList);
    }
};

class CppClxConfigInteger : public ClxConfigInteger
{
public:
    CppClxConfigInteger(const s1* name, ClxConfigList* parentList)
    {
        clxConfigInitIntegerParam(this, name, 0, parentList);
    }
};

class CppClxConfigString : public ClxConfigString
{
public:
    CppClxConfigString(const s1* name, ClxConfigList* parentList)
    {
        clxConfigInitStringParam(this, name, 0, parentList);
    }
};


#endif // #ifdef __cplusplus


#endif    // _ClxConfigParams_

