#ifndef _ClxMemoryPoolset_
#define _ClxMemoryPoolset_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxMemoryPoolset.h
* Description         Clarinox memory poolset definitions
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

/**
Define DEFAULT_MEMORY_POOL at the beginning of your application's main file, if you wish to use
the default memory pool provided by Clarinox SoftFrame for the operating system and CPU on which the application runs.
This macro will select the best poolset for the current system.
*/
#define DEFAULT_MEMORY_POOL                                                     \
    BufferDetails*  userSuppliedMemoryPool      = 0;                            \
    u4              userSuppliedMemoryPoolSize  = 0;

/**
Define USER_SUPPLIED_MEMORY_POOL at the beginning of your application's main file (or BSP), if you wish to define
your own set of memory pools. The following is an example of how to define a poolset:

\code

#include "ClxMemoryPoolset.h"

//   Below we will define a poolset with 21 pools of sized 4 bytes to 1.2 MB
//   At each row, the first number is the size of memory blocks in the corresponding memory pool
//   The second number is the number of memory blocks in the corresponding memory pool.
//   Note that the size of buffers in all pools are divisible by 4 (4, 8, 12, ... 12000000).

USER_SUPPLIED_MEMORY_POOL(
    { 4, 1200    }, 
    { 8, 400     }, 
    { 12, 1200   },
    { 16, 1200   }, 
    { 20, 200    }, 
    { 24, 400    },
    { 28, 400    }, 
    { 36, 600    }, 
    { 40, 50     },
    { 52, 20     }, 
    { 60, 40     }, 
    { 108, 60    },
    { 116, 15    }, 
    { 160, 20    }, 
    { 200, 20    },
    { 520, 8     },
    { 1068, 8    }, 
    { 1424, 20   }, 
    { 12000, 8   }, 
    { 2800000, 1 }, 
    { 12000000, 1}, );
\endcode
*/
#define USER_SUPPLIED_MEMORY_POOL( ... )                                        \
    BufferDetails userSuppliedMemoryPool_[] = { __VA_ARGS__ };                  \
    u4 userSuppliedMemoryPoolSize = sizeof( userSuppliedMemoryPool_ ) / sizeof( BufferDetails );\
    BufferDetails* userSuppliedMemoryPool= userSuppliedMemoryPool_;


#ifdef __cplusplus
extern "C" {
#endif


typedef struct 
{
    u4 size;
    u4 count;
} BufferDetails;


// _____________________________________________________________________________
//
// Function prototypes for memory allocating, re-allocating and de-allocating functions 
// _____________________________________________________________________________
//

typedef void* (*ClxAllocMemFunc) (u2 moduleID, u4 line, u4 size );
typedef void* (*ClxReallocMemFunc) (u2 moduleID, u4 line, void* mem, u4 size );
typedef void (*ClxFreeMemFunc) (void* ptr);



// _____________________________________________________________________________
//
// ClarinoxSoftFrame Memory PoolSet Access Functions 
// _____________________________________________________________________________
//

/**
Allocates a block of memory in the poolset
\param[ in ] moduleID The identifier of the module (file) which is calling this function
\param[ in ] line The line number of the code which is calling this function
\param[ in ] size The size of the memory block to be allocated
*/
extern void* clxPoolsetAlloc (u2 moduleID, u4 line, u4 size );

/**
Reallocates a block of memory in the poolset
\param[ in ] moduleID The identifier of the module (file) which is calling this function
\param[ in ] line The line number of the code which is calling this function
\param[ in ] mem A pointer to a block of memory which is already allocated in the poolset
\param[ in ] size The size of the new memory block to be allocated
\remarks
clxRealloc is used when a larger block of memory is needed for a buffer. clxRealloc allocates a larger block, copies the content of the buffer to the new block, and frees the old block. 
*/
extern void* clxPoolsetRealloc (u2 moduleID, u4 line, void* mem, u4 size );
/** \endcond */

/**
Frees a block of memory previously allocated by clxAlloc, or clxRealloc
\param[ in ] ptr a pointer to the memory block which is to be freed
\remarks
You should not use clxFree to free a memory block allocated by \b new operator, or \b NEW macro. 
*/
extern void clxPoolsetFree (void* ptr);

/**
Confirms the integrity of the memory pool elements. The check is done
against memory overruns, underruns and link list integrity. The function
generates an exception if encounters an integrity failure.
*/
extern void clxCheckMemoryCorruption();


// _____________________________________________________________________________
//
// Private heap support for memory poolset
// All the following functions SHALL be defined in the BSP
// _____________________________________________________________________________
//

/**
Initializes the private heap. When this function returns, Clarinox SoftFrame assumes the complete
ownership of the heap. No information about the heap is supposed to be reported to SoftFrame.
This function must be implemented, even if there is no need for initializing the heap.
*/
extern void clxMemoryInitPrivateHeap ();

/**
Destroys the private heap. When this function is called, Clarinox SoftFrame has already given up the ownership
of the heap.

This function must be implemented, even if there is no need for destroying the heap.
*/
extern void clxMemoryDestroyPrivateHeap ();

/**
Allocates a contiguous memory block in the private heap. If no memory block with the size requested is available,
this function must return NULL.

\param[ in ] allocationSize size of the contiguous memory block requested.

\return A pointer to the allocated memory block of size \p allocationSize.
*/
extern void* clxMemoryAllocateFromPrivateHeap (unsigned int allocationSize);

/**
Frees a block of memory in the private heap. The memory block must have been allocated by clxMemoryAllocateFromPrivateHeap function.

\param[ in ] buff A pointer to the buffer which is to be freed. This pointer must have been returned by clxMemoryAllocateFromPrivateHeap.
*/
extern void clxMemoryFreeFromPrivateHeap (void* buff);


#ifdef __cplusplus
}
#endif


#ifdef __cplusplus

/* Used internally */
class ClxPoolsetUniqueIdentifier {};

/**
Used by template classes which require an "Allocator" class. When used, The standard C heap
(malloc and free) will be used.
*/
class ClxStandardHeapAllocator
{
public:
    void* alloc(size_t size) const;
    void free(void* buf) const;
    void* realloc(void* buf, size_t size) const;

#if defined(CLX_DEBUG)
    void* alloc(size_t size, u2 moduleID, u4 line) const
    {
        return alloc(size);
    }
    void* realloc(void* buf, size_t size, u2 moduleID, u4 line) const
    {
        return realloc(buf, size);
    }
#endif    
};


/**
Used by template classes which require an "Allocator" class. When used, the memory
will be allocated from ClarinoxSoftFrame memory poolset.
*/
class ClxSoftframePoolsetAllocator
{
public:
    void* alloc(size_t size) const
    {
        return clxPoolsetAlloc(0, 0, (u4)size);
    }
    void free(void* buf) const
    {
        clxPoolsetFree(buf);
    }
    void* realloc(void* buf, size_t size) const
    {
        return clxPoolsetRealloc(0, 0, buf, (u4)size);
    }

#if defined(CLX_DEBUG)
    void* alloc(size_t size, u2 moduleID, u4 line) const
    {
        return clxPoolsetAlloc(moduleID, line, (u4)size);
    }
    void* realloc(void* buf, size_t size, u2 moduleID, u4 line) const
    {
        return clxPoolsetRealloc(moduleID, line, buf, (u4)size);
    }
#endif    
};


/**
Used by template classes which require an "Allocator" class. When used, a static memory with
fixed size will be used. Cannot be used when a dynamic allocation of memory buffer is required.
*/
template<size_t MAX_SIZE>
class ClxStaticMemoryAllocator
{
private:
    CLX_ALIGNED_BUFFER_4(buf_, MAX_SIZE);
public:
    void* alloc(size_t size) const
    {
        if (size == MAX_SIZE)
        {
            return (void*)buf_;
        }
        else
        {
            return NULL;
        }
    }
    void free(void* buf) const
    {}

    void* realloc(void* buf, size_t size) const
    {
        return NULL;
    }

#if defined(CLX_DEBUG)
    void* alloc(size_t size, u2 moduleID, u4 line) const
    {
        return alloc(size);
    }
    void* realloc(void* buf, size_t size, u2 moduleID, u4 line) const
    {
        return realloc(buf, size);
    }
#endif    
};

#endif // __cplusplus


#endif    // _ClxMemoryPoolset_

