#ifndef _ClxPrivateMemoryPool_
#define _ClxPrivateMemoryPool_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                ClxPrivateMemoryPoolHandle.h
* Description         Private Single-sized Memory Pool
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif


/**
A Private Memory Pool is a single contiguous chunk of memory which is divided into one or buffers of fixed size. Each fixed-size buffer in the pool may
be allocated and freed individually. After initialization of a private pool, neither the number of buffers in the pool nor the size of each buffer can be modified.

IMPORTANT : Access to a private memory pool is NOT thread safe. If the private pool is to be accessed by two or more threads, a multi-thread access
protection mechanism (e.g. such as a mutex) shall be applied externally by the application.
*/
typedef size_t ClxPrivateMemoryPoolHandle;

/**
Creates a private memory pool object and returns a handle to it.

\param[ in ] numberOfBuffers Number of fixed-size buffers which the private pool will contain.
\param[ in ] maxAllocationSize The maximum size which can be allocated using the function clxAllocatePrivateMemoryPoolBuffer().
\param[ in ] allocMemFunc The function which is called to allocate the entire private memory pool, as a single contiguous chunk of memory. This argument can be NULL. if set to NULL,
the memory for the private pool will be allocated from the ClarinoxSoftFrame poolset (In this case, ClarinoxSoftframe shall already have been initialized).

\return A handle to the private memory pool.
*/
ClxPrivateMemoryPoolHandle clxCreatePrivateMemoryPool(u4 numberOfBuffers, 
													  u4 maxAllocationSize,
													  ClxAllocMemFunc allocMemFunc);

/**
Allocates memory from a private pool set, and returns a pointer to the allocated buffer in the memory. 
If there is no free buffer in the pool, or if the requested size is more than the size of each buffer (as passed to #clxCreatePrivateMemoryPool),
this function will return NULL.

\param[ in ] pool The handle to the private pool as returned by #clxCreatePrivateMemoryPool.
\param[ in ] size The size of the buffer required to be allocated. This size cannot be more than the maxBufferSize argument passed to #clxCreatePrivateMemoryPool.
\return A pointer to the allocated buffer in the private pool. Will be NULL if there is no free buffer available, or the size argument is invalid.
*/
void* clxAllocatePrivateMemoryPoolBuffer(ClxPrivateMemoryPoolHandle poolHandle, u4 size);

/**
Frees a buffer previously allocated in the private pool by a call to clxAllocatePrivateMemoryPoolBuffer. If the function detects memory corruption, BLACKBOX will occur.

\param[ in ] buf A pointer to the buffer to be freed. This shall be the same the value returned by clxAllocatePrivateMemoryPoolBuffer.
*/
void clxFreePrivateMemoryPoolBuffer(void* buf);

/**
Determines if there is a free buffer available in the private memory pool.

\param[ in ] pool The handle to the private pool as returned by #clxCreatePrivateMemoryPool.
\param[ in ] size The size of the buffer required.

\return TRUE if there is a free buffer available.
FALSE if there is no free buffer, or the argument 'size' exceeds the maximum allocation size (as passed to #clxCreatePrivateMemoryPool).
*/
boolean clxIsPrivateMemoryPoolBufferAvailable(ClxPrivateMemoryPoolHandle poolHandle, u4 size);

/**
Destroys the private memory pool by freeing any memory allocated for it. The freeMemFunc function passed to #clxCreatePrivateMemoryPool will be used to free the memory.
After this function returns, the handle to the private memory will be invalid and SHALL not be used any longer.

\param[ in ] pool The handle to the private pool as returned by #clxCreatePrivateMemoryPool.
\param[ in ] freeMemFunc The function which is called by #clxDestroyPrivateMemoryPool() to free the memory allocated for the private memory pool. This argument may be NULL only if
allocMemFunc (as passed to clxCreatePrivateMemoryPool) was also NULL. Otherwise, it shall match allocMemFunc.
*/
void clxDestroyPrivateMemoryPool(ClxPrivateMemoryPoolHandle poolHandle, ClxFreeMemFunc freeMemFunc);

/**
Checks the entire private memory pool for corruption. If corruption is detected, BLACKBOX(s) will occur.

\param[ in ] pool The handle to the private pool as returned by #clxCreatePrivateMemoryPool.

\return If no BLACKBOX occurs, the function will return the number of buffers which are currently free (not allocated). This value can be used, for instance right before
destroying the private pool by a call to #clxDestroyPrivateMemoryPool, in oder to make sure all buffers are freed 
(in this case, return value shall be equal to numberOfBuffers as passed to #clxCreatePrivateMemoryPool).  
*/
u4 clxCheckPrivateMemoryPoolForCorruption(ClxPrivateMemoryPoolHandle poolHandle);


#ifdef __cplusplus
}
#endif



#ifdef __cplusplus

/**
Base class for a C++ object which is to be allocated in a private memory pool.
NOTE : It might not be safe to allocate a C++ object which does not derive from ClxPrivatePoolAllocator
in a private memory pool (e.g. for instance by using placement new operator). Always use this base class.

An instance of a class which derives from ClxPrivatePoolAllocator CANNOT be allocated anywhere other than a private memory pool.
Therefore, in order to allocate an object of the arbitrary type T (which does not derive from ClxPrivatePoolAllocator) in a private memory pool,
a new class can be defined which multiply derives from both T and ClxPrivatePoolAllocator, as follows:

// T is an arbitrary C++ class which has a VIRTUAL DESTRUCTOR
class T_PrivatePoolAllocator : public T, public ClxPrivatePoolAllocator
{
public:
	// Define any required constructor here if T has non-default constructors
};

In order to allocate an instance of T in a private pool called 'privatePool' :

T* obj = new(privatePool) T_PrivatePoolAllocator (constructor_args);

In order to destruct and de-allocate obj from the private pool:

delete obj;

However, T SHALL have a virtual destructor. Otherwise, the behaviour will be undefined.
*/
class ClxPrivatePoolAllocator
{
private:
	/* Cannot be used to allocate an object of this class (or a derived class): */
    void* operator new (size_t size)
    {
		assert(0);
        return NULL;
    }
    
	/* Cannot be used to allocate an object of this class (or a derived class): */
    void* operator new[] (size_t size)
    {
		assert(0);
        return NULL;
    }

	/* Cannot be used to allocate an object of this class (or a derived class): */
	void* operator new (size_t size, u2 moduleID, u4 line, const ClxPoolsetUniqueIdentifier& identifier)
	{
		assert(0);
		return NULL;
	}

	/* Cannot be used to allocate an object of this class (or a derived class): */
	void* operator new[] (size_t size, u2 moduleID, u4 line, const ClxPoolsetUniqueIdentifier& identifier)
	{
		assert(0);
		return NULL;
	}

public:
    virtual ~ClxPrivatePoolAllocator() 
    {}

    void* operator new (size_t size, ClxPrivateMemoryPoolHandle poolHandle)
    {
		return clxAllocatePrivateMemoryPoolBuffer(poolHandle, (u4)size);
    }

	void* operator new[](size_t size, ClxPrivateMemoryPoolHandle poolHandle)
    {
		return clxAllocatePrivateMemoryPoolBuffer(poolHandle, (u4)size);
    }

	void operator delete(void* buf, ClxPrivateMemoryPoolHandle poolHandle)
    {
        clxFreePrivateMemoryPoolBuffer(buf);
    }

	void operator delete[](void* buf, ClxPrivateMemoryPoolHandle poolHandle)
    {
        clxFreePrivateMemoryPoolBuffer(buf);
    }

    void operator delete(void* buf)
    {
        clxFreePrivateMemoryPoolBuffer(buf);
    }

    void operator delete[](void* buf)
    {
        clxFreePrivateMemoryPoolBuffer(buf);
    }
};


#endif

#endif // _ClxPrivateMemoryPool_
