#ifndef _ClxQueueTemplate_
#define _ClxQueueTemplate_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxQueueTemplate.h
* Description         ClxQueueTemplate header file
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include <assert.h>

#ifdef __cplusplus

/**
Generic template for a container object which is able to queue objects as a circular doubly-linked list.

The queued objects will be of type QueueableBase which could be either a C++ class or C structure. QueueableBase must at least have
the following two members:

struct (or class) QueueableBase
{
    QueueableBase* prev_;
    QueueableBase* next_;

    // Optionally other members
};

Since this is a circular linked list, both prev_ and next_ SHALL initially be set to a pointer to their parent object (of type QueueableBase).

The actual objects which are queued into the linked list could be directly of type QueueableBase (in case QueueableBase has more members than just prev_ and next_),
a class deriving from QueueableBase (in C++ code only), a structure or class which has a member of type QueueableBase, or any other approach, as long as the following
can be met:

- It should be possible to extract a pointer to the actual queued object from the pointer to its corresponding QueueableBase object. This is due to the fact, only a pointer
to QueueableBase object is stored in the container and not to its corresponding actual parent object. 
*/
template<typename QueueableBase>
class ClxQueueTemplate : public QueueableBase
{
private:
    /**
    A queue cannot be copied via its copy constructor.
    */
    ClxQueueTemplate(const ClxQueueTemplate& c) {}

protected:
    ClxQueueTemplate() : QueueableBase()
    {
        QueueableBase::prev_ = this;
        QueueableBase::next_ = this;    
    }

    ~ClxQueueTemplate()
    {
        QueueableBase::next_->prev_ = QueueableBase::prev_;
        QueueableBase::prev_->next_ = QueueableBase::next_;
        QueueableBase::prev_ = this;
        QueueableBase::next_ = this;    
    }

    /**
    queues a new item to this container by appending it AFTER a given existing item in the container.
    If the container is used as a FIFO (Item1 first, ItemN last), the existing item will be extracted first.


    Example:
    This container before the operation:
        
      |--> ItemN --> RootItem (This container) --> Item1 --> Item2 ---|
      |-------------------------- ... --- Item3 <---------------------|

    After calling appendAfter(NewItem, Item2) :

      |--> ItemN --> RootItem (This container) --> Item1 --> Item2 --> NewItem ---|
      |-------------------------- ... --- Item3 <---------------------------------|


    \param[ in ] newItem The new item to be queued into the container. This item cannot have already
    been queued in the same or different container.

    \param[ in ] prevItem The existing item, after which the new item will be added. This item must already have been
    queued in this container object.
    */
    void appendAfter( QueueableBase* newItem, QueueableBase* prevItem )
    {
        if (newItem->prev_ == newItem)
        {
            assert( newItem->next_ == newItem );
            newItem->prev_ = prevItem;
            newItem->next_ = prevItem->next_;
            prevItem->next_->prev_ = newItem;
            prevItem->next_ = newItem;
        }
    }

    /**
    queues a new item to this container by appending it BEFORE a given existing item in the container.
    If the container is used as a FIFO (Item1 first, ItemN last), the new item will be extracted first.

    Example:
    This container before the operation:

        
      |--> ItemN --> RootItem (This container) --> Item1 --> Item2 ---|
      |-------------------------- ... --- Item3 <---------------------|

    After calling appendBefore(NewItem, Item2) :

      |--> ItemN --> RootItem (This container) --> Item1 --> NewItem -> Item2 -----|
      |-------------------------- ... --- Item3 <----------------------------------|


    \param[ in ] newItem The new item to be queued into the container. This item cannot have already
    been queued in the same or different container.

    \param[ in ] nextItem The existing item, before which the new item will be added. This item must already have been
    queued in this container object.
    */
    void appendBefore( QueueableBase* newItem, QueueableBase* nextItem )
    {
        if (newItem->prev_ == newItem)
        {
            assert( newItem->next_ == newItem );
            newItem->next_ = nextItem;
            newItem->prev_ = nextItem->prev_;
            nextItem->prev_->next_ = newItem;
            nextItem->prev_ = newItem;
        }
    }

    /**
    adds zero or more new items to this container by appending all of them AFTER a given existing
    item in the container. The new items are transferred from another container object to this object. The order of the new items
    will be reserved. 
    After the appending operation, the passed queue will be empty (since all its items have been transferred to this container 
    and an item cannot be queued into two containers at the same time).


    Example:
    This containers before the operation:
        
      |--> Item1N --> RootItem10 (queue1) --> Item11 --> Item12 -------|
      |-------------------------- ... --- Item13 <---------------------|


      |--> Item2M --> RootItem20 (queue2) --> Item21 --> Item22 -------|
      |-------------------------- ... --- Item23 <---------------------|


    After calling queue1.appendAfter(queue2, Item12) :


      |--> Item1N --> RootItem10 (queue1) --> Item11 --> Item12 --> Item21 --> Item22 ---|
      |----------------- ... --- Item13 <-- Item2M <---- .... --- Item23 <---------------|


      |----> RootItem20 (queue2) ---|
      |-----------------------------|


    \param[ in ] queue The queue, all items of which will be transferred to this container. This queue will be empty
    when this function returns. If this queue is already empty, calling of this function will have no effect.

    \param[ in ] prevItem The existing item, after which the new items will be added. This item must already have been
    queued in this container object.
    */
    void appendAfter( ClxQueueTemplate& queue, QueueableBase* prevItem )
    {
        if (!queue.isEmpty())
        {
            queue.next_->prev_ = prevItem;
            queue.prev_->next_ = prevItem->next_;
            prevItem->next_->prev_ = queue.prev_;
            prevItem->next_ = queue.next_;

            /* We clear queue so it will not have any items any more: */
            queue.prev_ = &queue;
            queue.next_ = &queue;        
        }
    }

    /**
    adds zero or more new items to this container by appending all of them BEFORE a given existing
    item in the container. The new items are transferred from another container object to this object. The order of the new items
    will be reserved. 
    After the appending operation, the passed queue will be empty (since all its items have been transferred to this container 
    and an item cannot be queued into two containers at the same time).


    Example:
    This containers before the operation:
        
      |--> Item1N --> RootItem10 (queue1) --> Item11 --> Item12 -------|
      |-------------------------- ... --- Item13 <---------------------|


      |--> Item2M --> RootItem20 (queue2) --> Item21 --> Item22 -------|
      |-------------------------- ... --- Item23 <---------------------|


    After calling queue1.appendBefore(queue2, Item12) :


      |--> Item1N --> RootItem10 (queue1) --> Item11 --> Item21 --> Item22 ------------------|
      |---------------- .... --- Item13 <-- Item12 <-- Item2M <-- .... --- Item23 <----------|


      |----> RootItem20 (queue2) ---|
      |-----------------------------|

    \param[ in ] queue The queue, all items of which will be transferred to this container. This queue will be empty
    when this function returns. If this queue is already empty, calling of this function will have no effect.

    \param[ in ] nextItem The existing item, before which the new items will be added. This item must already have been
    queued in this container object.
    */
    void appendBefore( ClxQueueTemplate& queue, QueueableBase* nextItem )
    {
        if (!queue.isEmpty())
        {
            queue.prev_->next_ = nextItem;
            queue.next_->prev_ = nextItem->prev_;
            nextItem->prev_->next_ = queue.next_;
            nextItem->prev_ = queue.prev_;

            /* We clear queue so it will not have any items any more: */
            queue.prev_ = &queue;
            queue.next_ = &queue;
        }
    }

    /*
    detach() does not check whether "item" has been queued into
    this queue. The user must assure that.
    */
    void detach( QueueableBase* item )
    {
        item->next_->prev_ = item->prev_;
        item->prev_->next_ = item->next_;
        item->prev_ = item;
        item->next_ = item;
    }

    /**
    Checks if a given item is a valid item. A valid item is an item which is not this container itself.
    The function is used when traversing the container object. The traversal can be in forward direction (starting from ClxQueueTemplate.first())
    or in backward direction (starting from ClxQueueTemplate.last()). Since the container is a circular linked list, the traversal will never result
    in a NULL pointer. But, the traversal will eventually result in getting back to the root of the linked list, which is this container itself (as
    an object deriving from QueueableBase). This function can be called to check if the traversal attempt has ended.

    Example:
        ClxQueueTemplate<SomeType> queue;

        SomeType* current = queue.firstItem();

        while(queue.checkValidity(current))
        {
            // Process current
            current = current->next_;
        }
    */
    boolean checkValidity(QueueableBase* item) const
    {
        return (item != this);
    }

public:
    /**
    Returns the first item in the container. If the container is empty, the return value will be a pointer to this container, itself.
    */
    QueueableBase* first() const
    {
        return QueueableBase::next_;
    }

    /**
    Returns the last item in the container. If the container is empty, the return value will be a pointer to this container, itself.
    */
    QueueableBase* last() const
    {
        return QueueableBase::prev_;
    }

    /**
    Returns TRUE if this container is currently empty (there is item queued inside the this container at this moment),
    or FALSE otherwise.
    */
    boolean isEmpty() const
    {
        if (QueueableBase::next_ == this)
        {
            assert (QueueableBase::prev_ == this);
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }
};


/**
A C++-style queue for objects which are pure C structures.
Refer to ClxCQueueable.h for more information about the structure of C queueable objects (objects which can be queued in a queue of type ClxCQueue).
*/
template<typename CStruct>
class ClxCQueue : public ClxQueueTemplate<ClxCQueueable>
{
public:
    static void initItem(CStruct& item)
    {
        item.queueable.prev_ = &item.queueable;
        item.queueable.next_ = &item.queueable;
    }

    static boolean isItemQueued(CStruct& item)
    {
        return !(item.queueable.prev_ == item.queueable.next_);
    }

    static CStruct* next(CStruct& item)
    {
        return reinterpret_cast<CStruct*>(item.queueable.next_);
    }

    static CStruct* prev(CStruct& item)
    {
        return reinterpret_cast<CStruct*>(item.queueable.prev_);
    }

    void pushBack(CStruct* item)
    {
        appendBefore (&item->queueable, this);
    }

    void pushFront(CStruct* item)
    {
        appendAfter (&item->queueable, this);
    }

    CStruct* popFromFront()
    {
        if (isEmpty()) 
        {
            return NULL;
        }

        ClxCQueueable* ret = next_;
        detach(ret);
        return reinterpret_cast<CStruct*>(ret);
    }

    CStruct* popFromBack()
    {
        if (isEmpty()) 
        {
            return NULL;
        }

        ClxCQueueable* ret = prev_;
        detach(ret);
        return reinterpret_cast<CStruct*>(ret);
    }

    void detachItem(CStruct* item)
    {
        detach(&item->queueable);
    }

    void appendQueueBack(ClxCQueue& queue)
    {
        appendBefore(queue, this);
    }

    void appendQueueFront(ClxCQueue& queue)
    {
        appendAfter(queue, this);
    }

    boolean isValid(CStruct* item) const
    {
        return checkValidity(&item->queueable);
    }
};

#endif     // __cplusplus

#endif    // _ClxQueueTemplate_
