#ifndef _ClxSemaphore_
#define _ClxSemaphore_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxSemaphore.h
* Description         Includes target Operating System dependent definitions
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

// _____________________________________________________________________________
//
// ClxSemaphore
// _____________________________________________________________________________
//

#ifdef __cplusplus
extern "C" {
#endif


/**
A type for holding the semaphore handle for the current operating system.
Clarinox Semaphore APIs use this semaphore handle for most of the semaphore operations.
*/
typedef void* ClxSemaphore;

/**
The purpose of this function is to create a semaphore and initialize it.
Semaphore pointer is returned to be used in the other semaphore APIs.
\param[ in ] initialValue is the initial value of the semaphore.
\param[ in ] file is the file name where this function is called. Used for the debug version of the stack.
\param[ in ] line is the line number where this function is called. Used for the debug version of the stack.
Usually zero to indicate no events have occurred on the semaphore.
\return ClxSemaphore
\retval Non-zero         ClxSemaphore handle
\retval zero             clxCreateSemaphore failed
*/
extern  ClxSemaphore clxCreateSemaphore_( s4 initialValue, const char* file, unsigned int line );
#define clxCreateSemaphore( X ) clxCreateSemaphore_( X, __FILE__, __LINE__ )

/**
This function creates a binary semaphore and initializes it.
A binary semaphore is used for controlling only a single resource.
\param[ in ] initialValue is the initial value of the semaphore.
Usually zero to indicate no events have occurred on the semaphore.
\param[ in ] file is the file name where this function is called. Used for the debug version of the stack.
\param[ in ] line is the line number where this function is called. Used for the debug version of the stack.
\return ClxSemaphore
\retval Non-zero         ClxSemaphore handle
\retval zero             clxCreateBinarySemaphore failed
*/
extern  ClxSemaphore clxCreateBinarySemaphore_( s4 initialValue, const char* file, unsigned int line );
#define clxCreateBinarySemaphore( X ) clxCreateBinarySemaphore_( X, __FILE__, __LINE__ )

/**
This function deletes the semaphore specified by the handle.
** <!-- Parameters -->
\param[ in ] semaphore is the handle for the semaphore
*/
extern  void       clxDeleteSemaphore              ( ClxSemaphore semaphore );

/**
This function helps to restrict access to a shared resource and lock it for other's use.
The acquire operation decrements the value of the semaphore. If the value is already zero
then the current thread starts waiting for the semaphore to be released.
\param[ in ] semaphore is the handle for the semaphore
*/
extern  void       clxAcquireSemaphore             ( ClxSemaphore semaphore );

/**
The purpose of this function is to release an access to a shared resource
by unlocking it for other's use. If there are no threads blocked for this
semaphore then this call increments the value of the semaphore.
If there is a thread waiting for this semaphore, then the scheduler adds that thread into
the ready list for execution. Execution time would depend on the operating system's scheduling settings.
\param[ in ] semaphore is the handle for the semaphore
*/
extern  void       clxReleaseSemaphore             ( ClxSemaphore semaphore );

/**
This function is a non-blocking version of clxAcquireSemaphore.
\sa clxAcquireSemaphore. It does function very similar to the
clxAcquireSemaphore, except if the value of the semaphore is already zero then the function
returns with a failed status. If the resource is available,
then the function works exactly the same way as clxAcquireSemaphore would do.
\param[ in ] semaphore is the handle for the semaphore
\return status
\retval CLX_SUCCESS Operation completed successfully.
\retval CLX_FAIL    Semaphore is already taken
*/
extern  ClxResult  clxTryAcquireSemaphore          ( ClxSemaphore semaphore );

/**
This function is a timed version of clxAcquireSemaphore. It is very similar to the
clxAcquireSemaphore, except if the resource is busy it waits for the time given in milliseconds
parameter. The status returns as fail if the resource is not released within this waiting period.
\param[ in ] semaphore is the handle for the semaphore
\param[ in ] milliseconds is the time out period in milliseconds
\return status
\retval CLX_SUCCESS Operation completed successfully.
\retval CLX_FAIL    Semaphore could not be taken within given time (timed out)
*/
extern  ClxResult  clxAcquireSemaphoreTimed        ( ClxSemaphore semaphore, s4 milliseconds );

/**
ClxSemaphore.h contains the semaphore implementation
*/

#ifdef __cplusplus
}
#endif


#endif    // _ClxSemaphore_

