#ifndef __ClxSocketWrapper_h__
#define __ClxSocketWrapper_h__

/*******************************************************************************
*
* Project             ClarinoxWLAN
* File                ClxSocketWrapper.h
* Description         Declares ClxSocket related classes, types and macros
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#if defined ( CLX_SOCKETS_SUPPORT ) && defined (__cplusplus)


/**
Maximum size of the a socket address which an instance of ClxSocketAddressContainer can hold.
*/
#define CLX_SOCKETS_MAX_ADDRESS_SIZE        64



/**
ClxSocketAddressType enumerates different socket address types. 
*/
namespace ClxSocketAddressType
{
    enum Enum
    {
        Unknown = 0x0000,
        IP4     = 0x0001,
        IP6     = 0x0002
    };
}


/**
An instance of ClxSocketAddressContainer contains a socket (local or remote) address. 
This object does NOT represent a socket address of a specific type. It is only a container which can hold a socket address of any type.
When constructed, the container will not contain any valid address. A separate object (e.g. ClxSocketIp4Address) is used to encode a socket address to
an instance of ClxSocketAddressContainer. Also, an address stored in a ClxSocketAddressContainer object may be decoded into another object (e.g. ClxSocketIp4Address)
in order for the developer to access the actual parameters of the address (e.g. IP address and port).

All addresses passed to methods of a ClxSocket object are of type ClxSocketAddressContainer.
*/
class ClxSocketAddressContainer
{
protected:
    CLX_ALIGNED_BUFFER_4 (buf_, CLX_SOCKETS_MAX_ADDRESS_SIZE);

private:
    ClxSize  length_;

public:
    /**
    Initialize the object. This method is not be to called directly. It is called by encode() method of socket address objects (e.g. ClxSocketIp4Address).
    */
    void init(ClxSize length);

    /**
    Returns the type of the socket address currently stored in this container object. If there is address currently stored in this container,
    a value of ClxSocketAddressType::Invalid is returned.
    */
    ClxSocketAddressType::Enum  type() const;

public:
    /**
    Constructs an instance of ClxSocketAddressContainer which initially does not contain any valid socket address.
    */
    ClxSocketAddressContainer() : length_(0) {}

    /**
    Returns a constant void pointer to the internal container buffer. Not to be called by the developer. To be called by ClxSocket
    methods only.
    */
    const void* operator* () const  { return (const void*)buf_; }

    /**
    Returns a void pointer to the internal container buffer. Not to be called by the developer. To be called by ClxSocket
    methods only.
    */
    void* operator* () { return (void*)buf_; }

    /**
    Returns the length of the socket address currently stored in this container. Not to be called by the developer. To be called by ClxSocket
    methods only.
    */
    ClxSize length() const { return length_; }
};


/**
An instance of ClxSocketIp4Address represents a local or remote socket address including a IPv4 address and a port number. The port is either a UDP or TCP port.
In order to pass an address of this type to a ClxSocket object, the contents of this object must be encoded into an object of type ClxSocketAddressContainer by calling
ClxSocketIp4Address::encode() method. Also, in order to access the IPv4 address and port number stored in an object of type ClxSocketAddressContainer (if the method type() of
the ClxSocketAddressContainer object returns ClxSocketAddressType::IP4), the contents of the ClxSocketAddressContainer object can be decoded into an object of type ClxSocketIp4Address by calling
the ClxSocketIp4Address::decode() method.

NOTE : An instance of this class stores the IP address and port number of the the natural byte order (endianness) of the underlying platform, and NOT in the network byte order. If a conversion is required,
it will be automatically performed by this object.
*/
class ClxSocketIp4Address
{
private:
    u4 ip_;
    u2 port_;

public:
    /**
    Constructs an object of type ClxSocketIp4Address with IPv4 address and port number set to 0.
    */
    ClxSocketIp4Address() : ip_(0), port_(0) {}

    /**
    Constructs an object of type ClxSocketIp4Address with the specified IPv4 address and port number.
    
    \param[ in ] ip The IP address as a 4-byte integer encoded in the natural byte order (endianness) of the underlying platform.
    \param[ in ] port The port number as a 2-byte integer encoded in the natural byte order (endianness) of the underlying platform.
    */
    ClxSocketIp4Address(u4 ip, u2 port) : ip_(ip), port_(port) {}

    /**
    Constructs an object of type ClxSocketIp4Address and copies the contents of another object into this object.

    \param[ in ] c An object of type ClxSocketIp4Address which contents are to be copied into this newly-constructed object.
    */
    ClxSocketIp4Address(const ClxSocketIp4Address& c) : ip_(c.ip_), port_(c.port_) {}

    /*
    Returns the IPv4 address as a 32bit integer in the natural byte order (endianness) of the underlying platform.
    */
    u4 ip() const { return ip_; }

    /*
    Returns the TCP/UDP port number as a 16bit integer in the natural byte order (endianness) of the underlying platform.
    */
    u2 port() const { return port_; }

    /*
    Sets the IPv4 address. arg is in the natural byte order (endianness) of the underlying platform.
    */
    void ip(u4 arg) { ip_ = arg; }

    /*
    Sets the TCP/UDP port. arg is in the natural byte order (endianness) of the underlying platform.
    */
    void port(u2 arg) { port_ = arg; }

public:
    /*
    Extracts the IP address and/or the port from the ASCII representation.
    The ASCII representation (as pointed to by str argument) may be in one of the following formats:
        
        - a.b.c.d  IPv4 address where a, b, c, and d are decimal numbers ranging 0 to 255. Port number (as already stored in the object) will not change.

        - :xxxxx Port number where xxxxx is a decimal number ranging 0 to 65535. IP address (as already stored in the object) will not change.

        - a.b.c.d:xxxxx IPv4 and port number. Both IP address and port number will be stored in this object.
    
    If the format of either part (IP address or Port number) is not correct, the value already stored for that part in this object will not change. Each part
    is extracted independently.
    */
    ClxSocketIp4Address& operator=(const s1* str);

public:
    /**
    Encodes the current contents of this object into a socket address container of type ClxSocketAddressContainer.
    
    \param[ in ] container A reference to a container object of type ClxSocketAddressContainer to which the contents of this object is to be encoded. If container already
    contains a valid socket address, it will be overwritten.
    */
    void encode(ClxSocketAddressContainer& container) const;

    /**
    Decodes a socket address of type ClxSocketAddressType::IP4 stored in a container object into this object.

    \param[ in ] container A reference to a container object which contains a valid socket address of type ClxSocketAddressType::IP4 (as returned by container.type() method).
    \return TRUE if the socket address in the container object was successfully decoded into this object.
    FALSE if the container object does not contain a valid address of type ClxSocketAddressType::IP4.
    */
    boolean decode(const ClxSocketAddressContainer& container);
};


/**
Enumerates the type of a socket which is represented by an instance of ClxSocket class. 
From the socket type, the Layer 2, Layer 3 or Layer 4 protocol which is associated to an object of type ClxSocket is determined.
*/
namespace ClxSocketType
{
    enum ENUM
    {
        Invalid = 0,            /*!< The ClxSocket object is not associated to a valid socket */
        Stream,                 /*!< The ClxSocket object is associated to a TCP socket */
        Datagram,               /*!< The ClxSocket object is associated to a UDP socket */
        Raw,                    /*!< The ClxSocket object is associated to a Layer3 IP socket (the packets contain the IP header and all upper layer headers) */
        Ethernet                /*!< The ClxSocket object is associated to a Layer2 Ethernet socket (the packets contain Ethernet2 header and all upper layer headers) */
    };
}


class ClxSocket;


/****************************
ClxSocketSet
****************************/
class ClxSocketSet
{
protected:
    ClxSocketSet() {}
public:
    void                    reset();
    void                    add(ClxSocket& socket);
    boolean                 isInSet(ClxSocket& socket);
public:
    static ClxSocketSet&    create();
    static void             destroy(ClxSocketSet& set);
};



/**
An instance of ClxSocket represents a network socket. An object of type ClxSocket may represent a network socket belonging to the underlying platform, or an internal implementation.
In either case, The behaviour of a ClxSocket object (and its methods) is only a function of the socket type (as passed to the constructor) and independent from the underlying platform.

IMPORTANT : An object of ClxSocket is NOT copy-able nor clone-able. Therefore, only one instance of ClxSocket can represent an underlying network socket. An instance of ClxSocket CANNOT be associated
to an existing underlying network socket. Therefore, there is always a one-to-one relationship between a ClxSocket object and its associated network socket entity.

When a ClxSocket (which is already associated to a valid network socket) is destructed, the underlying associated network socket will be automatically closed. In multi-threaded environment, the application
must make sure the ClxSocket object will not be deleted until it is no longer used by any thread. Any thread is able to close the network socket by calling ClxSocket::close() method of the ClxSocket object.
*/
class ClxSocket 
{
protected:
    void* socket_;

protected:
    /**
    Not to be used directly.
    */
    ClxSocket(void* socket);

private:
    /** 
    A ClxSocket object cannot be copied by a copy constructor. 
    */
    ClxSocket(const ClxSocket& c) : socket_(NULL) {}

    /** 
    A ClxSocket object cannot be copied by assignment operator. 
    */
    ClxSocket& operator= (const ClxSocket& c) { return *this; }

public:
    /**
    Constructs an object of type ClxSocket, and associates it to a new underlying network socket. It also binds the network socket to a local address.
    The type of the ClxSocket object and the type of the local address, together, determine the protocols which are to be used with this network socket.
    
    \param[ in ] localAddress the local address to which this socket is to bind. The type of the local address (as returned by localAddress.type() method) must be compatible with the
    type of the socket which is passed as the second argument of this constructor. For instance, if socketType is ClxSocketType::Stream, ClxSocketType::Datagram or ClxSocketType::Raw,
    the type of the local address must be either ClxSocketAddressType::IP4 or ClxSocketAddressType::IP6. In any case, the type of the local address CANNOT be ClxSocketAddressType::Invalid.
    \param [ in ] socketType The type of the underlying network socket which this object will represent. The value CANNOT be ClxSocketType::Invalid.
    \param [ in ] nonblockingMode If TRUE, the socket will be in non-blocking mode. In this mode, all methods of this class will return immediately, and the requested task will complete asynchronously.
    If FALSE, the will be in blocking mode. In this mode, all requested tasks on the network socket will complete synchronously (the methods of this class will not return until the requested task is complete).
    \param[ out ] result A reference to a caller-provided variable of type ClxResult which on return will contain the result of the associating this object to an underlying network socket. If this value is not
    CLX_SUCCESS, this object is useless and must be destructed immediately.

    NOTE : In case of an error, this constructor will NOT throw an exception. So, the only way to know if there has been an error is to check the returned value of the result variable.
    */
    ClxSocket (const ClxSocketAddressContainer& localAddress, ClxSocketType::ENUM socketType, boolean nonblockingMode, ClxResult& result);
    
    /**
    Destructs this object. If the object is currently associated to a valid underlying network socket, it will be closed and all resources associated to that network socket will be freed.
    */
    virtual ~ClxSocket(); 

    /**
    Establishes a connection to a peer socket. If successful, this socket can then only send to and receive from this peer socket. Based on the actual type of the socket, this function may try to establish
    a connection to a remote device by protocol handshaking (e.g. in case of TCP), or it may just internally set the target of all sending and receiving operations to this address (e.g. in case of UDP).

    If this is a non-blocking socket, this function will return immediately. If the return value is CLX_SOCKET_EWOULDBLOCK, the connection cannot complete synchronously. Use #select method to establish the connection instead.
    If this is a blocking socket, this function will not return until the connection procedure is complete (either with success or in error).
    
    NOTE : Some socket types might not implement this function (e.g. ClxSocketType::Raw or ClxSocketType::Ethernet). In this case, this function will return an error.

    \param[ in ] peerAddress The address of the peer or remote device. The type of this address MUST be the same as the type of the local address as passed to the constructor.

    \return CLX_SUCCESS if the connection procedure has been successful.
    CLX_SOCKET_EWOULDBLOCK if the socket is in non-blocking mode and the connection procedure cannot complete asynchronously. Use #select method instead.
    Any other value will indicate an error.
    */
    ClxResult connect(const ClxSocketAddressContainer& peerAddress);

    /**
    Places the network socket into listening mode. A listening socket cannot be used to connect to a peer socket, or send and receive data. It can only be used to listen for incoming connection requests.
    Call this function when the local application is to act as a server.

    If this function succeeds, the application can accept incoming connection requests by calling #accept() method.

    \param[ in ] maxNumberOfPendingRequests Maximum number of connection requests which can be pending at any given time. Use this argument to limit the number of connections that the local server is able to handle.
    
    \return CLX_SUCCESS if the socket was successfully placed in listening mode.
    Any other value will indicate an error.
    */
    ClxResult listen(u4 maxNumberOfPendingRequests); 

    /**
    Accepts a connection request from a peer socket and returns a new ClxSocket object which is associated to this new connection. Before this function is called, the #listen() method must have been called in order
    to place this network socket into listening mode.

    In case of the listening socket being in the non-blocking mode, if there is no connection request pending when this function is called, the function will return immediately with the error CLX_SOCKET_EWOULDBLOCK.
    In case of the listening socket being in the blocking mode, if there is no connection request pending when this function is called, the function will block until a connection request is received and then accept it.

    \param[ out ] peerAddress A reference to a caller-provided container object of type ClxSocketAddressContainer which on a successful return will hold the address of the peer socket. The type of the address will be the same of the
    type of the local address as passed to the constructor of this object.
    
    \param[ out ] result A reference to a caller-provided to a variable of type ClxResult which on return will hold the result of the operation. This value will be as follows:
    CLX_SUCCESS if a connection request was successfully accepted. In this case, the return ClxSocket object will be associated to a valid underlying network socket.
    CLX_SOCKET_EWOULDBLOCK if the socket is in non-blocking mode and there is no incoming connection request currently pending to be accepted. Use #socket() method to wait for an incoming connection request and then call
    this function to accept the connection request.
    Any other value will indicate an error. In this case, and the CLX_SOCKET_EWOULDBLOCK case, the return ClxSocket object will not be associated to a valid underlying network socket 
    (ClxSocket::isValid() method for the returned object will return FALASE).

    \return A new ClxSocket object which is associated to a valid network socket which represents the new accepted connection (its isValid() method will return TRUE).
    If the procedure was not successful, this object will not be valid (its isValid() method will return FALSE), and must be destroyed as it cannot be used for any purpose.
    */
    ClxSocket accept(ClxSocketAddressContainer& peerAddress, ClxResult& result);

    /**
    Sends data to the connected peer socket. Based on the type of the network socket, The data will be sent as a separate packet, or might be encoded in one or more packets as part of a stream. 

    \param[ in ] data A pointer to the data which is to be sent to the connected peer socket.
    \param[ in ] length The length of the data, in bytes, to be sent to the peer socket. This can be 0 which, on a successful return, will set the #sent argument to 0 as well.
    \param[ out] sent A reference to a caller-provided variable of type ClxSize which on a successful return will hold the length of data, in bytes, which was actually sent
    to the peer socket. This value might be less than the length of the data which was passed to this function. If the function returns CLX_SUCCESS, this value can be set to 0 only if
    #length was 0 as well. If the function returns any value other than CLX_SUCCESS, this value must be ignored. 

    \return CLX_SUCCESS if the operation was successful, the value of #sent argument specifies the number of bytes which were actually sent to the peer socket.
    CLX_SOCKET_EWOULDBLOCK if the socket is in non-blocking mode and the sending operation cannot complete synchronously at this moment, call #select() method to wait until the socket is ready to accept outgoing
    data and then call this function to send data to the peer socket.
    Any other value will indicate an error.
    */
    ClxResult send(const u1* data, ClxSize length, ClxSize& sent);

    /**
    Sends data to an arbitrary remote socket. This is used when there is no connection to a peer socket. This function cannot be used with socket types which require a connection to a peer socket (e.g. TCP).
    Based on the type of the network socket, The data will be sent as a separate packet, or might be encoded in one or more packets as part of a stream. 

    This function can be used to send data to a single remote socket or multicast/broadcast it to several relevant remote sockets.

    \param[ in ] peerAddress A container object of type ClxSocketAddressContainer which must contain the address of the remote socket to which the data is to be sent.
    \param[ in ] data A pointer to the data which is to be sent to the remote socket.
    \param[ in ] length The length of the data, in bytes, to be sent to the remote socket. This can be 0 which, on a successful return, will set the #sent argument to 0 as well.
    \param[ out] sent A reference to a caller-provided variable of type ClxSize which on a successful return will hold the length of data, in bytes, which was actually sent
    to the remote socket. This value might be less than the length of the data which was passed to this function. If the function returns CLX_SUCCESS, this value can be set to 0 only if
    #length was 0 as well. If the function returns any value other than CLX_SUCCESS, this value must be ignored. 

    \return CLX_SUCCESS if the operation was successful, the value of #sent argument specifies the number of bytes which were actually sent to the remote socket.
    CLX_SOCKET_EWOULDBLOCK if the socket is in non-blocking mode and the sending operation cannot complete synchronously at this moment, call #select() method to wait until the socket is ready to accept outgoing
    data and then call this function to send data to the remote socket.
    Any other value will indicate an error.
    */
    ClxResult sendTo(const ClxSocketAddressContainer& peerAddress, const u1* data, ClxSize length, ClxSize& sent);

    /**
    Receives data from the connected peer socket and copies it into the caller-provided memory buffer. Based on the socket type, the received data might be a single packet or part of the incoming data stream.
    In the former case, if the size of the buffer provided is smaller than the size of the received packet, some part of the data in the packet may be lost.

    \param[ out ] buf A caller-provided buffer which on a successful return will contain the received data.
    \param[ in ] length Length of the caller-provided buffer, in bytes. This is the maximum length of the data which can be copied. The actual received data might be smaller.
    \param[ out ] received A reference to a caller-provided variable of type ClxSize which on a successful return will hold the actual number of bytes which were copied into the provided memory buffer.
    If this function returns CLX_SUCCESS, this value can be 0 only if the connection to the peer socket was closed before any data was received.

    \return CLX_SUCCESS if the operation was successful, the value of #received argument specifies the number of bytes which were actually copied into the provided buffer.
    CLX_SOCKET_EWOULDBLOCK if the socket is in non-blocking mode and there is no incoming data received from the peer socket and pending at this moment. call #select() method to wait until data is received from the peer socket, and
    then call this function is copy the received data into the provided buffer.
    Any other value will indicate an error.
    */
    ClxResult recv(u1* buf, ClxSize length, ClxSize& received);

    /**
    Receives data from any remote socket and copies it into the caller-provided memory buffer. This function cannot be used with socket types which require a connection to a peer socket (e.g. TCP).
    Based on the socket type, the received data might be a single packet or part of the incoming data stream.
    In the former case, if the size of the buffer provided is smaller than the size of the received packet, some part of the data in the packet may be lost.

    This function can be used to receive both unicast data (destined to our local address) and multicast/broadcast data.

    \param[ out ] peerAddress A reference to a caller-provided container object of type ClxSocketAddressContainer which on a successful return will contain the address of the remote socket from which the
    data has been received. The type of this address will be the same as the type of the local address as passed to the constructor.
    \param[ out ] buf A caller-provided buffer which on a successful return will contain the received data.
    \param[ in ] length Length of the caller-provided buffer, in bytes. This is the maximum length of the data which can be copied. The actual received data might be smaller.
    \param[ out ] received A reference to a caller-provided variable of type ClxSize which on a successful return will hold the actual number of bytes which were copied into the provided memory buffer.

    \return CLX_SUCCESS if the operation was successful, the value of #received argument specifies the number of bytes which were actually copied into the provided buffer.
    CLX_SOCKET_EWOULDBLOCK if the socket is in non-blocking mode and there is no incoming data received from a remote socket and pending at this moment. call #select() method to wait until data is received from a remote socket, and
    then call this function is copy the received data into the provided buffer.
    Any other value will indicate an error.
    */
    ClxResult recvFrom(ClxSocketAddressContainer& peerAddress, u1* buf, ClxSize length, ClxSize& received);

    /**
    Returns the local address of this socket.

    \param[ in ] localAddress A reference to a caller-provided container object of type ClxSocketAddressContainer which on a successful return will contain the local address of this socket.

    \return CLX_SUCCESS if the local address was retrieved successfully.
    Any other value will indicate an error.
    */
    ClxResult localAddress(ClxSocketAddressContainer& localAddress) const;

    /**
    Returns the address of the peer socket to which there is a connection.

    \param[ in ] peerAddress A reference to a caller-provided container object of type ClxSocketAddressContainer which on a successful return will contain the address of the peer socket.

    \return CLX_SUCCESS if the address of the peer socket was retrieved successfully.
    Any other value will indicate an error.
    */
    ClxResult peerAddress(ClxSocketAddressContainer& peerAddress) const;

    /**
    Sends an IOCTL request to the network interface or driver to which to socket is binding.

    NOTE : IOCTL requests are not standardized and each platform uses its own set of IOCTL requests. In order to make sure the code will be portable, avoid using
    this function.

    \param[ in ] request The type of the request as an integer.
    \param[ inout ] arg A pointer to the request data. The size of this data is determined by the type of the request. Based on the request type,
    this might be request input parameters, request output parameters, or a combination of both.

    \return CLX_SUCCESS The IOCTL request was successfully handled.
    Any other value will indicate an error.
    */
    ClxResult ioctl(u4 request, s1* arg);

    /**
    Returns the last error which is the result of the last socket operation which was performed in the context of the calling thread.

    NOTE : The last error is not associated to operations of a special socket, but to the operations of all sockets as long as they have been carried out in the context of the calling thread.
    That is why the function has been implemented as static.
    Each thread has its own last error indicator.

    This is basically the same as the return value of the last ClxSocket method which was called in the context of the calling thread.

    \return CLX_SUCCESS The last operation in the context of the calling thread has been successful.
    Any other value will indicate an error.
    */
    static ClxResult getLastError();

    /**
    Specifies if this object is valid. A ClxSocket object is valid if it is associated to a valid underlying network socket.
    This function is only used to check the validity of the ClxSocket object as returned by #accept() method.

    NOTE : If the ClxSocket object has initially been associated to a valid underlying network socket, but then the network socket became invalid for any reason (e.g. it was closed or the connection was
    disrupted), this function will STILL return TRUE. The returned value of this method only refers to the initial validity of the object.
    */
    boolean isValid() const;

    ClxResult select(ClxSocketSet* readfds, ClxSocketSet* writefds, ClxSocketSet* exceptfds, ClxTimeval *clxTimeout);

    /**
    Closes the OS socket associated to this object. After this function returns, this object cannot be associated to another OS socket. Therefore, it has to 
    be destructed and, if required, a new instance of ClxSocket shall be used for a new socket. 
    */
    void close();

public:
    /**
    Initializes the socket interface on the underlying platform. In order to make sure the code will be portable, this function MUST be called in all platforms.
    The function must be called before any ClxSocket object can be constructed and used.
    */
    static ClxResult initSocketInterface();

    /**
    Terminates the socket interface on the underlying platform. This function must be called when there is no need for socket interface and network programming any more.
    In order to make sure the code will be portable, this function MUST be called in all platforms.
    */
    static ClxResult terminateSocketInterface();

public:
    /**
    Used internally only. Not to be called by the developer.
    */
    void* osObject() const { return socket_; }
};



#endif     // defined ( CLX_SOCKETS_SUPPORT ) || defined (__cplusplus)

#endif   // __ClxSocketWrapper_h__
