#ifndef _ClxStandard_
#define _ClxStandard_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxStandard.h
* Description         Clarinox standard functions
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

///Unicode character type.
typedef s2 UniChar;

/**
Duplicate memory.
\param[ in ] data buffer to be duplicated.
\param[ in ] length size of \p data.
\return A new buffer whose content and size are same those of \p data.
*/
extern  void* clxMemDup( void* data, u4 length );

/**
Set memory with a specific byte value.
\param destination memory start address to be set.
\param c byte value to be used to set the memory.
\param count size of the area to be set in bytes.
\return \p destination if success, 0 otherwise.
*/
extern  void* clxMemSet( void* destination, s4 c, u4 count );

/**
Move memory content from one buffer to another.
\param destination destination buffer
\param source source buffer
\param n number of bytes that must be moved.
\return \p destination if success, 0 otherwise.
*/
extern  void* clxMemMove( void* destination, const void* source, u4 n );

/**
Copy memory content from one buffer to another.
\param destination destination buffer
\param source source buffer
\param n number of bytes that must be copied.
\return \p destination if success, 0 otherwise.
*/
extern  void* clxMemCpy( void* destination, const void* source, u4 n );

/**
Compare memory contents between two buffers.
\param source source buffer.
\param destination destination buffer.
\param n Number of bytes that must be compared.
\return 0 if same, the first encountered difference if not the same.
*/
extern  s4 clxMemCmp( const void* source, const void* destination, u4 n );

/**
Compare memory contents between two buffers. Comparison is case insensitive.
\param source source buffer.
\param destination destination buffer.
\param n Number of bytes that must be compared.
\return 0 if same, the first encountered difference if not the same.
*/
extern  s4 clxMemCmpCaseInsensitive ( const void* source, const void* destination, u4 n );
/**
Evaluate the length of a NULL terminated string.
\param str the string to be evaluated.
\return length of the string.
\note The evaluated length does not include the NULL character itself.
*/
extern  u4 clxStrLen( const s1* str );

/**
Compare two strings.
\param source source string.
\param destination destination string.
\return 0 if same, the first encountered difference if not the same.
*/
extern  s4 clxStrCmp( const s1* source, const s1* destination );

/**
Compare two strings.
\param source source string.
\param destination destination string.
\param n number of characters to be compared.
\return 0 if same, the first encountered difference if not the same.
*/
extern  s4 clxStrNCmp( const s1* source, const s1* destination, s4 n );

/**
Copy a string.
\param source source string.
\param destination destination string.
\return \p destination if success, 0 otherwise.
*/
extern  s1* clxStrCpy( s1* destination, const s1* source );

/**
Copies the first \p num characters of \p source to \p destination.
If the end of the \p source string (which is signalled by a null-character) is
found before num characters have been copied, \p destination is padded with
zeros until a total of \p num characters have been written to it.

No null-character is implicitly appended to the end of \p destination, so
\p destination will only be null-terminated if the length of the string in
\p source is less than num.

\param source string to be copied.
\param destination destination array where the content is to be copied.
\param num maximum number of characters to be copied from source
\return \p destination if success, 0 otherwise.
*/
extern  s1* clxStrNCpy( s1* destination, const s1* source, s4 num );

/**
Concatenate strings.
Appends a copy of the \p source string to the \p destination string.
The terminating null character in \p destination is overwritten by the first
character of \p source, and a new null-character is appended at the end of the
new string formed by the concatenation of both in \p destination.
\param source String to be appended. This should not overlap \p destination.
\param destination String to append to. Shall be large enough to contain the
       concatenated resulting string.
\return \p destination if success, 0 otherwise.
*/
extern  s1* clxStrCat( s1* destination, const s1* source );

/**
Append characters from string.
Appends the first \p num characters of \p source to \p destination,
plus a terminating null-character.
If the length of the string in \p source is less than \p num,
only the content up to the terminating null-character is copied.
\param[ in ] destination Destination array, which should contain a string,
             and be large enough to contain the concatenated resulting string,
             including the additional null-character.
\param[ in ] source string to be appended.
\param[ in ] num Maximum number of characters to be appended.
\return \p destination if success, 0 otherwise.
*/extern  s1* clxStrNCat( s1* destination, const s1* source, u4 num );

/**
Duplicate a string.
\param str Null terminated string to be duplicated.
\return A new string with the same content as that in \p str.
*/
extern  s1* clxStrDup( const s1* str );

/**
Find first occurrence of \p character in \p source string.
\param source Null terminated source string.
\param character character to be searched.
\return Pointer to the found character, 0 if not found.
*/
extern  s1* clxStrChr( const s1* source, s4 character );

/**
Find first occurrence of \p c in \p buffer.
\param buf memory buffer to search in. It doesn't need to be null-terminated. 
\param c character to be searched.
\param len size of buf in bytes.
\return Pointer to the found character, 0 if not found.
*/
extern  const void* clxMemChr(const void* buf, s4 c, u4 len);

/**
Find first occurrence of \p str string in \p buf memory buffer of size \p len. The search is case insensitive.
\param buf buffer to search str in.
\param str Null terminated sub string.
\param len size of buf in bytes.
\return Pointer to the first character of the found string inside the buffer, 0 if not found.
*/
extern  void* clxMemStrCaseInsensitive( void* buf, const s1* str, u4 len);

/**
Find last occurrence of \p character in \p source string.
\param source Null terminated source string.
\param character character to be searched.
\return Pointer to the found character, 0 if not found.
*/
extern  s1* clxRStrChr( const s1* source, s4 character );

/**
Find first occurrence of \p sub string in \p source string.
\param source Null terminated source string.
\param sub Null terminated sub string.
\return Pointer to the first occurrence of the sub string, 0 if not found.
*/
extern  s1* clxStrStr( const s1* source, const s1* sub );

/**
Find first occurrence of \p sub string in \p source memory buffer.
\param source Memory in which to look for substring.
\param destination Null terminated sub string.
\param sourceLen length of the memory buffer
\return Pointer to the first occurrence of the sub string, 0 if not found.
*/
extern void* clxMemStr( const void* source, const s1* destination,  u4 sourceLen);

/**
Converts a memory block from hex string (0 to 9, A to F, a to f) to unsigned integer. 
The conversion starts from the left-most character and continues until a non-hex character 
is encountered, or end of the buffer is reached. A maximum of 8 non-zero left-most 
characters will be considered (0x00000000 to 0xFFFFFFFF).
This function is the Hex counterpart of atoi() standard function.
\param[ in ] buf The buffer which holds the Hex number
\param[ in ] bufLen The length of the buffer
\return an unsigned integer containing the converted Hex number.
*/
extern u4 clxHexToInteger( void* buf, u4 bufLen ); 

/**
Convert a memory block from binary to ASCII. The result will have a "0x" prefix before the value.

The size of destination must be at least (2 + twice "length")

\param[ out ] destination The destination address which holds the converted ASCII value
\param[ in ] source The source address which contains data in hex format
\param[ in ] length The number of bytes that have to be converted to ASCII.
\return ClxResult code.
\retval CLX_SUCCESS Operation completed successfully.
\return CLX_FAIL clxBinaryToAscii failed.
*/
extern  ClxResult clxBinaryToAscii( u1* destination, const u1* source, u4 length );

/**
Convert a memory block from ASCII to binary.
\param[ in ] destination The destination address which holds the converted hex
             value.
\param[ in ] source The source address which contains data in ASCII format.
\param[ in ] length The number of bytes that have to be converted to hex
\return ClxResult code.
\retval CLX_SUCCESS Operation completed successfully.
\retval CLX_FAIL clxAsciiToBinary failed.
*/
extern  ClxResult clxAsciiToBinary( u1* destination, const u1* source, s4 length );

/**
Converts all characters in a string to uppercase characters.

\param[ in ] s the string to be converted to uppercase.
\return The same as s. This function changes the characters in the original buffer.
No new buffer will be created.
*/
extern  s1* clxStrUpr (s1* s);

/**
Converts all characters in a string to lowercase characters.

\param[ in ] s the string to be converted to lowercase.
\return The same as s. This function changes the characters in the original buffer.
No new buffer will be created.
*/
extern  s1* clxStrLwr (s1* s);

#if 0
/**
Convert null terminated ASCII string to null terminated Unicode string.
\param[ in ] str ASCII string to be converted.
*/
UniChar* convert( const s1* str );
s1* convert( const UniChar* str );
#endif

/**
Converts a big endian UCS2-formatted (two-bytes-per-character) ASCII string into normal single-byte-per-character representation.

\param[ in ] ucs2String A string of characters in UCS2 big endian format. The string is not null terminated.
\param[ in ] ucs2StringLength The length of ucs2String in bytes (NOT characters).
\param[ out ] asciiString A caller provided buffer into which to store the ASCII string. The size of this buffer
must be at least half of ucs2StringLength plus one byte for null character (ucs2StringLength/2 + 1). The result will be
null-terminated.

ret If TRUE the conversion was done successfully.
     If FALSE, the string stored in ucs2String is in big endian UCS2 format. 
*/
boolean convertUCS2BEToASCII( const u1* ucs2String, u4 ucs2StringLength, u1* asciiString);

/**
Converts a little endian UCS2-formatted (two-bytes-per-character) ASCII string into normal single-byte-per-character representation.

\param[ in ] ucs2String A string of characters in UCS2 little endian format. The string is not null terminated.
\param[ in ] ucs2StringLength The length of ucs2String in bytes (NOT characters).
\param[ out ] asciiString A caller provided buffer into which to store the ASCII string. The size of this buffer
must be at least half of ucs2StringLength plus one byte for null character (ucs2StringLength/2 + 1). The result will be
null-terminated.

ret If TRUE the conversion was done successfully.
     If FALSE, the string stored in ucs2String is in little endian UCS2 format. 
*/
boolean convertUCS2LEToASCII( const u1* ucs2String, u4 ucs2StringLength, u1* asciiString);

/** \file ClxStandard.h
    \brief Provides the standard function support for consistent execution.
*/


#endif    // _ClxStandard_
