#ifndef _ClxThread_
#define _ClxThread_

/******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxThread.h
* Description         Declares ClxThread APIs
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

/**
A typedef for generic thread id. ClxThread is not a pointer to the thread.
For more high level event driven architectures, use Module class.
\sa Module
*/
typedef void* ClxThreadID;
typedef void* ClxThreadHandle;

#define CLX_CALLBACK

/**
A typedef for thread function pointer.
*/
typedef ClxResult ( CLX_CALLBACK *ClxThreadFunction )( void* data );

/**
Clarinox internal thread priority index used by wireless stacks. 
If a user directly uses create thread function, they are not bound by these priority settings.
*/
enum ClxThreadPriority
{
    ClxThreadPriority_Highest = 0,
    ClxThreadPriority_High    = 1,
    ClxThreadPriority_Medium  = 2,
    ClxThreadPriority_Low     = 3,
    ClxThreadPriority_Lowest  = 4
};

/**
This function creates a thread in the system.
\param function is a pointer to the entry point of the thread
\param parameter is a generic parameter that may be passed to the thread
\param name is the name associated with the thread.
\param stackSize is the size of the stack required to run the thread.
\param platformPriorityTable priority table array, defines Clarinox thread priorities as defined in ClxThreadPriority enum set. 
Clarinox code internally uses these values for the threads required by the wireless stacks used.
\param priority current priority assigned for this specific thread.
A zero value indicates the default stack size is being used.
\return The thread handle. It is used for terminating the thread.
Will be NULL if the function was not successful.
*/
extern ClxThreadHandle  clxBeginThread( ClxThreadFunction function, void* parameter, const char* name, u4 stackSize, u2* platformPriorityTable = NULL, ClxThreadPriority priority = ClxThreadPriority_Medium);

/**
Deletes the thread. This function blocks until the thread has terminated.
Then it will cleanup after the thread.

\param[ in ] threadHandle the handle to the thread.
\param[ in ] timeout is the value in milliseconds for waiting graceful termination. 
When the thread returns or timeout occurs, then the RTOS delete function will be executed.
\remark This function does NOT force the thread to terminate.
It only waits until the thread terminates on its own.
*/
extern boolean          clxDeleteThread(ClxThreadHandle threadHandle, u4 timeout = 0xFFFFFFFF);

/**
The purpose of this function is to allow thread to relinquish control of the processor to another thread,
which has the same or higher priority.

This function can only be called within the context of a thread and will only work if the other thread
has the same priority.
\return status
\retval CLX_SUCCESS Operation completed successfully.
\retval CLX_FAIL    function failed.
*/
extern void             clxYieldCurrentThread();

/**
Each thread is assigned with a unique ID by the operating system;
this function gets the current thread id.
\return ClxThread
\retval current thread id.
*/
extern ClxThreadID        clxGetCurrentThreadId();

/**
Suspends the thread with the handle of threadHandle.
NOTE : Support of this operation is optional. If not supported by the underlying
OS (or BSP), a value of CLX_ERROR_INVALID_REQUEST will be returned.
*/
extern ClxResult          clxSuspendThread(ClxThreadHandle threadHandle);

/**
Resumes the thread with the handle of threadHandle. The thread must have been previously suspended
by a call to clxSuspendThread() function.
NOTE : Support of this operation is optional. If not supported by the underlying
OS (or BSP), a value of CLX_ERROR_INVALID_REQUEST will be returned.
*/
extern ClxResult          clxResumeThread(ClxThreadHandle threadHandle);

/**
In a multi-core platform, binds a thread to be scheduled and executed on only one of the processor cores.

NOTE : Support of this operation is optional. If not supported by the underlying
OS (or BSP), a value of CLX_SUCCESS will be returned.
*/
extern ClxResult          clxBindThreadToProcessor(ClxThreadHandle threadHandle, u4 processorIndex);

/**
Thread API functionality.
*/

#endif    // _ClxThread_
