#ifndef _TerminalEmulator_
#define _TerminalEmulator_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                TerminalEmulator.h
* Description         Declares TerminalEmulator
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Terminal Emulator is a SoftFrame module which emulates a text-based terminal (input and output) on a remote machine. In debug version, this module
communicates with ClarinoxDebugger virtual console via the same communication link used to send debugging data. In release version, a 
dedicated communication link will be assigned to Terminal Emulator. Over this dedicated link, raw text will be sent and received. Therefore, 
any standard Terminal Emulation application on the remote machine can be used (e.g. HyperTerminal, TetaTerm, PuTTY, ...). Terminal Emulator 
supports TCP/IP and Serial Ports.

If enabled, the Terminal Emulator communication link is established during Clarinox SoftFrame initialization, and terminated during Clarinox SoftFrame
termination. In release version, the user is able to provide the communication link configuration settings in the file SoftFrame.cfg. All configuration
settings' names start with TE_ (short for Terminal Emulator). Refer to the contents of the standard SoftFrame.cfg provided by Clarinox for more
information.

When enabled, the application developer can use Terminal Emulator API functions (defined below) to send and receive text and characters to and from the remote
machine. These functions can ONLY be used after the initialization of ClarinoxSoftFrame.

NOTE : It is safe to call these functions if the Terminal Emulator API is not enabled. In this case, these functions will return an error. Make sure these
functions are not running when the module is being terminated. Otherwise, the behaviour of these functions will be undefined.

Clarinox internal IO operations:
In rare cases, the code provided by Clarinox may need to use a text-based terminal for user interaction. The most common example is the Console User Interface Engine
(refer to ConsoleUIEngine.h). In these cases, all output operations are redirected to the user-defined print function, called userPrintfFunction. This is a function
pointer which can be set to a user-defined function. Also, all input operations are redirected to the user-defined function pointers userGetCharFunction (when a single character is
required) and userGetInputFunction (when a text string is required to be provided by the user). The Clarinox internal code never directly calls Terminal Emulator API functions. 

If it is desired to use the Terminal Emulator module for these internal IO operations, these user-defined functions must explicitly call Terminal Emulator API functions. In the sample BSP file
provided by Clarinox for each project, a sample implementation of these functions are provided which depicts how this can be achieved. Contact Clarinox if you do not have access to these sample functions.

NOTE : All Terminal Emulator API functions (except for clxTerminalEmulatorPrint) are thread-safe.
*/

/**
Specifies if the Terminal Emulator module is enabled at this moment. In debug version, the module is enabled as long as the debug sub-system is enabled and connected to ClarinoxDebugger.
In release version, the module is enabled if it has been configured by the user via the file SoftFrame.h and a communication link has been established to the remote machine.  

\return TRUE if the module is enabled. FALSE if it is not enabled.
*/
boolean clxTerminalEmulatorEnabled();

/*
Obtains a single (one-byte) character from the remote machine. If a character has already been received and stored in the internal buffer, it will be returned. Otherwise,
this function will block until a character is received from the remote machine, or an error occurs.

\param[ in ] echo If TRUE, the received character will be echoed back to the remote machine.
\param[ in ] asciiOnly If TRUE, only visible ASCII characters (32 and above) will be accepted as the input. Any non-ASCII received character will be ignored.
\param[ in ] flushInputQueue If TRUE, all characters already received and stored in the internal buffer will be removed. This function will wait for new characters.

\return this is the character received from the remote machine (of type s1).
If an error has occurred, the return value will be -1 (IMPORTANT : Check that the return value is not -1 before casting it to s1).
*/
s4 clxTerminalEmulatorGetChar(boolean echo,
                              boolean asciiOnly,
                              boolean flushInputQueue);

/**
Obtains a string of characters from the remote machine. The function will store the input characters in a caller-provided buffer, and it will not return
until a caller-defined specific character (called termination signal) is received, or the buffer becomes full, or an error occurs.
The returned string will not include the termination signal (it will be replaced by the null-termination character).

\param[ in ] buffer A caller-provided buffer which, on return, will hold the received input. If the function returns CLX_SUCCESS, The buffer will include a null-terminated string.
Otherwise, the contents of this buffer will be undefined.
\param[ in ] bufferSize The size of the buffer provided by the caller. This includes the null-termination character. Therefore, the maximum size of the returned string will be bufferSize-1.
\param[ in ] terminationSignal The character which the function will wait to receive before it returns. Normally, this will be a Carriage Return (0x0D) character. But, any character may be used. 
\param[ in ] echo If TRUE, the received characters will be echoed back to the remote machine.
\param[ in ] asciiOnly If TRUE, only visible ASCII characters (32 and above) will be accepted as the input. Any non-ASCII received character will be ignored.
\param[ in ] flushInputQueue If TRUE, all characters already received and stored in the internal buffer will be removed. This function will wait for new characters.

/return CLX_SUCCESS if the operation has been successful
CLX_FAIL otherwise
*/
ClxResult clxTerminalEmulatorGetInput(s1* buffer, 
                                      u4 bufferSize, 
                                      s1 terminationSignal,
                                      boolean echo,
                                      boolean asciiOnly,
                                      boolean flushInputQueue);

/**
Sends a string of characters to the remote machine.

NOTE : This function is not thread-safe. If two or more threads try to use this function at the same time, the result will
be undefined. If this is not desirable, an external protection mechanism must be employed.

\param[ in ] str The string of characters to be sent to the module. This string does not need to be null-terminated.
\param[ in ] strLen The length of the string provided by the caller.
*/
void clxTerminalEmulatorPrint(const u1* str, ClxSize strLen);


#ifdef __cplusplus
}
#endif


#endif    // _TerminalEmulator_

